﻿IF OBJECT_ID('TR__ActionGroupActions__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__ActionGroupActions__ValidateSequence]
GO

CREATE TRIGGER [TR__ActionGroupActions__ValidateSequence]
       ON [ActionGroupActions] FOR INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [ActionGroupActions] AGA
				   INNER JOIN (SELECT [ActionGroup_Id], [Action_Sequence],
									  [Actual_Action_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [ActionGroup_Id]
										   ORDER BY [ActionGroup_Id], [Action_Sequence])
									  FROM [ActionGroupActions]) AGA_ACTUAL
						   ON AGA.[ActionGroup_Id] = AGA_ACTUAL.[ActionGroup_Id]
						  AND AGA.[Action_Sequence] = AGA_ACTUAL.[Action_Sequence]
			 WHERE AGA.[Action_Sequence] <> AGA_ACTUAL.[Actual_Action_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE AGA.[ActionGroup_Id] = [ActionGroup_Id])
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE AGA.[ActionGroup_Id] = [ActionGroup_Id])
				   )
		) 
	BEGIN

		RAISERROR('An invalid Action_Sequence was provided for the ActionGroup.', 16, 1)
		ROLLBACK

	END

END
GO

GO
IF OBJECT_ID('TR__DeployableDependencies__CircularDependencyCheck') IS NOT NULL
DROP TRIGGER [TR__DeployableDependencies__CircularDependencyCheck]
GO

CREATE TRIGGER [TR__DeployableDependencies__CircularDependencyCheck]
ON [DeployableDependencies] AFTER UPDATE, INSERT AS
BEGIN

	SET NOCOUNT ON

	DECLARE @ResolvedDependencies TABLE(
		[Base_Deployable_Id] INT NOT NULL, 

		[Resolution_Depth] INT NOT NULL,
		[Dependant_Deployable_Id] INT NOT NULL, 
		[DependsOn_Deployable_Id] INT NOT NULL
	)

	DECLARE @depth INT, @row_count INT
	    SET @depth = 1
	    SET @row_count = 1 
	 
	INSERT INTO @ResolvedDependencies 
	  ([Base_Deployable_Id], [Resolution_Depth], [Dependant_Deployable_Id], [DependsOn_Deployable_Id])
	SELECT 
	   [Deployable_Id], 1, [Deployable_Id], [DependsOn_Deployable_Id]
	  FROM [inserted] 

	WHILE @row_count > 0
	BEGIN 
		SET @depth = @depth + 1

		-- insert all of the previous depth's dependecies from the actual table
		INSERT INTO @ResolvedDependencies 
		  ([Base_Deployable_Id], [Resolution_Depth], [Dependant_Deployable_Id], [DependsOn_Deployable_Id])
		SELECT
		  [Base_Deployable_Id], @depth, RD.[DependsOn_Deployable_Id], DD.[DependsOn_Deployable_Id]
		FROM @ResolvedDependencies RD
	         INNER JOIN [DeployableDependencies] DD
		             ON RD.[DependsOn_Deployable_Id] = DD.[Deployable_Id]
		WHERE RD.[Resolution_Depth] = @depth - 1

		SET @row_count = @@ROWCOUNT
	
		-- if a inserted deployable depends on itself (somewhere down the line) then shed tears
		IF EXISTS( SELECT * 
		             FROM [inserted] I
		                  INNER JOIN @ResolvedDependencies RD
		                          ON RD.[Resolution_Depth] > 1
		                         AND I.[Deployable_Id] = RD.[Base_Deployable_Id]
		                         AND I.[Deployable_Id] = RD.[Dependant_Deployable_Id] )

		BEGIN 
				RAISERROR('This insertion will cause a circular dependecy at depth %i, which is not supported.',16,2, @depth) 
				ROLLBACK
				RETURN 
		END 

		-- sanity check
		IF @depth > 1000 BEGIN
				RAISERROR('Depth exceeded 1000.',16,1) 
				ROLLBACK
				RETURN 
		END 
	END 


END
GO

GO
IF OBJECT_ID('[TR__DeployableDependencies__ValidateDeployableId]') IS NOT NULL DROP TRIGGER [TR__DeployableDependencies__ValidateDeployableId]
GO

CREATE TRIGGER [TR__DeployableDependencies__ValidateDeployableId] 
       ON [DeployableDependencies] FOR UPDATE, INSERT 
AS 
BEGIN
	IF EXISTS(SELECT *
				FROM [inserted] i
					 INNER JOIN [Deployables] D1 ON D1.[Deployable_Id] = i.[Deployable_Id]
					 INNER JOIN [Deployables] D2 ON D2.[Deployable_Id] = i.[DependsOn_Deployable_Id]
			   WHERE D1.[Application_Id] <> D2.[Application_Id])
		BEGIN
			RAISERROR('Cross-application deployables are not supported.', 16, 1)
			ROLLBACK
		END
END
GO

GO
IF OBJECT_ID('[TR__DeploymentPlanActionGroups_ValidateGoto]') IS NOT NULL DROP TRIGGER [TR__DeploymentPlanActionGroups_ValidateGoto]
GO

CREATE TRIGGER [TR__DeploymentPlanActionGroups_ValidateGoto] 
       ON [DeploymentPlanActionGroups] FOR UPDATE, INSERT
AS 
BEGIN
	IF EXISTS(SELECT *
				FROM [inserted] i
					 LEFT JOIN [DeploymentPlanActionGroups] DPAG_Success
					        ON DPAG_Success.[DeploymentPlanActionGroup_Id] = i.[OnSuccess_DeploymentPlanActionGroup_Id]
			   WHERE DPAG_Success.[OnSuccess_DeploymentPlanActionGroup_Id] IS NOT NULL
			     AND DPAG_Success.[DeploymentPlan_Id] <> i.[DeploymentPlan_Id])
		BEGIN
			RAISERROR('OnSuccess_DeploymentPlanActionGroup must refer to a group in the same plan.', 16, 1)
			ROLLBACK
		END

	IF EXISTS(SELECT *
				FROM [inserted] i
					 LEFT JOIN [DeploymentPlanActionGroups] DPAG_Failure
					        ON DPAG_Failure.[DeploymentPlanActionGroup_Id] = i.[OnFailure_DeploymentPlanActionGroup_Id]
			   WHERE DPAG_Failure.[OnFailure_DeploymentPlanActionGroup_Id] IS NOT NULL
			     AND DPAG_Failure.[DeploymentPlan_Id] <> i.[DeploymentPlan_Id])
		BEGIN
			RAISERROR('OnFailure_DeploymentPlanActionGroup must refer to a group in the same plan.', 16, 1)
			ROLLBACK
		END
END
GO

GO
IF OBJECT_ID('TR__DeploymentPlanActionGroups__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__DeploymentPlanActionGroups__ValidateSequence]
GO

CREATE TRIGGER [TR__DeploymentPlanActionGroups__ValidateSequence]
       ON [DeploymentPlanActionGroups] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [DeploymentPlanActionGroups] DPAG 
				   INNER JOIN (SELECT [DeploymentPlanActionGroup_Id], [DeploymentPlanActionGroup_Sequence],
									  [Actual_PlanActionGroup_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [DeploymentPlan_Id]
										   ORDER BY [DeploymentPlan_Id], [DeploymentPlanActionGroup_Sequence])
									  FROM [DeploymentPlanActionGroups]) DPAGS
						   ON DPAG.[DeploymentPlanActionGroup_Id] = DPAGS.[DeploymentPlanActionGroup_Id]
			 WHERE DPAG.[DeploymentPlanActionGroup_Sequence] <> DPAGS.[Actual_PlanActionGroup_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE DPAG.[DeploymentPlan_Id] = [DeploymentPlan_Id] )
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE DPAG.[DeploymentPlan_Id] = [DeploymentPlan_Id] )
				   )
		) 
	BEGIN

		RAISERROR('An invalid DeploymentPlanActionGroup_Sequence was provided for the DeploymentPlanActionGroup.', 16, 1)
		ROLLBACK

	END

END
GO

GO
IF OBJECT_ID('[TR__EventOccurenceDetails__ValidateDetailValue]') IS NOT NULL DROP TRIGGER [TR__EventOccurenceDetails__ValidateDetailValue]
GO

CREATE TRIGGER [TR__EventOccurenceDetails__ValidateDetailValue] 
       ON [EventOccurenceDetails] FOR UPDATE, INSERT 
AS 
BEGIN
	IF EXISTS
	(
		SELECT *
		  FROM inserted /*[EventOccurenceDetails]*/ EOD
			   INNER JOIN [EventTypeDetails] ETD
					   ON ETD.[Event_Code] = EOD.[Event_Code]
					  AND ETD.[Detail_Name] = EOD.[Detail_Name]
		 WHERE ETD.[Detail_Type] <> sql_variant_property(EOD.[Detail_Value], 'BaseType')
	)
		BEGIN
			RAISERROR('Event detail value/type mismatch.', 16, 1)
			ROLLBACK
		END
END
GO



GO

IF OBJECT_ID('[TR__ReleaseDeployables__ValidateDeployableId]') IS NOT NULL DROP TRIGGER [TR__ReleaseDeployables__ValidateDeployableId]
GO

CREATE TRIGGER [TR__ReleaseDeployables__ValidateDeployableId] 
       ON [ReleaseDeployables] FOR UPDATE,INSERT 
AS 
BEGIN

	/* If a deployable ID is found for the given application, or an imported deployabled ID is found, return without error */
	IF @@ROWCOUNT = 0
	OR EXISTS(SELECT * FROM [Deployables] D
			INNER JOIN [inserted] i ON 
				i.[Application_Id] = D.[Application_Id]
			AND	i.[Deployable_Id] = D.[Deployable_Id])
	OR EXISTS(SELECT * FROM [DeployablesImported] D
			INNER JOIN [inserted] i ON
				i.[Application_Id] = D.[Application_Id]
			AND	i.[Deployable_Id] = D.[Deployable_Id]
			AND i.[Application_Id] <> i.[Referenced_Application_Id])
	RETURN

	RAISERROR('An invalid Deployable ID was provided for this release', 16, 1)
	ROLLBACK
	
END
GO


GO
IF OBJECT_ID('TR__Releases__ValidateReleaseNumber') IS NOT NULL DROP TRIGGER [TR__Releases__ValidateReleaseNumber]
GO

GO
IF OBJECT_ID('TR__Releases__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__Releases__ValidateSequence]
GO

CREATE TRIGGER [TR__Releases__ValidateSequence]
       ON [Releases] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [Releases] R 
				   INNER JOIN (SELECT [Application_Id], 
                                      [Release_Number],
									  [Actual_Release_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [Application_Id]
										        ORDER BY [Application_Id], [Release_Sequence])
									  FROM [Releases]) R_ACTUAL
						   ON R.[Application_Id] = R_ACTUAL.[Application_Id]
                          AND R.[Release_Number] = R_ACTUAL.[Release_Number]
			 WHERE R.[Release_Sequence] <> R_ACTUAL.[Actual_Release_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE R.[Application_Id] = [Application_Id]
                                 AND R.[Release_Number] = [Release_Number] )
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE R.[Application_Id] = [Application_Id]
                                 AND R.[Release_Number] = [Release_Number] )
				   )
		) 
	BEGIN

		RAISERROR('An invalid Release_Sequence was provided for the Release.', 16, 1)
		ROLLBACK
	
	END
		
END
GO

GO
IF OBJECT_ID('TR__WorkflowSteps__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__WorkflowSteps__ValidateSequence]
GO

CREATE TRIGGER [TR__WorkflowSteps__ValidateSequence]
       ON [WorkflowSteps] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [WorkflowSteps] WS
				   INNER JOIN (SELECT [Workflow_Id],
				                      [Step_Sequence],
									  [Actual_Step_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [Workflow_Id]
										   ORDER BY [Workflow_Id], [Step_Sequence])
									  FROM [WorkflowSteps]) WS_ACTUAL
						   ON WS.[Workflow_Id] = WS_ACTUAL.[Workflow_Id]
						  AND WS.[Step_Sequence] = WS_ACTUAL.[Step_Sequence]
			 WHERE WS.[Step_Sequence] <> WS_ACTUAL.[Actual_Step_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE WS.[Workflow_Id] = [Workflow_Id])
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE WS.[Workflow_Id] = [Workflow_Id])
				   )
		) 
	BEGIN

		RAISERROR('An invalid Step_Sequence was provided for the Workflow Step.', 16, 1)
		ROLLBACK
	
	END
		
END
GO

GO
IF OBJECT_ID('ABS_MONTH_NUMBER') IS NOT NULL DROP FUNCTION ABS_MONTH_NUMBER
GO
CREATE FUNCTION ABS_MONTH_NUMBER
(
  @Date DATETIME
) RETURNS INT AS BEGIN
  
  RETURN
    /* Months Past Since Day 0 */
    ((DATEPART(yy,@Date) - 1900) * 12)
    /* Plus Months this year */
    + DATEPART(m,@Date)
END
GO

GO
IF OBJECT_ID('ABS_WEEK_NUMBER') IS NOT NULL DROP FUNCTION ABS_WEEK_NUMBER
GO
CREATE FUNCTION ABS_WEEK_NUMBER
(
  @DateTime DATETIME
) RETURNS INT AS BEGIN

  DECLARE @Dec DECIMAL(9,2)
  /* Convert to Days since 1900-01-01 */
  SET @Dec = CAST(@DateTime AS DECIMAL(9,2))
  /* Since 1900-01-01 is a Monday and Week start on Sunday, Add 1 */
  SET @Dec = @Dec + 1
  /* Floor of /7 is week # */
  RETURN FLOOR(@Dec/7)
END
GO

GO
IF OBJECT_ID('CompareBuildNumber') IS NOT NULL DROP FUNCTION CompareBuildNumber
GO

GO
IF OBJECT_ID('CompareReleaseNumber') IS NOT NULL DROP FUNCTION CompareReleaseNumber
GO

GO

IF OBJECT_ID('CsvToTable') IS NOT NULL DROP FUNCTION CsvToTable
GO

CREATE FUNCTION CsvToTable (@CSVString VARCHAR(MAX), @Delimiter	VARCHAR(10))
  RETURNS @tbl TABLE ([Value_Sequence] INT, [Value_Text] VARCHAR(MAX))

AS BEGIN 

    DECLARE @i INT, @j INT, @ctr INT
	SET @i = 1 

	WHILE @i <= len(@CSVString)
	BEGIN
		SET @j = CHARINDEX(@Delimiter, @CSVString, @i)
        SET @ctr = COALESCE(@ctr,0) + 1
		IF @j = 0 SET @j = LEN(@CSVString) + 1
		INSERT @tbl SELECT @ctr, SUBSTRING(@CSVString, @i, @j - @i)
		SELECT @i = @j + LEN(@Delimiter)
	END
	
    RETURN

END
GO



GO
IF OBJECT_ID('[GetSortableReleaseNumberWithValidation]') IS NOT NULL
	DROP FUNCTION [GetSortableReleaseNumberWithValidation]
GO

CREATE FUNCTION [GetSortableReleaseNumberWithValidation]
(
	@Release_Number RELEASENUMBER,
    @ReleaseNumber_Scheme_Name VARCHAR(20)
)
RETURNS @tbl TABLE(
  [Release_Sequence] DECIMAL(38,0), 
  [Valid_Indicator] CHAR(1), 
  [InvalidReason_Text] VARCHAR(100)) AS

BEGIN

    DECLARE @ERR VARCHAR(100), @RELNO DECIMAL(38,0)
    
    -- split into parts and digits
    DECLARE @Digits TABLE ([Part] INT, [Pos] INT, [Digit] CHAR(1), [Val] INT, [Seq] INT, [Pow] DECIMAL(38,0))
     INSERT @Digits
     SELECT [Part] = [Value_Sequence],
            [Pos] = LEN(R.[Value_Text]) - N.[Number] + 1,
            [Digit] = SUBSTRING(R.[Value_Text], N.[Number], 1),
            [Val] = NULL, [Seq] = NULL, [Pow] = NULL
      FROM  dbo.[CsvToTable](@Release_Number, '.') R INNER JOIN [Numbers] N
            ON N.[Number] BETWEEN 1 AND LEN(R.[Value_Text])
      WHERE N.[Number] BETWEEN 1 AND LEN(@Release_Number) --optimization
                                              
    -- set vals
    UPDATE @Digits SET [Val] = CHARINDEX([Digit], '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ') - 1

    -- validate
    IF @ReleaseNumber_Scheme_Name = 'MajorMinorRevision' BEGIN
      IF (SELECT MAX([Part]) FROM @Digits) <> 3 BEGIN SET @ERR='Release Numbers must have exactly three parts' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits) > 4 BEGIN SET @ERR='The maximum length for a part is 4' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Pos] > 1) BEGIN SET @ERR='Only the last digit of a Release Number part may be a character' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Part] NOT IN (SELECT [Part] FROM @Digits WHERE [Digit] LIKE '[0-9]')) BEGIN SET @ERR='Each part must contain a number' GOTO ERR END
    END ELSE IF @ReleaseNumber_Scheme_Name = 'MajorMinor' BEGIN
      IF (SELECT MAX([Part]) FROM @Digits) <> 2 BEGIN SET @ERR='Release Number must have exactly two parts'  GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=1) > 4 BEGIN SET @ERR='The maximum length for the first part is 4.' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=2) > 8 BEGIN SET @ERR='The maximum length for the second part is 8.' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Pos] > 1 AND [Part] = 1) BEGIN SET @ERR='Only the last digit of the first part may be a character' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Part] NOT IN (SELECT [Part] FROM @Digits WHERE [Digit] LIKE '[0-9]')) BEGIN SET @ERR='Each part must contain a number' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Part]=2) > 4 BEGIN SET @ERR='The maximum length for the non-numeric portion of the second part is 4' GOTO ERR END
    END ELSE IF @ReleaseNumber_Scheme_Name = 'DateBased' BEGIN
      IF (SELECT MAX([Part]) FROM @Digits) > 2 BEGIN SET @ERR='Release Number must have at most two parts' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=1) > 8 BEGIN SET @ERR='The maximum length for the first part is 8' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=2) > 4 BEGIN SET @ERR='The maximum length for the second part is 4' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Part] = 1) BEGIN SET @ERR='The first part may only contain numbers' GOTO ERR END
    END


    -- munge digits
    IF @ReleaseNumber_Scheme_Name = 'MajorMinorRevision' BEGIN
      -- shift all parts left one if there's no char in first position
      UPDATE @Digits SET [Pos] = [Pos] + 1
       WHERE [Part] NOT IN (SELECT [Part] FROM @Digits WHERE [Digit] LIKE '[A-Z]')

    END ELSE IF @ReleaseNumber_Scheme_Name = 'MajorMinor' BEGIN
      -- move all chars from part 2 to part 3
      UPDATE @Digits SET [Part] = 3 
       WHERE [Part] = 2
         AND [Pos] <= (SELECT MAX([Pos]) FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Part]=2)
      -- shift part 2 right by number of chars moved
      UPDATE @Digits SET [Pos] = [Pos] - (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=3)
       WHERE [Part] = 2 AND EXISTS (SELECT * FROM @Digits WHERE [Part]=3)

    END ELSE IF @ReleaseNumber_Scheme_Name = 'DateBased' BEGIN
      -- move part 1 to part 2, and part 2 to part 3
      UPDATE @Digits SET [Part] = [Part] + 1 WHERE [Part] IN (1,2)

    END


    /* 
    -- insert missing digits (debug)
    INSERT @Digits SELECT NPART.[Number], NPOS.[Number], '0', 0, NULL, NULL
     FROM [Numbers] NPART CROSS JOIN [Numbers] NPOS
          LEFT JOIN @Digits D ON NPOS.[Number] = D.[Pos] AND NPART.[Number] = D.[Part]
    WHERE NPART.[Number] BETWEEN 1 AND 3 AND NPOS.[Number] BETWEEN 1 AND 4
      AND D.[Part] IS NULL
    */

    -- set seqs, pow
    UPDATE @Digits SET [Seq] = (3 - [Part]) * 4  + [Pos]
    UPDATE @Digits SET [Pow] = POWER(CAST(36 AS DECIMAL(38,0)),[Seq]-1)

    -- SELECT * FROM @Digits ORDER BY [Part], [Pos] DESC -- (debug)
    
    SET @RELNO = (SELECT SUM([Pow] * [Val]) FROM @Digits)
    
    INSERT INTO @tbl VALUES(@RELNO, 'Y', NULL)
    GOTO RET

ERR:
    -- PRINT @ERR
    INSERT INTO @tbl VALUES(NULL, 'N', @ERR)

RET:
    RETURN

END
GO

GO
IF OBJECT_ID('IsAuthorizedForTask') IS NOT NULL DROP FUNCTION [IsAuthorizedForTask]
GO

CREATE FUNCTION [IsAuthorizedForTask]
(
	@Directory_Provider_Id INT,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX),
	@Task_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Environment_Id INT
) RETURNS BIT AS
BEGIN

	DECLARE @RETVAL BIT
	SET @RETVAL = 0

	SET @ApplicationGroup_Id = NULLIF(@ApplicationGroup_Id, 0)
	SET @Application_Id = NULLIF(@Application_Id, 0)
	SET @Environment_Id = NULLIF(@Environment_Id, 0)

	IF @Directory_Provider_Id IS NULL
		SET @Directory_Provider_Id = CAST((SELECT [Value_Text]
		                                     FROM [Configuration]
											WHERE [Scope_Name] = 'CoreEx'
											  AND [Key_Name] = 'DirectoryProvider') AS INT)

	/* If application group is not specified but the application is, look up the group */
	IF @ApplicationGroup_Id IS NULL AND @Application_Id IS NOT NULL
		SELECT @ApplicationGroup_Id = [ApplicationGroup_Id] FROM [Applications] WHERE [Application_Id] = @Application_Id

	SET @RETVAL = COALESCE(
	(
		SELECT TOP 1 CASE [PrivilegeType_Code]
                     WHEN 'G' THEN 1 WHEN 'D' THEN 0 END
		  FROM [Privileges] P
			   INNER JOIN [RoleTasks] RT
					   ON RT.[Role_Id] = P.[Role_Id]
			   INNER JOIN [Tasks] T
					   ON T.[Task_Id] = RT.[Task_Id]
		 WHERE (( /* User */
				   P.[PrincipalType_Code] = 'U'
				   AND P.[Principal_Name] = @User_Name
				 ) OR (
				   /* Group */ 
				   P.[PrincipalType_Code] = 'G'
				   AND P.[Principal_Name] IN 
					 (SELECT [Value_Text]
						FROM [dbo].[CsvToTable](@GroupNames_Csv, ',') ) 
				))
			AND (
				/* Filter out denys that are not scopeable */
				P.[PrivilegeType_Code] <> 'D'
				OR
				(T.[Application_Scopeable_Indicator] = 'Y' AND P.[Application_Id] IS NOT NULL)
				OR
				(T.[Environment_Scopeable_Indicator] = 'Y' AND P.[Environment_Id] IS NOT NULL)
                OR
                (P.[Application_Id] IS NULL AND P.[Environment_Id] IS NULL)
				OR
				(T.Application_Scopeable_Indicator = 'N' AND T.[Environment_Scopeable_Indicator] = 'N')
			) AND ((
			      /* If not app scopeable, include regardless of input */			
                  T.[Application_Scopeable_Indicator] = 'N'
                  OR
			      /* If no app group specified, only system-scope is acceptable */			
			      (@ApplicationGroup_Id IS NULL AND P.[ApplicationGroup_Id] IS NULL)
			      OR
			      /* If app group is specified, system or app group scopes are acceptable */
			      (@ApplicationGroup_Id IS NOT NULL AND
			         (P.[ApplicationGroup_Id] IS NULL
			          OR P.[ApplicationGroup_Id] = @ApplicationGroup_Id)
			      )
				) AND (
			      /* If not app scopeable, include regardless of input */			
                  T.[Application_Scopeable_Indicator] = 'N'
                  OR
			      /* If no app specified, only system-scope is acceptable */			
			      (@Application_Id IS NULL AND P.[Application_Id] IS NULL)
			      OR
			      /* If app is specified, system or app scopes are acceptable */
			      (@Application_Id IS NOT NULL AND
			         (P.[Application_Id] IS NULL
			          OR P.[Application_Id] = @Application_Id)
			      )
				) AND (
			      /* If not env scopeable, include regardless of input */			
                  T.[Environment_Scopeable_Indicator] = 'N'
                  OR
			      /* If no env specified, only system-scope is acceptable */			
			      (@Environment_Id IS NULL AND P.[Environment_Id] IS NULL)
			      OR
			      /* If env is specified, system or env scopes are acceptable */
			      (@Environment_Id IS NOT NULL AND
			         (P.[Environment_Id] IS NULL
			          OR P.[Environment_Id] = @Environment_Id)
			      )
				))
			AND T.[Task_Id] = @Task_Id
			AND P.[Directory_Provider_Id] = @Directory_Provider_Id
		 ORDER BY CHARINDEX([PrincipalType_Code], 'UG'), /* Users (Explicit) always override Groups (Implicit) */
                  CASE WHEN P.[Application_Id] IS NOT NULL THEN 0 ELSE 1 END, /* Scopes always override system */
                  CASE WHEN P.[Environment_Id] IS NOT NULL THEN 0 ELSE 1 END, 
                  CHARINDEX([PrivilegeType_Code], 'DG') /* Deny Always overrides Grant */
	),0)

	RETURN @RETVAL
END
GO

GO

IF OBJECT_ID('IsBuildNumberValid') IS NOT NULL DROP FUNCTION IsBuildNumberValid
GO

CREATE FUNCTION IsBuildNumberValid
(
	@Application_Id INT,
	@Build_Number BUILDNUMBER
) RETURNS BIT AS
BEGIN

	DECLARE @RETVAL BIT  SET @RETVAL = 0

	/* BuildNumber is always numeric-only */
	IF @Build_Number NOT LIKE '%[^0-9]%' SET @RETVAL = 1
	
	RETURN @RETVAL
END
GO

GO
IF OBJECT_ID('[IsPromotionApproved]') IS NOT NULL
	DROP FUNCTION [IsPromotionApproved]
GO

CREATE FUNCTION [IsPromotionApproved]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT
) RETURNS CHAR(1) AS
BEGIN

	DECLARE @Workflow_Id INT
	SET @Workflow_Id = (SELECT [Workflow_Id]
	                      FROM [Releases]
						 WHERE [Application_Id] = @Application_Id
						   AND [Release_Number] = @Release_Number)

	DECLARE @ApprovalCounts TABLE
	(
		[Requirement_Id] INT NOT NULL,
		[Approval_Count] INT NOT NULL
	)

	INSERT INTO @ApprovalCounts
	(
		[Requirement_Id],
		[Approval_Count]
    )
	SELECT BAR.[Original_Requirement_Id],
		   [Approval_Count] = COUNT(*)
	  FROM [BuildApprovalsReceived] BAR
	 WHERE BAR.[Application_Id] = @Application_Id
	   AND BAR.[Release_Number] = @Release_Number
	   AND BAR.[Build_Number] = @Build_Number
	   AND BAR.[Environment_Id] = @Environment_Id
	 GROUP BY BAR.[Original_Requirement_Id]

	RETURN CASE WHEN EXISTS (

		SELECT *
		  FROM [BuildPromotionRequirements] BPR
		       LEFT JOIN @ApprovalCounts AC
			           ON AC.[Requirement_Id] = BPR.[Requirement_Id]
		 WHERE BPR.[Environment_Id] = @Environment_Id
		   AND (BPR.[Workflow_Id] IS NULL OR BPR.[Workflow_Id] = @Workflow_Id)
		   AND (AC.[Approval_Count] IS NULL OR AC.[Approval_Count] < COALESCE(BPR.[RequiredInGroup_Count], 1))

	)
	THEN 'N'
	ELSE 'Y'
	END
END

GO
IF OBJECT_ID('IsReleaseNumberValid') IS NOT NULL DROP FUNCTION IsReleaseNumberValid
GO


GO
IF OBJECT_ID('MAXOF') IS NOT NULL DROP FUNCTION MAXOF
GO
CREATE FUNCTION MAXOF
( 
  @a SQL_VARIANT, 
  @b SQL_VARIANT
) RETURNS SQL_VARIANT AS BEGIN 

  IF @a > @b RETURN @a ELSE RETURN @b
  RETURN NULL 
END
GO

GO

IF OBJECT_ID('Session_GetUserName') IS NOT NULL DROP FUNCTION Session_GetUserName
GO

CREATE FUNCTION Session_GetUserName () RETURNS USERNAME
AS BEGIN

	DECLARE @Context_Info VARCHAR(MAX)
	SET @Context_Info = CAST(CONTEXT_INFO() AS VARCHAR(MAX))

	IF CHARINDEX(CHAR(0), @Context_Info) > 0
		SET @Context_Info = SUBSTRING(@Context_Info, 0, CHARINDEX(CHAR(0), @Context_Info))
	RETURN @Context_Info

END
GO

GO

IF OBJECT_ID('SortableBuildNumber') IS NOT NULL DROP FUNCTION SortableBuildNumber
GO

CREATE FUNCTION SortableBuildNumber
(
	@Application_Id INT,
	@Build_Number BUILDNUMBER
) RETURNS VARCHAR(MAX) AS
BEGIN
	/* Dec/Set Return Value */
	DECLARE @RETVAL VARCHAR(MAX)
	SET @RETVAL = NULL

	/* Dec/set BuildNumber Type Length */
	DECLARE @BuildNumber_Length INT
	SET @BuildNumber_Length = 10

	/* Dec/set Scheme */
	DECLARE @BuildNumber_Scheme_Name VARCHAR(20)
	SET @BuildNumber_Scheme_Name = 
		(SELECT [BuildNumber_Scheme_Name] FROM [Applications]
	      WHERE [Application_Id] = @Application_Id)

	/*******************
     * ERROR CONDITION 
     ******************/
	IF 0=dbo.IsBuildNumberValid(@Application_Id, @Build_Number) OR @BuildNumber_Length IS NULL
		BEGIN
			SET @RETVAL = NULL
		END 

	/*******************
     * Scheme Sequential or DateTimeBased
     ******************/
	ELSE IF @BuildNumber_Scheme_Name IN ('Sequential','DateTimeBased','Unique')
		BEGIN
			/* Simply return a padded string */
			SET @RETVAL = RIGHT( SPACE(@BuildNumber_Length) + @Build_Number , @BuildNumber_Length)
		END

	RETURN @RETVAL

END
GO

GO
IF OBJECT_ID('Variables_CascadedValuesByApplication') IS NOT NULL DROP VIEW Variables_CascadedValuesByApplication
GO
IF OBJECT_ID('Variables_CascadedValuesByExecution') IS NOT NULL DROP VIEW Variables_CascadedValuesByExecution
GO
IF OBJECT_ID('Variables_GetValuesForExecution') IS NOT NULL DROP FUNCTION Variables_GetValuesForExecution
GO


CREATE FUNCTION [Variables_GetValuesForExecution]
(
	@ExecutionId INT,
	@Server_Id INT,
	@Deployable_Id INT
) RETURNS @vvals TABLE ([Variable_Name] VARCHAR(50), [Value_Text] VARCHAR(MAX), [Scope_Code] CHAR(1), [Sensitive_Indicator] CHAR(1))
BEGIN

	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
	DECLARE @Environment_Id INT
	DECLARE @Release_Number RELEASENUMBER
	DECLARE @Build_Number BUILDNUMBER

	SELECT @Application_Id = BE.[Application_Id],
	       @Environment_Id = BE.[Environment_Id],
		   @Release_Number = BE.[Release_Number],
		   @Build_Number = BE.[Build_Number],
		   @ApplicationGroup_Id = A.[ApplicationGroup_Id]
	  FROM [BuildExecutions] BE
	       INNER JOIN [Applications] A
		           ON A.[Application_Id] = BE.[Application_Id]
     WHERE BE.[Execution_Id] = @ExecutionId

	DECLARE @VDEC_Ids TABLE ([VariableDeclaration_Id] INT)
	DECLARE @VDEF_Ids TABLE ([VariableDefinition_Id] INT)

	INSERT @VDEF_Ids
	SELECT VDEF.[VariableDefinition_Id]
	  FROM [VariableDefinitions] VDEF
	 WHERE (VDEF.[Application_Id] IS NULL OR VDEF.[Application_Id] = @Application_Id)
	   AND (VDEF.[ApplicationGroup_Id] IS NULL OR VDEF.[ApplicationGroup_Id] = @ApplicationGroup_Id)
	   AND (VDEF.[Release_Number] IS NULL OR VDEF.[Release_Number] = @Release_Number)
	   AND (VDEF.[Build_Number] IS NULL OR VDEF.[Build_Number] = @Build_Number)
	   AND (VDEF.[Environment_Id] IS NULL OR VDEF.[Environment_Id] = @Environment_Id)
	   AND (VDEF.[Execution_Id] IS NULL OR VDEF.[Execution_Id] = @ExecutionId)
	   AND (VDEF.[Server_Id] IS NULL OR VDEF.[Server_Id] = @Server_Id)
	   AND (VDEF.[Deployable_Id] IS NULL OR VDEF.[Deployable_Id] = @Deployable_Id)

	INSERT @VDEC_Ids
	SELECT VDEC.[VariableDeclaration_Id]
	  FROM [VariableDeclarations] VDEC
	 WHERE (VDEC.[Scoped_Application_Id] IS NULL OR VDEC.[Scoped_Application_Id] = @Application_Id)
	   AND (VDEC.[Scoped_Environment_Id] IS NULL OR VDEC.[Scoped_Environment_Id] = @Environment_Id)
	   AND ((@Server_Id IS NULL AND VDEC.[Scope_Code] <> 'V') OR @Server_Id IS NOT NULL)
	   AND ((@Deployable_Id IS NULL AND VDEC.[Scope_Code] <> 'D') OR @Deployable_Id IS NOT NULL)
	   AND NOT EXISTS(SELECT * FROM [VariableDefinitions] _VDEF
					   WHERE _VDEF.[VariableDefinition_Id] IN (SELECT [VariableDefinition_Id] FROM @VDEF_Ids)
						 AND _VDEF.[Variable_Name] = VDEC.[Variable_Name]
						 AND _VDEF.[Scope_Code] = VDEC.[Scope_Code])

	INSERT INTO @vvals ([Variable_Name], [Value_Text], [Scope_Code], [Sensitive_Indicator])
	SELECT [Variable_Name], [Value_Text], [Scope_Code], [Sensitive_Indicator]
	  FROM (SELECT *, [Rank] = RANK() OVER (PARTITION BY [Variable_Name] ORDER BY CHARINDEX([Scope_Code], 'SEVGADRBPX') DESC)
			  FROM (SELECT [Variable_Name], [Value_Text], [Scope_Code], [Sensitive_Indicator], NULL AS [Scoped_Environment_Id]
					  FROM [VariableDefinitions]
					 WHERE [VariableDefinition_Id] IN (SELECT [VariableDefinition_Id] FROM @VDEF_Ids)
						   UNION ALL
					SELECT [Variable_Name], [DefaultValue_Text], [Scope_Code], [Sensitive_Indicator], [Scoped_Environment_Id]
					  FROM [VariableDeclarations]
					 WHERE [VariableDeclaration_Id] IN (SELECT [VariableDeclaration_Id] FROM @VDEC_Ids))_)_
	 WHERE [Rank] = 1
  
  RETURN
END
GO

GO

IF OBJECT_ID('GetNextBuildNumber') IS NOT NULL DROP FUNCTION GetNextBuildNumber
GO

CREATE FUNCTION GetNextBuildNumber
(
	@Application_Id  INT,
	@Build_Number BUILDNUMBER
)
RETURNS BUILDNUMBER
AS

BEGIN

	
	DECLARE @Ret BUILDNUMBER SET @Ret = NULL
	DECLARE @Now DATETIME SET @Now = GETDATE()

	DECLARE @BuildNumber_Scheme_Name VARCHAR(15)
	SELECT @BuildNumber_Scheme_Name = [BuildNumber_Scheme_Name]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

	IF @BuildNumber_Scheme_Name = 'DateTimeBased'
	BEGIN

		DECLARE @DTB_Build_Number [BUILDNUMBER]
		SET @DTB_Build_Number = /* YYMMDDBBB (where B is build sequence) */
			RIGHT(CAST(DATEPART(YEAR,@Now) AS VARCHAR(4)),2) +
			RIGHT('0' + CAST(DATEPART(MONTH,@Now) AS VARCHAR(2)),2) +
			RIGHT('0' + CAST(DATEPART(DAY,@Now) AS VARCHAR(2)),2)

		/* Find the next build sequence */
		DECLARE @Build_Sequence INT
		SELECT @Build_Sequence = CAST(MAX(RIGHT([Build_Number], 3)) AS INT)
		  FROM [Builds]
		 WHERE [Application_Id] = @Application_Id
		   AND [Build_Number] LIKE @DTB_Build_Number + '%'

		SET @Build_Sequence = COALESCE(@Build_Sequence, 0) + 1

		SET @Ret = @DTB_Build_Number + RIGHT('00' + CAST(@Build_Sequence AS VARCHAR(3)), 3)

	END

	ELSE IF @BuildNumber_Scheme_Name = 'Sequential'
	BEGIN
		SET @Ret = COALESCE(CAST(@Build_Number AS INT), 0) + 1
	END
	ELSE IF @BuildNumber_Scheme_Name = 'Unique'
	BEGIN
	    
	    SELECT @Ret = COALESCE(MAX(CAST([Build_Number] AS INT)), 0) + 1
	     FROM [Builds]
	    WHERE [Application_Id] = @Application_Id
	    
	END

	RETURN @Ret

END
	
GO


GO

IF OBJECT_ID('GetNextReleaseNumber') IS NOT NULL DROP FUNCTION GetNextReleaseNumber
GO

CREATE FUNCTION GetNextReleaseNumber
(
	@Application_Id  INT,
    @Release_Number RELEASENUMBER
)
RETURNS RELEASENUMBER
AS

BEGIN

	DECLARE @New_Release_Number RELEASENUMBER  SET @New_Release_Number = NULL

    DECLARE @ReleaseNumber_Scheme_Name VARCHAR(20)
	 SELECT @ReleaseNumber_Scheme_Name = [ReleaseNumber_Scheme_Name]
	   FROM [Applications]
	  WHERE [Application_Id] = @Application_Id

	IF @ReleaseNumber_Scheme_Name = 'DateBased'
	BEGIN
		/* Try to autoincrement a freeform release number */
		SET @New_Release_Number = CHARINDEX('.', @Release_Number)
		IF CHARINDEX('.', @Release_Number) <> 0
			SET @New_Release_Number = LEFT(@Release_Number, CHARINDEX('.', @Release_Number) - 1)
		ELSE
			SET @New_Release_Number = @Release_Number
		IF ISNUMERIC(@New_Release_Number) = 1
			SET @New_Release_Number = CAST(@New_Release_Number + 1 AS VARCHAR(10))
		ELSE
			SET @New_Release_Number = NULL
	END
	ELSE
	BEGIN
		/* Try to autoincrement a major-minor/major-minor-rev release number */
		DECLARE @LastDotIndex INT
		SET @LastDotIndex = CHARINDEX('.', REVERSE(@Release_Number)) - 1
		SET @New_Release_Number = RIGHT(@Release_Number, @LastDotIndex)
		IF ISNUMERIC(@New_Release_Number) = 1
			SET @New_Release_Number = LEFT(@Release_Number, LEN(@Release_Number) - @LastDotIndex) + CAST(@New_Release_Number + 1 AS VARCHAR(10))
		ELSE
			SET @New_Release_Number = NULL
	END

	RETURN @New_Release_Number

END
	
GO


GO
IF OBJECT_ID('[GetSortableReleaseNumber]') IS NOT NULL
	DROP FUNCTION [GetSortableReleaseNumber]
GO

CREATE FUNCTION [GetSortableReleaseNumber]
(
	@Release_Number RELEASENUMBER,
    @ReleaseNumber_Scheme_Name VARCHAR(20)
)
RETURNS DECIMAL(38,0) AS

BEGIN

    RETURN (SELECT [Release_Sequence]
              FROM dbo.[GetSortableReleaseNumberWithValidation](@Release_Number,@ReleaseNumber_Scheme_Name))

END
GO

GO
IF OBJECT_ID('[ActionGroupActions_Extended]') IS NOT NULL DROP VIEW [ActionGroupActions_Extended]
GO

CREATE VIEW [ActionGroupActions_Extended] AS

    SELECT AGA.[ActionGroup_Id],
           AGA.[Action_Sequence],
           AGA.[Server_Id],
           AGA.[Long_Description],
           AGA.[ResumeNextOnFailure_Indicator],
           AGA.[Action_Configuration],
           AGA.[Short_Description],
           AGA.[Active_Indicator],
           AGA.[Retry_Count],
           AGA.[LogFailureAsWarning_Indicator],
	       AGA.[Server_Variable_Name],
	       AGA.[Target_Server_Id],
	       AGA.[Target_Server_Variable_Name],
	       AGA.[ExtensionConfiguration_Id],
           [Configuration_Profile_Name] = E.[Profile_Name],
           SRC_S.[Server_Name],
           [Target_Server_Name] = TRG_S.[Server_Name],
           SRC_S.[ServerType_Code],
           [Target_ServerType_Code] = TRG_S.[ServerType_Code]

      FROM [ActionGroupActions] AGA
INNER JOIN [ActionGroups] AG
        ON AG.[ActionGroup_Id] = AGA.[ActionGroup_Id]
 LEFT JOIN [ExtensionConfigurations] E
        ON E.[ExtensionConfiguration_Id] = AGA.[ExtensionConfiguration_Id]
 LEFT JOIN [Servers] SRC_S
        ON SRC_S.[Server_Id] = AGA.[Server_Id]
 LEFT JOIN [Servers] TRG_S
        ON TRG_S.[Server_Id] = AGA.[Target_Server_Id]

GO


GO
IF OBJECT_ID('[ActionGroupUsage_Slim]') IS NOT NULL DROP VIEW [ActionGroupUsage_Slim]
GO

CREATE VIEW [ActionGroupUsage_Slim]
AS
SELECT [ActionGroup_Id] = DPAG.[ActionGroup_Id],
       DPAG.[DeploymentPlanActionGroup_Id],
       DP.[DeploymentPlan_Name],
       A.[Application_Id],
       A.[Application_Name],
       AG.[ApplicationGroup_Id],
       AG.[ApplicationGroup_Name],
       E.[Environment_Id],
       E.[Environment_Name]

  FROM [DeploymentPlanActionGroups] DPAG
       INNER JOIN [DeploymentPlans] DP
               ON DP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
		LEFT JOIN [WorkflowSteps] WS
		       ON WS.[DeploymentPlan_Id] = DP.[DeploymentPlan_Id]
		LEFT JOIN [ApplicationWorkflows] AW
		       ON AW.[Workflow_Id] = WS.[Workflow_Id]
        LEFT JOIN [Applications] A
               ON A.[Application_Id] = AW.[Application_Id]
        LEFT JOIN [ApplicationGroups] AG
               ON A.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]
        LEFT JOIN [Environments] E
               ON E.[Environment_Id] = WS.[Environment_Id]

GO

GO
IF OBJECT_ID('[ApplicationDeploymentPlans]') IS NOT NULL
	DROP VIEW [ApplicationDeploymentPlans]
GO

CREATE VIEW [ApplicationDeploymentPlans]
AS

SELECT DP.[DeploymentPlan_Id],
       DP.[DeploymentPlan_Name],
	   A.[Application_Id],
	   A.[Application_Name],
	   AG.[ApplicationGroup_Id],
	   AG.[ApplicationGroup_Name],
	   E.[Environment_Id],
	   E.[Environment_Name],
       [Environment_Display_Sequence] = E.[Display_Sequence],
	   W.[Workflow_Id],
	   W.[Workflow_Name],
	   [Workflow_Step_Sequence] = WS.[Step_Sequence],
       [BuildStep_Indicator] = 'N',
	   DP.[AllowLocalChanges_Indicator],
       [DeploymentPlan_Shared_Indicator] = CASE WHEN 
                                                    DP.[DeploymentPlan_Id] IN 
                                                         (SELECT [PlanId] FROM 
                                                            (SELECT [PlanId] = [DeploymentPlan_Id] FROM [WorkflowSteps]
                                                              UNION ALL 
                                                             SELECT [PlanId] = [Build_DeploymentPlan_Id] FROM [Workflows]) AS [AllPlanIds]
                                                        GROUP BY [PlanId]
                                                        HAVING COUNT([PlanId]) > 1)
                                                
                                                THEN 'Y'
                                                ELSE 'N'
                                            END
  FROM [ApplicationWorkflows] AW
       INNER JOIN [Workflows] W
	           ON W.[Workflow_Id] = AW.[Workflow_Id]
       INNER JOIN [WorkflowSteps] WS
	           ON WS.[Workflow_Id] = AW.[Workflow_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = AW.[Application_Id]
	   INNER JOIN [Environments] E
	           ON E.[Environment_Id] = WS.[Environment_Id]
	   INNER JOIN [DeploymentPlans] DP
	           ON DP.[DeploymentPlan_Id] = WS.[DeploymentPlan_Id]
        LEFT JOIN [ApplicationGroups] AG
		       ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]
UNION ALL 
SELECT 
	   DP.[DeploymentPlan_Id],
       DP.[DeploymentPlan_Name],
	   A.[Application_Id],
	   A.[Application_Name],
	   AG.[ApplicationGroup_Id],
	   AG.[ApplicationGroup_Name],
	   NULL,
	   NULL,
       NULL,
	   W.[Workflow_Id],
	   W.[Workflow_Name],
	   NULL,
       [BuildStep_Indicator] = 'Y',
	   DP.[AllowLocalChanges_Indicator],
       [DeploymentPlan_Shared_Indicator] = CASE WHEN 
                                                    DP.[DeploymentPlan_Id] IN 
                                                         (SELECT [PlanId] FROM 
                                                            (SELECT [PlanId] = [DeploymentPlan_Id] FROM [WorkflowSteps]
                                                              UNION ALL 
                                                             SELECT [PlanId] = [Build_DeploymentPlan_Id] FROM [Workflows]) AS [AllPlanIds]
                                                        GROUP BY [PlanId]
                                                        HAVING COUNT([PlanId]) > 1)
                                                
                                                THEN 'Y'
                                                ELSE 'N'
                                            END
  FROM [ApplicationWorkflows] AW
       INNER JOIN [Workflows] W
	           ON W.[Workflow_Id] = AW.[Workflow_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = AW.[Application_Id]
	   INNER JOIN [DeploymentPlans] DP
	           ON DP.[DeploymentPlan_Id] = W.[Build_DeploymentPlan_Id]
        LEFT JOIN [ApplicationGroups] AG
		       ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]
GO

GO
IF OBJECT_ID('[ApplicationDeploymentPlans_Extended]') IS NOT NULL
	DROP VIEW [ApplicationDeploymentPlans_Extended]
GO

GO
IF OBJECT_ID('[ApplicationEnvironments_Merged]') IS NOT NULL
	DROP VIEW [ApplicationEnvironments_Merged]
GO

GO
IF OBJECT_ID('[Applications_Extended]') IS NOT NULL
	DROP VIEW [Applications_Extended]
GO

CREATE VIEW [Applications_Extended]
AS
	SELECT A.[Application_Id],
		   A.[Application_Name],
		   A.[Application_Description],
		   A.[ReleaseNumber_Scheme_Name],
		   A.[BuildNumber_Scheme_Name],
		   A.[AllowMultipleActiveReleases_Indicator],
		   A.[AllowMultipleActiveBuilds_Indicator],
		   A.[IssueTracking_Provider_Id],
		   A.[IssueTracking_CategoryIdList_Text],
		   A.[IssueTracking_Filter_Configuration],
           A.[IssueTracking_AllowBuildMasterIssues_Indicator],
		   A.[CreatedBy_User_Name],
		   A.[CreatedOn_Date],
		   A.[ModifiedBy_User_Name],
		   A.[ModifiedOn_Date],
		   A.[Dashboard_Text],
		   A.[ApplicationGroup_Id],
		   AG.[ApplicationGroup_Name],
		   AG.[ApplicationGroup_Description],
		   [Active_Releases_Count] = (SELECT COUNT(*) 
		                                FROM [Releases] 
		                               WHERE [Application_Id] = A.[Application_Id] 
		                                 AND [ReleaseStatus_Name] = 'Active'),
		   A.[VariableSupport_Code],
           [Issues_LastSynchronized_Date] = (SELECT MAX([IssueTracker_LastSync_Date]) 
                                               FROM [Issues] 
                                              WHERE [Application_Id] = A.[Application_Id])
	  FROM [Applications] A
		   LEFT JOIN [ApplicationGroups] AG
				  ON A.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]

GO
IF OBJECT_ID('[Environments_LatestDeployedReleases]') IS NOT NULL DROP VIEW [Environments_LatestDeployedReleases]
GO

IF OBJECT_ID('[Applications_LatestDeployedReleases]') IS NOT NULL DROP VIEW [Applications_LatestDeployedReleases]
GO

CREATE VIEW [Applications_LatestDeployedReleases] AS

SELECT A.[Application_Id]
	  ,A.[ApplicationGroup_Id]
	  ,A.[Application_Name]
	  ,AG.[ApplicationGroup_Name]
	  ,E.[Environment_Id]
      ,E.[Environment_Name]
      ,E.[Display_Sequence]
	  ,BE.[Release_Number]
	  ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])
	  ,BE.[Build_Number]
	  ,BE.[Execution_Id]
      ,BE.[ExecutionStart_Date]
	  ,BE.[ExecutionStatus_Name]
	  ,[Workflow_Step_Sequence] = dWS.[Step_Sequence]
  FROM [Applications] A
       JOIN (SELECT _AW.[Application_Id], _WS.[Environment_Id], MAX(_WS.[Step_Sequence]) [Step_Sequence]
               FROM [WorkflowSteps] _WS
                    JOIN [Workflows] _W
                      ON _W.[Workflow_Id] = _WS.[Workflow_Id]
                     AND _W.[Active_Indicator] = 'Y'
                    JOIN [ApplicationWorkflows] _AW
					  ON _AW.[Workflow_Id] = _W.[Workflow_Id]
              GROUP BY _AW.[Application_Id], _WS.[Environment_Id]
			  UNION ALL
			  SELECT _AW.[Application_Id], NULL [Environment_Id], NULL [Step_Sequence]
               FROM [ApplicationWorkflows] _AW
                    JOIN [Workflows] _W
                      ON _W.[Workflow_Id] = _AW.[Workflow_Id]
                     AND _W.[Active_Indicator] = 'Y'
					 AND (_W.[Build_DeploymentPlan_Id] IS NOT NULL OR 
					      _W.[BuildImporterTemplate_Configuration] IS NOT NULL)			  
              GROUP BY _AW.[Application_Id]
			  ) dWS
         ON dWS.[Application_Id] = A.[Application_Id]

       LEFT JOIN [Environments] E
         ON E.[Environment_Id] = dWS.[Environment_Id]
       LEFT JOIN [BuildExecutions] BE
         ON BE.[Execution_Id] = (SELECT TOP 1 [Execution_Id]
                                   FROM [BuildExecutions] _BE
                                  WHERE _BE.[Application_Id] = A.[Application_Id]
                                    AND (_BE.[Environment_Id] = E.[Environment_Id]
									     OR (_BE.[Environment_Id] IS NULL AND E.[Environment_Id] IS NULL))
                                  ORDER BY [ExecutionStart_Date] DESC)
       LEFT JOIN [Releases] R
         ON R.[Application_Id] = A.[Application_Id]
        AND R.[Release_Number] = BE.[Release_Number]
         
       LEFT JOIN [ApplicationGroups] AG
         ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]



GO
IF OBJECT_ID('[BuildApprovalsReceived_Extended]') IS NOT NULL DROP VIEW [BuildApprovalsReceived_Extended]
GO

CREATE VIEW [BuildApprovalsReceived_Extended] AS

SELECT BAR.[Approval_Id],
	   A.[Application_Id],
       A.[Application_Name],
       R.[Release_Number],
	   [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number]),
       BAR.[Build_Number],
       E.[Environment_Id],
       E.[Environment_Name],
	   BAR.[Original_Requirement_Id],
       BAR.[User_Name],
       BAR.[Comments_Text],
       BAR.[Received_Date],
	   BAR.[RequirementType_Code],
	   BAR.[Requirement_Description],
	   R.[Workflow_Id]

  FROM [BuildApprovalsReceived] BAR
       INNER JOIN [Applications] A
	           ON A.[Application_Id] = BAR.[Application_Id]
       INNER JOIN [Environments] E
               ON E.[Environment_Id] = BAR.[Environment_Id]
	   INNER JOIN [Releases] R
	           ON R.[Application_Id] = BAR.[Application_Id]
			  AND R.[Release_Number] = BAR.[Release_Number]

GO
IF OBJECT_ID('[BuildApprovals_Extended]') IS NOT NULL DROP VIEW [BuildApprovals_Extended]
GO

GO

IF OBJECT_ID('[BuildArtifacts_Extended]') IS NOT NULL DROP VIEW [BuildArtifacts_Extended]
GO

CREATE VIEW [BuildArtifacts_Extended] AS

	SELECT BA.[Application_Id]
          ,A.[Application_Name]
		  ,BA.[Release_Number]
		  ,BA.[Build_Number]
		  ,BA.[Deployable_Id]
		  ,D.[Deployable_Name]
		  ,BA.[Artifact_Name]
		  ,BA.[Execution_Id]
		  ,BA.[Created_Date]
		  ,BE.[Environment_Id]
		  ,ENV.[Environment_Name]

	  FROM [BuildArtifacts] BA
            LEFT JOIN [Applications] A
                   ON A.[Application_Id] = BA.[Application_Id]
		    LEFT JOIN [BuildExecutions] BE
				   ON BA.[Execution_Id] = BE.[Execution_Id]
		    LEFT JOIN [Environments] ENV
				   ON BE.[Environment_Id] = ENV.[Environment_Id]
		    LEFT JOIN [Deployables] D
				   ON BA.[Deployable_Id] = D.[Deployable_Id]

GO


GO
IF OBJECT_ID('[BuildArtifacts_Slim]') IS NOT NULL DROP VIEW [BuildArtifacts_Slim]
GO

CREATE VIEW [BuildArtifacts_Slim] AS

	SELECT BA.[Application_Id]
		  ,BA.[Release_Number]
		  ,BA.[Build_Number]
		  ,BA.[Deployable_Id]
		  ,BA.[Artifact_Name]
		  ,BA.[Execution_Id]
		  ,BA.[Created_Date]
		  ,B.[BuildStatus_Name]

	  FROM [BuildArtifacts] BA
		   INNER JOIN [Builds] B
				   ON BA.[Application_Id] = B.[Application_Id] AND BA.[Release_Number] = B.[Release_Number] AND BA.[Build_Number] = B.[Build_Number]

GO


GO
IF OBJECT_ID('[BuildExecutions_ActionsInProgress]') IS NOT NULL DROP VIEW [BuildExecutions_ActionsInProgress]
GO

GO
IF OBJECT_ID('[BuildExecutions_Extended]') IS NOT NULL DROP VIEW [BuildExecutions_Extended]
GO

CREATE VIEW [BuildExecutions_Extended] AS

	SELECT BE.[Execution_Id]
		  ,BE.[Application_Id]
		  ,BE.[Release_Number]
		  ,BE.[Build_Number]
		  ,B.[BuildStatus_Name]
		  ,R.[Workflow_Id]
		  ,[Workflow_Name] = R.[Original_Workflow_Name]
		  ,BE.[Environment_Id]
		  ,ENV.[Environment_Name]
		  ,BE.[ExecutionStart_Date]
		  ,BE.[ExecutionEnded_Date]
		  ,BE.[ExecutionStatus_Name]
		  ,BE.[CreatedBy_User_Name]
		  ,BE.[CreatedOn_Date]
		  ,BE.[CanceledBy_User_Name]
		  ,BE.[CanceledOn_Date]
		  ,BE.[LastActionReexecutionOf_Execution_Id]
		  ,A.[Application_Name]
		  ,BP.[PromotionStatus_Name]
		  ,BP.[Promoted_Date]
		  ,BP.[PromotedBy_Name]
		  ,BE.[WarningLogEntry_Indicator]
          ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
		  ,R.[ReleaseStatus_Name]
		  ,BE.[Initial_BuildExecution_DeploymentPlan_Id]
		  ,[Initial_BuildExecution_DeploymentPlan_Name] = BEDP.[Original_DeploymentPlan_Name]
		  ,A.[VariableSupport_Code]
		  ,A.[ApplicationGroup_Id]
		  ,R.[Release_Sequence]
		  ,BE.[ReexecutionState_Xml]
		  ,[Initial_DeploymentPlan_Id] = COALESCE(WS.[DeploymentPlan_Id], WF.[Build_DeploymentPlan_Id])
		  ,B.[BuildImporter_Configuration]
		  ,WF.[Build_AutoPromote_Indicator]

	  FROM [BuildExecutions] BE
		   INNER JOIN [Releases] R
				  ON R.[Application_Id] = BE.[Application_Id]
				 AND R.[Release_Number] = BE.[Release_Number]
		   LEFT JOIN [Workflows] WF
				  ON R.[Workflow_Id] = WF.[Workflow_Id]
		   LEFT JOIN [Environments] ENV
				  ON BE.[Environment_Id] = ENV.[Environment_Id]
		   INNER JOIN [Builds] B
				  ON BE.[Application_Id] = B.[Application_Id]
				 AND BE.[Release_Number] = B.[Release_Number]
				 AND BE.[Build_Number] = B.[Build_Number]
		   INNER JOIN [Applications] A
				  ON A.[Application_Id] = BE.[Application_Id]
		    LEFT JOIN [BuildPromotions] BP
				   ON BE.[Application_Id] = BP.[Application_Id]
				  AND BE.[Release_Number] = BP.[Release_Number]
				  AND BE.[Build_Number] = BP.[Build_Number]
				  AND BE.[Environment_Id] = BP.[Environment_Id]
			LEFT JOIN [BuildExecution_DeploymentPlans] BEDP
			       ON BEDP.[BuildExecution_DeploymentPlan_Id] = BE.[Initial_BuildExecution_DeploymentPlan_Id]
			LEFT JOIN [WorkflowSteps] WS
			       ON WS.[Workflow_Id] = WF.[Workflow_Id]
				  AND WS.[Environment_Id] = BE.[Environment_Id]

GO


GO
IF OBJECT_ID('[BuildExecutions_PendingCancellation]') IS NOT NULL DROP VIEW [BuildExecutions_PendingCancellation]
GO

CREATE VIEW [BuildExecutions_PendingCancellation] AS

	SELECT BE.[Execution_Id]
	  FROM [BuildExecutions] BE
	 WHERE BE.[ExecutionStatus_Name] = 'Canceled'

GO

GO
IF OBJECT_ID('[BuildExecutions_ActionGroupActions_Extended]') IS NOT NULL DROP VIEW [BuildExecutions_ActionGroupActions_Extended]
GO

CREATE VIEW [BuildExecutions_ActionGroupActions_Extended] AS

SELECT BE.[Execution_Id],
	   [BuildExecutionStatus_Name] = BE.[ExecutionStatus_Name],
	   BE.[Environment_Id],

	   ENV.[Environment_Name],

       B.[Application_Id],
       B.[Release_Number],
       B.[Build_Number],

       BEAGA.[Original_Action_Sequence],
       BEAGA.[ExecutionAction_Description],

       BEDPAG.[Original_DeploymentPlanActionGroup_Sequence],
	   BEDPAG.[Original_DeploymentPlanActionGroup_Name],

       BEAGA.[ExecutionStarted_Date],
       BEAGA.[ExecutionEnded_Date],
       [ExecutionStatus_Name] = COALESCE(BEAGA.[ExecutionStatus_Name], 'Pending'),

       BEDPAG.[Original_Deployable_Name],
       BEDPAG.[Original_Deployable_Application_Name],

	   BEAGA.[WarningLogEntry_Indicator],
	   A.[VariableSupport_Code]

  FROM [BuildExecutions] BE
	   INNER JOIN [Environments] ENV
			   ON BE.[Environment_Id] = ENV.[Environment_Id]
       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
               ON BEDP.[Execution_Id] = BE.[Execution_Id]
       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
               ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
       INNER JOIN [BuildExecution_DeploymentPlanActionGroups] BEDPAG
               ON BEDPAG.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
              AND BEDPAG.[Original_DeploymentPlanActionGroup_Sequence] = BEAGA.[Original_DeploymentPlanActionGroup_Sequence]
       INNER JOIN [Builds] B
               ON B.[Application_Id] = BE.[Application_Id]
              AND B.[Release_Number] = BE.[Release_Number]
              AND B.[Build_Number] = BE.[Build_Number]
	   INNER JOIN [Releases] R
			   ON B.[Application_Id] = R.[Application_Id]
			  AND B.[Release_Number] = R.[Release_Number]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = BE.[Application_Id]
GO


GO

IF OBJECT_ID('[BuildOutputs_Extended]') IS NOT NULL DROP VIEW [BuildOutputs_Extended]
GO

CREATE VIEW [BuildOutputs_Extended] AS


    SELECT  BO.[Execution_Id]
           ,BO.[Output_Sequence]
           ,BO.[Output_Name]
           ,BO.[BuildExecution_PlanAction_Id]
           ,BO.[Output_Bytes]
           ,BO.[OutputType_Code]
           
           ,E.[Application_Id]
           ,E.[Release_Number]
           ,E.[Build_Number]
           ,E.[Environment_Id]
           ,E.[ExecutionStart_Date]
           ,E.[ExecutionStatus_Name]
       
      FROM [BuildOutputs] BO
INNER JOIN [BuildExecutions] E
        ON E.[Execution_Id] = BO.[Execution_Id]


GO


GO
IF OBJECT_ID('[BuildPromotionRequirements_Extended]') IS NOT NULL DROP VIEW [BuildPromotionRequirements_Extended]
GO

CREATE VIEW [BuildPromotionRequirements_Extended] AS

SELECT BPR.[Requirement_Id],
       BPR.[Workflow_Id],
	   BPR.[Environment_Id],
	   BPR.[Principal_Name],
	   BPR.[RequiredInGroup_Count],
	   BPR.[Requirement_Description],
	   BPR.[Requirement_Configuration],
	   BPR.[CreatedBy_User_Name],
	   BPR.[CreatedOn_Date],
	   BPR.[ModifiedBy_User_Name],
	   BPR.[ModifiedOn_Date],

	   [RequirementType_Code] = CASE
	                                WHEN [Requirement_Configuration] IS NOT NULL THEN 'A'
									WHEN [RequiredInGroup_Count] IS NOT NULL THEN 'G'
									ELSE 'U'
								END

  FROM [BuildPromotionRequirements] BPR

GO


GO
IF OBJECT_ID('[BuildPromotionRequirements_VerificationRequired]') IS NOT NULL DROP VIEW [BuildPromotionRequirements_VerificationRequired]
GO

CREATE VIEW [BuildPromotionRequirements_VerificationRequired] AS

	SELECT A.[Application_Id],
	       A.[Application_Name],
		   R.[Release_Number],
		   [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number]),
		   B.[Build_Number],
		   E.[Environment_Id],
		   E.[Environment_Name],
		   W.[Workflow_Id],
		   [Workflow_Name] = R.[Original_Workflow_Name],
		   [Workflow_Step_Sequence] = WS.[Step_Sequence],
		   BPR.[Requirement_Id],
		   BPR.[Requirement_Configuration],
		   BPR.[Requirement_Description]

	  FROM [Builds] B
	       INNER JOIN [Applications] A
		           ON A.[Application_Id] = B.[Application_Id]
	       INNER JOIN [Releases] R
		           ON R.[Application_Id] = B.[Application_Id]
				  AND R.[Release_Number] = B.[Release_Number]
		   INNER JOIN [Workflows] W
		           ON W.[Workflow_Id] = R.[Workflow_Id]
		   INNER JOIN [WorkflowSteps] WS
		           ON WS.[Workflow_Id] = W.[Workflow_Id]
		   INNER JOIN [Environments] E
		           ON E.[Environment_Id] = WS.[Environment_Id]
		    LEFT JOIN [BuildPromotions] BP
			       ON BP.[Application_Id] = B.[Application_Id]
				  AND BP.[Release_Number] = B.[Release_Number]
				  AND BP.[Build_Number] = B.[Build_Number]
				  AND BP.[Environment_Id] = WS.[Environment_Id]
		   INNER JOIN [BuildPromotionRequirements] BPR
		           ON BPR.[Environment_Id] = WS.[Environment_Id]
				  AND (BPR.[Workflow_Id] IS NULL OR BPR.[Workflow_Id] = W.[Workflow_Id])
	 WHERE B.[BuildStatus_Name] = 'Active'
	   AND BP.[Application_Id] IS NULL
	   AND BPR.[Requirement_Configuration] IS NOT NULL

GO


GO
IF OBJECT_ID('[BuildPromotions_Extended]') IS NOT NULL DROP VIEW [BuildPromotions_Extended]
GO

CREATE VIEW [BuildPromotions_Extended] AS

SELECT BP.[Application_Id]
      ,BP.[Release_Number]
      ,BP.[Build_Number]
      ,W.[Workflow_Id]
      ,BP.[Environment_Id]
      ,BP.[Promoted_Date]
      ,BP.[PromotedBy_Name]
      ,BP.[Comments_Text]
      ,BP.[PromotionStatus_Name]
	  ,BP.[StatusOverriddenBy_User_Name]
	  ,BP.[StatusOverriddenReason_Text]
	  ,BP.[StatusOverridden_Date]
	  ,BP.[Forced_Indicator]

      ,A.[Application_Name]

      ,WS.[Step_Sequence]

      ,B.[BuildStatus_Name]

      ,[Workflow_Name] = R.[Original_Workflow_Name]

      ,E.[Environment_Name]

	  ,BE.[Execution_Id] AS [Last_Execution_Id]
	  ,BE.[ExecutionStart_Date] AS [Last_ExecutionStart_Date]
	  ,BE.[ExecutionStatus_Name] AS [Last_ExecutionStatus_Name]
      ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
	  ,[Contingent_Indicator] = CASE WHEN EXISTS(SELECT *
	                                               FROM [BuildPromotionContingencies] BPC
												  WHERE BPC.[Contingent_Application_Id] = BP.[Application_Id]
												    AND BPC.[Contingent_Release_Number] = BP.[Release_Number]
													AND BPC.[Contingent_Build_Number] = BP.[Build_Number]
													AND BPC.[Contingent_Environment_Id] = BP.[Environment_Id]) THEN 'Y' ELSE 'N' END

  FROM [BuildPromotions] BP
       INNER JOIN [Applications] A
               ON A.[Application_Id] = BP.[Application_Id]
       INNER JOIN [Builds] B
               ON B.[Application_Id] = BP.[Application_Id]
              AND B.[Release_Number] = BP.[Release_Number]
              AND B.[Build_Number] = BP.[Build_Number]
       INNER JOIN [Releases] R
               ON R.[Application_Id] = BP.[Application_Id]
              AND R.[Release_Number] = BP.[Release_Number]
       LEFT JOIN [Workflows] W
               ON W.[Workflow_Id] = R.[Workflow_Id]
       INNER JOIN [WorkflowSteps] WS
               ON WS.[Workflow_Id] = W.[Workflow_Id]
              AND WS.[Environment_Id] = BP.[Environment_Id]
       INNER JOIN [Environments] E        
               ON E.[Environment_Id] = WS.[Environment_Id]
	   LEFT JOIN [BuildExecutions] BE
	          ON BE.[Execution_Id] = 
	             (
	              SELECT TOP 1 [Execution_Id]
	                FROM [BuildExecutions] _BE
	               WHERE _BE.[Application_Id] = BP.[Application_Id]
	                 AND _BE.[Release_Number] = BP.[Release_Number]
	                 AND _BE.[Build_Number] = BP.[Build_Number]
	                 AND _BE.[Environment_Id] = BP.[Environment_Id]
	               ORDER BY [ExecutionStart_Date] DESC
	             )

GO


GO
IF OBJECT_ID('[BuildPromotions_RequiredApprovals]') IS NOT NULL DROP VIEW [BuildPromotions_RequiredApprovals]
GO

GO
IF OBJECT_ID('[BuildPromotions_RequiredApprovalsByUser]') IS NOT NULL DROP VIEW [BuildPromotions_RequiredApprovalsByUser]
GO

GO

IF OBJECT_ID('[BuildSchedules_Extended]') IS NOT NULL DROP VIEW [BuildSchedules_Extended]
GO

CREATE VIEW [BuildSchedules_Extended] AS

    SELECT
           BS.[Schedule_Id]
          ,BS.[Schedule_Name]
          ,BS.[Application_Id]
          ,A.[Application_Name]
          ,BS.[Active_Indicator]
          ,BS.[CreatedBy_User_Name]
          ,BS.[CreatedOn_Date]
          ,BS.[ModifiedBy_User_Name]
          ,BS.[ModifiedOn_Date]
          ,BS.[Schedule_Type]
          ,BS.[StartTime_Minutes]
          ,BS.[EndTime_Minutes]
          ,BS.[DailyInterval_Minutes]
          ,BS.[Recurrence_Type]
          ,BS.[RecurrenceFrequency_Value]
          ,BS.[RecurrenceWeeklyDay_Mask]
          ,BS.[RecurrenceMonthlyDay_Value]
          ,BS.[RecurrenceMonthlyRelativity_Type]
          ,BS.[RecurrenceMonthlyRelativityDay_Type]
          ,BS.[RecurrenceMonthlyRelativityWeek_Type]
          ,BS.[RecurrenceMonthlyRelativityWeekDay_Value]
          ,BS.[Url_AllowedIP_CSV]
          ,BS.[Url_Username_Text]
          ,BS.[Url_Password_Text]
          ,BS.[SourceControl_Path_Name]
          ,BS.[SourceControl_Provider_Id]
          ,[SourceControl_Provider_Name] = P.[Provider_Name]
          ,BS.[SourceControl_QuietPeriod_Minutes]
          ,BS.ReleaseFilter_ReleaseType_Code
          ,BS.Releasefilter_Workflow_Id
          ,BS.[BuildImporter_Configuration]


      FROM [BuildSchedules] BS

INNER JOIN [Applications] A
        ON A.[Application_Id] = BS.[Application_Id]

LEFT JOIN [Providers] P
        ON P.[Provider_Id] = BS.[SourceControl_Provider_Id]

GO


GO
IF OBJECT_ID('[BuildSchedules_Scheduled]') IS NOT NULL DROP VIEW [BuildSchedules_Scheduled]
GO

CREATE VIEW BuildSchedules_Scheduled AS

SELECT
  [Schedule_Id],

  A.[Application_Id],
  A.[Application_Name],

  [Schedule_Name],

  [RunAt_Date] = CASE WHEN [Schedule_Type] = 'Recurring' THEN BS.[CreatedOn_Date] ELSE NULL END,
  [RunAt_Minutes] = [StartTime_Minutes],
  
  [Url_AllowedIP_CSV],
  [Url_Username_Text],
  [Url_Password_Text],
  [SourceControl_Path_Name],
  [SourceControl_Provider_Id],
  
  [Schedule_Type],
  [SourceControl_QuietPeriod_Minutes],
  [ReleaseFilter_ReleaseType_Code],
  [Releasefilter_Workflow_Id],
  [BuildImporter_Configuration]
FROM 
  [BuildSchedules] BS
  JOIN [Applications] A ON A.[Application_Id] = BS.[Application_Id]

 WHERE [Schedule_Type] <> 'Recurring'
	AND [Active_Indicator] = 'Y'

UNION ALL


SELECT
  [Schedule_Id],

  A.[Application_Id],
  A.[Application_Name],

  [Schedule_Name],

  [RunAt_Date] = [Date],
  [RunAt_Minutes] =[Number],
  [Url_AllowedIP_CSV],
  [Url_Username_Text],
  [Url_Password_Text],
  [SourceControl_Path_Name],
  [SourceControl_Provider_Id],
  [Schedule_Type],
  [SourceControl_QuietPeriod_Minutes],
  [ReleaseFilter_ReleaseType_Code],
  [Releasefilter_Workflow_Id],
  [BuildImporter_Configuration]
FROM 
  [BuildSchedules] BS
  JOIN [Applications] A ON A.[Application_Id] = BS.[Application_Id]
  CROSS JOIN [Calendar] C
  CROSS JOIN [Numbers] N
WHERE 
  [Active_Indicator] = 'Y' AND
  [Number] BETWEEN [StartTime_Minutes] AND COALESCE([EndTime_Minutes],[StartTime_Minutes]) AND
  ([StartTime_Minutes] + [Number]) % COALESCE([DailyInterval_Minutes],1) = 0 AND
  [Schedule_Type] = 'Recurring' AND
  ( (
      [Recurrence_Type] = 'Monthly' AND
      ( (
          [RecurrenceMonthlyDay_Value] IS NOT NULL AND
            [RecurrenceMonthlyDay_Value] = DATEPART(d,[Date])
        ) OR (
          [RecurrenceMonthlyDay_Value] IS NULL AND
          ( (
               [RecurrenceMonthlyRelativity_Type] = 'Day' AND 
               ( /* [RecurrenceMonthlyRelativityDay_Type] can only be 'Last' */
                  DATEPART(m,[Date]+1) <> DATEPART(m,[Date])
               )
            ) OR (
               [RecurrenceMonthlyRelativity_Type] = 'Week' AND 
               (  [RecurrenceMonthlyRelativityWeekDay_Value] = 
                    DATEPART(weekday,[Date]) AND
                  CEILING(DATEPART(day,[Date])/7.0) =
                    CASE [RecurrenceMonthlyRelativityWeek_Type]
                      WHEN 'First'	THEN 1
                      WHEN 'Second' THEN 2
                      WHEN 'Third'	THEN 3
                      WHEN 'Fourth' THEN 4
                      WHEN 'Last' THEN CASE 
                        WHEN DATEPART(month,[Date]+7) <> DATEPART(month,[Date]) 
                             THEN CEILING(DATEPART(day,[Date])/7.0)
                        ELSE -1 END
                    END  
               )
            )
          )
        )
      ) AND
      0 = (dbo.ABS_MONTH_NUMBER([Date]) - dbo.ABS_MONTH_NUMBER(BS.[CreatedOn_Date])) % [RecurrenceFrequency_Value]
    ) OR (
      [Recurrence_Type] = 'Weekly' AND
      'Y' = SUBSTRING([RecurrenceWeeklyDay_Mask],DATEPART(dw,[Date]),1) AND
      0 = (dbo.ABS_WEEK_NUMBER([Date]) - dbo.ABS_WEEK_NUMBER(BS.[CreatedOn_Date])) % [RecurrenceFrequency_Value]
    ) OR (
      [Recurrence_Type] = 'Daily' AND
      0 = (CAST([Date] AS INT) - CAST(BS.[CreatedOn_Date] AS INT)) % [RecurrenceFrequency_Value]
    )
  )
GO


GO
IF OBJECT_ID('[BuildScheduleVariableDefinitions_Extended]') IS NOT NULL DROP VIEW [BuildScheduleVariableDefinitions_Extended]
GO

CREATE VIEW [BuildScheduleVariableDefinitions_Extended] AS

SELECT BSVD.[Schedule_Id],
       BSVD.[VariableDeclaration_Id],
       VDEC.[Variable_Name],
       VDEC.[Scope_Code],
       VDEC.[Required_Indicator],
       VDEC.[DefaultValue_Text],
       VDEC.[Variable_Configuration],
       BSVD.[Value_Text],
       [Application_Id] = VDEC.[Scoped_Application_Id]
  FROM [BuildScheduleVariableDefinitions] BSVD
       INNER JOIN [VariableDeclarations] VDEC
               ON BSVD.[VariableDeclaration_Id] = VDEC.[VariableDeclaration_Id]
GO

GO
IF OBJECT_ID('[Builds_Extended]') IS NOT NULL DROP VIEW [Builds_Extended]
GO

CREATE VIEW [Builds_Extended] AS


SELECT B.[Release_Number]
      ,B.[Build_Number]
      ,B.[BuildStatus_Name]
      ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])
      ,R.[Target_Date]
      ,[Sortable_Build_Number] = dbo.SortableBuildNumber(R.[Application_Id], B.[Build_Number])
	  ,B.[Build_Sequence]

      ,R.[ReleaseStatus_Name]
      ,[Numeric_Release_Number] = R.[Release_Sequence]
      ,[Release_Notes_Text] = R.[Notes_Text]
      ,B.[Rejection_Notes]
      ,B.[RejectedBy_User_Name]
      ,B.[Rejected_Date]
      ,B.[CreatedBy_User_Name]
      ,B.[CreatedOn_Date]
      ,B.[ModifiedBy_User_Name]
      ,B.[ModifiedOn_Date]
	  ,B.[BuildImporter_Configuration]

	  ,A.[Application_Id]
	  ,A.[Application_Name]

	  ,W.[Workflow_Id]
	  ,[Workflow_Name] = R.[Original_Workflow_Name]
      ,W.[AutoDeployRelease_Indicator]
      ,W.[AllowOutOfSequencePromotions_Indicator]
      ,W.[AutoCancelReleases_Indicator]

	  ,WS_1ST.[Step_Sequence] AS [First_Step_Sequence]
	  ,E_1ST.[Environment_Id] AS [First_Environment_Id]
	  ,E_1ST.[Environment_Name] AS [First_Environment_Name]

	  ,WS_CUR.[Step_Sequence] AS [Current_Step_Sequence]
	  ,E_CUR.[Environment_Id] AS [Current_Environment_Id]
	  ,E_CUR.[Environment_Name] AS [Current_Environment_Name]
	  ,BP_CUR.[PromotedBy_Name] AS [Current_PromotedBy_Name]
	  ,BP_CUR.[Promoted_Date] AS [Current_Promoted_Date]
	  ,BP_CUR.[PromotionStatus_Name] AS [Current_PromotionStatus_Name]
	  ,BP_CUR.[Environment_Id] AS [Current_Promotion_Environment_Id]
      ,BP_CUR.[Comments_Text] AS [Current_Promotion_Comments_Text]

	  ,[TotalExecutions_Count] =
	       (SELECT COUNT([Execution_Id])
			  FROM [dbo].[BuildExecutions] _BE
	         WHERE _BE.[Application_Id] = B.[Application_Id]
			   AND _BE.[Release_Number] = B.[Release_Number]
			   AND _BE.[Build_Number] = B.[Build_Number])

	  ,BE.[Execution_Id] AS [Current_Execution_Id]
	  ,BE_E.[Environment_Id] AS [Current_Execution_Environment_Id]
	  ,BE_E.[Environment_Name] AS [Current_Execution_Environment_Name]
	  ,BE.[ExecutionStart_Date] AS [Current_ExecutionStart_Date]
	  ,BE.[ExecutionStatus_Name] AS [Current_ExecutionStatus_Name]

	  ,WS_NXT.[Step_Sequence] AS [Next_Step_Sequence]
	  ,E_NXT.[Environment_Id] AS [Next_Environment_Id]
	  ,E_NXT.[Environment_Name] AS [Next_Environment_Name]

	  ,WS_FNL.[Step_Sequence] AS [Final_Step_Sequence]
	  ,E_FNL.[Environment_Id] AS [Final_Environment_Id]
	  ,E_FNL.[Environment_Name] AS [Final_Environment_Name]

	  ,[CurrentExecution_WarningLogEntry_Indicator] = BE.[WarningLogEntry_Indicator]

  FROM [Builds] B
       INNER JOIN [Releases] R
               ON B.[Application_Id] = R.[Application_Id]
              AND B.[Release_Number] = R.[Release_Number]
       INNER JOIN [Applications] A
               ON R.[Application_Id] = A.[Application_Id]
	   LEFT JOIN [Workflows] W
               ON R.[Workflow_Id] = W.[Workflow_Id]

       LEFT JOIN [WorkflowSteps] WS_1ST 
              ON WS_1ST.[Workflow_Id] = W.[Workflow_Id]
             AND WS_1ST.[Step_Sequence] =
                  (SELECT MIN(WS.[Step_Sequence])
                     FROM [WorkflowSteps] WS
                    WHERE WS.[Workflow_Id] = R.[Workflow_Id])
       LEFT JOIN [Environments] E_1ST
              ON E_1ST.[Environment_Id] = WS_1ST.[Environment_Id]

       LEFT JOIN [WorkflowSteps] WS_CUR
              ON WS_CUR.[Workflow_Id] = W.[Workflow_Id]
             AND WS_CUR.[Step_Sequence] =
                  (
					SELECT MAX(_WS.[Step_Sequence])
					  FROM [BuildPromotions] _BP
					  JOIN [WorkflowSteps] _WS
						ON R.[Workflow_Id] = _WS.[Workflow_Id]
					   AND _BP.[Environment_Id] = _WS.[Environment_Id]
					 WHERE _BP.[Application_Id] = B.[Application_Id]
					   AND _BP.[Release_Number] = B.[Release_Number]
					   AND _BP.[Build_Number] = B.[Build_Number]
				  )
	   LEFT JOIN [BuildPromotions] BP_CUR
			  ON B.[Application_Id] = BP_CUR.[Application_Id]
			 AND B.[Release_Number] = BP_CUR.[Release_Number]
			 AND B.[Build_Number] = BP_CUR.[Build_Number]
			 AND BP_CUR.[Promoted_Date] =
				 ( SELECT TOP 1 [Promoted_Date]
					 FROM [BuildPromotions] _BP_CUR
					WHERE _BP_CUR.[Application_Id] = B.[Application_Id]
					  AND _BP_CUR.[Release_Number] = B.[Release_Number]
					  AND _BP_CUR.[Build_Number] = B.[Build_Number]
				   ORDER BY [Promoted_Date] DESC )
       LEFT JOIN [Environments] E_CUR
              ON BP_CUR.[Environment_Id] = E_CUR.[Environment_Id]

	   /* Last (most recent) promotion */
	   LEFT JOIN [BuildPromotions] BP_LAST
			  ON B.[Application_Id] = BP_LAST.[Application_Id]
			 AND B.[Release_Number] = BP_LAST.[Release_Number]
			 AND B.[Build_Number] = BP_LAST.[Build_Number]
			 AND BP_LAST.[Promoted_Date] = 
				( SELECT TOP 1 [Promoted_Date]
					FROM [BuildPromotions] _BP_LAST
				   WHERE _BP_LAST.[Application_Id] = B.[Application_Id]
					 AND _BP_LAST.[Release_Number] = B.[Release_Number]
					 AND _BP_LAST.[Build_Number] = B.[Build_Number]
				  ORDER BY [Promoted_Date] DESC )

       LEFT JOIN [WorkflowSteps] WS_NXT
              ON WS_NXT.[Workflow_Id] = W.[Workflow_Id]
             AND B.[BuildStatus_Name] <> 'Rejected'
             AND WS_NXT.[Step_Sequence] =
                  (SELECT MIN(WS.[Step_Sequence])
                     FROM [WorkflowSteps] WS
                    WHERE WS.[Workflow_Id] = R.[Workflow_Id]
                      AND WS.[Step_Sequence] > COALESCE(WS_CUR.[Step_Sequence],0) )
       LEFT JOIN [Environments] E_NXT
              ON E_NXT.[Environment_Id] = WS_NXT.[Environment_Id]

	   LEFT JOIN [WorkflowSteps] WS_FNL
              ON WS_FNL.[Workflow_Id] = W.[Workflow_Id]
             AND WS_FNL.[Step_Sequence] =
                  (SELECT MAX(WS.[Step_Sequence])
                     FROM [WorkflowSteps] WS
                    WHERE WS.[Workflow_Id] = R.[Workflow_Id])
	   LEFT JOIN [Environments] E_FNL
              ON E_FNL.[Environment_Id] = WS_FNL.[Environment_Id]
	
	   LEFT JOIN [BuildExecutions] BE
	          ON BE.[Execution_Id] = 
	             (
	              SELECT TOP 1 [Execution_Id]
	                FROM [BuildExecutions] _BE
	               WHERE _BE.[Application_Id] = BP_CUR.[Application_Id]
	                 AND _BE.[Release_Number] = BP_CUR.[Release_Number]
	                 AND _BE.[Build_Number] = BP_CUR.[Build_Number]
	               ORDER BY [ExecutionStart_Date] DESC
	             )
	   LEFT JOIN [Environments] BE_E
	          ON BE_E.[Environment_Id] = BE.[Environment_Id]
GO


GO
IF OBJECT_ID('[Builds_RequiredPromotions]') IS NOT NULL DROP VIEW [Builds_RequiredPromotions]
GO

CREATE VIEW [Builds_RequiredPromotions] AS

SELECT B.[Application_Id],
       B.[Release_Number],
       B.[Build_Number],
       B.[BuildStatus_Name],

       R.[Workflow_Id],
       R.[ReleaseStatus_Name],

       E.[Environment_Id],
       WS.[Step_Sequence],

       E.[Environment_Name],

       BP.[Promoted_Date],
       BP.[PromotedBy_Name],
       BP.[Comments_Text],
       BP.[PromotionStatus_Name],
       BP.[StatusOverriddenBy_User_Name],
       BP.[StatusOverriddenReason_Text],
       BP.[StatusOverridden_Date]

  FROM [Builds] B
       INNER JOIN [Releases] R
	           ON R.[Application_Id] = B.[Application_Id]
			  AND R.[Release_Number] = B.[Release_Number]
	   INNER JOIN [WorkflowSteps] WS
	           ON WS.[Workflow_Id] = R.[Workflow_Id]
	   INNER JOIN [Environments] E
	           ON E.[Environment_Id] = WS.[Environment_Id]
	    LEFT JOIN [BuildPromotions] BP
		       ON BP.[Application_Id] = B.[Application_Id]
			  AND BP.[Release_Number] = B.[Release_Number]
			  AND BP.[Build_Number] = B.[Build_Number]
			  AND BP.[Environment_Id] = WS.[Environment_Id]

GO
IF OBJECT_ID('[BuildTestResults_AllGroups]') IS NOT NULL DROP VIEW [BuildTestResults_AllGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE VIEW [BuildTestResults_AllGroups] WITH SCHEMABINDING AS

SELECT BTR.[Group_Name],
       [Group_Count] = COUNT_BIG(*)
  FROM dbo.[BuildTestResults] BTR
 GROUP BY BTR.[Group_Name]

GO

CREATE UNIQUE CLUSTERED INDEX [IX__BuildTestResults_AllGroups]
	ON [BuildTestResults_AllGroups] ([Group_Name])

GO

GO
IF OBJECT_ID('[BuildTestResults_Extended]') IS NOT NULL DROP VIEW [BuildTestResults_Extended]
GO

CREATE VIEW [BuildTestResults_Extended] AS

SELECT BTR.[Execution_Id]
      ,BTR.[Group_Name]
      ,BTR.[Test_Sequence]
      ,BTR.[Test_Name]
      ,BTR.[TestStatus_Code]
      ,[TestPassed_Indicator] = CASE BTR.[TestStatus_Code] WHEN 'P' THEN 'Y' ELSE 'N' END
      ,BTR.[TestResult_Text]
      ,BTR.[TestStarted_Date]
      ,BTR.[TestEnded_Date]
      ,BTR.[BuildExecution_PlanAction_Id]
      ,BEAGA.[BuildExecution_DeploymentPlan_Id]
      ,BEAGA.[Original_DeploymentPlanActionGroup_Sequence]
      ,BEAGA.[Original_Action_Sequence],
      A.[Application_Id],
      A.[Application_Name],
      R.[Release_Number],
      [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])

  FROM [BuildTestResults] BTR
	   INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
			   ON BTR.[BuildExecution_PlanAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
	   INNER JOIN [BuildExecutions] BE
	           ON BTR.[Execution_Id] = BE.[Execution_Id]
	   INNER JOIN [Releases] R
	           ON BE.[Application_Id] = R.[Application_Id]
	          AND BE.[Release_Number] = R.[Release_Number]
	   INNER JOIN [Applications] A
	           ON BE.[Application_Id] = A.[Application_Id]
GO


GO
IF OBJECT_ID('[BuildTestResults_TestGroupResults]') IS NOT NULL DROP VIEW [BuildTestResults_TestGroupResults]
GO

CREATE VIEW [BuildTestResults_TestGroupResults] AS

SELECT TR.[Execution_Id],
       TR.[Group_Name],
       BE.[Application_Id],
       BE.[Release_Number],
       BE.[Build_Number],
       COUNT(*) AS [Total_Tests], 
       COUNT(NULLIF(NULLIF(TR.[TestStatus_Code], 'F'), 'I')) AS [Passed_Tests],
       COUNT(NULLIF(NULLIF(TR.[TestStatus_Code], 'P'), 'I')) AS [Failed_Tests],
       COUNT(NULLIF(NULLIF(TR.[TestStatus_Code], 'P'), 'F')) AS [Inconclusive_Tests]
  FROM [BuildTestResults] TR
  INNER JOIN [BuildExecutions] BE
          ON BE.[Execution_Id] = TR.[Execution_Id]
 GROUP BY 
    TR.[Execution_Id], 
    TR.[Group_Name],
    BE.[Application_Id],
    BE.[Release_Number],
    BE.[Build_Number]

GO
IF OBJECT_ID('[ChangeControlEnvironments_Extended]') IS NOT NULL DROP VIEW [ChangeControlEnvironments_Extended]
GO

CREATE VIEW [ChangeControlEnvironments_Extended] AS

	SELECT CCE.[ChangeControl_Id],
	       CCE.[Environment_Id],
	       CC.[ChangeControl_Name],
	       E.[Environment_Name]
	  FROM [ChangeControlEnvironments] CCE
INNER JOIN [Environments] E
	    ON E.[Environment_Id] = CCE.[Environment_Id]
INNER JOIN [ChangeControls] CC
        ON CC.[ChangeControl_Id] = CCE.[ChangeControl_Id]

GO

GO
IF OBJECT_ID('[ChangeControlPerformances_Extended]') IS NOT NULL DROP VIEW [ChangeControlPerformances_Extended]
GO

CREATE VIEW [ChangeControlPerformances_Extended] AS

	SELECT CCP.[ChangeControl_Id],
	       CCP.[Environment_Id],
	       CCP.[PerformedBy_User_Name],
	       CCP.[Performed_Date],
	       CCP.[Notes_Text],
	       CCP.[Execution_Id],
	       E.[Environment_Name],
	       CC.[Application_Id],
	       CC.[Release_Number],
	       CC.[Group_Name],
	       CC.[Description_Text],
	       CC.[ChangeControl_Name],
	       CC.[Principal_Name],
	       CC.[PrincipalType_Code],
	       CC.[CreatedBy_User_Name],
	       CC.[ModifiedBy_User_Name],
	       CC.[CreatedOn_Date],
	       CC.[ModifiedOn_Date],
	       CC.[PerformanceType_Code],
	       CC.[Active_Indicator],
           [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
	  FROM [ChangeControlPerformances] CCP
INNER JOIN [Environments] E
	    ON  CCP.[Environment_Id] = E.[Environment_Id]
INNER JOIN [ChangeControls] CC
        ON CCP.[ChangeControl_Id] = CC.[ChangeControl_Id]
           INNER JOIN [Releases] R
                   ON R.[Application_Id] = CC.[Application_Id]
                  AND R.[Release_Number] = CC.[Release_Number]
GO

GO
IF OBJECT_ID('[ChangeControlPerformances_ReleaseSummary]') IS NOT NULL DROP VIEW [ChangeControlPerformances_ReleaseSummary]
GO

CREATE VIEW [ChangeControlPerformances_ReleaseSummary] AS

	    SELECT  CC.[ChangeControl_Id],
	     E.[Environment_Name],
	     E.[Environment_Id],
       CCP.[ChangeControlPerformance_Id],
       CCP.[Notes_Text],
       CCP.[PerformedBy_User_Name],
       CCP.[Performed_Date],
           [ChangeControlPerformed_Indicator] = CASE 
                WHEN [ChangeControlPerformance_Id] IS NULL 
                    THEN 'N' 
                    ELSE 'Y' 
                END
          FROM [ChangeControlEnvironments] CCE
    INNER JOIN [ChangeControls] CC
            ON CC.[ChangeControl_Id] = CCE.[ChangeControl_Id]
    LEFT JOIN  [ChangeControlPerformances] CCP
            ON CCP.[ChangeControl_Id] = CC.[ChangeControl_Id]
            AND CCP.[Environment_Id] = CCE.[Environment_Id]
    INNER JOIN [Environments] E
            ON E.[Environment_Id] = CCE.[Environment_Id]

GO

GO
IF OBJECT_ID('[ConfigurationFileDeployments_Extended]') IS NOT NULL DROP VIEW [ConfigurationFileDeployments_Extended]
GO

CREATE VIEW [ConfigurationFileDeployments_Extended] AS

SELECT CFD.[ConfigurationFileDeployment_Id]
      ,CFD.[Execution_Id]
      ,CFD.[ConfigurationFile_Id]
	  ,CF.[ConfigurationFile_Name]
	  ,CF.[FilePath_Text]
      ,CFD.[Instance_Name]
      ,CFD.[Version_Number]
      ,CFD.[Server_Id]
      ,CFD.[DeployedTo_Path]
      ,CFD.[DeployedBy_User_Name]
      ,CFD.[DeployedOn_Date]

      ,CFI.[Environment_Id]

      ,CF.[Deployable_Id]
      ,[ConfigurationFile_Active_Indicator] = CF.[Active_Indicator]

      ,D.[Deployable_Name]
      ,D.[Application_Id]

      ,S.[Server_Name]
      ,E.[Environment_Name]

	  ,R.[Release_Number]
	  ,[Release_Name] = COALESCE(R.[Release_Number], R.[Release_Name])
	  ,BE.[Build_Number]
	  ,[Execution_Environment_Id] = BE.[Environment_Id]
	  ,[Execution_Environment_Name] = BE_E.[Environment_Name]

 FROM [ConfigurationFileDeployments] CFD
      INNER JOIN [ConfigurationFileInstances] CFI
              ON CFI.[ConfigurationFile_Id] = CFD.[ConfigurationFile_Id]
             AND CFI.[Instance_Name] = CFD.[Instance_Name]
       LEFT JOIN [Environments] E
              ON E.[Environment_Id] = CFI.[Environment_Id]
      INNER JOIN [ConfigurationFiles] CF
              ON CF.[ConfigurationFile_Id] = CFD.[ConfigurationFile_Id]
      INNER JOIN [ConfigurationFileVersions] CFV
              ON CFV.[ConfigurationFile_Id] = CFD.[ConfigurationFile_Id]
             AND CFV.[Instance_Name] = CFD.[Instance_Name]
             AND CFV.[Version_Number] = CFD.[Version_Number]
      INNER JOIN [Deployables] D
              ON D.[Deployable_Id] = CF.[Deployable_Id]
      INNER JOIN [Servers] S
              ON S.[Server_Id] = CFD.[Server_Id]
	   LEFT JOIN [BuildExecutions] BE
	          ON BE.[Execution_Id] = CFD.[Execution_Id]
	   LEFT JOIN [Releases] R
	          ON R.[Application_Id] = BE.[Application_Id]
			 AND R.[Release_Number] = BE.[Release_Number]
	   LEFT JOIN [Environments] BE_E
	          ON BE_E.[Environment_Id] = BE.[Environment_Id]

GO

GO
IF OBJECT_ID('[ConfigurationFileInstances_Extended]') IS NOT NULL DROP VIEW [ConfigurationFileInstances_Extended]
GO

CREATE VIEW [ConfigurationFileInstances_Extended] AS

SELECT CFI.[ConfigurationFile_Id]
      ,CFI.[Instance_Name]
      ,CFI.[Active_Indicator]
      ,CFI.[Environment_Id]
      ,E.[Environment_Name]
	  ,E.[Display_Sequence]

      ,CF.[Deployable_Id]
      ,CF.[FilePath_Text]
      ,[ConfigurationFile_Active_Indicator] = CF.[Active_Indicator]

      ,D.[Application_Id]

      ,[LatestVersion_Version_Number] = CFV.[Version_Number]
      ,[LatestVersion_VersionNotes_Text] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[VersionNotes_Text] ELSE CFV_NonNull.[VersionNotes_Text] END
      ,[LatestVersion_File_Date] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[File_Date] ELSE CFV_NonNull.[File_Date] END
      ,[LatestVersion_File_Bytes] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[File_Bytes] ELSE CFV_NonNull.[File_Bytes] END
      ,[LatestVersion_CreatedOn_Date] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[CreatedOn_Date] ELSE CFV_NonNull.[CreatedOn_Date] END
      ,[LatestVersion_CreatedByUser_Name] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[CreatedBy_User_Name] ELSE CFV_NonNull.[CreatedBy_User_Name] END

      ,CFI.[Template_Indicator]
      ,CFI.[Template_Instance_Name]
      ,[Template_Instance_Count] = (    
                             SELECT CASE WHEN CFI.[Template_Indicator] = 'Y' THEN COUNT(*) ELSE NULL END
                             FROM [ConfigurationFileInstances] 
                             WHERE [ConfigurationFile_Id] = CFI.[ConfigurationFile_Id] 
                               AND [Active_Indicator] = 'Y'
                               AND [Template_Instance_Name] = CFI.[Instance_Name]
                               )
	  ,CFI.[TransformType_Code]

  FROM [ConfigurationFileInstances] CFI
       INNER JOIN [ConfigurationFiles] CF
               ON CFI.[ConfigurationFile_Id] = CF.[ConfigurationFile_Id]
       INNER JOIN [Deployables] D
               ON D.[Deployable_Id] = CF.[Deployable_Id]

	    LEFT JOIN (SELECT [ConfigurationFile_Id],
		                  [Instance_Name],
						  [Latest_Version_Number] = MAX(CFV_Inner.[Version_Number]),
						  [Last_NonNull_Version] = (SELECT MAX(CFV_Inner_Inner.[Version_Number])
						                              FROM [ConfigurationFileVersions] CFV_Inner_Inner
													 WHERE CFV_Inner_Inner.[ConfigurationFile_Id] = CFV_Inner.[ConfigurationFile_Id]
													   AND CFV_Inner_Inner.[Instance_Name] = CFV_Inner.[Instance_Name]
													   AND CFV_Inner_Inner.[Version_Number] < MAX(CFV_Inner.[Version_Number])
													   AND CFV_Inner_Inner.[Null_Indicator] = 'N')
		             FROM [ConfigurationFileVersions] CFV_Inner
					GROUP BY [ConfigurationFile_Id], [Instance_Name]) CFV_Latest
			   ON CFV_Latest.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
			  AND CFV_Latest.[Instance_Name] = CFI.[Instance_Name]

		LEFT JOIN [ConfigurationFileVersions] CFV
		       ON CFV.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
			  AND CFV.[Instance_Name] = CFI.[Instance_Name]
			  AND CFV.[Version_Number] = CFV_Latest.[Latest_Version_Number]

		LEFT JOIN [ConfigurationFileVersions] CFV_NonNull
		       ON CFV.[Null_Indicator] = 'Y'
			  AND CFV_NonNull.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
			  AND CFV_NonNull.[Instance_Name] = CFI.[Instance_Name]
			  AND CFV_NonNull.[Version_Number] = CFV_Latest.[Last_NonNull_Version]

        LEFT JOIN [Environments] E
               ON E.[Environment_Id] = CFI.[Environment_Id]
GO


GO

IF OBJECT_ID('[ConfigurationFiles_Extended]') IS NOT NULL DROP VIEW [ConfigurationFiles_Extended]
GO

CREATE VIEW [ConfigurationFiles_Extended] AS

SELECT C.[ConfigurationFile_Id]

      ,C.[Deployable_Id]
      ,C.[FilePath_Text]
	  ,C.[ConfigurationFile_Name]
	  ,C.[Description_Text]
      ,D.[Application_Id]
      ,C.[Active_Indicator]
      ,C.[CreatedBy_User_Name]
      ,C.[CreatedOn_Date]
      ,C.[ModifiedBy_User_Name]
      ,C.[ModifiedOn_Date]

      ,D.[Deployable_Name]

      ,[LatestDeployment_ConfigurationFileDeployment_Id] = CFD.[ConfigurationFileDeployment_Id]
      ,[LatestDeployment_Execution_Id] = CFD.[Execution_Id]
      ,[LatestDeployment_Instance_Name] = CFD.[Instance_Name]
      ,[LatestDeployment_Version_Number] = CFD.[Version_Number]
      ,[LatestDeployment_Server_Id] = CFD.[Server_Id]
      ,[LatestDeployment_DeployedTo_Path]= CFD.[DeployedTo_Path]
      ,[LatestDeployment_DeployedBy_User_Name] = CFD.[DeployedBy_User_Name]
      ,[LatestDeployment_DeployedOn_Date] = CFD.[DeployedOn_Date]
      ,[LatestDeployment_Server_Name] = S_CFD.[Server_Name]
  
      ,CFV.[Instance_Name]  [LatestVersion_Instance_Name]
      ,CFV.[Version_Number]  [LatestVersion_Version_Number]
      ,CFV.[VersionNotes_Text]  [LatestVersion_VersionNotes_Text]
      ,CFV.[File_Date]  [LatestVersion_File_Date]
      ,CFV.[CreatedBy_User_Name]  [LatestVersionCreatedBy_User_Name]
      ,CFV.[CreatedOn_Date]  [LatestVersionCreatedOn_Date]

 FROM [ConfigurationFiles] C
       INNER JOIN [Deployables] D
               ON C.[Deployable_Id] = D.[Deployable_Id]
       LEFT  JOIN (SELECT [ConfigurationFile_Id]
                         ,MAX([ConfigurationFileDeployment_Id]) [ConfigurationFileDeployment_Id]
                     FROM [ConfigurationFileDeployments] _CFD
                    GROUP BY [ConfigurationFile_Id]) CFD_MAX
               ON CFD_MAX.[ConfigurationFile_Id]  = C.[ConfigurationFile_Id] 
       LEFT  JOIN [ConfigurationFileDeployments] CFD
               ON CFD.[ConfigurationFileDeployment_Id]= CFD_MAX.[ConfigurationFileDeployment_Id]
       LEFT  JOIN [Servers] S_CFD
               ON S_CFD.[Server_Id] = CFD.[Server_Id]
  
       LEFT  JOIN (SELECT ROW_NUMBER() OVER (PARTITION BY [ConfigurationFile_Id] 
                                                 ORDER BY [File_Date] DESC) [Row_Number],
                          [ConfigurationFile_Id], [Instance_Name], [Version_Number]
                     FROM [ConfigurationFileVersions]) CFV_MAX
               ON CFV_MAX.[ConfigurationFile_Id] = C.[ConfigurationFile_Id]
              AND CFV_MAX.[Row_Number] = 1
       LEFT  JOIN [ConfigurationFileVersions] CFV
               ON CFV.[ConfigurationFile_Id] = CFV_MAX.[ConfigurationFile_Id]
              AND CFV.[Instance_Name] = CFV_MAX.[Instance_Name]
              AND CFV.[Version_Number] = CFV_MAX.[Version_Number]
GO


GO
IF OBJECT_ID('[ConfigurationFileVersions_Combined]') IS NOT NULL DROP VIEW [ConfigurationFileVersions_Combined]
GO

CREATE VIEW [ConfigurationFileVersions_Combined] AS

SELECT CF.[ConfigurationFile_Id],
       CFV.[Version_Number],
	   CF.[ConfigurationFile_Name],
	   CF.[FilePath_Text],
	   [Version_CreatedOn_Date] = CFV.[CreatedOn_Date],
	   [Version_CreatedBy_User_Name] = CFV.[CreatedBy_User_Name]
  FROM [ConfigurationFiles] CF
       INNER JOIN (SELECT [ConfigurationFile_Id],
                          [Version_Number],
	                      [CreatedOn_Date] = MAX([CreatedOn_Date]),
						  [CreatedBy_User_Name] = MAX([CreatedBy_User_Name])
                     FROM [ConfigurationFileVersions]
                    GROUP BY [ConfigurationFile_Id], [Version_Number]) CFV
			   ON CFV.[ConfigurationFile_Id] = CF.[ConfigurationFile_Id]

GO


GO
IF OBJECT_ID('[ConfigurationFileVersions_Extended]') IS NOT NULL DROP VIEW [ConfigurationFileVersions_Extended]
GO

CREATE VIEW [ConfigurationFileVersions_Extended] AS

SELECT CFV.[ConfigurationFile_Id],
       CFV.[Instance_Name],
       CFV.[Version_Number],
	   CFI.[Environment_Id],
	   E.[Environment_Name],
	   CFI.[Active_Indicator],
	   D.[Deployable_Id],
	   D.[Deployable_Name],
	   CFI.[Template_Indicator],
	   CFI.[Template_Instance_Name],
	   CFI.[TransformType_Code],
	   CF.[FilePath_Text],
	   A.[Application_Id],
	   A.[Application_Name],
	   AG.[ApplicationGroup_Id],
	   AG.[ApplicationGroup_Name],
	   [VersionNotes_Text] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[VersionNotes_Text] ELSE CFV_Last.[VersionNotes_Text] END,
	   [File_Date] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[File_Date] ELSE CFV_Last.[File_Date] END,
	   [File_Bytes] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[File_Bytes] ELSE CFV_Last.[File_Bytes] END,
	   [CreatedBy_User_Name] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[CreatedBy_User_Name] ELSE CFV_Last.[CreatedBy_User_Name] END,
	   [CreatedOn_Date] = CASE WHEN CFV.[Null_Indicator] = 'N' THEN CFV.[CreatedOn_Date] ELSE CFV_Last.[CreatedOn_Date] END,
	   CFV.[Null_Indicator]

  FROM [ConfigurationFileVersions] CFV
       INNER JOIN [ConfigurationFiles] CF
	           ON CF.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
	   INNER JOIN [ConfigurationFileInstances] CFI
	           ON CFI.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
			  AND CFI.[Instance_Name] = CFV.[Instance_Name]
	    LEFT JOIN [Environments] E
	           ON E.[Environment_Id] = CFI.[Environment_Id]
	   INNER JOIN [Deployables] D
	           ON D.[Deployable_Id] = CF.[Deployable_Id]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = D.[Application_Id]
	    LEFT JOIN [ApplicationGroups] AG
	           ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]
	    LEFT JOIN (SELECT [ConfigurationFile_Id],
		                  [Instance_Name],
						  [Version_Number],
						  [Last_NonNull_Version] = (SELECT MAX(CFV_Inner_Inner.[Version_Number])
						                              FROM [ConfigurationFileVersions] CFV_Inner_Inner
													 WHERE CFV_Inner_Inner.[ConfigurationFile_Id] = CFV_Inner.[ConfigurationFile_Id]
													   AND CFV_Inner_Inner.[Instance_Name] = CFV_Inner.[Instance_Name]
													   AND CFV_Inner_Inner.[Version_Number] < CFV_Inner.[Version_Number]
													   AND CFV_Inner_Inner.[Null_Indicator] = 'N')
		             FROM [ConfigurationFileVersions] CFV_Inner
					WHERE [Null_Indicator] = 'Y') CFV2
			   ON CFV.[Null_Indicator] = 'Y'
			  AND CFV2.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
			  AND CFV2.[Instance_Name] = CFV.[Instance_Name]
			  AND CFV2.[Version_Number] = CFV.[Version_Number]
		LEFT JOIN [ConfigurationFileVersions] CFV_Last
		       ON CFV.[Null_Indicator] = 'Y'
			  AND CFV_Last.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
			  AND CFV_Last.[Instance_Name] = CFV.[Instance_Name]
			  AND CFV_Last.[Version_Number] = CFV2.[Last_NonNull_Version]
GO


GO
IF OBJECT_ID('[ConfigurationFileVersions_ModifiedInstances]') IS NOT NULL DROP VIEW [ConfigurationFileVersions_ModifiedInstances]
GO

CREATE VIEW [ConfigurationFileVersions_ModifiedInstances] AS

SELECT CFV.[ConfigurationFile_Id],
       CFV.[Version_Number],
	   CFV.[Instance_Name]
  FROM [ConfigurationFileVersions] CFV
 WHERE CFV.[Null_Indicator] = 'N'

GO


GO

IF OBJECT_ID('[DashboardGadgets_Extended]') IS NOT NULL DROP VIEW [DashboardGadgets_Extended]
GO

CREATE VIEW [DashboardGadgets_Extended] AS

SELECT G.[Gadget_Id],
       G.[Dashboard_Id],
       D.[Application_Id],
       A.[Application_Name],
       D.[User_Name],
       D.[DashboardScope_Code],
       G.[Dashboard_Zone],
       G.[Display_Sequence],
       G.[DashboardGadget_Name],
       G.[DashboardGadget_Description],
       G.[DashboardGadget_Configuration],
       G.[CreatedBy_User_Name],
       G.[CreatedOn_Date],
       G.[ModifiedBy_User_Name],
       G.[ModifiedOn_Date]
  FROM [DashboardGadgets] G
       INNER JOIN [Dashboards] D
               ON D.[Dashboard_Id] = G.[Dashboard_Id]
       LEFT JOIN [Applications] A
              ON A.[Application_Id] = D.[Application_Id]
GO


GO

IF OBJECT_ID('[DatabaseChangeScriptExecutions_Extended]') IS NOT NULL
	DROP VIEW [DatabaseChangeScriptExecutions_Extended]
GO

CREATE VIEW [DatabaseChangeScriptExecutions_Extended]
AS


SELECT SX.[ScriptExecution_Id]
      ,SX.[Script_Id]
      ,SX.[ExecutedOn_Date]
      ,SX.[ExecutedBy_User_Name]
      ,SX.[Success_Indicator]
      ,SX.[Log_Text]
      ,SX.[Execution_Id]
      ,SX.[Provider_Id]

      ,S.[Application_Id]
      ,S.[Release_Number]
      ,S.[Deployable_Id]
      ,S.[Script_Name]
      ,[Script_Active_Indicator] = S.[Active_Indicator]
      ,D.[Deployable_Name]

      ,P.[Provider_Name]

      ,E.[Environment_Id]
      ,E.[Environment_Name]
      ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])

  FROM [DatabaseChangeScriptExecutions] SX
       INNER JOIN [DatabaseChangeScripts] S
               ON SX.[Script_Id] = S.[Script_Id]
       INNER JOIN [Deployables] D
               ON S.[Deployable_Id] = D.[Deployable_Id]
       INNER JOIN [Releases] R
               ON S.[Release_Number] = R.[Release_Number]
              AND S.[Application_Id] = R.[Application_Id]
       LEFT  JOIN [Providers] P
               ON SX.[Provider_Id] = P.[Provider_Id]
       LEFT  JOIN [DatabaseChangeScriptProviders] DBP
               ON DBP.[Provider_Id] = P.[Provider_Id]
       LEFT  JOIN [Environments] E
               ON DBP.[Environment_Id] = E.[Environment_Id]


GO

IF OBJECT_ID('[DatabaseChangeScriptProviders_Extended]') IS NOT NULL DROP VIEW [DatabaseChangeScriptProviders_Extended]
GO

CREATE VIEW [DatabaseChangeScriptProviders_Extended] AS

SELECT DBP.[Provider_Id]

      ,P.[Provider_Name]
      ,P.[Provider_Description]
      ,P.[Provider_Configuration]
      ,P.[ExecuteOn_Server_Id]
      ,P.[CreatedBy_User_Name]
      ,P.[CreatedOn_Date]
      ,P.[ModifiedBy_User_Name]
      ,P.[ModifiedOn_Date]

      ,A.[Application_Id]
      ,A.[Application_Name]

      ,E.[Environment_Id]
      ,E.[Environment_Name]

  FROM [DatabaseChangeScriptProviders] DBP
       INNER JOIN [Providers] P
               ON DBP.[Provider_Id] = P.[Provider_Id]
       INNER JOIN [Applications] A
               ON DBP.[Application_Id] = A.[Application_Id]

       INNER JOIN [Environments] E
               ON DBP.[Environment_Id] = E.[Environment_Id]
GO


GO

IF OBJECT_ID('[DatabaseChangeScripts_Extended]') IS NOT NULL DROP VIEW [DatabaseChangeScripts_Extended]
GO

CREATE VIEW [DatabaseChangeScripts_Extended] AS

	SELECT dS.[Script_Id]
		  ,dS.[Application_Id]
		  ,dS.[Release_Number]
		  ,dS.[Deployable_Id]
		  ,dS.[Script_Name]
		  ,dS.[Script_Text]
		  ,dS.[Active_Indicator]
		  ,dS.[CreatedBy_User_Name]
		  ,dS.[CreatedOn_Date]
		  ,dS.[ModifiedBy_User_Name]
		  ,dS.[ModifiedOn_Date]

          ,A.[Application_Name]
          ,[Numeric_Release_Number] = R.[Release_Sequence]

		  ,[LastExecuted_ScriptExecution_Id] = SX.[ScriptExecution_Id]
		  ,[LastExecutedOn_Date] = SX.[ExecutedOn_Date]
		  ,[LastExecutedBy_User_Name] = SX.[ExecutedBy_User_Name]
		  ,[LastExecuted_Success_Indicator] = SX.[Success_Indicator]
		  ,[LastExecuted_Log_Text] = SX.[Log_Text]
		  ,[LastExecuted_Execution_Id] = SX.[Execution_Id]
		  ,[LastExecuted_Provider_Id] = SX.[Provider_Id]

		  ,[LastExecuted_Provider_Name] = P.[Provider_Name]

		  ,D.[Deployable_Name]
          ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])

	  FROM [DatabaseChangeScripts] dS
	       INNER JOIN [Applications] A
	               ON dS.[Application_Id] = A.[Application_Id]
           INNER JOIN [Releases] R
                   ON R.[Application_Id] = dS.[Application_Id]
                  AND R.[Release_Number] = dS.[Release_Number]
	       INNER JOIN [Deployables] D
	               ON dS.[Deployable_Id] = D.[Deployable_Id]
	       LEFT  JOIN (
	                    SELECT [Script_Id], MAX([ScriptExecution_Id]) [ScriptExecution_Id]
	                      FROM [DatabaseChangeScriptExecutions] GROUP BY [Script_Id]
	                  ) SX_MAX
	               ON SX_MAX.[Script_Id] = dS.[Script_Id]
	       LEFT  JOIN [DatabaseChangeScriptExecutions] SX
	               ON SX.[ScriptExecution_Id] = SX_MAX.[ScriptExecution_Id]
		   LEFT  JOIN [Providers] P
				   ON SX.[Provider_Id] = P.[Provider_Id]
GO


GO

IF OBJECT_ID('[DeployablesImported_Extended]') IS NOT NULL DROP VIEW [DeployablesImported_Extended]
GO

CREATE VIEW [DeployablesImported_Extended] AS

SELECT D.[Deployable_Id]
      ,D.[Deployable_Name]
      ,D.[CreatedBy_User_Name]
      ,D.[CreatedOn_Date]
      ,D.[ModifiedBy_User_Name]
      ,D.[ModifiedOn_Date]
	
      ,A.[Application_Id]
      ,A.[Application_Name]
      
      ,[ImportedFrom_Application_Id] = DA.[Application_Id]
      ,[ImportedFrom_Application_Name] = DA.[Application_Name]

FROM [DeployablesImported] DI
INNER JOIN [Applications] A
  ON A.[Application_Id] = DI.[Application_Id]
INNER JOIN [Deployables] D
  ON D.[Deployable_Id] = DI.[Deployable_Id]
INNER JOIN [Applications] DA
  ON DA.[Application_Id] = D.[Application_Id]
  
GO


GO

IF OBJECT_ID('[Deployables_Extended]') IS NOT NULL DROP VIEW [Deployables_Extended]
GO

CREATE VIEW [Deployables_Extended] AS

SELECT D.[Deployable_Id]
      ,D.[Deployable_Name]
      ,D.[DeployableType_Code]
      ,D.[CreatedBy_User_Name]
      ,D.[CreatedOn_Date]
      ,D.[ModifiedBy_User_Name]
      ,D.[ModifiedOn_Date]

      ,A.[Application_Id]
      ,A.[Application_Name]

      ,[Dependencies_Count] =
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[Deployable_Id] = D.[Deployable_Id])
      ,[Dependants_Count] = 
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[DependsOn_Deployable_Id] = D.[Deployable_Id])

FROM [Deployables] D
JOIN [Applications] A
  ON A.[Application_Id] = D.[Application_Id]
GO


GO
IF OBJECT_ID('[DeploymentPlanActionGroups_Extended]') IS NOT NULL DROP VIEW [DeploymentPlanActionGroups_Extended]
GO

CREATE VIEW [DeploymentPlanActionGroups_Extended] AS

	SELECT DPAG.[DeploymentPlanActionGroup_Id],
	       DPAG.[DeploymentPlanActionGroup_Sequence],
	       DPAG.[DeploymentPlan_Id],
	       DPAG.[Deployable_Name],
	       [Active_Indicator] = DPAG.[Active_Indicator],
	       [ActionGroup_Name] = COALESCE(DPAG.[ActionGroup_Name], AG.[ActionGroup_Name]),
	       [ActionGroup_Description] = COALESCE(DPAG.[ActionGroup_Description], AG.[ActionGroup_Description]),
	       [Predicate_Configuration] = COALESCE(DPAG.[Predicate_Configuration], AG.[Predicate_Configuration]),
	       [Server_Id] = CASE WHEN DPAG.[Server_Variable_Name] IS NULL THEN COALESCE(DPAG.[Server_Id], AG.[Server_Id]) ELSE NULL END,
	       [Server_Variable_Name] = CASE WHEN DPAG.[Server_Id] IS NULL THEN COALESCE(DPAG.[Server_Variable_Name], AG.[Server_Variable_Name]) ELSE NULL END,
	       [ServerType_Code] = CASE WHEN DPAG.[Server_Variable_Name] IS NULL THEN S.[ServerType_Code] ELSE NULL END,
	       [Server_Name] = CASE WHEN DPAG.[Server_Variable_Name] IS NULL THEN S.[Server_Name] ELSE NULL END,
	       DPAG.[Iterate_Servers_Indicator],
	       DPAG.[Iterate_Deployables_Indicator],
	       DPAG.[Parallel_Indicator],
	       DPAG.[OnSuccess_DeploymentPlanActionGroup_Id],
		   [OnSuccess_DeploymentPlanActionGroup_Sequence] = S_DPAG.[DeploymentPlanActionGroup_Sequence],
		   [OnSuccess_DeploymentPlanActionGroup_Name] = COALESCE(S_DPAG.[ActionGroup_Name], S_AG.[ActionGroup_Name]),
	       DPAG.[OnFailure_DeploymentPlanActionGroup_Id],
		   [OnFailure_DeploymentPlanActionGroup_Sequence] = F_DPAG.[DeploymentPlanActionGroup_Sequence],
		   [OnFailure_DeploymentPlanActionGroup_Name] = COALESCE(F_DPAG.[ActionGroup_Name], F_AG.[ActionGroup_Name]),
	       DPAG.[OnFailure_Continue_Indicator],

	       [DeploymentPlanActionGroup_ActionGroup_Name] = DPAG.[ActionGroup_Name],
	       [DeploymentPlanActionGroup_ActionGroup_Description] = DPAG.[ActionGroup_Description],
	       [DeploymentPlanActionGroup_Predicate_Configuration] = DPAG.[Predicate_Configuration],
	       [DeploymentPlanActionGroup_ModifiedBy_User_Name] = DPAG.[ModifiedBy_User_Name],
	       [DeploymentPlanActionGroup_ModifiedOn_Date] = DPAG.[ModifiedOn_Date],
	       [DeploymentPlanActionGroup_Server_Id] = DPAG.[Server_Id],
	       [DeploymentPlanActionGroup_Server_Variable_Name] = DPAG.[Server_Variable_Name],

	       [ActionGroup_Id] = AG.[ActionGroup_Id],
	       [ActionGroup_CreatedBy_User_Name] = AG.[CreatedBy_User_Name],
	       [ActionGroup_CreatedOn_Date] = AG.[CreatedOn_Date],
	       [ActionGroup_ModifiedBy_User_Name] = AG.[ModifiedBy_User_Name],
	       [ActionGroup_ModifiedOn_Date] = AG.[ModifiedOn_Date],
	       [ActionGroup_ActionGroup_Name] = AG.[ActionGroup_Name],
	       [ActionGroup_Predicate_Configuration] = AG.[Predicate_Configuration],
	       [ActionGroup_ActionGroup_Description] = AG.[ActionGroup_Description],
	       [ActionGroup_Server_Id] = AG.[Server_Id],
	       [ActionGroup_Server_Variable_Name] = AG.[Server_Variable_Name],

	       [Referenced_Application_Id] = R_A.[Application_Id],
	       [Referenced_Application_Name] = R_A.[Application_Name],

	       [DeploymentPlanActionGroup_Count] = (SELECT COUNT(*)
						                         FROM [DeploymentPlanActionGroups] _DPAG
							                    WHERE _DPAG.[ActionGroup_Id] = DPAG.[ActionGroup_Id])

	  FROM	[DeploymentPlanActionGroups] DPAG
            INNER JOIN [ActionGroups] AG
                    ON DPAG.[ActionGroup_Id] = AG.[ActionGroup_Id]
             LEFT JOIN [Deployables] R_D
                    ON R_D.[Deployable_Name] = DPAG.[Deployable_Name]
                   AND R_D.[Application_Id] = DPAG.[Deployable_Application_Id]
             LEFT JOIN [Applications] R_A
                    ON R_A.[Application_Id] = R_D.[Application_Id]
             LEFT JOIN [Servers] S
                    ON S.[Server_Id] = COALESCE(DPAG.[Server_Id], AG.[Server_Id])
			 LEFT JOIN [DeploymentPlanActionGroups] S_DPAG
			        ON DPAG.[OnSuccess_DeploymentPlanActionGroup_Id] = S_DPAG.[DeploymentPlanActionGroup_Id]
			 LEFT JOIN [ActionGroups] S_AG
			        ON S_DPAG.[ActionGroup_Id] = S_AG.[ActionGroup_Id]
			 LEFT JOIN [DeploymentPlanActionGroups] F_DPAG
			        ON DPAG.[OnFailure_DeploymentPlanActionGroup_Id] = F_DPAG.[DeploymentPlanActionGroup_Id]
			 LEFT JOIN [ActionGroups] F_AG
			        ON F_DPAG.[ActionGroup_Id] = F_AG.[ActionGroup_Id]
GO


GO
IF OBJECT_ID('[DeploymentPlans_ActionUsage]') IS NOT NULL DROP VIEW [DeploymentPlans_ActionUsage]
GO

CREATE VIEW [DeploymentPlans_ActionUsage] AS

SELECT DPAG.[DeploymentPlanActionGroup_Id],
       [ActionGroup_Name] = COALESCE(DPAG.[ActionGroup_Name], AG.[ActionGroup_Name]),
       DPAG.[DeploymentPlanActionGroup_Sequence],
       DPAG.[ActionGroup_Id],
       DP.[DeploymentPlan_Id],
       DP.[DeploymentPlan_Name],
       AGA.[Action_Sequence],
       [Uses_DeploymentPlan_Id] = DP_Uses.[DeploymentPlan_Id],
       [Uses_DeploymentPlan_Name] = DP_Uses.[DeploymentPlan_Name]
  FROM [DeploymentPlanActionGroups] DPAG
       INNER JOIN [DeploymentPlans] DP
               ON DP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
       INNER JOIN [ActionGroups] AG
               ON AG.[ActionGroup_Id] = DPAG.[ActionGroup_Id]
       INNER JOIN [ActionGroupActions] AGA
               ON DPAG.[ActionGroup_Id] = AGA.[ActionGroup_Id]
       INNER JOIN [DeploymentPlans] DP_Uses
               ON DP_Uses.[DeploymentPlan_Id] = AGA.[Action_Configuration].value('/*[1]/Properties[1]/@DeploymentPlanId', 'INT')
 WHERE CAST(AGA.[Action_Configuration].query('local-name(/*[1])') AS VARCHAR(200)) = 'Inedo.BuildMaster.Extensibility.Actions.BuildMaster.ExecuteDeploymentPlanAction'
   AND AGA.[Action_Configuration].value('/*[1]/@Assembly', 'VARCHAR(200)') = 'BuildMasterExtensions'

GO


GO

IF OBJECT_ID('[EnvironmentServers_Extended]') IS NOT NULL DROP VIEW [EnvironmentServers_Extended]
GO

CREATE VIEW [EnvironmentServers_Extended] AS

	SELECT  ES.[Environment_Id],
            E.[Environment_Name],
            [Environment_Active_Indicator] = E.[Active_Indicator],
			S.[Server_Id],
			S.[Server_Name],
			S.[Agent_Configuration],
			S.[ServerType_Code],
			[Server_Active_Indicator] = S.[Active_Indicator]
      FROM	[Servers] S
 LEFT JOIN	[EnvironmentServers] ES
        ON	S.[Server_Id] = ES.[Server_Id]
 LEFT JOIN	[Environments] E
        ON	ES.[Environment_Id] = E.[Environment_Id]

GO


GO
IF OBJECT_ID('[EventListeners_Extended]') IS NOT NULL DROP VIEW [EventListeners_Extended]
GO

CREATE VIEW [EventListeners_Extended] AS

SELECT L.*,
	   W.[Workflow_Id],
       W.[Workflow_Name],
	   E.[Environment_Name],
	   [Single_Application_Id] = A.[Application_Id],
	   [Single_Application_Name] = A.[Application_Name]
  FROM [EventListeners] L
       LEFT  JOIN [Workflows] W
	           ON W.[Workflow_Id] = L.[BuildStep_Workflow_Id]
			   OR W.[Workflow_Id] = L.[WorkflowStep_Workflow_Id]
       LEFT  JOIN [Environments] E
	           ON E.[Environment_Id] = L.[WorkflowStep_Environment_Id]
       LEFT  JOIN [ApplicationWorkflows] AW
               ON AW.[Workflow_Id] = W.[Workflow_Id]
              AND 1 = (SELECT COUNT(*) FROM [ApplicationWorkflows] _AW
                        WHERE _AW.[Workflow_Id] = W.[Workflow_Id])
       LEFT  JOIN [Applications] A
	           ON A.[Application_Id] = AW.[Application_Id]

GO
IF OBJECT_ID('[EventListeners_ForEvent]') IS NOT NULL DROP VIEW [EventListeners_ForEvent]
GO

CREATE VIEW [EventListeners_ForEvent] AS

SELECT N.*

      ,EO.[EventOccurence_Id]
      ,EO.[Event_Code]
      ,EO.[User_Name]
      ,EO.[Occurence_Date]
      ,EO.[Application_Id]

  FROM [EventOccurences] EO
       INNER JOIN [EventListenerEvents] NEC
               ON EO.[Event_Code] = NEC.[Event_Code]
       INNER JOIN [EventListeners] N
               ON N.[EventListener_Id] = NEC.[EventListener_Id]
 WHERE N.[Active_Indicator] = 'Y'




GO
IF OBJECT_ID('[EventNotifiers]') IS NOT NULL DROP VIEW [EventNotifiers]
GO

CREATE VIEW [EventNotifiers] AS

SELECT N.[Notifier_Id]
      ,N.[Notifier_Name]
      ,N.[Notifier_Configuration]
      ,N.[Owner_User_Name]

      ,EO.[EventOccurence_Id]
      ,EO.[Event_Code]
      ,EO.[User_Name]
      ,EO.[Occurence_Date]
      ,EO.[Application_Id]

  FROM [EventOccurences] EO
       INNER JOIN [NotifierEventCodes] NEC
               ON EO.[Event_Code] = NEC.[Event_Code]
       INNER JOIN [Notifiers] N
               ON N.[Notifier_Id] = NEC.[Notifier_Id]
 WHERE N.[Active_Indicator] = 'Y'

GO
IF OBJECT_ID('[EventOccurences_Extended]') IS NOT NULL
	DROP VIEW [EventOccurences_Extended]
GO

CREATE VIEW [EventOccurences_Extended]
AS

	SELECT EO.*
	      ,ET.[Event_Description]
	      ,A.[Application_Name]
	  FROM [EventOccurences] EO
	       INNER JOIN [EventTypes] ET	              
	               ON ET.[Event_Code] = EO.[Event_Code]
	       LEFT  JOIN [Applications] A
	               ON EO.[Application_Id] = A.[Application_Id]


GO
IF OBJECT_ID('[Issues_Extended]') IS NOT NULL DROP VIEW [Issues_Extended]
GO

CREATE VIEW [Issues_Extended] AS
	SELECT I.[Application_Id],
	       A.[Application_Name],
		   A.[ApplicationGroup_Id],
		   AG.[ApplicationGroup_Name],
		   I.[Release_Number],
		   [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number]),
           R.[Release_Sequence],
		   I.[Issue_Id],
		   I.[Title_Text],
		   I.[Description_Text],
		   I.[Closed_Indicator],
		   I.[ClosedOn_Date],
           I.[OpenedOn_Original_Build_Number],
           [OpenedOn_Original_Build_Sequence] = B_OPENED.[Build_Sequence],
           I.[ClosedOn_Original_Build_Number],
           [ClosedOn_Original_Build_Sequence]= B_CLOSED.[Build_Sequence],
		   I.[CreatedBy_User_Name],
		   I.[CreatedOn_Date],
		   I.[IssueTracker_IssueId_Text],
		   I.[IssueTracker_IssueStatus_Text],
		   I.[IssueTracker_LastSync_Date],
           I.[IssueTracker_IssueUrl_Text],
           [BuildMasterIssue_Indicator] = CASE WHEN I.[IssueTracker_IssueId_Text] IS NULL THEN 'Y' ELSE 'N' END
                                          
	  FROM [Issues] I
	       INNER JOIN [Applications] A
		           ON A.[Application_Id] = I.[Application_Id]
		    LEFT JOIN [ApplicationGroups] AG
			       ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]
		    LEFT JOIN [Releases] R
		           ON R.[Application_Id] = I.[Application_Id]
				  AND R.[Release_Number] = I.[Release_Number]
            LEFT JOIN [Builds] B_OPENED
                   ON B_OPENED.[Application_Id] = I.[Application_Id]
                  AND B_OPENED.[Release_Number] = I.[Release_Number]
                  AND B_OPENED.[Build_Number] = I.[OpenedOn_Original_Build_Number]
            LEFT JOIN [Builds] B_CLOSED
                   ON B_CLOSED.[Application_Id] = I.[Application_Id]
                  AND B_CLOSED.[Release_Number] = I.[Release_Number]
                  AND B_CLOSED.[Build_Number] = I.[ClosedOn_Original_Build_Number]
GO

GO
IF OBJECT_ID('LogMessages_Extended') IS NOT NULL DROP VIEW LogMessages_Extended
GO

CREATE VIEW LogMessages_Extended AS
SELECT L.[LogMessage_Id]
      ,L.[Read_Indicator]
      ,L.[Logged_Date]
      ,L.[Message_Text]
      ,L.[ExceptionStackTrace_Text]
      ,L.[Server_Id]
      ,S.[Server_Name]
  FROM [LogMessages] L
       LEFT JOIN [Servers] S
               ON L.[Server_Id] = S.[Server_Id]       
GO




GO
IF OBJECT_ID('ManualActionsInProgress_Extended') IS NOT NULL DROP VIEW ManualActionsInProgress_Extended
GO

CREATE VIEW [ManualActionsInProgress_Extended] AS
	SELECT A.[Application_Id],
		   A.[Application_Name],
 	       R.[Release_Number],
 	       [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number]),
 	       BE.[Build_Number],
 	       BE.[Execution_Id],
	       BEAGA.[Original_Action_Sequence],
	       MAIP.[ActionComplete_Indicator],
		   MAIP.[AssignedTo_Principal_Name],
		   MAIP.[CompletedBy_User_Name],
		   BEAGA.[ExecutionAction_Description],
		   MAIP.[Task_Name],
		   MAIP.[Task_Description],
	       [Action_ExecutionStarted_Date] = BEAGA.[ExecutionStarted_Date],
	       MAIP.[BuildExecution_PlanAction_Id]
	  FROM [ManualActionsInProgress] MAIP
	       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
	               ON MAIP.[BuildExecution_PlanAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
		   INNER JOIN [BuildExecution_DeploymentPlanActionGroups] BEDPAG
				   ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDPAG.[BuildExecution_DeploymentPlan_Id]
				  AND BEAGA.[Original_DeploymentPlanActionGroup_Sequence] = BEDPAG.[Original_DeploymentPlanActionGroup_Sequence]
		   INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		           ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
	       INNER JOIN [BuildExecutions] BE
	               ON BE.[Execution_Id] = BEDP.[Execution_Id]
		   INNER JOIN [Applications] A
				   ON BE.[Application_Id] = A.[Application_Id]
		   INNER JOIN [Releases] R
		           ON R.[Release_Number] = BE.[Release_Number]
		          AND R.[Application_Id] = BE.[Application_Id]


GO
IF OBJECT_ID('[Plans_MostCommonActions]') IS NOT NULL DROP VIEW [Plans_MostCommonActions]
GO

CREATE VIEW [Plans_MostCommonActions] AS

SELECT [Action_Type],
       [Usage_Count] = COUNT([Action_Type])
  FROM (SELECT [Action_Type] = CAST([Action_Configuration].query('local-name(/*[1])') AS VARCHAR(200)) + ',' + [Action_Configuration].value('/*[1]/@Assembly', 'VARCHAR(100)')
          FROM [ActionGroupActions]) AT
 GROUP BY AT.[Action_Type]

GO


GO
IF OBJECT_ID('[Privileges_Extended]') IS NOT NULL
	DROP VIEW [Privileges_Extended]
GO

CREATE VIEW [Privileges_Extended]
AS

	SELECT P.[Privilege_Id]
		  ,P.[Directory_Provider_Id]
		  ,P.[Principal_Name]
		  ,P.[PrincipalType_Code]
		  ,P.[PrivilegeType_Code]

		  ,R.[Role_Id]
		  ,R.[Role_Name]

		  ,A.[Application_Id]
		  ,A.[Application_Name]

		  ,E.[Environment_Id]
		  ,E.[Environment_Name]

		  ,AG.[ApplicationGroup_Id]
		  ,AG.[ApplicationGroup_Name]

	  FROM [Privileges] P
		   INNER JOIN [Roles] R
				   ON P.[Role_Id] = R.[Role_Id]

		   LEFT JOIN [Applications] A
				  ON P.[Application_Id] = A.[Application_Id]

		   LEFT JOIN [Environments] E
				  ON P.[Environment_Id] = E.[Environment_Id]
				  
		   LEFT JOIN [ApplicationGroups] AG
				  ON P.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]

GO


GO

IF OBJECT_ID('[ReleaseDeployables_Extended]') IS NOT NULL DROP VIEW [ReleaseDeployables_Extended]
GO

CREATE VIEW [ReleaseDeployables_Extended] AS

SELECT D.[Deployable_Id]
      ,D.[Deployable_Name]
      ,D.[CreatedBy_User_Name]
      ,D.[CreatedOn_Date]
      ,D.[ModifiedBy_User_Name]
      ,D.[ModifiedOn_Date]

      ,A.[Application_Id]
      ,A.[Application_Name]

      ,[Dependencies_Count] =
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[Deployable_Id] = D.[Deployable_Id])
      ,[Dependants_Count] = 
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[DependsOn_Deployable_Id] = D.[Deployable_Id])
      ,RD.[Release_Number]
      ,RD.[InclusionType_Code]
      ,RD.[Referenced_Release_Number]
      ,RD.[Referenced_Application_Id]
      ,[Referenced_Release_Name] = COALESCE(RREF.[Release_Name],RREF.[Release_Number])
      
      ,[Referenced_Application_Name] = AREF.[Application_Name]

FROM [ReleaseDeployables] RD
     INNER JOIN [Deployables] D
             ON D.[Deployable_Id] = RD.[Deployable_Id]
     INNER JOIN [Applications] A
             ON A.[Application_Id] = RD.[Application_Id]
     LEFT JOIN [Applications] AREF
			 ON RD.[Referenced_Application_Id] = AREF.[Application_Id]
     LEFT JOIN [Releases] RREF
			 ON RD.[Referenced_Application_Id] = RREF.[Application_Id]
            AND RD.[Referenced_Release_Number] = RREF.[Release_Number]
GO




GO
IF OBJECT_ID('[ReleaseNotes_Extended]') IS NOT NULL DROP VIEW [ReleaseNotes_Extended]
GO

CREATE VIEW [ReleaseNotes_Extended] AS

SELECT RN.*
      ,E.[Environment_Name]
  FROM [ReleaseNotes] RN
       LEFT  JOIN [Environments] E
               ON E.[Environment_Id] = RN.[Environment_Id]
GO

GO

IF OBJECT_ID('[Releases_Extended]') IS NOT NULL DROP VIEW [Releases_Extended]
GO

CREATE VIEW [Releases_Extended] AS

SELECT R.[Application_Id]
	  ,R.[Release_Number]
      ,R.[ReleaseStatus_Name]
      ,R.[Notes_Text]
	  ,R.[CreatedBy_User_Name]
	  ,R.[CreatedOn_Date]
	  ,R.[ModifiedBy_User_Name]
	  ,R.[ModifiedOn_Date]
	  ,R.[CancelledBy_User_Name]
	  ,R.[CancelledReason_Text]
	  ,R.[CancelledOn_Date]
      ,R.[Target_Date]
      ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])
      ,R.[Release_Sequence]

	  ,A.[Application_Name]

	  ,W.[Workflow_Id]
	  ,[Workflow_Name] = R.[Original_Workflow_Name]
	  ,W.[Color_Code]
      ,W.[AutoDeployRelease_Indicator]
      ,W.[AllowOutOfSequencePromotions_Indicator]
      ,W.[AutoCancelReleases_Indicator]
      ,W.[AutoIncrementRelease_Indicator]
      ,W.[BuildImporterTemplate_Configuration]

	  ,[Latest_Step_Sequence] = WS_LAT.[Step_Sequence]
	  ,[Latest_Environment_Id] = E_LAT.[Environment_Id]
	  ,[Latest_Environment_Name] = E_LAT.[Environment_Name]
	  ,[Latest_Build_Number] = B_LAT.[Build_Number]
	  ,[Latest_Promoted_Date] = BP_LAT.[Promoted_Date]
	  ,[Latest_Promoted_Build_Number] = BP_LAT.[Build_Number]
	  ,[Latest_Promoted_Environment_Id] = BP_LAT.[Environment_Id]
      ,[Latest_Promotion_Status] = BP_LAT.[PromotionStatus_Name]
	  ,[Latest_PromotedBy_Name] = BP_LAT.[PromotedBy_Name]
	  ,[Latest_PromotionComments_Text] = BP_LAT.[Comments_Text]

	  ,[NextLatest_Step_Sequence] = WS_NXTL.[Step_Sequence]
	  ,[NextLatest_Environment_Id] = E_NXTL.[Environment_Id]
	  ,[NextLatest_Environment_Name] = E_NXTL.[Environment_Name]
	  ,[NextLatest_IsApproved_Indicator] = [dbo].[IsPromotionApproved]
                                           (
                                                R.[Application_Id], 
                                                R.[Release_Number], 
                                                B_LAT.[Build_Number], 
                                                WS_NXTL.[Environment_Id]
                                           )

	  ,[Furthest_Step_Sequence] = WS_FUR.[Step_Sequence]
	  ,[Furthest_Environment_Id] = E_FUR.[Environment_Id]
	  ,[Furthest_Environment_Name] = E_FUR.[Environment_Name]
	  ,[Furthest_Build_Number] = B_FUR.[Build_Number]
	  ,[Furthest_Promoted_Date] = BP_FUR.[Promoted_Date]
	  ,[Furthest_PromotedBy_Name] = BP_FUR.[PromotedBy_Name]

	  ,[NextFurthest_Step_Sequence] = WS_NXTF.[Step_Sequence]
	  ,[NextFurthest_Environment_Id] = E_NXTF.[Environment_Id]
	  ,[NextFurthest_Environment_Name] = E_NXTF.[Environment_Name]


	  ,[First_Step_Sequence] = WS_1ST.[Step_Sequence]
	  ,[First_Environment_Id] = E_1ST.[Environment_Id]
	  ,[First_Environment_Name] = E_1ST.[Environment_Name]

	  ,[Final_Step_Sequence] = WS_FNL.[Step_Sequence]
	  ,[Final_Environment_Id] = E_FNL.[Environment_Id]
	  ,[Final_Environment_Name] = E_FNL.[Environment_Name]

	  ,[Builds_Count] = (SELECT COUNT(*)
						   FROM [Builds] _B
						  WHERE R.[Application_Id] = _B.[Application_Id]
							AND R.[Release_Number] = _B.[Release_Number])

  FROM [Releases] R
	   INNER JOIN [Applications] A
               ON R.[Application_Id] = A.[Application_Id]
	    LEFT JOIN [Workflows] W
               ON R.[Workflow_Id] = W.[Workflow_Id]

        /* Latest (i.e. most recently created) Build */
        LEFT JOIN [Builds] B_LAT
               ON B_LAT.[Application_Id] = R.[Application_Id]
              AND B_LAT.[Release_Number] = R.[Release_Number]
              AND B_LAT.[Build_Number] = 
                     (SELECT MAX([Build_Number])
                        FROM [Builds]
                       WHERE [Application_Id] = R.[Application_Id]
                         AND [Release_Number] = R.[Release_Number]
                         AND [BuildStatus_Name] <> 'Rejected')
        LEFT JOIN [WorkflowSteps] WS_LAT
               ON WS_LAT.[Workflow_Id] = W.[Workflow_Id]
              AND WS_LAT.[Step_Sequence] =
              (
					SELECT TOP 1 _WS.[Step_Sequence]
					  FROM [BuildPromotions] _BP
					  JOIN [WorkflowSteps] _WS
						ON _BP.[Environment_Id] = _WS.[Environment_Id]
					 WHERE _BP.[Application_Id] = R.[Application_Id]
					   AND _BP.[Release_Number] = R.[Release_Number]
					   AND _BP.[Build_Number] = B_LAT.[Build_Number]
					ORDER BY [Promoted_Date] DESC	
			  )

		/* Latest (most recent) Promotion */
		LEFT JOIN [BuildPromotions] BP_LAT
			   ON BP_LAT.[Application_Id] = R.[Application_Id]
			  AND BP_LAT.[Release_Number] = R.[Release_Number]
			  AND BP_LAT.[Promoted_Date] =
				( SELECT TOP 1 [Promoted_Date]
					FROM [BuildPromotions] _BP_LAT
				   WHERE _BP_LAT.[Application_Id] = R.[Application_Id]
					 AND _BP_LAT.[Release_Number] = R.[Release_Number]
				  ORDER BY [Promoted_Date] DESC )
        LEFT JOIN [Environments] E_LAT
               ON BP_LAT.[Environment_Id] = E_LAT.[Environment_Id]

      /* Next LAtest Workflow Step */
        LEFT JOIN [WorkflowSteps] WS_NXTL
               ON WS_NXTL.[Workflow_Id] = R.[Workflow_Id]
              AND WS_NXTL.[Step_Sequence] =
                  (SELECT MIN(_WS.[Step_Sequence])
                     FROM [WorkflowSteps] _WS
                    WHERE _WS.[Workflow_Id] = WS_LAT.[Workflow_Id]
                      AND _WS.[Step_Sequence] > COALESCE(WS_LAT.[Step_Sequence],0) )
        LEFT JOIN [Environments] E_NXTL
               ON E_NXTL.[Environment_Id] = WS_NXTL.[Environment_Id]

        /* Furthest (i.e. longest down the workflow line) Build */
        LEFT JOIN [WorkflowSteps] WS_FUR
               ON WS_FUR.[Workflow_Id] = W.[Workflow_Id]
              AND WS_FUR.[Step_Sequence] =
              (
					SELECT MAX(_WS.[Step_Sequence])
					  FROM [BuildPromotions] _BP
						  INNER JOIN [Builds] _B 
								  ON _B.[Application_Id] = _BP.[Application_Id]
								 AND _B.[Release_Number] = _BP.[Release_Number]
								 AND _B.[Build_Number] = _BP.[Build_Number]
								 AND _B.[BuildStatus_Name] <> 'Rejected'
							JOIN [WorkflowSteps] _WS
							  ON R.[Workflow_Id] = _WS.[Workflow_Id]
							 AND _BP.[Environment_Id] = _WS.[Environment_Id]
					 WHERE _BP.[Application_Id] = R.[Application_Id]
					   AND _BP.[Release_Number] = R.[Release_Number]
              )
        LEFT JOIN [Builds] B_FUR
               ON B_FUR.[Application_Id] = R.[Application_Id]
              AND B_FUR.[Release_Number] = R.[Release_Number]
              AND B_FUR.[BuildStatus_Name] <> 'Rejected' /* optimization */
              AND 
              (
                  ( B_FUR.[Build_Number] = 
                         (SELECT MAX(_BP.[Build_Number])
                            FROM [BuildPromotions] _BP
                                 INNER JOIN [Builds] _B 
                                         ON _B.[Application_Id] = _BP.[Application_Id]
                                        AND _B.[Release_Number] = _BP.[Release_Number]
                                        AND _B.[Build_Number] = _BP.[Build_Number]
                                        AND _B.[BuildStatus_Name] = 'Active'
                           WHERE _BP.[Application_Id] = R.[Application_Id]
                             AND _BP.[Release_Number] = R.[Release_Number]
                             AND R.[Workflow_Id] = WS_FUR.[Workflow_Id]
                             AND _BP.[Environment_Id] = WS_FUR.[Environment_Id]))
               OR
                  ( B_FUR.[BuildStatus_Name] = 'Deployed')
              )   
        LEFT JOIN [BuildPromotions] BP_FUR
               ON BP_FUR.[Application_Id] = B_FUR.[Application_Id]
              AND BP_FUR.[Release_Number] = B_FUR.[Release_Number]
              AND BP_FUR.[Build_Number] = B_FUR.[Build_Number]
              AND BP_FUR.[Environment_Id] = (SELECT TOP 1 [Environment_Id]
                                               FROM [BuildPromotions] _BP
                                              WHERE BP_FUR.[Application_Id] = _BP.[Application_Id]
                                                AND BP_FUR.[Release_Number] = _BP.[Release_Number]
                                                AND BP_FUR.[Build_Number] = _BP.[Build_Number]
                                              ORDER BY [Promoted_Date] DESC)
        LEFT JOIN [Environments] E_FUR
               ON BP_FUR.[Environment_Id] = E_FUR.[Environment_Id]

      /* Next Furthest Workflow Step */
        LEFT JOIN [WorkflowSteps] WS_NXTF
               ON WS_NXTF.[Workflow_Id] = R.[Workflow_Id]
              AND WS_NXTF.[Step_Sequence] =
                  (SELECT MIN(_WS.[Step_Sequence])
                     FROM [WorkflowSteps] _WS
                    WHERE _WS.[Workflow_Id] = WS_FUR.[Workflow_Id]
                      AND _WS.[Step_Sequence] > COALESCE(WS_FUR.[Step_Sequence],0) )
        LEFT JOIN [Environments] E_NXTF
               ON E_NXTF.[Environment_Id] = WS_NXTF.[Environment_Id]

        /* First Workflow Step */
        LEFT JOIN [WorkflowSteps] WS_1ST 
               ON WS_1ST.[Workflow_Id] = W.[Workflow_Id]
              AND WS_1ST.[Step_Sequence] =
                  (SELECT MIN(WS.[Step_Sequence])
                     FROM [WorkflowSteps] WS
                    WHERE WS.[Workflow_Id] = R.[Workflow_Id])
        LEFT JOIN [Environments] E_1ST
               ON E_1ST.[Environment_Id] = WS_1ST.[Environment_Id]

  

        /* Final Step */
        LEFT JOIN [WorkflowSteps] WS_FNL
               ON WS_FNL.[Workflow_Id] = R.[Workflow_Id]
              AND WS_FNL.[Step_Sequence] =
                  (SELECT MAX(_WS.[Step_Sequence])
                     FROM [WorkflowSteps] _WS
                    WHERE _WS.[Workflow_Id] = R.[Workflow_Id])
        LEFT JOIN [Environments] E_FNL
               ON E_FNL.[Environment_Id] = WS_FNL.[Environment_Id]

GO

GO
IF OBJECT_ID('Releases_RequiredPromotions') IS NOT NULL DROP VIEW [Releases_RequiredPromotions]
GO

CREATE VIEW [Releases_RequiredPromotions] AS

SELECT R.[Application_Id]
      ,R.[Release_Number]
      ,R.[ReleaseStatus_Name]

      ,R.[Workflow_Id]
      ,[Step_Sequence] = COALESCE(WS.[Step_Sequence], BP_IDX.[Actual_Step_Sequence])

      ,E.[Environment_Id]
      ,E.[Environment_Name]
      
      ,BP.[PromotionStatus_Name]
      ,BP.[Promoted_Date]

      ,B.[Build_Number]
      ,B.[BuildStatus_Name]

  FROM [Releases] R
        LEFT JOIN [WorkflowSteps] WS
               ON WS.[Workflow_Id] = R.[Workflow_Id]
              AND R.[ReleaseStatus_Name] = 'Active'

        LEFT JOIN (SELECT _BP.[Application_Id]
                         ,_BP.[Release_Number]
                         ,_BP.[Environment_Id]
                         ,[Build_Number] = LTRIM(RTRIM(MAX([dbo].SortableBuildNumber(_BP.[Application_Id], _BP.[Build_Number]))))
                         ,[Actual_Step_Sequence] = ROW_NUMBER() OVER (ORDER BY MAX([Promoted_Date]))
                     FROM [BuildPromotions] _BP
                          INNER JOIN [Releases] _R
                                  ON _R.[Application_Id] = _BP.[Application_Id]
                                 AND _R.[Release_Number] = _BP.[Release_Number]
                          INNER JOIN [Builds] _B
                                  ON _B.[Application_Id] = _BP.[Application_Id]
                                 AND _B.[Release_Number] = _BP.[Release_Number]
                                 AND _B.[Build_Number] = _BP.[Build_Number]
                    WHERE _R.[ReleaseStatus_Name] = 'Cancelled'
                       OR _B.[BuildStatus_Name] <> 'Rejected'
                    GROUP BY _BP.[Application_Id]
                            ,_BP.[Release_Number]
                            ,_BP.[Environment_Id]) BP_IDX
               ON BP_IDX.[Application_Id] = R.[Application_Id]
              AND BP_IDX.[Release_Number] = R.[Release_Number]
              AND BP_IDX.[Environment_Id] = COALESCE(WS.[Environment_Id], BP_IDX.[Environment_Id])

       INNER JOIN [Environments] E
               ON E.[Environment_Id] = COALESCE(WS.[Environment_Id], BP_IDX.[Environment_Id])

        LEFT JOIN [BuildPromotions] BP
               ON BP.[Application_Id] = BP_IDX.[Application_Id]
              AND BP.[Release_Number] = BP_IDX.[Release_Number]
              AND BP.[Build_Number] = BP_IDX.[Build_Number]
              AND BP.[Environment_Id] = BP_IDX.[Environment_Id]
        LEFT JOIN [Builds] B
               ON B.[Application_Id] = BP.[Application_Id]
              AND B.[Release_Number] = BP.[Release_Number]
              AND B.[Build_Number] = BP.[Build_Number]

GO
IF OBJECT_ID('[RetentionPolicies_Extended]') IS NOT NULL DROP VIEW [RetentionPolicies_Extended]
GO

CREATE VIEW [RetentionPolicies_Extended] AS

SELECT RP.[RetentionPolicy_Id],
       RP.[RetentionPolicy_Name],
	   RP.[PurgeType_Code],
	   RP.[CreatedOn_Date],
	   RP.[CreatedBy_User_Name],
	   RP.[ModifiedOn_Date],
	   RP.[ModifiedBy_User_Name],
	   
	   RP.[PurgeIn_Application_Id],
	   [PurgeIn_Application_Name] = A.[Application_Name],
	   RP.[PurgeIn_ApplicationGroup_Id],
	   [PurgeIn_ApplicationGroup_Name] = AG.[ApplicationGroup_Name],
	   RP.[PurgeIn_Workflow_Id],
	   [PurgeIn_Workflow_Name] = W.[Workflow_Name],
	   RP.[PurgeIn_Environment_Id],
	   [PurgeIn_Environment_Name] = E.[Environment_Name],
	   RP.[PurgeIn_BuildStep_Indicator],
	   RP.[PurgeIn_OnlyDeployedReleases_Indicator],
	   RP.[PurgeIn_OnlyRejectedBuilds_Indicator],

	   RP.[Retention_Count],
	   RP.[Retention_Size],
	   RP.[Retention_Days_Count],

	   RP.[PurgeWindow_Start_Minutes],
       RP.[PurgeWindow_Duration_Minutes],
	   RP.[PurgeNow_Indicator],

       RP.[Active_Indicator],

	   RPPH.[RetentionPoliciesPurgeHistoryEntry_Id],
	   [LastStart_Date] = RPPH.[Start_Date]

  FROM [RetentionPolicies] RP
       LEFT JOIN [Applications] A
	          ON A.[Application_Id] = RP.[PurgeIn_Application_Id]
	   LEFT JOIN [ApplicationGroups] AG
	          ON AG.[ApplicationGroup_Id] = RP.[PurgeIn_ApplicationGroup_Id]
	   LEFT JOIN [Workflows] W
	          ON W.[Workflow_Id] = RP.[PurgeIn_Workflow_Id]
	   LEFT JOIN [Environments] E
	          ON E.[Environment_Id] = RP.[PurgeIn_Environment_Id]
	   LEFT JOIN (SELECT [RetentionPoliciesPurgeHistoryEntry_Id],
						 [Original_RetentionPolicy_Id],
						 [Start_Date],
	                     [Rank] = ROW_NUMBER() OVER (PARTITION BY [Original_RetentionPolicy_Id] ORDER BY [Start_Date] DESC)
	                FROM [RetentionPoliciesPurgeHistory]) RPPH
			  ON RPPH.[Original_RetentionPolicy_Id] = RP.[RetentionPolicy_Id]
			 AND RPPH.[Rank] = 1
GO

GO
IF OBJECT_ID('[dbo].[RoleTasks_Extended]') IS NOT NULL
	DROP VIEW [dbo].[RoleTasks_Extended]
GO

CREATE VIEW [dbo].[RoleTasks_Extended]
AS

	SELECT RT.[Role_Id]

		  ,T.[Task_Id]
		  ,T.[Task_Name]
          ,T.[Application_Scopeable_Indicator]
          ,T.[Environment_Scopeable_Indicator]

	  FROM [dbo].[RoleTasks] RT
		   INNER JOIN [dbo].[Tasks] T
				   ON RT.[Task_Id] = T.[Task_Id]

GO


GO
IF OBJECT_ID('[Scripts_Extended]') IS NOT NULL DROP VIEW [Scripts_Extended]
GO

CREATE VIEW [Scripts_Extended] AS

SELECT S.[Script_Id],
       S.[Script_Name],
       S.[Description_Text],
       S.[Active_Indicator],
       S.[AutoSync_Indicator],
       S.[ScriptType_Code],
       S.[CreatedBy_User_Name],
       S.[CreatedOn_Date],
       S.[ModifiedBy_User_Name],
       S.[ModifiedOn_Date],
       [LatestVersion_Number] = SV.[Version_Number],
       [LatestVersion_Bytes] = SV.[Script_Bytes]

  FROM [Scripts] S
        LEFT JOIN (SELECT _SV.[Script_Id],
                          [Version_Number] = MAX(_SV.[Version_Number])
                     FROM [ScriptVersions] _SV
                    GROUP BY _SV.[Script_Id]) _Latest_SV
               ON S.[Script_Id] = _Latest_SV.[Script_Id]
        LEFT JOIN [ScriptVersions] SV
               ON _Latest_SV.[Script_Id] = SV.[Script_Id]
              AND _Latest_SV.[Version_Number] = SV.[Version_Number]
GO


GO
IF OBJECT_ID('[ScriptTypes_Usage]') IS NOT NULL DROP VIEW [ScriptTypes_Usage]
GO

CREATE VIEW [ScriptTypes_Usage] AS

	SELECT S.[ScriptType_Code],
	       [Usage_Count] = COUNT(S.[Script_Id])
	  FROM [Scripts] S
	 WHERE S.[Active_Indicator] = 'Y'
	 GROUP BY S.[ScriptType_Code]

GO


GO
IF OBJECT_ID('[VariableDefinitions_Extended]') IS NOT NULL DROP VIEW [VariableDefinitions_Extended]
GO

CREATE VIEW [VariableDefinitions_Extended] AS

SELECT VDEF.[VariableDefinition_Id]
      ,VDEF.[Variable_Name]
      ,VDEF.[Environment_Id]
      ,VDEF.[Server_Id]
      ,VDEF.[ApplicationGroup_Id]
      ,VDEF.[Application_Id]
      ,VDEF.[Deployable_Id]
      ,VDEF.[Release_Number]
      ,VDEF.[Build_Number]
      ,VDEF.[Value_Text]
      ,VDEF.[Execution_Id]
	  ,VDEF.[Scope_Code]

      ,VDEC.[VariableDeclaration_Id]
      ,VDEC.[Scoped_Application_Id]
      ,VDEC.[Required_Indicator]
      ,VDEC.[Variable_Configuration]
      ,VDEC.[DefaultValue_Text]
      ,VDEC.[Scoped_Environment_Id]

      ,E.[Environment_Name]
      ,S.[Server_Name]
      ,AG.[ApplicationGroup_Name]
      ,A.[Application_Name]
      ,D.[Deployable_Name]
      ,VDEF.[Sensitive_Indicator]

  FROM [VariableDefinitions] VDEF
       LEFT JOIN [VariableDeclarations] VDEC
              ON VDEF.[Variable_Name] = VDEC.[Variable_Name]
             AND (VDEC.[Scope_Code] NOT IN ('R','B','P','X') OR
                  VDEF.[Application_Id] = VDEC.[Scoped_Application_Id])
             AND (VDEC.[Scope_Code] NOT IN ('P','X') OR
                  VDEF.[Environment_Id] = VDEC.[Scoped_Environment_Id])
             AND VDEC.[Scope_Code] = VDEF.[Scope_Code]
       LEFT JOIN [Applications] A
                 ON VDEF.[Application_Id] = A.[Application_Id]
       LEFT JOIN [ApplicationGroups] AG
                 ON VDEF.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]
       LEFT JOIN [Environments] E
                 ON VDEF.[Environment_Id] = E.[Environment_Id]
       LEFT JOIN [Deployables] D
                 ON VDEF.[Deployable_Id] = D.[Deployable_Id]
       LEFT JOIN [Servers] S
                 ON VDEF.[Server_Id] = S.[Server_Id]
GO

GO
IF OBJECT_ID('[VariableDefinitions_System]') IS NOT NULL DROP VIEW [VariableDefinitions_System]
GO

CREATE VIEW [VariableDefinitions_System] AS

SELECT VDEF.*
      ,E.[Environment_Name]
      ,S.[Server_Name]
      ,AG.[ApplicationGroup_Name]
      ,[Application_Name] = COALESCE(A.[Application_Name],AD.[Application_Name])
      ,D.[Deployable_Name]
  FROM [VariableDefinitions] VDEF
       LEFT JOIN [Applications] A
              ON VDEF.[Application_Id] = A.[Application_Id]
       LEFT JOIN [ApplicationGroups] AG
              ON VDEF.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]
       LEFT JOIN [Environments] E
              ON VDEF.[Environment_Id] = E.[Environment_Id]
       LEFT JOIN [Deployables] D
              ON VDEF.[Deployable_Id] = D.[Deployable_Id]
       LEFT JOIN [Applications] AD
              ON D.[Application_Id] = AD.[Application_Id]
       LEFT JOIN [Servers] S
              ON VDEF.[Server_Id] = S.[Server_Id]
  
  WHERE [Scope_Code] IN ('S','E','V','G','A','D')

GO

GO
IF OBJECT_ID('[Variable_Values]') IS NOT NULL DROP VIEW [Variable_Values]
GO

CREATE VIEW [Variable_Values] AS

SELECT DISTINCT
	   VDEF.[Variable_Name]
      ,VDEF.[Value_Text]
	  ,VDEF.[Scope_Code]
	  ,VDEF.[Sensitive_Indicator]

  FROM [VariableDefinitions] VDEF
GO

GO
IF OBJECT_ID('[WorkflowSteps_Extended]') IS NOT NULL DROP VIEW [WorkflowSteps_Extended]
GO

CREATE VIEW [WorkflowSteps_Extended] AS

SELECT WS.[Workflow_Id]
	  ,W.[Workflow_Name]
	  ,W.[Color_Code]
      ,WS.[Step_Sequence]
	  ,WS.[AutoPromote_Indicator]

      ,WS.[Environment_Id]
      ,E.[Environment_Name]
	  ,[Environment_Display_Sequence] = E.[Display_Sequence]
	  ,DP.[DeploymentPlan_Id]
	  ,DP.[DeploymentPlan_Name]

	  ,[Previous_Step_Sequence] = WS_PREV.[Step_Sequence]
	  ,[Previous_Environment_Id] = WS_PREV.[Environment_Id]
	  ,[Previous_Environment_Name] = E_PREV.[Environment_Name]

	  ,[Next_Step_Sequence] = WS_NEXT.[Step_Sequence]
	  ,[Next_Environment_Id] = WS_NEXT.[Environment_Id]
	  ,[Next_Environment_Name] = E_NEXT.[Environment_Name]

	  ,[Single_Application_Id] = A.[Application_Id]
	  ,[Single_Application_Name] = A.[Application_Name]

FROM [WorkflowSteps] WS
	 INNER JOIN [Workflows] W
			 ON WS.[Workflow_Id] = W.[Workflow_Id]

     INNER JOIN [Environments] E
       ON WS.[Environment_Id] = E.[Environment_Id]

	LEFT  JOIN [ApplicationWorkflows] AW
			ON AW.[Workflow_Id] = W.[Workflow_Id]
			AND 1 = (SELECT COUNT(*) FROM [ApplicationWorkflows] _AW
					WHERE _AW.[Workflow_Id] = W.[Workflow_Id])

	LEFT  JOIN [Applications] A
			ON A.[Application_Id] = AW.[Application_Id]

	/* Previous Step */
	LEFT JOIN [WorkflowSteps] WS_PREV
		   ON WS_PREV.[Workflow_Id] = WS.[Workflow_Id]
		  AND WS_PREV.[Environment_Id] =
			( SELECT TOP 1 _PREV.[Environment_Id]
				FROM [WorkflowSteps] _PREV
			   WHERE _PREV.[Workflow_Id] = WS.[Workflow_Id]
				 AND _PREV.[Step_Sequence] < WS.[Step_Sequence]
			  ORDER BY [Step_Sequence] DESC )
	LEFT JOIN [Environments] E_PREV
		   ON WS_PREV.[Environment_Id] = E_PREV.[Environment_Id]

	/* Next Step */
	LEFT JOIN [WorkflowSteps] WS_NEXT
		   ON WS_NEXT.[Workflow_Id] = WS.[Workflow_Id]
		  AND WS_NEXT.[Environment_Id] =
			( SELECT TOP 1 _NEXT.[Environment_Id]
				FROM [WorkflowSteps] _NEXT
			   WHERE _NEXT.[Workflow_Id] = WS.[Workflow_Id]
				 AND _NEXT.[Step_Sequence] > WS.[Step_Sequence]
			  ORDER BY [Step_Sequence] )
	LEFT JOIN [Environments] E_NEXT
		   ON WS_NEXT.[Environment_Id] = E_NEXT.[Environment_Id]

	LEFT JOIN [DeploymentPlans] DP
	       ON DP.[DeploymentPlan_Id] = WS.[DeploymentPlan_Id]

GO


GO
IF OBJECT_ID('[Workflows_Extended]') IS NOT NULL DROP VIEW [Workflows_Extended]
GO

CREATE VIEW [Workflows_Extended] AS

	SELECT W.[Workflow_Id]
          ,W.[Workflow_Name]
          ,W.[Active_Indicator]
          ,W.[CreatedBy_User_Name]
          ,W.[CreatedOn_Date]
          ,W.[ModifiedBy_User_Name]
          ,W.[ModifiedOn_Date]
          ,W.[AutoIncrementRelease_Indicator]
          ,W.[AutoDeployRelease_Indicator]
          ,W.[AllowOutOfSequencePromotions_Indicator]
          ,W.[AutoCancelReleases_Indicator]
		  ,W.[Build_DeploymentPlan_Id]
		  ,W.[BuildImporterTemplate_Configuration]
		  ,W.[AllowLocalChanges_Indicator]

		  ,[Releases_Count] = (SELECT COUNT(*) FROM [Releases]
	                            WHERE [Workflow_Id] = W.[Workflow_Id])
		  
          ,[InUse_Indicator] =
				(CASE WHEN EXISTS(SELECT * 
									FROM [Releases]
								   WHERE [Workflow_Id] = W.[Workflow_Id]
									 AND [ReleaseStatus_Name] = 'Active') 
					  THEN 'Y' ELSE 'N' END)

          ,W.[Color_Code]
		  ,W.[Global_Indicator]

		  ,[Single_Application_Id] = A.[Application_Id]
		  ,[Single_Application_Name] = A.[Application_Name]

		  ,[Build_AutoPromote_Indicator]
	  FROM [Workflows] W
	       LEFT  JOIN [ApplicationWorkflows] AW
		           ON AW.[Workflow_Id] = W.[Workflow_Id]
	              AND 1 = (SELECT COUNT(*) FROM [ApplicationWorkflows] _AW
	                        WHERE _AW.[Workflow_Id] = W.[Workflow_Id])

           LEFT  JOIN [Applications] A
	               ON A.[Application_Id] = AW.[Application_Id]
GO

GO
IF OBJECT_ID('[__BuildMaster_StoredProcInfo]') IS NOT NULL
	DROP TABLE [__BuildMaster_StoredProcInfo]
GO

CREATE TABLE [__BuildMaster_StoredProcInfo]
(
    [StoredProc_Name] SYSNAME,
    CONSTRAINT [PK__BuildMaster_StoredProcInfo]
        PRIMARY KEY ([StoredProc_Name]),
    [Internal_Indicator] YNINDICATOR,
    [ReturnType_Name] VARCHAR(100) NULL,
    CONSTRAINT [CK__BuildMaster_StoredProcInfo__ReturnType_Name_Domain]
        CHECK ([ReturnType_Name] IS NULL OR [ReturnType_Name] IN ('void', 'DataRow', 'DataTable', 'DataSet')),
    [DataTableNames_Csv] VARCHAR(255) NULL,
    [OutputPropertyNames_Csv] VARCHAR(255) NULL,
    [Description_Text] VARCHAR(MAX),
    [Remarks_Text] VARCHAR(MAX)
)
GO

IF OBJECT_ID('[__BuildMaster_AddStoredProcInfo]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_AddStoredProcInfo]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_AddStoredProcInfo]
(
	@StoredProc_Name SYSNAME,
	@Internal_Indicator YNINDICATOR,
	@ReturnType_Name VARCHAR(100),
	@DataTableNames_Csv VARCHAR(255),
	@OutputPropertyNames_Csv VARCHAR(100),
	@Description_Text VARCHAR(MAX),
	@Remarks_TEXT VARCHAR(MAX) = NULL
)
AS
BEGIN
	
	IF EXISTS(SELECT * FROM [__BuildMaster_StoredProcInfo] WHERE [StoredProc_Name]=@StoredProc_Name)
		DELETE [__BuildMaster_StoredProcInfo] WHERE [StoredProc_Name]=@StoredProc_Name

    INSERT INTO [__BuildMaster_StoredProcInfo]
    (
         [StoredProc_Name]
        ,[Internal_Indicator]
        ,[ReturnType_Name]
        ,[DataTableNames_Csv]
        ,[OutputPropertyNames_Csv]
        ,[Description_Text]
        ,[Remarks_Text]
    )
    VALUES
    (
         @StoredProc_Name
	    ,@Internal_Indicator
	    ,@ReturnType_Name
	    ,@DataTableNames_Csv
	    ,@OutputPropertyNames_Csv
	    ,@Description_Text
	    ,@Remarks_Text
    )

END
GO

GRANT EXECUTE ON [__BuildMaster_AddStoredProcInfo] TO [BuildMasterUser_Role]
GO

IF OBJECT_ID('[__BuildMaster_GetStoredProcInfo]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_GetStoredProcInfo]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_GetStoredProcInfo]
(
	@StoredProc_Name SYSNAME = NULL
)
AS
BEGIN
    
    SET NOCOUNT ON
    
    IF @StoredProc_Name IS NULL
	    SELECT * FROM [__BuildMaster_StoredProcInfo]
	ElSE
		SELECT * FROM [__BuildMaster_StoredProcInfo] WHERE [StoredProc_Name] = @StoredProc_Name

END
GO

GRANT EXECUTE ON [__BuildMaster_GetStoredProcInfo] TO [BuildMasterUser_Role]
GO

GO
IF OBJECT_ID('[__BuildMaster_GetEventReference]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_GetEventReference]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_GetEventReference]
AS
BEGIN
	
	SELECT [Event_Code], [Event_Description], [StoredProc_Name] = ROUTINE_NAME
	  FROM INFORMATION_SCHEMA.ROUTINES 
		   INNER JOIN [EventTypes]
				   ON ROUTINE_DEFINITION LIKE '%' + [Event_Code] + '%' 
				  AND ROUTINE_TYPE='PROCEDURE'
	 ORDER BY 1

END
GO

GRANT EXECUTE ON [__BuildMaster_GetEventReference] TO [BuildMasterUser_Role]
GO

GO
IF OBJECT_ID('[__BuildMaster_GetSecurityTasks]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_GetSecurityTasks]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_GetSecurityTasks]
AS
BEGIN
	
	SELECT * FROM [Tasks]

END
GO

GRANT EXECUTE ON [__BuildMaster_GetSecurityTasks] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_FillVersionGaps',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Ensures that there are no gaps in configuration file instance versions.'
GO

IF OBJECT_ID('[ConfigurationFiles_FillVersionGaps]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_FillVersionGaps]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_FillVersionGaps]
(
	@ConfigurationFile_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @InstanceGaps TABLE
	(
		[Instance_Name] VARCHAR(100) NOT NULL,
		[Version_Number] INT NOT NULL,
		[Count] INT NOT NULL
	)

	INSERT INTO @InstanceGaps
	(
		[Instance_Name],
		[Version_Number],
		[Count]
	)
	SELECT CFV.[Instance_Name],
		   [Version_Number] = MAX([Version_Number]),
		   [Count] = MAX(CFV_Max.[Max_Version_Number]) - MAX([Version_Number])
	  FROM [ConfigurationFileVersions] CFV
	       INNER JOIN (SELECT [ConfigurationFile_Id],
	                          [Max_Version_Number] = MAX([Version_Number])
	                     FROM [ConfigurationFileVersions]
	                    GROUP BY [ConfigurationFile_Id]) CFV_Max
			       ON CFV_Max.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
	 WHERE CFV.[ConfigurationFile_Id] = @ConfigurationFile_Id
	 GROUP BY CFV.[ConfigurationFile_Id], CFV.[Instance_Name]
	HAVING MAX(CFV.[Version_Number]) < MAX(CFV_Max.[Max_Version_Number])
	
	INSERT INTO [ConfigurationFileVersions]
	(
		[ConfigurationFile_Id],
		[Instance_Name],
		[Version_Number],
		[VersionNotes_Text],
		[File_Date],
		[File_Bytes],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	SELECT CFV.[ConfigurationFile_Id],
	       CFV.[Instance_Name],
		   CFV.[Version_Number] + N.[Number],
		   NULL,
		   NULL,
		   NULL,
		   NULL,
		   NULL
	  FROM @InstanceGaps IG
	       INNER JOIN [Numbers] N
		           ON N.[Number] >= 1 AND N.[Number] <= IG.[Count]
		   INNER JOIN [ConfigurationFileVersions] CFV
		           ON CFV.[ConfigurationFile_Id] = @ConfigurationFile_Id
				  AND CFV.[Instance_Name] = IG.[Instance_Name]
				  AND CFV.[Version_Number] = IG.[Version_Number]

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ConfigurationFiles_FillVersionGaps] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_RaiseEvent',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Events_RaiseEvent]') IS NOT NULL
	DROP PROCEDURE [Events_RaiseEvent]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_RaiseEvent]
(
	@Event_Code VARCHAR(8),
	@Application_Id INT = NULL,
	@detail1 SQL_VARIANT = NULL,
	@detail2 SQL_VARIANT = NULL,
	@detail3 SQL_VARIANT = NULL,
	@detail4 SQL_VARIANT = NULL,
	@detail5 SQL_VARIANT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @User_Name USERNAME
		SET @User_Name = dbo.Session_GetUserName()

	INSERT INTO [EventOccurences]
	 ([Event_Code],[User_Name],[Occurence_Date],[Application_Id])
	 VALUES
	 (@Event_Code, @User_Name, GETUTCDATE(), @Application_Id)

	DECLARE @EventOccurence_Id INT
		SET @EventOccurence_Id = SCOPE_IDENTITY()

	INSERT INTO [EventOccurenceDetails]
	 ([EventOccurence_Id],[Event_Code],[Detail_Name],[Detail_Value])
	SELECT @EventOccurence_Id
		  ,[Event_Code]
		  ,[Detail_Name]
		  ,CASE [Detail_Sequence]
				WHEN 1 THEN @detail1
				WHEN 2 THEN @detail2
				WHEN 3 THEN @detail3
				WHEN 4 THEN @detail4
				WHEN 5 THEN @detail5
				ELSE NULL END
	  FROM [EventTypeDetails]
	 WHERE [Event_Code] = @Event_Code

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Events_RaiseEvent] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'HandleError',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[HandleError]') IS NOT NULL DROP PROCEDURE [HandleError]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [HandleError] AS BEGIN
   -- Return if there is no error information to retrieve.
    IF ERROR_NUMBER() IS NULL
        RETURN;

    DECLARE 
        @ErrorMessage    NVARCHAR(4000),
        @ErrorNumber     INT,
        @ErrorSeverity   INT,
        @ErrorState      INT,
        @ErrorLine       INT,
        @ErrorProcedure  NVARCHAR(200);

    -- Assign variables to error-handling functions that 
    -- capture information for RAISERROR.
    SELECT 
        @ErrorNumber = ERROR_NUMBER(),
        @ErrorSeverity = ERROR_SEVERITY(),
        @ErrorState = ERROR_STATE(),
        @ErrorLine = ERROR_LINE(),
        @ErrorProcedure = REPLACE(ISNULL(ERROR_PROCEDURE(), '-'),'`','''');

    -- Building the message string that will contain original
    -- error information.
    SELECT @ErrorMessage = N'%d`%d`%d`%s`%d`'+ ERROR_MESSAGE();

    -- Raise an error: msg_str parameter of RAISERROR will contain
    -- the original error information.
    RAISERROR 
        (
        @ErrorMessage, 
        @ErrorSeverity, 
        42,               
        @ErrorNumber,    -- parameter: original error number.
        @ErrorSeverity,  -- parameter: original error severity.
        @ErrorState,     -- parameter: original error state.
        @ErrorProcedure, -- parameter: original error procedure name.
        @ErrorLine       -- parameter: original error line number.
        );
END
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_InvalidateApprovals',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes received approvals for a pending promotion.'
GO

IF OBJECT_ID('[Promotions_InvalidateApprovals]') IS NOT NULL 
	DROP PROCEDURE [Promotions_InvalidateApprovals]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_InvalidateApprovals]
(
	@Workflow_Id INT,
	@Environment_Id INT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE BAR
	  FROM [BuildApprovalsReceived] BAR
	        LEFT JOIN [BuildPromotions] BP
		           ON BP.[Application_Id] = BAR.[Application_Id]
				  AND BP.[Release_Number] = BAR.[Release_Number]
				  AND BP.[Build_Number] = BAR.[Build_Number]
				  AND BP.[Environment_Id] = BAR.[Environment_Id]
		   INNER JOIN [Releases] R
		           ON R.[Application_Id] = BAR.[Application_Id]
				  AND R.[Release_Number] = BAR.[Release_Number]
		   INNER JOIN [WorkflowSteps] WS
			       ON WS.[Workflow_Id] = R.[Workflow_Id]
	 WHERE BP.[Application_Id] IS NULL
	   AND WS.[Environment_Id] = @Environment_Id
	   AND (@Workflow_Id IS NULL OR WS.[Workflow_Id] = @Workflow_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_InvalidateApprovals] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_EditChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the text of a database change script.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[AdminEdit_EditChangeScript]') IS NOT NULL DROP PROCEDURE [AdminEdit_EditChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_EditChangeScript]
(
	@Script_Id INT,
    @Script_Text NVARCHAR(MAX)
)
AS
BEGIN

    SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    UPDATE [DatabaseChangeScripts]
       SET [Script_Text] = @Script_Text,
           [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
           [ModifiedOn_Date] = GETUTCDATE()
     WHERE [Script_Id] = @Script_Id

     

    COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_EditChangeScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_PurgeArtifact',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an artifact from the specified build.'
GO


IF OBJECT_ID('[AdminEdit_PurgeArtifact]') IS NOT NULL DROP PROCEDURE [AdminEdit_PurgeArtifact]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_PurgeArtifact]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Artifact_Name VARCHAR(50)
)
AS
BEGIN

    DELETE [BuildArtifacts]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number
       AND [Artifact_Name] = @Artifact_Name

END
GO

GRANT EXECUTE ON [AdminEdit_PurgeArtifact] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_PurgeBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire build and all associated data from the system.'
GO

IF OBJECT_ID('[AdminEdit_PurgeBuild]') IS NOT NULL
	DROP PROCEDURE [AdminEdit_PurgeBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_PurgeBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [Applications] WHERE [Application_Id] = @Application_Id)
		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)
	
	IF NOT EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number)
		RAISERROR('Release %s of application %i does not exist.', 16, 1, @Release_Number, @Application_Id)

    IF NOT EXISTS(SELECT * FROM [Builds] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number AND [Build_Number] = @Build_Number)
		RAISERROR('Build %s of application %i of release %s does not exist.', 16, 1, @Build_Number, @Application_Id, @Release_Number)
		
	DECLARE @Application_Name VARCHAR(50)
	SELECT @Application_Name = [Application_Name]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

	DELETE [ReleaseNotes]
	 WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number

	DELETE [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number

	DELETE [BuildArtifacts]
	 WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number

	DELETE [BuildPromotionContingencies]
	 WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number

	DELETE [BuildApprovalsReceived]
	 WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number

    DELETE [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number

	DECLARE @Executions TABLE ([Execution_Id] INT NOT NULL)
	INSERT INTO @Executions ([Execution_Id])
	SELECT [Execution_Id]
	  FROM [BuildExecutions]
	 WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number

	DECLARE @BuildExecution_DeploymentPlans TABLE ([BuildExecution_DeploymentPlan_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_DeploymentPlans ([BuildExecution_DeploymentPlan_Id])
	SELECT [BuildExecution_DeploymentPlan_Id]
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DECLARE @BuildExecution_ActionGroupActions TABLE ([BuildExecution_ActionGroupAction_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_ActionGroupActions ([BuildExecution_ActionGroupAction_Id])
	SELECT [BuildExecution_ActionGroupAction_Id]
	  FROM [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	DELETE [BuildTestResults]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE [BuildOutputs]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE [BuildExecution_ActionGroupActionLogEntries]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

    DELETE [BuildExecution_ActionGroupActionVariableValues]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE [ManualActionsInProgress]
	 WHERE [BuildExecution_PlanAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE [BuildExecution_DeploymentPlanActionGroups]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	UPDATE [BuildExecutions]
	   SET [Initial_BuildExecution_DeploymentPlan_Id] = NULL
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE [BuildExecution_DeploymentPlans]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	DELETE [DatabaseChangeScriptExecutions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE [BuildExecutions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE [BuildPromotions]
	 WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number

	DELETE [Builds]
	 WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number


	 /* Raise Event */
	--EXEC [Events_RaiseEvent] 'RELDEL',NULL,@Application_Name,@Release_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_PurgeBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_UpdateBuildStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the status of a build.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[AdminEdit_UpdateBuildStatus]') IS NOT NULL DROP PROCEDURE [AdminEdit_UpdateBuildStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_UpdateBuildStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
    @BuildStatus_Name VARCHAR(8)
)
AS
BEGIN

    SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    IF EXISTS(SELECT * FROM [Builds]
               WHERE [Application_Id] = @Application_Id
                 AND [Release_Number] = @Release_Number
                 AND [Build_Number] = @Build_Number
                 AND [BuildStatus_Name] = @BuildStatus_Name)
    BEGIN
        ROLLBACK
        RETURN
    END

    IF (@BuildStatus_Name = 'Rejected')
    BEGIN

        UPDATE [Builds]
           SET [BuildStatus_Name] = @BuildStatus_Name,
               [Rejection_Notes] = 'Rejected via Administrative Edit',
               [RejectedBy_User_Name] = [dbo].Session_GetUserName(),
               [Rejected_Date] = GETUTCDATE(),
               [ModifiedBy_User_Name] = [dbo].Session_GetUserName(),
               [ModifiedOn_Date] = GETUTCDATE()
         WHERE [Application_Id] = @Application_Id
           AND [Release_Number] = @Release_Number
           AND [Build_Number] = @Build_Number

    END
    ELSE
    BEGIN

        UPDATE [Builds]
           SET [BuildStatus_Name] = @BuildStatus_Name,
               [Rejection_Notes] = NULL,
               [RejectedBy_User_Name] = NULL,
               [Rejected_Date] = NULL,
               [ModifiedBy_User_Name] = [dbo].Session_GetUserName(),
               [ModifiedOn_Date] = GETUTCDATE()
         WHERE [Application_Id] = @Application_Id
           AND [Release_Number] = @Release_Number
           AND [Build_Number] = @Build_Number

    END


    COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_UpdateBuildStatus] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_UpdateExecutionStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the status of an execution.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[AdminEdit_UpdateExecutionStatus]') IS NOT NULL DROP PROCEDURE [AdminEdit_UpdateExecutionStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_UpdateExecutionStatus]
(
	@Execution_Id INT,
    @ExecutionStatus_Name VARCHAR(10)
)
AS
BEGIN

    SET NOCOUNT ON


BEGIN TRY
	BEGIN TRANSACTION

    IF EXISTS(SELECT * FROM [BuildExecutions]
               WHERE [Execution_Id] = @Execution_Id
                 AND [ExecutionStatus_Name] = @ExecutionStatus_Name)
    BEGIN
        ROLLBACK
        RETURN
    END

    IF (@ExecutionStatus_Name = 'Canceled')
    BEGIN

        UPDATE [BuildExecutions]
           SET [ExecutionStatus_Name] = @ExecutionStatus_Name,
               [CanceledBy_User_Name] = [dbo].Session_GetUserName(),
               [CanceledOn_Date] = GETUTCDATE()
         WHERE [Execution_Id] = @Execution_Id

    END
    ELSE
    BEGIN

        UPDATE [BuildExecutions]
           SET [ExecutionStatus_Name] = @ExecutionStatus_Name,
               [CanceledBy_User_Name] = NULL,
               [CanceledOn_Date] = NULL
         WHERE [Execution_Id] = @Execution_Id

    END

    
    COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_UpdateExecutionStatus] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_UpdatePromotionStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the status of a promotion.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[AdminEdit_UpdatePromotionStatus]') IS NOT NULL DROP PROCEDURE [AdminEdit_UpdatePromotionStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_UpdatePromotionStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
    @Environment_Id INT,
    @PromotionStatus_Name VARCHAR(20)
)
AS
BEGIN

    SET NOCOUNT ON


BEGIN TRY
	BEGIN TRANSACTION

    IF EXISTS(SELECT * FROM [BuildPromotions]
               WHERE [Application_Id] = @Application_Id
                 AND [Release_Number] = @Release_Number
                 AND [Build_Number] = @Build_Number
                 AND [Environment_Id] = @Environment_Id
                 AND [PromotionStatus_Name] = @PromotionStatus_Name)
    BEGIN
        ROLLBACK
        RETURN
    END


    UPDATE [BuildPromotions]
       SET [PromotionStatus_Name] = @PromotionStatus_Name,
           [StatusOverriddenBy_User_Name] = [dbo].Session_GetUserName(),
           [StatusOverridden_Date] = GETUTCDATE(),
           [StatusOverriddenReason_Text] = 'Overridden via Administrative Edit'
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number
       AND [Environment_Id] = @Environment_Id

    
    COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_UpdatePromotionStatus] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'AdminEdit_UpdateReleaseStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the status of a release.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[AdminEdit_UpdateReleaseStatus]') IS NOT NULL DROP PROCEDURE [AdminEdit_UpdateReleaseStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [AdminEdit_UpdateReleaseStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
    @ReleaseStatus_Name VARCHAR(9)
)
AS
BEGIN

    SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    -- don't do anything if the status isn't changing
    IF EXISTS(SELECT * FROM [Releases]
               WHERE [Application_Id] = @Application_Id
                 AND [Release_Number] = @Release_Number
                 AND [ReleaseStatus_Name] = @ReleaseStatus_Name)
    BEGIN
        ROLLBACK
        RETURN
    END
    
    IF (@ReleaseStatus_Name = 'Cancelled')
    BEGIN

        UPDATE [Releases]
           SET [ReleaseStatus_Name] = @ReleaseStatus_Name,
               [CancelledBy_User_Name] = [dbo].Session_GetUserName(),
               [CancelledOn_Date] = GETUTCDATE(),
               [CancelledReason_Text] = 'Cancelled via Administrative Edit',
               [ModifiedOn_Date] = GETUTCDATE(),
               [ModifiedBy_User_Name] = [dbo].Session_GetUserName()
         WHERE [Application_Id] = @Application_Id
           AND [Release_Number] = @Release_Number

    END
    ELSE
    BEGIN

        UPDATE [Releases]
           SET [ReleaseStatus_Name] = @ReleaseStatus_Name,
               [CancelledBy_User_Name] = NULL,
               [CancelledOn_Date] = NULL,
               [CancelledReason_Text] = NULL,
               [ModifiedOn_Date] = GETUTCDATE(),
               [ModifiedBy_User_Name] = [dbo].Session_GetUserName()
         WHERE [Application_Id] = @Application_Id
           AND [Release_Number] = @Release_Number

    END

    COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [AdminEdit_UpdateReleaseStatus] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_CreateOrUpdateApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ApplicationGroup_Id',
    /* Description_Text        */ 'Creates or updates a new application group'
GO

IF OBJECT_ID('[ApplicationGroups_CreateOrUpdateApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_CreateOrUpdateApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_CreateOrUpdateApplicationGroup]
(
	@ApplicationGroup_Id INT = NULL OUT,
	@ApplicationGroup_Name VARCHAR(50),
	@ApplicationGroup_Description VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION
	
	IF @ApplicationGroup_Id IS NULL
	BEGIN
		INSERT INTO [ApplicationGroups]
		(
			[ApplicationGroup_Name],
			[ApplicationGroup_Description],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@ApplicationGroup_Name,
			@ApplicationGroup_Description,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		
		SET @ApplicationGroup_Id = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		UPDATE [ApplicationGroups]
		   SET [ApplicationGroup_Name] = @ApplicationGroup_Name,
			   [ApplicationGroup_Description] = @ApplicationGroup_Description,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id
	END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ApplicationGroups_CreateOrUpdateApplicationGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_DeleteApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an application group'
GO

IF OBJECT_ID('[ApplicationGroups_DeleteApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_DeleteApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_DeleteApplicationGroup]
(
	@ApplicationGroup_Id INT
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION
	
        UPDATE [Applications]
           SET [ApplicationGroup_Id] = NULL
         WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

        DELETE [EventListenerApplicationGroups]
         WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

        DELETE [Privileges]
         WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

        DELETE [VariableDefinitions]
         WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

        DELETE [RetentionPolicies]
         WHERE [PurgeIn_ApplicationGroup_Id] = @ApplicationGroup_Id

		DELETE [ApplicationGroups]
		 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ApplicationGroups_DeleteApplicationGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_GetApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ApplicationGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an application group by ID.'
GO

IF OBJECT_ID('[ApplicationGroups_GetApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_GetApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_GetApplicationGroup]
(
	@ApplicationGroup_Id INT
)
AS
BEGIN

SET NOCOUNT ON
	SELECT *
	  FROM [ApplicationGroups]
	 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

END
GO

GRANT EXECUTE ON [ApplicationGroups_GetApplicationGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_GetApplicationGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ApplicationGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the application groups'
GO

IF OBJECT_ID('[ApplicationGroups_GetApplicationGroups]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_GetApplicationGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_GetApplicationGroups]
AS
BEGIN

SET NOCOUNT ON
	SELECT * FROM [ApplicationGroups] ORDER BY [ApplicationGroup_Name]

END
GO

GRANT EXECUTE ON [ApplicationGroups_GetApplicationGroups] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_GetOrCreateApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ApplicationGroup_Id',
    /* Description_Text        */ 'Gets or creates a new application group using the name'
GO

IF OBJECT_ID('[ApplicationGroups_GetOrCreateApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_GetOrCreateApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_GetOrCreateApplicationGroup]
(
	@ApplicationGroup_Name VARCHAR(50),
    @ApplicationGroup_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Existing_ApplicationGroup_Id INT
    DECLARE @Existing_ApplicationGroup_Name VARCHAR(50)

    SELECT @Existing_ApplicationGroup_Name = [ApplicationGroup_Name],
           @Existing_ApplicationGroup_Id = [ApplicationGroup_Id]
      FROM [ApplicationGroups]
     WHERE [ApplicationGroup_Name] = @ApplicationGroup_Name
    
    IF (@ApplicationGroup_Name IS NULL OR LTRIM(RTRIM(@ApplicationGroup_Name)) = '')
    BEGIN
        
        SET @ApplicationGroup_Id = NULL

    END
	ELSE IF (@Existing_ApplicationGroup_Id IS NULL)
	BEGIN

		INSERT INTO [ApplicationGroups]
		(
			[ApplicationGroup_Name],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@ApplicationGroup_Name,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		
		SET @ApplicationGroup_Id = SCOPE_IDENTITY()

	END
    ELSE
    BEGIN

        SET @ApplicationGroup_Id = @Existing_ApplicationGroup_Id

    END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ApplicationGroups_GetOrCreateApplicationGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_AddImportedDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds an imported deployable to an application'
GO

IF OBJECT_ID('[Applications_AddImportedDeployable]') IS NOT NULL DROP PROCEDURE [Applications_AddImportedDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_AddImportedDeployable]
(
	@Deployable_Id INT,
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
		
		IF NOT EXISTS(
			SELECT * FROM DeployablesImported
			WHERE [Deployable_Id] = @Deployable_Id
			  AND [Application_Id] = @Application_Id
		)
		BEGIN
			INSERT INTO [DeployablesImported]
			(
				[Deployable_Id],
				[Application_Id]
			)
			VALUES
			(
				@Deployable_Id,
				@Application_Id
			)
		END

		/* Raise Application Modified Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_AddImportedDeployable] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_CreateApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Application_Id',
    /* Description_Text        */ 'Creates a new application'
GO

IF OBJECT_ID('[Applications_CreateApplication]') IS NOT NULL DROP PROCEDURE [Applications_CreateApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_CreateApplication]
(
	@Application_Name VARCHAR(50),
	@ReleaseNumber_Scheme_Name VARCHAR(20) = NULL,
	@IssueTracking_Provider_Id INT = NULL,
	@BuildNumber_Scheme_Name VARCHAR(15) = NULL,
	@AllowMultipleActiveReleases_Indicator YNINDICATOR  = NULL,
	@AllowMultipleActiveBuilds_Indicator YNINDICATOR  = NULL,
	@ApplicationGroup_Id INT = NULL,
	@Application_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION

	/* Set defaults */
	SET @ReleaseNumber_Scheme_Name = ISNULL(@ReleaseNumber_Scheme_Name, 'MajorMinor')
	SET @BuildNumber_Scheme_Name = ISNULL(@BuildNumber_Scheme_Name, 'Sequential')
	SET @AllowMultipleActiveReleases_Indicator = ISNULL(@AllowMultipleActiveReleases_Indicator, 'N')
	SET @AllowMultipleActiveBuilds_Indicator = ISNULL(@AllowMultipleActiveBuilds_Indicator, 'N')

	INSERT INTO [Applications] (
		[Application_Name], 
		[ReleaseNumber_Scheme_Name],
		[BuildNumber_Scheme_Name],
		[IssueTracking_Provider_Id],
		[AllowMultipleActiveReleases_Indicator],
		[AllowMultipleActiveBuilds_Indicator],
		[ApplicationGroup_Id],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[ModifiedBy_User_Name],
		[ModifiedOn_Date],
		[VariableSupport_Code],
        [IssueTracking_AllowBuildMasterIssues_Indicator]
	)
	VALUES (
		@Application_Name,
		@ReleaseNumber_Scheme_Name,
		@BuildNumber_Scheme_Name,
		@IssueTracking_Provider_Id,
		@AllowMultipleActiveReleases_Indicator,
		@AllowMultipleActiveBuilds_Indicator,
		@ApplicationGroup_Id,
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		'N',
        'Y'
	)
	SET @Application_Id = SCOPE_IDENTITY()
	
	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPNEW',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_CreateApplication] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_CreateOrUpdateDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Deployable_Id',
    /* Description_Text        */ 'Creates a deployable for a specified application, or updates an existing one'
GO


IF OBJECT_ID('[Applications_CreateOrUpdateDeployable]') IS NOT NULL DROP PROCEDURE [Applications_CreateOrUpdateDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_CreateOrUpdateDeployable]
(
	@Deployable_Id INT OUT,
	@Application_Id INT,
	@Deployable_Name VARCHAR(50),
	@DeployableType_Code CHAR(4)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Deployable_Id IS NOT NULL
	BEGIN

		DECLARE @Old_Deployable_Name VARCHAR(50)
		SET @Old_Deployable_Name = (SELECT [Deployable_Name]
		                              FROM [Deployables]
									 WHERE [Deployable_Id] = @Deployable_Id)

		UPDATE [Deployables]
		   SET [Deployable_Name] = @Deployable_Name,
		       [DeployableType_Code] = @DeployableType_Code,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Deployable_Id] = @Deployable_Id

		IF @@ROWCOUNT=0 RAISERROR('Deployable %i does not exist.',16,1,@Deployable_Id)

		IF @Old_Deployable_Name <> @Deployable_Name
		BEGIN

			-- Update deployable names in deployment plans
			UPDATE DPAG
			   SET DPAG.[Deployable_Name] = @Deployable_Name
			  FROM [DeploymentPlanActionGroups] DPAG
			       LEFT JOIN [ApplicationDeploymentPlans] ADP
				          ON ADP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
			 WHERE (DPAG.[Deployable_Application_Id] = @Application_Id AND DPAG.[Deployable_Name] = @Old_Deployable_Name)
			    OR (DPAG.[Deployable_Application_Id] IS NULL AND ADP.[Application_Id] = @Application_Id AND DPAG.[Deployable_Name] = @Old_Deployable_Name)

		END

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	END
	ELSE
	BEGIN
		INSERT INTO [Deployables]
		(
			[Application_Id],
			[Deployable_Name],
			[DeployableType_Code],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
        (
			@Application_Id,
			@Deployable_Name,
			@DeployableType_Code,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		SET @Deployable_Id = SCOPE_IDENTITY()

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPNWD',@Application_Id,@Deployable_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_CreateOrUpdateDeployable] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_DeleteImportedDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an existing imported deployable relation, but does not delete the deployable from the host application'
GO

IF OBJECT_ID('[Applications_DeleteImportedDeployable]') IS NOT NULL DROP PROCEDURE [Applications_DeleteImportedDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_DeleteImportedDeployable]
(
	@Deployable_Id INT,
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [DeployablesImported]
		WHERE [Deployable_Id] = @Deployable_Id
		  AND [Application_Id] = @Application_Id

		/* Raise Application Modified Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_DeleteImportedDeployable] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_EditApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Edits an existing application.'
GO


IF OBJECT_ID('[Applications_EditApplication]') IS NOT NULL DROP PROCEDURE [Applications_EditApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_EditApplication]
(
	@Application_Id INT,
	@Application_Name VARCHAR(50) = NULL,
	@Application_Description VARCHAR(MAX) = NULL,
	@ApplicationGroup_Id INT,
	@AllowMultipleActiveReleases_Indicator YNINDICATOR = NULL,
	@AllowMultipleActiveBuilds_Indicator YNINDICATOR = NULL,
	@VariableSupport_Code CHAR(1),
    @ReleaseNumber_Scheme_Name VARCHAR(20),
	@BuildNumber_Scheme_Name VARCHAR(15) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
       SET [Application_Name] = @Application_Name
		  ,[Application_Description] = @Application_Description
		  ,[ApplicationGroup_Id] = @ApplicationGroup_Id
		  ,[AllowMultipleActiveReleases_Indicator] = @AllowMultipleActiveReleases_Indicator
		  ,[AllowMultipleActiveBuilds_Indicator] = @AllowMultipleActiveBuilds_Indicator
		  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		  ,[ModifiedOn_Date] = GETUTCDATE()
		  ,[VariableSupport_Code] = @VariableSupport_Code
          ,[ReleaseNumber_Scheme_Name] = @ReleaseNumber_Scheme_Name
		  ,[BuildNumber_Scheme_Name] = COALESCE(@BuildNumber_Scheme_Name, [BuildNumber_Scheme_Name])
     WHERE [Application_Id] = @Application_Id
	IF @@ROWCOUNT=0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_EditApplication] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_FindDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Searches for deployables in the system.'
GO

IF OBJECT_ID('[Applications_FindDeployables]') IS NOT NULL DROP PROCEDURE [Applications_FindDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_FindDeployables]
(
	@SearchTerm_Text VARCHAR(50),
	@Max_Count INT
)
AS
BEGIN

	SET NOCOUNT ON

	SET @SearchTerm_Text = NULLIF(@SearchTerm_Text, '')
	IF @SearchTerm_Text IS NOT NULL
	BEGIN
		SET @SearchTerm_Text = REPLACE(@SearchTerm_Text, '_', '\_')
		SET @SearchTerm_Text = REPLACE(@SearchTerm_Text, '%', '\%')
		SET @SearchTerm_Text = REPLACE(@SearchTerm_Text, '[', '\[')
		SET @SearchTerm_Text = @SearchTerm_Text + '%'
	END

	IF @Max_Count IS NOT NULL
		SET ROWCOUNT @Max_Count

	SELECT *
	  FROM [Deployables_Extended]
     WHERE @SearchTerm_Text IS NULL
	    OR [Deployable_Name] LIKE @SearchTerm_Text ESCAPE '\'
		OR [Application_Name] LIKE @SearchTerm_Text ESCAPE '\'

END
GO

GRANT EXECUTE ON [Applications_FindDeployables] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Applications_Extended,Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of an application along with the associated deployables'
GO


IF OBJECT_ID('[Applications_GetApplication]') IS NOT NULL DROP PROCEDURE [Applications_GetApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetApplication]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Applications_Extended]
     WHERE [Application_Id] = @Application_Id

	SELECT * 
	  FROM [Deployables_Extended]
     WHERE [Application_Id] = @Application_Id
	ORDER BY [Deployable_Name]

END
GO

GRANT EXECUTE ON [Applications_GetApplication] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetApplications',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified number of applications in the system, or all if null is passed in'
GO


IF OBJECT_ID('[Applications_GetApplications]') IS NOT NULL DROP PROCEDURE [Applications_GetApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetApplications]
(
    @Application_Count INT
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Application_Count IS NOT NULL
        SET ROWCOUNT @Application_Count

	SELECT * FROM [Applications_Extended]
	ORDER BY [ApplicationGroup_Name], [Application_Name]

END
GO

GRANT EXECUTE ON [Applications_GetApplications] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetApplicationsVisibleToUser',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the applications which the specified user is allowed to view.'
GO


IF OBJECT_ID('[Applications_GetApplicationsVisibleToUser]') IS NOT NULL DROP PROCEDURE [Applications_GetApplicationsVisibleToUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetApplicationsVisibleToUser]
(
    @User_Name USERNAME,
    @GroupNames_Csv VARCHAR(MAX),
    @Directory_Provider_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

-- Applications_ViewOverview = 40

SELECT AE.*
  FROM [Applications_Extended] AE
 WHERE dbo.IsAuthorizedForTask(@Directory_Provider_Id, @User_Name, @GroupNames_Csv, 40, AE.[ApplicationGroup_Id], AE.[Application_Id], NULL) = 1

END
GO

GRANT EXECUTE ON [Applications_GetApplicationsVisibleToUser] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDependentApplications',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all applications with imported deployables from a specified application'
GO

IF OBJECT_ID('[Applications_GetDependentApplications]') IS NOT NULL DROP PROCEDURE [Applications_GetDependentApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDependentApplications]
(
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

        SELECT * 
          FROM [Applications_Extended]
         WHERE [Application_Id] IN (SELECT [Application_Id]     
                                      FROM [DeployablesImported_Extended] 
                                     WHERE [ImportedFrom_Application_Id] = @Application_Id)

END
GO

GRANT EXECUTE ON [Applications_GetDependentApplications] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified deployable'
GO


IF OBJECT_ID('[Applications_GetDeployable]') IS NOT NULL DROP PROCEDURE [Applications_GetDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDeployable]
(
	@Deployable_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Deployables_Extended]
     WHERE [Deployable_Id] = @Deployable_Id

END
GO

GRANT EXECUTE ON [Applications_GetDeployable] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDeployableDependencies',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DeployableDependencies',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of dependencies for the deployables in a given application or a specific deployable if one is specified'
GO

IF OBJECT_ID('[Applications_GetDeployableDependencies]') IS NOT NULL DROP PROCEDURE [Applications_GetDeployableDependencies]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDeployableDependencies]
(
	@Application_Id INT,
	@Deployable_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		SELECT * FROM [DeployableDependencies]
		 WHERE [Deployable_Id] IN (SELECT [Deployable_Id] FROM [Deployables] 
		                            WHERE [Application_Id] = @Application_Id)
		   AND [Deployable_Id] = COALESCE(@Deployable_Id,[Deployable_Id])

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_GetDeployableDependencies] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the deployables for a specific application'
GO

IF OBJECT_ID('[Applications_GetDeployables]') IS NOT NULL DROP PROCEDURE [Applications_GetDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDeployables]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Deployables_Extended]
     WHERE [Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [Applications_GetDeployables] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetImportedDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DeployablesImported_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set a imported deployables for a specified application'
GO

IF OBJECT_ID('[Applications_GetImportedDeployables]') IS NOT NULL DROP PROCEDURE [Applications_GetImportedDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetImportedDeployables]
(
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		SELECT * FROM DeployablesImported_Extended
		WHERE @Application_Id IS NULL 
		   OR [Application_Id] = @Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_GetImportedDeployables] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetNavigationData',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Applications_Extended,Builds_Extended,Workflows_Extended,ApplicationDeploymentPlans',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the information required to display the application navigation bar.'
GO


IF OBJECT_ID('[Applications_GetNavigationData]') IS NOT NULL DROP PROCEDURE [Applications_GetNavigationData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetNavigationData]
(
    @Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Applications_Extended]
	 WHERE [Application_Id] = @Application_Id

	SELECT *
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [BuildStatus_Name] = 'Active'
	 ORDER BY [CreatedOn_Date] DESC

	SELECT W.*
	  FROM [Workflows_Extended] W
	       INNER JOIN [ApplicationWorkflows] AW
		           ON AW.[Workflow_Id] = W.[Workflow_Id]
	 WHERE AW.[Application_Id] = @Application_Id
	   AND W.[Active_Indicator] = 'Y'
	 ORDER BY [Workflow_Name]

	SELECT *
	  FROM [ApplicationDeploymentPlans]
	 WHERE [Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [Applications_GetNavigationData] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetWorkflowSteps',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'WorkflowSteps_Extended,ApplicationDeploymentPlans',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the workflow steps for a specific application, and any build plans associated with that application.'
GO

IF OBJECT_ID('[Applications_GetWorkflowSteps]') IS NOT NULL DROP PROCEDURE [Applications_GetWorkflowSteps]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetWorkflowSteps]
(
	@Application_Id INT,
	@IncludeInactive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT WS.*
	  FROM [WorkflowSteps_Extended] WS
	       INNER JOIN [ApplicationWorkflows] AW
		           ON AW.[Workflow_Id] = WS.[Workflow_Id]
		   INNER JOIN [Workflows] W
		           ON W.[Workflow_Id] = WS.[Workflow_Id]
     WHERE AW.[Application_Id] = @Application_Id
	   AND (@IncludeInactive_Indicator = 'Y' OR W.[Active_Indicator] = 'Y')

    SELECT * 
      FROM [ApplicationDeploymentPlans]
     WHERE [Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [Applications_GetWorkflowSteps] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_PurgeApplicationData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire application and all associated data from the system'
GO

IF OBJECT_ID('[Applications_PurgeApplicationData]') IS NOT NULL
	DROP PROCEDURE [Applications_PurgeApplicationData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_PurgeApplicationData]
(
	@Application_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [Applications] WHERE [Application_Id] = @Application_Id)
		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)

	/* 1) Delete data without sub-application dependencies */
	DELETE
	  FROM [BuildScheduleVariableDefinitions]
	 WHERE [Schedule_Id] IN (SELECT [Schedule_Id] 
		                            FROM [BuildSchedules] WHERE [Application_Id] = @Application_Id)
	 
	DELETE
	  FROM [BuildSchedules]
	 WHERE [Application_Id] = @Application_Id	 

	DELETE
	  FROM [Privileges]
	 WHERE [Application_Id] = @Application_Id


	/* Find dependent data: */
	DECLARE @ApplicationLocalWorkflows TABLE ([Workflow_Id] INT NOT NULL)
	INSERT INTO @ApplicationLocalWorkflows ([Workflow_Id])
	SELECT W.[Workflow_Id]
	  FROM [ApplicationWorkflows] AW
     INNER JOIN [Workflows] W
        ON AW.[Workflow_Id] = W.[Workflow_Id]
	 WHERE [Application_Id] = @Application_Id
       AND W.[Global_Indicator] = 'N'

	DECLARE @Deployables TABLE ([Deployable_Id] INT NOT NULL)
	INSERT INTO @Deployables ([Deployable_Id])
	SELECT [Deployable_Id]
	  FROM [Deployables]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @Releases TABLE ([Release_Number] RELEASENUMBER NOT NULL)
	INSERT INTO @Releases ([Release_Number])
	SELECT [Release_Number]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @Builds TABLE ([Release_Number] RELEASENUMBER NOT NULL, [Build_Number] BUILDNUMBER)
	INSERT INTO @Builds ([Release_Number], [Build_Number])
	SELECT [Release_Number], [Build_Number]
	  FROM [Builds]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @Executions TABLE ([Execution_Id] INT NOT NULL)
	INSERT INTO @Executions ([Execution_Id])
	SELECT [Execution_Id]
	  FROM [BuildExecutions]
	 WHERE [Application_Id] = @Application_Id
	 
	DECLARE @BuildExecutionActionGroupActions TABLE ([BuildExecution_ActionGroupAction_Id] INT NOT NULL)
	INSERT INTO @BuildExecutionActionGroupActions
	SELECT BEAGA.[BuildExecution_ActionGroupAction_Id]
	  FROM [BuildExecution_ActionGroupActions] BEAGA
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		           ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEDP.[Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	-- Plans should get deleted if they are anonymous and only used by this application
	DECLARE @DeploymentPlans TABLE ([DeploymentPlan_Id] INT NOT NULL)
	INSERT INTO @DeploymentPlans
	SELECT DISTINCT ADP.[DeploymentPlan_Id]
	  FROM [ApplicationDeploymentPlans] ADP
	       INNER JOIN [DeploymentPlans] DP
		           ON DP.[DeploymentPlan_Id] = ADP.[DeploymentPlan_Id]
	 WHERE ADP.[Application_Id] = @Application_Id
	   AND DP.[DeploymentPlan_Name] IS NULL
	   AND ADP.[DeploymentPlan_Id] NOT IN (SELECT [DeploymentPlan_Id] FROM [ApplicationDeploymentPlans] WHERE [Application_Id] <> @Application_Id)

	DECLARE @ConfigurationFiles TABLE ([ConfigurationFile_Id] INT NOT NULL)
	INSERT INTO @ConfigurationFiles ([ConfigurationFile_Id])
	SELECT [ConfigurationFile_Id]
	  FROM [ConfigurationFiles]
	 WHERE [Deployable_Id] IN ( SELECT [Deployable_Id] FROM @Deployables )

	DECLARE @DatabaseChangeScripts TABLE ([Script_Id] INT NOT NULL)
	INSERT INTO @DatabaseChangeScripts ([Script_Id])
	SELECT [Script_Id]
	  FROM [DatabaseChangeScripts]
	 WHERE [Application_Id] = @Application_Id
	 
	DECLARE @ChangeControls TABLE ([ChangeControl_Id] INT NOT NULL)
	INSERT INTO @ChangeControls ([ChangeControl_Id])
	 SELECT [ChangeControl_Id]
	   FROM [ChangeControls]
	  WHERE [Application_Id] = @Application_Id
	  
	DECLARE @Dashboards TABLE ([Dashboard_Id] INT NOT NULL)
	INSERT INTO @Dashboards ([Dashboard_Id])
	 SELECT [Dashboard_Id]
	   FROM [Dashboards]
	  WHERE [Application_Id] = @Application_Id

	/* 2) Delete data with sub-application dependencies */
	/* 2a) Variables */

	DELETE
	  FROM [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id
        OR [Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)

	DELETE
	  FROM [VariableDeclarations]
	 WHERE [Scoped_Application_Id] = @Application_Id

	DELETE
	  FROM [ReleaseNotes]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [BuildPromotionContingencies]
	 WHERE [Application_Id] = @Application_Id
	    OR [Contingent_Application_Id] = @Application_Id

	/* 2b) Configuration */
	DELETE
	  FROM [ConfigurationFileDeployments]
	 WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
	  FROM [ConfigurationFileVersions]
	 WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )

	DELETE
	  FROM [ConfigurationFileInstances]
	 WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )

	DELETE
	  FROM [ReleaseConfigurationFiles]
	 WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )

	DELETE
	  FROM [ConfigurationFiles]
	 WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	/* 2c) Builds */
	DELETE
	  FROM [BuildApprovalsReceived]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [PromotionRequirementStatuses]
	 WHERE [Application_Id] = @Application_Id	 


	DELETE
	  FROM [BuildArtifacts]
	 WHERE [Application_Id] = @Application_Id


	DELETE
	  FROM [BuildTestResults]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )
	 
	 DELETE
	  FROM [BuildOutputs]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildExecution_ActionGroupActionVariableValues]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecutionActionGroupActions)

	DELETE
	  FROM [BuildExecution_ActionGroupActionLogEntries]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecutionActionGroupActions)


	DELETE
	  FROM [ManualActionsInProgress]
	 WHERE [BuildExecution_PlanAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecutionActionGroupActions)


	DELETE
	  FROM [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecutionActionGroupActions)


	DELETE BEDPAG
	  FROM [BuildExecution_DeploymentPlanActionGroups] BEDPAG
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		           ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEDPAG.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEDP.[Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)


	UPDATE [BuildExecutions]
	   SET [Initial_BuildExecution_DeploymentPlan_Id] = NULL
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)


	/* Have to delete change script executions before Build Executions... */
	DELETE
	  FROM [DatabaseChangeScriptExecutions]
	 WHERE [Script_Id] IN ( SELECT [Script_Id] FROM @DatabaseChangeScripts )
	 
	DELETE
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

    DELETE
	  FROM [BuildPromotionsScheduled]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [BuildPromotions]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [Issues_BuildStatusChanges]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [Builds]
	 WHERE [Application_Id] = @Application_Id


	/* 2d) Change Scripts */
	DELETE
	  FROM [DatabaseChangeScriptProviders]
	 WHERE [Application_Id] = @Application_Id


	DELETE
	  FROM [DatabaseChangeScripts]
	 WHERE [Application_Id] = @Application_Id
	 

	/* 2d2) Change Controls */
	DELETE 
      FROM [ChangeControlEnvironments]
     WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)
    
    DELETE
      FROM [ChangeControlPerformances]
     WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)
    
    DELETE 
      FROM [ChangeControls]
     WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)
	 


	/* 2e) Releases */
	DELETE [ReleaseDeployables]
	 WHERE [Application_Id] = @Application_Id
	    OR [Referenced_Application_Id] = @Application_Id


	DELETE [Releases]
	 WHERE [Application_Id] = @Application_Id


	/* 2f) Deployment Plans */
	DECLARE @TmpDeploymentPlanActionGroups TABLE ([DeploymentPlanActionGroup_Id] INT)
	 INSERT @TmpDeploymentPlanActionGroups
	 SELECT [DeploymentPlanActionGroup_Id] 
	   FROM [DeploymentPlanActionGroups]
	  WHERE [DeploymentPlan_Id] IN (SELECT [DeploymentPlan_Id] FROM @DeploymentPlans)

	 WHILE ((SELECT COUNT(*) FROM @TmpDeploymentPlanActionGroups) > 0)
	 BEGIN
		DECLARE @DeploymentPlanActionGroup_Id INT
		SET @DeploymentPlanActionGroup_Id = (SELECT TOP 1 [DeploymentPlanActionGroup_Id] FROM @TmpDeploymentPlanActionGroups)
		
		EXEC [Plans_DeleteDeploymentPlanActionGroup] @DeploymentPlanActionGroup_Id

		DELETE @TmpDeploymentPlanActionGroups WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id
	 END
	    
	DECLARE @ActionGroups TABLE ([ActionGroup_Id] INT NOT NULL)	 
	INSERT INTO @ActionGroups ([ActionGroup_Id])
	SELECT [ActionGroup_Id]
	  FROM [ActionGroups] AG
	 WHERE NOT EXISTS (SELECT * FROM [DeploymentPlanActionGroups] _DPAG
	                     WHERE _DPAG.[ActionGroup_Id] = AG.[ActionGroup_Id]) 

    DELETE [ActionGroupActions]
     WHERE [ActionGroup_Id] IN (SELECT [ActionGroup_Id] FROM @ActionGroups)

	DELETE [ActionGroups] FROM [ActionGroups]
	 WHERE [ActionGroup_Id] IN (SELECT [ActionGroup_Id] FROM @ActionGroups)

    UPDATE [Workflows]
       SET [Build_DeploymentPlan_Id] = NULL
     WHERE [Build_DeploymentPlan_Id] IN (SELECT [DeploymentPlan_Id] FROM @DeploymentPlans)      

    UPDATE [WorkflowSteps]
       SET [DeploymentPlan_Id] = NULL
	 WHERE [DeploymentPlan_Id] IN (SELECT [DeploymentPlan_Id] FROM @DeploymentPlans)

	DELETE [DeploymentPlans]
	 WHERE [DeploymentPlan_Id] IN (SELECT [DeploymentPlan_Id] FROM @DeploymentPlans)

	UPDATE [DeploymentPlanActionGroups]
	   SET [Deployable_Application_Id] = NULL
	 WHERE [Deployable_Application_Id] = @Application_Id

    /* 2f2) EventListeners */

    DECLARE @EventListeners_ToDelete TABLE ([Id] INT)

	INSERT @EventListeners_ToDelete ([Id])
	SELECT [EventListener_Id] 
      FROM [EventListeners]
	 WHERE [WorkflowStep_Workflow_Id] IN (SELECT [Workflow_Id] FROM @ApplicationLocalWorkflows)
        OR [BuildStep_Workflow_Id] IN (SELECT [Workflow_Id] FROM @ApplicationLocalWorkflows)

	DELETE [EventListenerApplicationGroups] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListenerApplications] WHERE [Application_Id] = @Application_Id OR [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListenerEnvironments] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListenerEvents] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListeners] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)

	/* 2g) Workflows */

	DELETE
	  FROM [BuildPromotionRequirements]
	 WHERE [Workflow_Id] IN ( SELECT [Workflow_Id] FROM @ApplicationLocalWorkflows )

	DELETE
	  FROM [WorkflowSteps]
	 WHERE [Workflow_Id] IN ( SELECT [Workflow_Id] FROM @ApplicationLocalWorkflows )

    DELETE 
      FROM [ApplicationWorkflows]
     WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [Workflows]
	 WHERE [Workflow_Id] IN ( SELECT [Workflow_Id] FROM @ApplicationLocalWorkflows )


	/* 2h) Deployables */
	DELETE [DeployableDependencies]
	 WHERE [Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)
	    OR [DependsOn_Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)

	DELETE [DeployablesImported]
	 WHERE [Application_Id] = @Application_Id
	    OR [Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)
 	
	DELETE
	  FROM [Deployables]
	 WHERE [Application_Id] = @Application_Id
	 
 DECLARE @Application_Name SQL_VARIANT
	SET @Application_Name = (SELECT [Application_Name] FROM [Applications] WHERE [Application_Id] = @Application_Id)

	/* Dashboards */
	
	DELETE [DashboardGadgets]
	 WHERE [Dashboard_Id] IN (SELECT [Dashboard_Id] FROM @Dashboards)
	
	DELETE [Dashboards]
	 WHERE [Dashboard_Id] IN (SELECT [Dashboard_Id] FROM @Dashboards)
	
	/* Event Occurrences */

	 DELETE [EventOccurenceDetails]
	  FROM [EventOccurenceDetails] EOD
	       INNER JOIN [EventOccurences] ED
	               ON ED.[EventOccurence_Id] = EOD.[EventOccurence_Id]
	 WHERE ED.[Application_Id] = @Application_Id

	DELETE [EventOccurences]
	 WHERE [Application_Id] = @Application_Id

    DELETE [RetentionPolicies]
     WHERE [PurgeIn_Application_Id] = @Application_Id

	/* 3) Application */
	DELETE
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPDEL',NULL,@Application_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_PurgeApplicationData] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_PurgeDeployableData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges a deployable and all its associations from the system.'
GO

IF OBJECT_ID('[Applications_PurgeDeployableData]') IS NOT NULL
	DROP PROCEDURE [Applications_PurgeDeployableData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_PurgeDeployableData]
(
	@Deployable_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [Deployables] WHERE [Deployable_Id] = @Deployable_Id)
		RAISERROR('Deployable %i does not exist.', 16, 1, @Deployable_Id)

	DECLARE @ConfigurationFiles TABLE ([ConfigurationFile_Id] INT NOT NULL)
    INSERT INTO @ConfigurationFiles ([ConfigurationFile_Id])
    SELECT [ConfigurationFile_Id]
      FROM [ConfigurationFiles]
     WHERE [Deployable_Id] = @Deployable_Id

	DECLARE @DatabaseChangeScripts TABLE ([Script_Id] INT NOT NULL)
    INSERT INTO @DatabaseChangeScripts ([Script_Id])
    SELECT [Script_Id]
      FROM [DatabaseChangeScripts]
     WHERE [Deployable_Id] = @Deployable_Id

    DELETE [DeployableDependencies]
     WHERE [Deployable_Id] = @Deployable_Id
	    OR [DependsOn_Deployable_Id] = @Deployable_Id

	DELETE [DeployablesImported]
	 WHERE [Deployable_Id] = @Deployable_Id

	DELETE
      FROM [BuildArtifacts]
     WHERE [Deployable_Id] = @Deployable_Id

	DELETE
      FROM [ConfigurationFileDeployments]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )

    DELETE
      FROM [ReleaseConfigurationFiles]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )

	DELETE
      FROM [ConfigurationFileVersions]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [ConfigurationFileInstances]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [ConfigurationFiles]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [DatabaseChangeScriptExecutions]
     WHERE [Script_Id] IN ( SELECT [Script_Id] FROM @DatabaseChangeScripts )


	DELETE
      FROM [DatabaseChangeScripts]
     WHERE [Deployable_Id] = @Deployable_Id

	DELETE 
      FROM [ReleaseDeployables]
     WHERE [Deployable_Id] = @Deployable_Id 

    DELETE
      FROM [VariableDefinitions]
     WHERE [Deployable_Id] = @Deployable_Id 


	DECLARE @Deployable_Name SQL_VARIANT, @Application_Id INT
	 SELECT @Deployable_Name = [Deployable_Name] 
	       ,@Application_Id = [Application_Id] 
       FROM [Deployables] 
      WHERE [Deployable_Id] = @Deployable_Id

	DELETE 
      FROM [Deployables]
     WHERE [Deployable_Id] = @Deployable_Id 

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPDLD',@Application_Id,@Deployable_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_PurgeDeployableData] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SetDashboard',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the dashboard text for an application'
GO


IF OBJECT_ID('[Applications_SetDashboard]') IS NOT NULL DROP PROCEDURE [Applications_SetDashboard]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SetDashboard]
(
	@Application_Id INT,
	@Dashboard_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
       SET [Dashboard_Text] = @Dashboard_Text
     WHERE [Application_Id] = @Application_Id
	IF @@ROWCOUNT=0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_SetDashboard] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SetDeployableDependencies',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the dependency relationships between the specified deployable and its dependents'
GO

IF OBJECT_ID('[Applications_SetDeployableDependencies]') IS NOT NULL DROP PROCEDURE [Applications_SetDeployableDependencies]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SetDeployableDependencies]
(
	@Deployable_Id INT,
	@DependsOn_Deployables_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [DeployableDependencies]
		 WHERE [Deployable_Id] = @Deployable_Id

		INSERT INTO [DeployableDependencies]
			(
				[Deployable_Id],
				[DependsOn_Deployable_Id]
			)
		SELECT
				@Deployable_Id,
				CAST(CSV.[Value_Text] AS INT)
		  FROM [dbo].[CsvToTable](@DependsOn_Deployables_Csv, ',') CSV

		DECLARE @Application_Id INT 
		    SET @Application_Id = (SELECT [Application_Id] 
			                         FROM [Deployables]
			                        WHERE [Deployable_Id] = @Deployable_Id)

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_SetDeployableDependencies] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SetIssueTracker',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the issue tracking provider of a specified application.'
GO

IF OBJECT_ID('[Applications_SetIssueTracker]') IS NOT NULL
	DROP PROCEDURE [Applications_SetIssueTracker]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SetIssueTracker]
(
	@Application_Id INT,
	@IssueTracking_Provider_Id INT,
	@IssueTracking_CategoryIdList_Text VARCHAR(MAX) = NULL,
	@IssueTracking_Filter_Configuration XML = NULL,
    @IssueTracking_AllowBuildMasterIssues_Indicator YNINDICATOR = NULL
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
	   SET [IssueTracking_Provider_Id] = @IssueTracking_Provider_Id
		  ,[IssueTracking_CategoryIdList_Text] = @IssueTracking_CategoryIdList_Text
		  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		  ,[ModifiedOn_Date] = GETUTCDATE()
		  ,[IssueTracking_Filter_Configuration] = @IssueTracking_Filter_Configuration
          ,[IssueTracking_AllowBuildMasterIssues_Indicator] = COALESCE(@IssueTracking_AllowBuildMasterIssues_Indicator, [IssueTracking_AllowBuildMasterIssues_Indicator])
	 WHERE [Application_Id] = @Application_Id

	IF @@ROWCOUNT=0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_SetIssueTracker] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_ValidateHealth',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'HasDeployable_Indicator,HasActiveWorkflow_Indicator,HasRelease_Indicator,HasDeploymentPlans_Indicator',
    /* Description_Text        */ 'Indicates whether an application should redirect to the application health page.'
GO


IF OBJECT_ID('[Applications_ValidateHealth]') IS NOT NULL DROP PROCEDURE [Applications_ValidateHealth]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_ValidateHealth]
(
	@Application_Id INT,
    @HasDeployable_Indicator CHAR(1) OUT,
    @HasActiveWorkflow_Indicator CHAR(1) OUT,
    @HasRelease_Indicator CHAR(1) OUT,
    @HasDeploymentPlans_Indicator CHAR(1) OUT
)
AS
BEGIN

	SET NOCOUNT ON

    SET @HasDeployable_Indicator = 'N'
    SET @HasActiveWorkflow_Indicator = 'N'
    SET @HasRelease_Indicator = 'N'
    SET @HasDeploymentPlans_Indicator = 'N'

    IF EXISTS(SELECT * FROM [Deployables] WHERE [Application_Id] = @Application_Id)
        SET @HasDeployable_Indicator = 'Y'

    IF EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id)
        SET @HasRelease_Indicator = 'Y'

    IF EXISTS(SELECT * FROM [ApplicationWorkflows] WHERE [Application_Id] = @Application_Id)
        SET @HasActiveWorkflow_Indicator = 'Y'

    IF EXISTS(SELECT * FROM [ApplicationDeploymentPlans] WHERE [Application_Id] = @Application_Id)
        SET @HasDeploymentPlans_Indicator = 'Y'
	
END
GO

GRANT EXECUTE ON [Applications_ValidateHealth] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_CreateOrReplaceArtifact',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a reference to an artifact in the artifact library'
GO


IF OBJECT_ID('[Artifacts_CreateOrReplaceArtifact]') IS NOT NULL DROP PROCEDURE [Artifacts_CreateOrReplaceArtifact]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_CreateOrReplaceArtifact]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Deployable_Id INT,
	@Artifact_Name VARCHAR(50),

	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [BuildArtifacts]
	 WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number
       AND (
             (@Deployable_Id IS NULL AND [Deployable_Id] IS NULL)
             OR
             ([Deployable_Id] = @Deployable_Id)
           )
	   AND [Artifact_Name] = @Artifact_Name

	INSERT INTO [BuildArtifacts]
		([Application_Id], [Release_Number], [Build_Number], [Deployable_Id], [Artifact_Name], [Execution_Id], [Created_Date])
	VALUES
		(@Application_Id, @Release_Number, @Build_Number, @Deployable_Id, @Artifact_Name, @Execution_Id, GETUTCDATE())

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Artifacts_CreateOrReplaceArtifact] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_GetArtifact',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an artifact based on the specified criteria.'
GO


IF OBJECT_ID('[Artifacts_GetArtifact]') IS NOT NULL DROP PROCEDURE [Artifacts_GetArtifact]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_GetArtifact]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Artifact_Name VARCHAR(50)
)
AS
BEGIN

	SELECT *
	  FROM [BuildArtifacts_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number
       AND [Artifact_Name] = @Artifact_Name

END
GO

GRANT EXECUTE ON [Artifacts_GetArtifact] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_GetArtifacts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of artifacts based on the specified criteria.'
GO


IF OBJECT_ID('[Artifacts_GetArtifacts]') IS NOT NULL DROP PROCEDURE [Artifacts_GetArtifacts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_GetArtifacts]
(
	@Application_Id INT = NULL,
	@Release_Number RELEASENUMBER = NULL,
	@Build_Number BUILDNUMBER = NULL,
	@Deployable_Id INT = NULL,
    @Artifact_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

    IF (@Artifact_Count IS NOT NULL)
        SET ROWCOUNT @Artifact_Count

	SELECT *
	  FROM [BuildArtifacts_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@Release_Number IS NULL OR [Release_Number] = @Release_Number)
	   AND (@Build_Number IS NULL OR [Build_Number] = @Build_Number)
	   AND (@Deployable_Id IS NULL OR [Deployable_Id] IS NULL OR [Deployable_Id] = @Deployable_Id)
	 ORDER BY [Created_Date] DESC

END
GO

GRANT EXECUTE ON [Artifacts_GetArtifacts] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_GetArtifactsByBuildStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Slim',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of artifacts that are associated with a specified build status, optionally filtered by application and release.'
GO


IF OBJECT_ID('[Artifacts_GetArtifactsByBuildStatus]') IS NOT NULL DROP PROCEDURE [Artifacts_GetArtifactsByBuildStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_GetArtifactsByBuildStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@BuildStatus_Name VARCHAR(8)
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Application_Id IS NULL
	BEGIN
		SELECT *
		  FROM [BuildArtifacts_Slim]
		 WHERE [BuildStatus_Name] = @BuildStatus_Name
	END
	ELSE IF @Release_Number IS NULL
	BEGIN
		SELECT *
		  FROM [BuildArtifacts_Slim]
		 WHERE [Application_Id] = @Application_Id
		   AND [BuildStatus_Name] = @BuildStatus_Name
	END
	ELSE
	BEGIN
		SELECT *
		  FROM [BuildArtifacts_Slim]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [BuildStatus_Name] = @BuildStatus_Name
	END
END
GO

GRANT EXECUTE ON [Artifacts_GetArtifactsByBuildStatus] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildMaster_GetAlerts',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Alerts_Xml',
    /* Description_Text        */ 'Gets the alerts.'
GO

IF OBJECT_ID('[BuildMaster_GetAlerts]') IS NOT NULL DROP PROCEDURE [BuildMaster_GetAlerts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildMaster_GetAlerts]
(
	@Current_User_Name VARCHAR(255),
	@Current_User_Groups_Csv VARCHAR(MAX),
	@Alerts_Xml XML OUT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @DirectoryProvider_Id INT
    SET @DirectoryProvider_Id = (SELECT [Value_Text]
                                   FROM [Configuration]
								   WITH (READUNCOMMITTED)
                                  WHERE [Scope_Name] = 'CoreEx'
                                    AND [Key_Name] = 'DirectoryProvider')

SET @Alerts_Xml =
    (SELECT
           [@IgnoreServiceWarning] = COALESCE((SELECT [Value_Text]
                                        FROM [Configuration]
										WITH (READUNCOMMITTED)
                                       WHERE [Scope_Name] = 'Web'
                                         AND [Key_Name] = 'IgnoreServiceWarning'), 'False'),

           [@ErrorsInLog] = (CASE WHEN EXISTS(SELECT *
                                                FROM [LogMessages]
												WITH (READUNCOMMITTED)
                                               WHERE [Read_Indicator] = 'N')
                                         THEN 'True'
                                         ELSE 'False' END),

           [@ServersInErrorStatus] = (SELECT COUNT(*) 
                                        FROM [Servers] 
                                        WITH (READUNCOMMITTED)
                                       WHERE [ServerType_Code] = 'S'
                                         AND [Active_Indicator] = 'Y'
                                         AND [ServerStatus_Code] = 'ERROR'),

           [@TotalServers] = (SELECT COUNT(*) 
                                        FROM [Servers] 
                                        WITH (READUNCOMMITTED)
                                       WHERE [ServerType_Code] = 'S'
                                         AND [Active_Indicator] = 'Y'),

           [@BmUpdateReminder] = (SELECT [Value_Text]
                                    FROM [Configuration]
									WITH (READUNCOMMITTED)
                                   WHERE [Scope_Name] = 'CoreEx'
                                     AND [Key_Name] = 'BuildMasterUpdateCheck_ShowReminder'),

           [@ExtUpdateReminder] = (SELECT [Value_Text]
                                     FROM [Configuration]
									 WITH (READUNCOMMITTED)
                                    WHERE [Scope_Name] = 'CoreEx'
                                      AND [Key_Name] = 'ExtensionUpdateCheck_ShowReminder'),
       
           [@ManualAction] = (SELECT CASE WHEN COUNT(*) = 0 THEN 'False' ELSE 'True' END
                                FROM [ManualActionsInProgress]
							    WITH (READUNCOMMITTED)
                               WHERE [ActionComplete_Indicator] = 'N'
                                 AND [AssignedTo_Principal_Name] IN
                                     (SELECT [Value_Text] FROM dbo.CsvToTable(@Current_User_Groups_Csv, ',')
                                      UNION ALL
                                      SELECT @Current_User_Name)),
       
           [@CanConfigure] = CASE WHEN dbo.IsAuthorizedForTask(@DirectoryProvider_Id, @Current_User_Name, @Current_User_Groups_Csv, 62, NULL, NULL, NULL) = 0 THEN 'False' ELSE 'True' END,

           [@CanManageExtensions] = CASE WHEN dbo.IsAuthorizedForTask(@DirectoryProvider_Id, @Current_User_Name, @Current_User_Groups_Csv, 39, NULL, NULL, NULL) = 0 THEN 'False' ELSE 'True' END,

		   [ExecutionsInProgress] = (SELECT [@ExecutionId] = [Execution_Id],
		                                    [@ApplicationId] = [Application_Id]
		                               FROM [BuildExecutions]
									   WITH (READUNCOMMITTED)
									  WHERE [ExecutionStatus_Name] = 'Executing'
                                        AND dbo.IsAuthorizedForTask(@DirectoryProvider_Id, @Current_User_Name, @Current_User_Groups_Csv,58, NULL, [Application_Id], [Environment_Id]) = 1
									    FOR XML PATH('Execution'), TYPE)

   FOR XML PATH('Alerts'))

END
GO

GRANT EXECUTE ON [BuildMaster_GetAlerts] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildMaster_GetCurrentDateTime',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Current_Date',
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[BuildMaster_GetCurrentDateTime]') IS NOT NULL DROP PROCEDURE [BuildMaster_GetCurrentDateTime]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildMaster_GetCurrentDateTime]
(
	@Current_Date DATETIME OUT
)
AS
BEGIN

	SET NOCOUNT ON
    
    SET @Current_Date = GETUTCDATE()

END
GO

GRANT EXECUTE ON [BuildMaster_GetCurrentDateTime] TO [BuildMasterUser_Role]

GO
IF OBJECT_ID('[BuildMaster_GetUsageData]') IS NOT NULL DROP PROCEDURE [BuildMaster_GetUsageData]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildOutputs_AddOutput',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a build output report to a specified build execution'
GO

IF OBJECT_ID('[BuildOutputs_AddOutput]') IS NOT NULL DROP PROCEDURE [BuildOutputs_AddOutput]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildOutputs_AddOutput]
(
	@BuildExecution_PlanAction_Id INT,

	@Output_Name VARCHAR(50),
	@Output_Bytes VARBINARY(MAX),
    @OutputType_Code CHAR(1)
)
AS
BEGIN

	SET NOCOUNT ON

	BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Execution_Id INT
	SELECT @Execution_Id = BEDP.[Execution_Id]
	  FROM [BuildExecution_ActionGroupActions] BEAGA
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		           ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEAGA.[BuildExecution_ActionGroupAction_Id] = @BuildExecution_PlanAction_Id

	DECLARE @Output_Sequence INT
	SET @Output_Sequence = COALESCE((SELECT MAX([Output_Sequence])
	                                   FROM [BuildOutputs]
							          WHERE [Execution_Id] = @Execution_Id), 0) + 1

	INSERT INTO [BuildOutputs]
	(
		[Execution_Id],
		[Output_Sequence],
		[Output_Name],
	    [Output_Bytes],
		[OutputType_Code],
		[BuildExecution_PlanAction_Id]
	)
	VALUES
	(
		@Execution_Id,
		@Output_Sequence,
		@Output_Name,
		@Output_Bytes,
		@OutputType_Code,
		@BuildExecution_PlanAction_Id
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildOutputs_AddOutput] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildOutputs_GetOutput',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildOutputs_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the build output report for a specified build and sequence number'
GO


IF OBJECT_ID('[BuildOutputs_GetOutput]') IS NOT NULL DROP PROCEDURE [BuildOutputs_GetOutput]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildOutputs_GetOutput]
(
	@Execution_Id INT,
	@Output_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildOutputs_Extended]
	 WHERE [Execution_Id] = @Execution_Id
	   AND [Output_Sequence] = @Output_Sequence

END
GO

GRANT EXECUTE ON [BuildOutputs_GetOutput] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildOutputs_GetOutputs',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildOutputs_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the build output reports for a specified build'
GO


IF OBJECT_ID('[BuildOutputs_GetOutputs]') IS NOT NULL DROP PROCEDURE [BuildOutputs_GetOutputs]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildOutputs_GetOutputs]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Application_Id INT
	DECLARE @Release_Number RELEASENUMBER
	DECLARE @Build_Number BUILDNUMBER
	
	SELECT @Application_Id = [Application_Id],
	       @Release_Number = [Release_Number],
	       @Build_Number = [Build_Number]
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] = @Execution_Id
	 

	SELECT *
	  FROM [BuildOutputs_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
  ORDER BY [Output_Sequence]

END
GO

GRANT EXECUTE ON [BuildOutputs_GetOutputs] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateRecurring',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a recurring build based on the specified parameters, or updates an existing schedule if one with the given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateRecurring]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateRecurring]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateRecurring]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
	@StartTime_Minutes INT,
	@EndTime_Minutes SMALLINT,
	@DailyInterval_Minutes SMALLINT,
	@Recurrence_Type VARCHAR(10),
	@RecurrenceFrequency_Value INT,
	@RecurrenceWeeklyDay_Mask CHAR(7),
	@RecurrenceMonthlyDay_Value TINYINT,
	@RecurrenceMonthlyRelativity_Type VARCHAR(4),
    @RecurrenceMonthlyRelativityDay_Type VARCHAR(5),
    @RecurrenceMonthlyRelativityWeek_Type VARCHAR(7),
    @RecurrenceMonthlyRelativityWeekDay_Value TINYINT,
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@Releasefilter_Workflow_Id INT,
    @BuildImporter_Configuration XML,
    @Active_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[StartTime_Minutes]
			,[EndTime_Minutes]
			,[DailyInterval_Minutes]
			,[Recurrence_Type]
			,[RecurrenceFrequency_Value]
			,[RecurrenceWeeklyDay_Mask]
			,[RecurrenceMonthlyDay_Value]
			,[RecurrenceMonthlyRelativity_Type]
			,[RecurrenceMonthlyRelativityDay_Type]
			,[RecurrenceMonthlyRelativityWeek_Type]
			,[RecurrenceMonthlyRelativityWeekDay_Value]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[ReleaseFilter_ReleaseType_Code]
            ,[Releasefilter_Workflow_Id]
            ,[BuildImporter_Configuration])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,'Recurring'
			,@StartTime_Minutes
			,@EndTime_Minutes
			,@DailyInterval_Minutes
			,@Recurrence_Type
			,@RecurrenceFrequency_Value
			,@RecurrenceWeeklyDay_Mask
			,@RecurrenceMonthlyDay_Value
			,@RecurrenceMonthlyRelativity_Type
			,@RecurrenceMonthlyRelativityDay_Type
			,@RecurrenceMonthlyRelativityWeek_Type
			,@RecurrenceMonthlyRelativityWeekDay_Value
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@ReleaseFilter_ReleaseType_Code
			,@Releasefilter_Workflow_Id
            ,@BuildImporter_Configuration)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		
		DECLARE @Schedule_Type VARCHAR(15)
		SELECT @Schedule_Type = [Schedule_Type] 
		  FROM [BuildSchedules]
		 WHERE [Schedule_Id] = @Schedule_Id
		
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id) 
		IF @Schedule_Type <> 'Recurring' RAISERROR('Existing BuildSchedule #%i is not a Recurring schedule, it is %s.', 16, 1, @Schedule_Id, @Schedule_Type)
	
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Application_Id] = @Application_Id
			  ,[Active_Indicator] = @Active_Indicator
			  ,[StartTime_Minutes] = @StartTime_Minutes
			  ,[EndTime_Minutes] = @EndTime_Minutes
			  ,[DailyInterval_Minutes] = @DailyInterval_Minutes
			  ,[Recurrence_Type] = @Recurrence_Type
			  ,[RecurrenceFrequency_Value] = @RecurrenceFrequency_Value
			  ,[RecurrenceWeeklyDay_Mask] = @RecurrenceWeeklyDay_Mask
			  ,[RecurrenceMonthlyDay_Value] = @RecurrenceMonthlyDay_Value
			  ,[RecurrenceMonthlyRelativity_Type] = @RecurrenceMonthlyRelativity_Type
			  ,[RecurrenceMonthlyRelativityDay_Type] = @RecurrenceMonthlyRelativityDay_Type
			  ,[RecurrenceMonthlyRelativityWeek_Type] = @RecurrenceMonthlyRelativityWeek_Type
			  ,[RecurrenceMonthlyRelativityWeekDay_Value] = @RecurrenceMonthlyRelativityWeekDay_Value
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[Releasefilter_Workflow_Id] = @Releasefilter_Workflow_Id
              ,[BuildImporter_Configuration] = @BuildImporter_Configuration
		 WHERE [Schedule_Id] = @Schedule_Id

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateRecurring] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateSchedule',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a scheduled build based on the specified parameters, or updates an existing scheduled build if one with the given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateSchedule]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateSchedule]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateSchedule]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
    @Active_Indicator YNINDICATOR,
    @Schedule_Type VARCHAR(15),
	@StartTime_Minutes INT,
	@EndTime_Minutes SMALLINT,
	@DailyInterval_Minutes SMALLINT,
	@Recurrence_Type VARCHAR(10),
	@RecurrenceFrequency_Value INT,
	@RecurrenceWeeklyDay_Mask CHAR(7),
	@RecurrenceMonthlyDay_Value TINYINT,
	@RecurrenceMonthlyRelativity_Type VARCHAR(4),
    @RecurrenceMonthlyRelativityDay_Type VARCHAR(5),
    @RecurrenceMonthlyRelativityWeek_Type VARCHAR(7),
    @RecurrenceMonthlyRelativityWeekDay_Value TINYINT,
	@Url_AllowedIP_CSV varchar(MAX),
	@Url_Username_Text varchar(50),
	@Url_Password_Text varchar(50),
	@SourceControl_Path_Name varchar(250),
	@SourceControl_Provider_Id int,
	@SourceControl_QuietPeriod_Minutes INT,
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@Releasefilter_Workflow_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[StartTime_Minutes]
			,[EndTime_Minutes]
			,[DailyInterval_Minutes]
			,[Recurrence_Type]
			,[RecurrenceFrequency_Value]
			,[RecurrenceWeeklyDay_Mask]
			,[RecurrenceMonthlyDay_Value]
			,[RecurrenceMonthlyRelativity_Type]
			,[RecurrenceMonthlyRelativityDay_Type]
			,[RecurrenceMonthlyRelativityWeek_Type]
			,[RecurrenceMonthlyRelativityWeekDay_Value]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[Url_AllowedIP_CSV]
			,[Url_Username_Text]
			,[Url_Password_Text]
			,[SourceControl_Path_Name]
			,[SourceControl_Provider_Id]
			,[SourceControl_QuietPeriod_Minutes]
			,[ReleaseFilter_ReleaseType_Code]
			,[Releasefilter_Workflow_Id])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,@Schedule_Type
			,@StartTime_Minutes
			,@EndTime_Minutes
			,@DailyInterval_Minutes
			,@Recurrence_Type
			,@RecurrenceFrequency_Value
			,@RecurrenceWeeklyDay_Mask
			,@RecurrenceMonthlyDay_Value
			,@RecurrenceMonthlyRelativity_Type
			,@RecurrenceMonthlyRelativityDay_Type
			,@RecurrenceMonthlyRelativityWeek_Type
			,@RecurrenceMonthlyRelativityWeekDay_Value
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@Url_AllowedIP_CSV
			,@Url_Username_Text
			,@Url_Password_Text
			,@SourceControl_Path_Name
			,@SourceControl_Provider_Id
			,@SourceControl_QuietPeriod_Minutes
			,@ReleaseFilter_ReleaseType_Code
			,@Releasefilter_Workflow_Id)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Application_Id] = @Application_Id
			  ,[Active_Indicator] = @Active_Indicator
			  ,[Schedule_Type] = @Schedule_Type
			  ,[StartTime_Minutes] = @StartTime_Minutes
			  ,[EndTime_Minutes] = @EndTime_Minutes
			  ,[DailyInterval_Minutes] = @DailyInterval_Minutes
			  ,[Recurrence_Type] = @Recurrence_Type
			  ,[RecurrenceFrequency_Value] = @RecurrenceFrequency_Value
			  ,[RecurrenceWeeklyDay_Mask] = @RecurrenceWeeklyDay_Mask
			  ,[RecurrenceMonthlyDay_Value] = @RecurrenceMonthlyDay_Value
			  ,[RecurrenceMonthlyRelativity_Type] = @RecurrenceMonthlyRelativity_Type
			  ,[RecurrenceMonthlyRelativityDay_Type] = @RecurrenceMonthlyRelativityDay_Type
			  ,[RecurrenceMonthlyRelativityWeek_Type] = @RecurrenceMonthlyRelativityWeek_Type
			  ,[RecurrenceMonthlyRelativityWeekDay_Value] = @RecurrenceMonthlyRelativityWeekDay_Value
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[Url_AllowedIP_CSV] = @Url_AllowedIP_CSV
			  ,[Url_Username_Text] = @Url_Username_Text
			  ,[Url_Password_Text] = @Url_Password_Text
			  ,[SourceControl_Path_Name] = @SourceControl_Path_Name
			  ,[SourceControl_Provider_Id] = @SourceControl_Provider_Id
			  ,[SourceControl_QuietPeriod_Minutes] = @SourceControl_QuietPeriod_Minutes
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[Releasefilter_Workflow_Id] = @Releasefilter_Workflow_Id
		 WHERE [Schedule_Id] = @Schedule_Id
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id)

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateSchedule] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateSCMTriggered',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a source control triggered automated build based on the specified parameters, or updates an existing one if one with a given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateSCMTriggered]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateSCMTriggered]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateSCMTriggered]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
	@SourceControl_Path_Name VARCHAR(250),
	@SourceControl_Provider_Id INT,
	@SourceControl_QuietPeriod_Minutes INT,
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@Releasefilter_Workflow_Id INT,
	@Active_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[SourceControl_Path_Name]
			,[SourceControl_Provider_Id]
			,[SourceControl_QuietPeriod_Minutes]
			,[ReleaseFilter_ReleaseType_Code]
			,[Releasefilter_Workflow_Id])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,'SCM Triggered'
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@SourceControl_Path_Name
			,@SourceControl_Provider_Id
			,@SourceControl_QuietPeriod_Minutes
			,@ReleaseFilter_ReleaseType_Code
			,@Releasefilter_Workflow_Id)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		
		DECLARE @Schedule_Type VARCHAR(15)
		SELECT @Schedule_Type = [Schedule_Type] 
		  FROM [BuildSchedules]
		 WHERE [Schedule_Id] = @Schedule_Id
		
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id) 
		IF @Schedule_Type <> 'SCM Triggered' RAISERROR('Existing BuildSchedule #%i is not an SCM Triggered schedule, it is %s.', 16, 1, @Schedule_Id, @Schedule_Type)
	
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Active_Indicator] = @Active_Indicator
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[SourceControl_Path_Name] = @SourceControl_Path_Name
			  ,[SourceControl_Provider_Id] = @SourceControl_Provider_Id
			  ,[SourceControl_QuietPeriod_Minutes] = @SourceControl_QuietPeriod_Minutes
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[Releasefilter_Workflow_Id] = @Releasefilter_Workflow_Id
		 WHERE [Schedule_Id] = @Schedule_Id
		

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateSCMTriggered] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateURLTriggered',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a URL-triggered build based on the specified parameters, or updates an existing schedule if one with the given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateURLTriggered]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateURLTriggered]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateURLTriggered]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
	@Url_AllowedIP_CSV VARCHAR(MAX),
	@Url_Username_Text VARCHAR(50),
	@Url_Password_Text VARCHAR(50),
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@Releasefilter_Workflow_Id INT,
    @BuildImporter_Configuration XML,
	@Active_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[Url_AllowedIP_CSV]
			,[Url_Username_Text]
			,[Url_Password_Text]
			,[ReleaseFilter_ReleaseType_Code]
			,[Releasefilter_Workflow_Id]
            ,[BuildImporter_Configuration])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,'URL Triggered'
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@Url_AllowedIP_CSV
			,@Url_Username_Text
			,@Url_Password_Text
			,@ReleaseFilter_ReleaseType_Code
			,@Releasefilter_Workflow_Id
            ,@BuildImporter_Configuration)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		
		DECLARE @Schedule_Type VARCHAR(15)
		SELECT @Schedule_Type = [Schedule_Type] 
		  FROM [BuildSchedules]
		 WHERE [Schedule_Id] = @Schedule_Id
		
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id) 
		IF @Schedule_Type <> 'URL Triggered' RAISERROR('Existing BuildSchedule #%i is not a URL Triggered schedule, it is %s.', 16, 1, @Schedule_Id, @Schedule_Type)
	
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Application_Id] = @Application_Id
			  ,[Active_Indicator] = @Active_Indicator
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[Url_AllowedIP_CSV] = @Url_AllowedIP_CSV
			  ,[Url_Username_Text] = @Url_Username_Text
			  ,[Url_Password_Text] = @Url_Password_Text
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[Releasefilter_Workflow_Id] = @Releasefilter_Workflow_Id
              ,[BuildImporter_Configuration] = @BuildImporter_Configuration
		 WHERE [Schedule_Id] = @Schedule_Id

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateURLTriggered] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_DeleteBuildSchedule',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified build schedule'
GO

IF OBJECT_ID('[dbo].[BuildSchedules_DeleteBuildSchedule]') IS NOT NULL
	DROP PROCEDURE [dbo].[BuildSchedules_DeleteBuildSchedule]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[BuildSchedules_DeleteBuildSchedule]
(
	@Schedule_Id INT
) AS
BEGIN


SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT, @Schedule_Name SQL_VARIANT
	 SELECT @Application_Id = [Application_Id], @Schedule_Name = [Schedule_Name]
	   FROM [BuildSchedules]
	  WHERE [Schedule_Id] = @Schedule_Id

	DELETE
	  FROM [BuildScheduleVariableDefinitions]
	 WHERE [Schedule_Id] = @Schedule_Id

	DELETE
	  FROM [BuildSchedules]
	 WHERE [Schedule_Id] = @Schedule_Id

	IF @@ROWCOUNT <> 0 EXEC [Events_RaiseEvent] 'BSHDEL', @Application_Id, @Schedule_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[BuildSchedules_DeleteBuildSchedule] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_GetBuildSchedule',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'BuildSchedules_Extended,BuildScheduleVariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified build schedule'
GO

IF OBJECT_ID('[dbo].[BuildSchedules_GetBuildSchedule]') IS NOT NULL
	DROP PROCEDURE [dbo].[BuildSchedules_GetBuildSchedule]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[BuildSchedules_GetBuildSchedule]
(
	@Schedule_Id INT
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildSchedules_Extended]
	 WHERE [Schedule_Id] = @Schedule_Id
	IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id)

    SELECT *
      FROM [BuildScheduleVariableDefinitions_Extended]
     WHERE [Schedule_Id] = @Schedule_Id

END
GO

GRANT EXECUTE ON [dbo].[BuildSchedules_GetBuildSchedule] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_GetBuildSchedules',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildSchedules_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of builds schedules for the specified application and active indicator, or all if null is supplied'
GO

IF OBJECT_ID('[BuildSchedules_GetBuildSchedules]') IS NOT NULL
	DROP PROCEDURE [BuildSchedules_GetBuildSchedules]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_GetBuildSchedules]
(
	@Application_Id INT = NULL,
	@Active_Indicator YNINDICATOR = NULL
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildSchedules_Extended]
	 WHERE ( @Application_Id IS NULL OR [Application_Id] = @Application_Id )
	   AND [Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator])

END
GO

GRANT EXECUTE ON [BuildSchedules_GetBuildSchedules] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_GetScheduled',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildSchedules_Scheduled',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of build schedules that are scheduled to be executed between the specified dates'
GO

IF OBJECT_ID('BuildSchedules_GetScheduled') IS NOT NULL DROP PROCEDURE BuildSchedules_GetScheduled
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_GetScheduled]
(
	@From_Date DATETIME,
	@To_Date DATETIME 
) AS BEGIN

	SET NOCOUNT ON

	DECLARE @fDat DATETIME, @fMin INT, @tDat DATETIME, @tMin INT

	SET @fDat = DATEADD(dd, DATEDIFF(dd,0,@From_Date), 0)
	SET @fMin = DATEPART(MINUTE, @From_Date)
	          + DATEPART(HOUR, @From_Date) * 60
	SET @tDat = DATEADD(dd, DATEDIFF(dd,0,@To_Date), 0)
	SET @tMin = DATEPART(MINUTE, @To_Date)
	          + DATEPART(HOUR, @To_Date) * 60

	SELECT * FROM [BuildSchedules_Scheduled]
	 WHERE [RunAt_Date] BETWEEN @fDat AND @tDat
	   AND [RunAt_Minutes] BETWEEN @fMin AND @tMin
	    OR ([RunAt_Date] IS NULL AND [RunAt_Minutes] IS NULL)
      

END
GO

GRANT EXECUTE ON [BuildSchedules_GetScheduled] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_SetVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates a variable definition to use with a scheduled build',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_SetVariableDefinition]') IS NOT NULL DROP PROCEDURE [BuildSchedules_SetVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_SetVariableDefinition]
(
	@Schedule_Id INT,
	@VariableDeclaration_Id INT,
	@Value_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	IF @Value_Text IS NULL OR @Value_Text = ''
	BEGIN

		DELETE [BuildScheduleVariableDefinitions]
		 WHERE [Schedule_Id] = @Schedule_Id
		   AND [VariableDeclaration_Id] = @VariableDeclaration_Id

	END
	ELSE IF EXISTS(SELECT *
		            FROM [BuildScheduleVariableDefinitions]
				   WHERE [Schedule_Id] = @Schedule_Id
				     AND [VariableDeclaration_Id] = @VariableDeclaration_Id)
	BEGIN
		
		UPDATE [BuildScheduleVariableDefinitions]
		   SET [Value_Text] = @Value_Text
		 WHERE [Schedule_Id] = @Schedule_Id
		   AND [VariableDeclaration_Id] = @VariableDeclaration_Id
          
	END
	ELSE
	BEGIN

		INSERT INTO [BuildScheduleVariableDefinitions]
		([Schedule_Id], [VariableDeclaration_Id], [Value_Text])
		VALUES
		(@Schedule_Id, @VariableDeclaration_Id, @Value_Text)

	END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_SetVariableDefinition] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_AddApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'This procedure is deprecated and will be removed in a future version of BuildMaster.'
GO

IF OBJECT_ID('[Builds_AddApproval]') IS NOT NULL DROP PROCEDURE [Builds_AddApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_AddApproval]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@Approval_Sequence INT,
	@User_Name USERNAME,
	@GroupNames_List VARCHAR(MAX),
	@Comments_Text VARCHAR(255)
)
AS
BEGIN

SET NOCOUNT ON

	DECLARE @RequirementIds TABLE ([Requirement_Id] INT NOT NULL)

	INSERT INTO @RequirementIds
	SELECT BPR.[Requirement_Id]
	  FROM [BuildPromotionRequirements] BPR
	       INNER JOIN [ApplicationWorkflows] AW
		           ON AW.[Workflow_Id] = BPR.[Workflow_Id]
	 WHERE AW.[Application_Id] = @Application_Id
	   AND BPR.[Environment_Id] = @Environment_Id
	   AND BPR.[Principal_Name] = @User_Name

	DECLARE @Requirement_Id INT

	WHILE EXISTS(SELECT * FROM @RequirementIds) BEGIN

		SELECT TOP 1 @Requirement_Id = [Requirement_Id]
		  FROM @RequirementIds

		EXEC [Promotions_AddApproval] @Application_Id, @Release_Number, @Build_Number, @Environment_Id, @Requirement_Id, @User_Name, @Comments_Text

		DELETE @RequirementIds
		 WHERE [Requirement_Id] = @Requirement_Id

	END

END
GO

GRANT EXECUTE ON [Builds_AddApproval] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_DeployBuild',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ '',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks a build and its associated release as deployed based on the settings in its workflow.'
GO

IF OBJECT_ID('[Builds_DeployBuild]') IS NOT NULL DROP PROCEDURE [Builds_DeployBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_DeployBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER,
    @Workflow_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    BEGIN TRY
	BEGIN TRANSACTION

        DECLARE @AutoIncrementRelease_Indicator YNINDICATOR
        DECLARE @AutoCancelReleases_Indicator YNINDICATOR

        IF (SELECT [ReleaseStatus_Name] FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number) <> 'Active'
			RAISERROR('Can only deploy an active release.', 16, 1)

		IF (SELECT [BuildStatus_Name] FROM [Builds] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number AND [Build_Number] = @Build_Number) <> 'Active'
			RAISERROR('Can only deploy an active build.', 16, 1)

        SELECT @AutoIncrementRelease_Indicator = [AutoIncrementRelease_Indicator],
               @AutoCancelReleases_Indicator = [AutoCancelReleases_Indicator]
          FROM [Workflows]
         WHERE [Workflow_Id] = @Workflow_Id

        IF @AutoIncrementRelease_Indicator IS NULL
			RAISERROR('Invalid workflow.', 16, 1)

	    UPDATE [Releases]
		   SET [ReleaseStatus_Name] = 'Deployed'
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number

        DECLARE @Release_Sequence BIGINT

        SELECT @Release_Sequence = [Release_Sequence]
          FROM [Releases]
         WHERE [Application_Id] = @Application_Id 
           AND [Release_Number] = @Release_Number

        IF (@AutoCancelReleases_Indicator = 'Y')
        BEGIN

            /* Cancel existing releases that are older */
		    DECLARE @CNCL_Release_Number RELEASENUMBER
		    DECLARE CNCL_Release_Cursor CURSOR FOR
		    SELECT [Release_Number]
			    FROM [Releases]
			    WHERE [Application_Id] = @Application_Id
			    AND [ReleaseStatus_Name] = 'Active'
			    AND [Release_Sequence] < @Release_Sequence

		    OPEN CNCL_Release_Cursor
		    FETCH NEXT FROM CNCL_Release_Cursor INTO @CNCL_Release_Number
		    WHILE @@FETCH_STATUS <> -1
		    BEGIN
			    DECLARE @CancelledReason_Text VARCHAR(100) SET @CancelledReason_Text = 'Superseded by ' + @Release_Number
			    EXEC [Releases_CancelRelease]
				    @Application_Id = @Application_Id,
				    @Release_Number = @CNCL_Release_Number,
				    @CancelledReason_Text = @CancelledReason_Text

			    FETCH NEXT FROM CNCL_Release_Cursor INTO @CNCL_Release_Number
		    END

		    CLOSE CNCL_Release_Cursor
		    DEALLOCATE CNCL_Release_Cursor

        END

		UPDATE [Builds]
		   SET [BuildStatus_Name] = 'Deployed'
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number

		UPDATE [Builds]
		   SET [BuildStatus_Name] = 'Rejected',
			   [RejectedBy_User_Name] = dbo.Session_GetUserName(),
			   [Rejected_Date] = GETUTCDATE()
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [BuildStatus_Name] = 'Active'

		INSERT INTO [ReleaseConfigurationFiles]
		(
			[Application_Id],
			[Release_Number],
			[ConfigurationFile_Id],
			[Version_Number]
		)
		SELECT @Application_Id,
		       @Release_Number,
			   CF.[ConfigurationFile_Id],
			   CFV_Latest.[Version_Number]
		  FROM [ConfigurationFiles] CF
		       INNER JOIN [Deployables] D
			           ON D.[Deployable_Id] = CF.[Deployable_Id]
		       INNER JOIN (SELECT [ConfigurationFile_Id],
			                      [Version_Number] = MAX([Version_Number])
			                 FROM [ConfigurationFileVersions]
							GROUP BY [ConfigurationFile_Id]) CFV_Latest
					   ON CFV_Latest.[ConfigurationFile_Id] = CF.[ConfigurationFile_Id]
		 WHERE D.[Application_Id] = @Application_Id
		   AND CF.[ConfigurationFile_Id] NOT IN (SELECT [ConfigurationFile_Id]
		                                           FROM [ReleaseConfigurationFiles] RCF
												  WHERE RCF.[Application_Id] = @Application_Id
												    AND RCF.[Release_Number] = @Release_Number)
		
		/* If there are no other active releases, check for autoincrement setting */
		IF NOT EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [ReleaseStatus_Name] = 'Active')
		BEGIN
			
		    /* Check if auto-increment is enabled */
		    IF @AutoIncrementRelease_Indicator = 'Y'
		    BEGIN
				    
                DECLARE @New_Release_Number RELEASENUMBER
			    SET @New_Release_Number = dbo.GetNextReleaseNumber(@Application_Id, @Release_Number)
				
			    /* If the autoincrement was successful */
			    IF NOT @New_Release_Number IS NULL
			    BEGIN
				    /* Make sure there is no identical existing release number */
				    IF NOT EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @New_Release_Number)
				    BEGIN
					    DECLARE @ReleaseDeployables_Xml XML
					    SET @ReleaseDeployables_Xml =
						    (SELECT [Deployable_Id] '@Deployable_Id',
								    [InclusionType_Code] '@InclusionType_Code',
								    [Referenced_Release_Number] '@Referenced_Release_Number',
								    [Referenced_Application_Id] '@Referenced_Application_Id'
							    FROM [ReleaseDeployables]
							    WHERE [Application_Id] = @Application_Id
							    AND [Release_Number] = @Release_Number
							    FOR XML PATH('ReleaseDeployable'), ROOT('ReleaseDeployables'))

					    /* Create the release with the same deployables */
					    EXEC [Releases_CreateOrUpdateRelease]
							    @Application_Id,
							    @New_Release_Number,
							    @Workflow_Id,
							    NULL,
							    NULL,
							    NULL,
							    @ReleaseDeployables_Xml
					           
				    END
			    END
		    END
		END


	    EXEC [Events_RaiseEvent] 'RELDEP',@Application_Id,@Release_Number

	COMMIT
    END TRY BEGIN CATCH
	    IF XACT_STATE()<>0 ROLLBACK
	    EXEC [HandleError]
    END CATCH

END
GO

GRANT EXECUTE ON [Builds_DeployBuild] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Builds_GetAllPromotionRequiredApprovalsByUser]') IS NOT NULL DROP PROCEDURE [Builds_GetAllPromotionRequiredApprovalsByUser]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for a specified build'
GO


IF OBJECT_ID('[Builds_GetBuild]') IS NOT NULL DROP PROCEDURE [Builds_GetBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	IF (@@ROWCOUNT = 0) RAISERROR('{App#%i} %s Build %s does not exist.', 16, 1, @Application_Id, @Release_Number, @Build_Number)

END
GO

GRANT EXECUTE ON [Builds_GetBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetBuilds',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for all builds that match the specified criteria'
GO


IF OBJECT_ID('[Builds_GetBuilds]') IS NOT NULL DROP PROCEDURE [Builds_GetBuilds]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetBuilds]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER = NULL,
	@BuildStatus_Name VARCHAR(8) = NULL,
	@Build_Count INT = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	--Use RowCount to limit results 
	IF (@Build_Count IS NOT NULL AND @Build_Count > 0)
	BEGIN
		SET ROWCOUNT @Build_Count
	END

	SELECT *
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
	   AND [BuildStatus_Name] = COALESCE(@BuildStatus_Name, [BuildStatus_Name])
	  ORDER BY [Numeric_Release_Number] DESC, [Build_Sequence] DESC, [CreatedOn_Date] DESC

END
GO

GRANT EXECUTE ON [Builds_GetBuilds] TO [BuildMasterUser_Role]

GO

 EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetDependentPromotions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all promotions that are contingent upon the specified promotion.'
GO

IF OBJECT_ID('[Builds_GetDependentPromotions]') IS NOT NULL DROP PROCEDURE [Builds_GetDependentPromotions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetDependentPromotions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

 
	SELECT BP.* 
	  FROM [BuildPromotionContingencies] BPC
		   INNER JOIN [BuildPromotions_Extended] BP
				   ON BPC.[Application_Id] = BP.[Application_Id]
				  AND BPC.[Release_Number] = BP.[Release_Number]
				  AND BPC.[Build_Number] = BP.[Build_Number]
				  AND BPC.[Environment_Id] = BP.[Environment_Id]
	 WHERE BPC.[Contingent_Application_Id] = @Application_Id
	   AND BPC.[Contingent_Release_Number] = @Release_Number
	   AND BPC.[Contingent_Build_Number] = @Build_Number
	   AND BPC.[Contingent_Environment_Id] = @Environment_Id
       AND BP.[BuildStatus_Name] = 'Active'
 

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_GetDependentPromotions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecution',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all details regarding a specific build execution'
GO

IF OBJECT_ID('[Builds_GetExecution]') IS NOT NULL DROP PROCEDURE [Builds_GetExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecution]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id


END
GO

GRANT EXECUTE ON [Builds_GetExecution] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Builds_GetExecutionActionsInProgress]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionActionsInProgress]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionExtended',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended,BuildExecution_DeploymentPlans,BuildExecution_DeploymentPlanActionGroups,BuildExecution_ActionGroupActions,BuildExecution_ActionGroupActionLogEntries',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all details regarding a specific build execution.'
GO

IF OBJECT_ID('[Builds_GetExecutionExtended]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionExtended]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionExtended]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id

	SELECT *
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [Execution_Id] = @Execution_Id

	SELECT BEDPAG.*
	  FROM [BuildExecution_DeploymentPlanActionGroups] BEDPAG
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
	               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEDPAG.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEDP.[Execution_Id] = @Execution_Id
	 
	SELECT BEAGA.*
	  FROM [BuildExecution_ActionGroupActions] BEAGA
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
	               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEDP.[Execution_Id] = @Execution_Id

	SELECT BEAGALE.*
	  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
	       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		           ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
		   INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		           ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
	 WHERE BEDP.[Execution_Id] = @Execution_Id
	   AND BEAGA.[Original_Action_Sequence] = 0
	   AND BEAGA.[Original_DeploymentPlanActionGroup_Sequence] = 0
	   AND BEDP.[Original_DeploymentPlan_Id] = 0
	 ORDER BY BEAGALE.[LogEntry_Sequence]

END
GO

GRANT EXECUTE ON [Builds_GetExecutionExtended] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionLog',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended,BuildExecution_DeploymentPlanActionGroups,BuildExecution_ActionGroupActions,BuildExecution_ActionGroupActionLogEntries,BuildExecution_ActionGroupActionVariableValues',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for a specified build execution, and plan or action if specified.'
GO

IF OBJECT_ID('[Builds_GetExecutionLog]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionLog]
(
	@Execution_Id INT = NULL,
    @BuildExecution_DeploymentPlan_Id INT = NULL,
    @Original_DeploymentPlanActionGroup_Sequence INT = NULL,
	@BuildExecution_ActionGroupAction_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Execution_Id IS NOT NULL
	BEGIN

		-- Executions_Extended
		SELECT *
		  FROM [BuildExecutions_Extended]
		 WHERE [Execution_Id] = @Execution_Id

		-- BuildExecution_DeploymentPlanActionGroups
		SELECT BEDPAG.*
		  FROM [BuildExecution_DeploymentPlanActionGroups] BEDPAG
		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEDPAG.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEDP.[Execution_Id] = @Execution_Id

		-- BuildExecution_ActionGroupActions
		SELECT BEAGA.*
		  FROM [BuildExecution_ActionGroupActions] BEAGA
  		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEDP.[Execution_Id] = @Execution_Id

		-- BuildExecution_ActionGroupActionLogEntries
		SELECT BEAGALE.*
		  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
		       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
			           ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
  		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEDP.[Execution_Id] = @Execution_Id

		-- BuildExecution_ActionGroupActionVariableValues
		SELECT BEAGAVV.*
		  FROM [BuildExecution_ActionGroupActionVariableValues] BEAGAVV
			   INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		               ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGAVV.[BuildExecution_ActionGroupAction_Id]
  		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEDP.[Execution_Id] = @Execution_Id

	END
	ELSE IF @BuildExecution_DeploymentPlan_Id IS NOT NULL
	BEGIN

		-- Executions_Extended
		SELECT BEX.*
		  FROM [BuildExecution_DeploymentPlans] BEDP
		       INNER JOIN [BuildExecutions_Extended] BEX
		               ON BEX.[Execution_Id] = BEDP.[Execution_Id]
		 WHERE BEDP.[BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id

		-- BuildExecution_DeploymentPlanActionGroups
		SELECT *
		  FROM [BuildExecution_DeploymentPlanActionGroups]
		 WHERE [BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		   AND (@Original_DeploymentPlanActionGroup_Sequence IS NULL OR [Original_DeploymentPlanActionGroup_Sequence] = @Original_DeploymentPlanActionGroup_Sequence)

		-- BuildExecution_ActionGroupActions
		SELECT *
		  FROM [BuildExecution_ActionGroupActions]
		 WHERE [BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
	       AND (@Original_DeploymentPlanActionGroup_Sequence IS NULL OR [Original_DeploymentPlanActionGroup_Sequence] = @Original_DeploymentPlanActionGroup_Sequence)

		-- BuildExecution_ActionGroupActionLogEntries
		SELECT BEAGALE.*
		  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
		       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
			           ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
		 WHERE BEAGA.[BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		   AND (@Original_DeploymentPlanActionGroup_Sequence IS NULL OR [Original_DeploymentPlanActionGroup_Sequence] = @Original_DeploymentPlanActionGroup_Sequence)

		-- BuildExecution_ActionGroupActionVariableValues
		SELECT BEAGAVV.*
		  FROM [BuildExecution_ActionGroupActionVariableValues] BEAGAVV
			   INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		               ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGAVV.[BuildExecution_ActionGroupAction_Id]
		 WHERE BEAGA.[BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		   AND (@Original_DeploymentPlanActionGroup_Sequence IS NULL OR [Original_DeploymentPlanActionGroup_Sequence] = @Original_DeploymentPlanActionGroup_Sequence)

	END
	ELSE IF @BuildExecution_ActionGroupAction_Id IS NOT NULL
	BEGIN

		-- Executions_Extended
		SELECT BEX.*
		  FROM [BuildExecution_ActionGroupActions] BEAGA
		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		       INNER JOIN [BuildExecutions_Extended] BEX
		               ON BEX.[Execution_Id] = BEDP.[Execution_Id]
		 WHERE BEAGA.[BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

		-- BuildExecution_DeploymentPlanActionGroups
		SELECT BEDPAG.*
		  FROM [BuildExecution_ActionGroupActions] BEAGA
		       INNER JOIN [BuildExecution_DeploymentPlanActionGroups] BEDPAG
		               ON BEDPAG.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		              AND BEDPAG.[Original_DeploymentPlanActionGroup_Sequence] = BEAGA.[Original_DeploymentPlanActionGroup_Sequence]
		 WHERE BEAGA.[BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

		-- BuildExecution_ActionGroupActions
		SELECT *
		  FROM [BuildExecution_ActionGroupActions]
		 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

		-- BuildExecution_ActionGroupActionLogEntries
		SELECT *
		  FROM [BuildExecution_ActionGroupActionLogEntries]
		 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

		-- BuildExecution_ActionGroupActionVariableValues
		SELECT *
		  FROM [BuildExecution_ActionGroupActionVariableValues]
		 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

	END

END
GO

GRANT EXECUTE ON [Builds_GetExecutionLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionLogForActions',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecution_ActionGroupActionLogEntries',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the log entries for the actions specified.'
GO

IF OBJECT_ID('[Builds_GetExecutionLogForActions]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionLogForActions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionLogForActions]
(
	@BuildExecution_ActionGroupActions_Csv VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT BEAGALE.*
	  FROM dbo.CsvToTable(@BuildExecution_ActionGroupActions_Csv, ',') T
	       INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
	               ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = T.[Value_Text]

END
GO

GRANT EXECUTE ON [Builds_GetExecutionLogForActions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the build executions for the specified build'
GO


IF OBJECT_ID('[Builds_GetExecutions]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT = NULL,
	@Execution_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF NULLIF(@Execution_Count, 0) IS NOT NULL
		SET ROWCOUNT @Execution_Count

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND (@Release_Number IS NULL OR @Release_Number = [Release_Number])
	   AND (@Build_Number IS NULL OR @Build_Number = [Build_Number])
	   AND (@Environment_Id IS NULL OR @Environment_Id = [Environment_Id])
	 ORDER BY [ExecutionStart_Date] DESC

END
GO

GRANT EXECUTE ON [Builds_GetExecutions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionsByServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets recent build executions for the specified server.'
GO


IF OBJECT_ID('[Builds_GetExecutionsByServer]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionsByServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionsByServer]
(
	@Server_Id INT,
	@Execution_Count INT = 50
)
AS
BEGIN

	SET NOCOUNT ON

	SET @Execution_Count = COALESCE(@Execution_Count, 50)

	DECLARE @Server_Name VARCHAR(50)
	SET @Server_Name = (SELECT [Server_Name]
	                      FROM [Servers]
	                     WHERE [Server_Id] = @Server_Id)

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] IN
	 (
		SELECT DISTINCT TOP(@Execution_Count) BEDP.[Execution_Id]
		  FROM [BuildExecution_ActionGroupActions] BEAGA
		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
		 WHERE BEAGA.[Original_Server_Name] = @Server_Name
		 ORDER BY BEDP.[Execution_Id] DESC
	 )

END
GO

GRANT EXECUTE ON [Builds_GetExecutionsByServer] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionsInProgress',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all executions in the executing state optionally filtered by application.'
GO


IF OBJECT_ID('[Builds_GetExecutionsInProgress]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionsInProgress]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionsInProgress]
(
	@Application_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND [ExecutionStatus_Name] = 'Executing'
	 ORDER BY [ExecutionStart_Date] DESC

END
GO

GRANT EXECUTE ON [Builds_GetExecutionsInProgress] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPartialExecutionLog',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecution_ActionGroupActionLogEntries',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the latest log entries for a specified build execution.'
GO

IF OBJECT_ID('[Builds_GetPartialExecutionLog]') IS NOT NULL DROP PROCEDURE [Builds_GetPartialExecutionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPartialExecutionLog]
(
	@Execution_Id INT,
    @After_BuildExecution_ActionGroupAction_Id INT,
	@After_LogEntry_Sequence INT,
    @IncludeDebug_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

    SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

		SELECT BEAGALE.*
		  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
		       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		               ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
		       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		               ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
			   INNER JOIN [BuildExecutions] BE
			           ON BE.[Execution_Id] = BEDP.[Execution_Id]
			   INNER JOIN [Builds] B
			           ON B.[Application_Id] = BE.[Application_Id]
					  AND B.[Release_Number] = BE.[Release_Number]
					  AND B.[Build_Number] = BE.[Build_Number]
		 WHERE BEDP.[Execution_Id] = @Execution_Id
		   AND (@IncludeDebug_Indicator = 'Y' OR BEAGALE.[LogEntry_Level] > 0)
		   AND (BEAGA.[Original_Action_Sequence] > 0 OR (B.[BuildImporter_Configuration] IS NOT NULL AND BE.[Environment_Id] IS NULL))
		   AND (@After_BuildExecution_ActionGroupAction_Id IS NULL
		        OR (BEAGALE.[BuildExecution_ActionGroupAction_Id] > @After_BuildExecution_ActionGroupAction_Id)
		        OR (BEAGALE.[BuildExecution_ActionGroupAction_Id] = @After_BuildExecution_ActionGroupAction_Id
		            AND (@After_LogEntry_Sequence IS NULL OR BEAGALE.[LogEntry_Sequence] > @After_LogEntry_Sequence)))
		 ORDER BY BEAGALE.[BuildExecution_ActionGroupAction_Id]

END
GO

GRANT EXECUTE ON [Builds_GetPartialExecutionLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the set of promotions that have occurred to the specified environment'
GO


IF OBJECT_ID('[Builds_GetPromotion]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotion]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT
) AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildPromotions_Extended]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [Environment_Id] = @Environment_Id
	IF (@@ROWCOUNT = 0) RAISERROR('{App#%i} %s Build %i to {Env#%i} does not exist.', 16, 1, @Application_ID,@Release_Number,@Build_Number,@Environment_Id)

END
GO

GRANT EXECUTE ON [Builds_GetPromotion] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Builds_GetPromotionApproval]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotionApproval]
GO

GO
IF OBJECT_ID('[Builds_GetPromotionApprovals]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotionApprovals]
GO

GO
IF OBJECT_ID('[Builds_GetPromotionRequiredApprovals]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotionRequiredApprovals]
GO

GO
IF OBJECT_ID('[Builds_GetPromotionRequiredApprovalsByUser]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotionRequiredApprovalsByUser]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the promotions that have occurred for a specified build'
GO


IF OBJECT_ID('[Builds_GetPromotions]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotions]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
) AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildPromotions_Extended]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

END
GO

GRANT EXECUTE ON [Builds_GetPromotions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetRequiredPromotions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Builds_Extended,Builds_RequiredPromotions',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all the promotions that must occur for a build from start to finish.'
GO

IF OBJECT_ID('[Builds_GetRequiredPromotions]') IS NOT NULL DROP PROCEDURE [Builds_GetRequiredPromotions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetRequiredPromotions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	SELECT *
	  FROM [Builds_RequiredPromotions]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	 ORDER BY [Step_Sequence]
END
GO

GRANT EXECUTE ON [Builds_GetRequiredPromotions] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_PurgeBuildData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire build and all associated data from the system.'
GO

IF OBJECT_ID('[Builds_PurgeBuildData]') IS NOT NULL
	DROP PROCEDURE [Builds_PurgeBuildData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_PurgeBuildData]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Name VARCHAR(50)
	SELECT @Application_Name = [Application_Name]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id


	/* 1) Delete data without sub-release dependencies */
	DELETE [ReleaseNotes]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DELETE [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DELETE [BuildArtifacts]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DELETE [BuildPromotionContingencies]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DELETE [BuildApprovalsReceived]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

    DELETE [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number


	/* 2) Delete data with sub-build dependencies */
	/* Dependencies */
	DECLARE @Executions TABLE ([Execution_Id] INT NOT NULL)
	INSERT INTO @Executions ([Execution_Id])
	SELECT [Execution_Id]
	  FROM [BuildExecutions]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DECLARE @BuildExecution_DeploymentPlans TABLE ([BuildExecution_DeploymentPlan_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_DeploymentPlans ([BuildExecution_DeploymentPlan_Id])
	SELECT [BuildExecution_DeploymentPlan_Id]
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DECLARE @BuildExecution_ActionGroupActions TABLE ([BuildExecution_ActionGroupAction_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_ActionGroupActions ([BuildExecution_ActionGroupAction_Id])
	SELECT [BuildExecution_ActionGroupAction_Id]
	  FROM [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)


	/* 2a) Builds */
	DELETE
	  FROM [BuildTestResults]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildOutputs]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildExecution_ActionGroupActionLogEntries]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

    DELETE
	  FROM [BuildExecution_ActionGroupActionVariableValues]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE
	  FROM [ManualActionsInProgress]
	 WHERE [BuildExecution_PlanAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE
	  FROM [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN  (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE
	  FROM [BuildExecution_DeploymentPlanActionGroups]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	UPDATE [BuildExecutions]
	   SET [Initial_BuildExecution_DeploymentPlan_Id] = NULL
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	DELETE [DatabaseChangeScriptExecutions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE [ConfigurationFileDeployments]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

    DELETE [BuildPromotionsScheduled]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DELETE
	  FROM [BuildPromotions]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DELETE
	  FROM [Issues_BuildStatusChanges]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DELETE
	  FROM [Builds]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number


	--EXEC [Events_RaiseEvent] 'BLDDEL',@Application_Id,@Application_Name,@Release_Number,@Build_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_PurgeBuildData] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_PurgeExecutionLogs',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'LogMessagesDeleted_Count',
    /* Description_Text        */ 'Purges execution logs in a specific build before a specified time.'
GO

IF OBJECT_ID('[Builds_PurgeExecutionLogs]') IS NOT NULL
	DROP PROCEDURE [Builds_PurgeExecutionLogs]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_PurgeExecutionLogs]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@BuildStep_Indicator YNINDICATOR,
	@Before_Date DATETIME,
	@LogMessagesDeleted_Count BIGINT = NULL OUT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE BEAGALE
	  FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
	       INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
		           ON BEAGA.[BuildExecution_ActionGroupAction_Id] = BEAGALE.[BuildExecution_ActionGroupAction_Id]
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
		           ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
	       INNER JOIN [BuildExecutions] BE
		           ON BE.[Execution_Id] = BEDP.[Execution_Id]
	 WHERE BE.[Application_Id] = @Application_Id
	   AND BE.[Release_Number] = @Release_Number
	   AND BE.[Build_Number] = @Build_Number
	   AND (@BuildStep_Indicator = 'N' OR BE.[Environment_Id] IS NULL)
	   AND (@Environment_Id IS NULL OR BE.[Environment_Id] = @Environment_Id)
	   AND (@Before_Date IS NULL OR BEAGA.[ExecutionEnded_Date] < @Before_Date)

	SET @LogMessagesDeleted_Count = ROWCOUNT_BIG()

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_PurgeExecutionLogs] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_RejectBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Rejects the specified build'
GO


IF OBJECT_ID('[Builds_RejectBuild]') IS NOT NULL DROP PROCEDURE [Builds_RejectBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_RejectBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Rejection_Notes VARCHAR(255) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	UPDATE [Builds]
	   SET [BuildStatus_Name] = 'Rejected'
		  ,[Rejection_Notes] = @Rejection_Notes
		  ,[Rejected_Date] = GETUTCDATE()
		  ,[RejectedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [BuildStatus_Name] <> 'Deployed'
	IF (@@ROWCOUNT = 0) RAISERROR('{App#%i} %s Build %s does not exist.', 16, 1, @Application_Id, @Release_Number, @Build_Number)
	
	-- Cancel all pending and executing build executions when rejecting this build
	UPDATE [BuildExecutions]
	   SET [ExecutionStatus_Name] = 'Canceled'
		  ,[CanceledBy_User_Name] = dbo.Session_GetUserName()
		  ,[CanceledOn_Date] = GETUTCDATE()
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [ExecutionStatus_Name] IN ('Pending', 'Executing')
	
	EXEC [Events_RaiseEvent] 'BLDRJC', @Application_Id, @Release_Number, @Build_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_RejectBuild] TO [BuildMasterUser_Role]

GO
IF OBJECT_ID('[Builds_RemoveApproval]') IS NOT NULL DROP PROCEDURE [Builds_RemoveApproval]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_ValidateForPromotion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Eligible_Indicator,IneligibleReason_Text',
    /* Description_Text        */ 'Indicates whether a build is eligible for a promotion to the next environment in its workflow'
GO


IF OBJECT_ID('[Builds_ValidateForPromotion]') IS NOT NULL DROP PROCEDURE [Builds_ValidateForPromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_ValidateForPromotion]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Eligible_Indicator CHAR(1) OUT,
	@IneligibleReason_Text VARCHAR(MAX) OUT,
	@ForcePromotion_Indicator YNINDICATOR = 'N',
	@DependencyContingent_Indicator YNINDICATOR = 'N',
	@PromoteTo_Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	IF @ForcePromotion_Indicator IS NULL
	BEGIN
		SET @ForcePromotion_Indicator = 'N'
	END

	DECLARE @ReleaseStatus_Name VARCHAR(8)
	DECLARE @BuildStatus_Name VARCHAR(8)
	DECLARE @Workflow_Id INT
	DECLARE @Current_Environment_Id INT
    DECLARE @Current_Environment_Name VARCHAR(50)
    DECLARE @Next_Step_Sequence INT
	DECLARE @Next_Environment_Id INT
    DECLARE @Next_Environment_Name VARCHAR(50)
    DECLARE @AllowOutOfSequencePromotions_Indicator YNINDICATOR
    DECLARE @AutoDeployRelease_Indicator YNINDICATOR

    SELECT @Current_Environment_Name = [Environment_Name]
      FROM [Environments]
     WHERE [Environment_Id] = @PromoteTo_Environment_Id

	SELECT @ReleaseStatus_Name = [ReleaseStatus_Name],
	       @BuildStatus_Name = [BuildStatus_Name],
	       @Next_Step_Sequence = [Next_Step_Sequence],
	       @Workflow_Id = [Workflow_Id],
           @Current_Environment_Id = [Current_Environment_Id],
		   @Next_Environment_Id = [Next_Environment_Id],
           @Next_Environment_Name = [Next_Environment_Name],
           @AllowOutOfSequencePromotions_Indicator = [AllowOutOfSequencePromotions_Indicator],
           @AutoDeployRelease_Indicator = [AutoDeployRelease_Indicator]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number


	DECLARE @Last_Promotion_Status VARCHAR(20)

	IF @Current_Environment_Id IS NULL
	BEGIN

		SELECT TOP 1 @Last_Promotion_Status = [ExecutionStatus_Name]
		  FROM [BuildExecutions_Extended]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] IS NULL
		 ORDER BY [Execution_Id] DESC
	END
	ELSE
	BEGIN

		SELECT @Last_Promotion_Status = [PromotionStatus_Name]
		  FROM [BuildPromotions]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] = @Current_Environment_Id
	END
       
	/* Promotions to any environment AllowOutOfSequencePromotions_Indicator */
	IF ((@PromoteTo_Environment_Id <> @Next_Environment_Id) AND @ForcePromotion_Indicator = 'N' AND @AllowOutOfSequencePromotions_Indicator = 'N')
   		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 
			  'Cannot promote to environment "' + @Current_Environment_Name + '" '
			  + 'because it''s not the next environment in the workflow "' + @Next_Environment_Name + '" '
              + 'and the workflow is not set up to allow promotions to arbitrary environments.' 
			RETURN
		END

	/* Release must be active */
	IF @ReleaseStatus_Name <> 'Active'
   		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 
			  'The release status is '
			  + @ReleaseStatus_Name 
			  + '; only builds for active releases may be promoted.'
			RETURN
		END

	/* Build must be active */
	IF @BuildStatus_Name <> 'Active'
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 
			  'The build status is '
			  + @BuildStatus_Name
			  + '; only Active builds may be promoted.'
			RETURN
		END

	/* last promotion status needs to have succeeded */
	IF @Last_Promotion_Status IS NOT NULL AND @Last_Promotion_Status IN ('Failed','Executing','AwaitingExecution') 
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text =
				'The previous promotion status is '
				+ @Last_Promotion_Status
				+ ' and must be completed or overridden before the build can be promoted again.'
			RETURN
		END

	/* there is a next workflow step */
	IF @Next_Step_Sequence IS NULL AND @AllowOutOfSequencePromotions_Indicator = 'N' AND @AutoDeployRelease_Indicator = 'Y'
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 'There is no step that the build can be promoted to.'
			RETURN
		END

	/* approvals checked off */
    IF (SELECT [dbo].[IsPromotionApproved](@Application_Id, @Release_Number, @Build_Number, @PromoteTo_Environment_Id)) = 'N'
			   AND @ForcePromotion_Indicator = 'N'
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 'Not all required approvals are present.'
			RETURN
		END
	
	/* Cannot promote to their final environment until all referenced dependencies are promoted to their respective final environments */	
	IF EXISTS(SELECT * FROM Builds_Extended BE
	           WHERE [Next_Environment_Id] = [Final_Environment_Id]
	             AND BE.[Build_Number] = @Build_Number				
	             AND BE.[Release_Number] = @Release_Number
	             AND BE.[Application_Id] = @Application_Id
	             AND (
	                   EXISTS 
	                   (
	                     SELECT * 
	                       FROM ReleaseDeployables _RD
	                            INNER JOIN Releases _R 
	                                    ON _RD.Application_Id = BE.[Application_Id]
	                                   AND _RD.Release_Number = BE.[Release_Number]
	                                   AND _RD.Referenced_Release_Number = _R.Release_Number 
	                                   AND _RD.Referenced_Application_Id = _R.Application_Id
	                                   AND _R.[ReleaseStatus_Name] <> 'Deployed'
	                   ) AND (
	                     @DependencyContingent_Indicator <> 'Y'
	                     OR NOT EXISTS 
	                     ( /* Ensure all dependencies have a non-reject build that has been promoted to final environment */
	                       SELECT * 
	                         FROM ReleaseDeployables _RD
	                               INNER JOIN Builds_Extended _B
	                                       ON _RD.Referenced_Release_Number = _B.Release_Number 
	                                      AND _RD.Referenced_Application_Id = _B.Application_Id
	                        WHERE _RD.Application_Id = BE.[Application_Id]
	                         AND _RD.Release_Number = BE.[Release_Number]
	                         AND _B.[Current_Environment_Id] = _B.[Final_Environment_Id]
	                         AND _B.[BuildStatus_Name] <> 'Rejected'
	                     )
	                   )
				)
		) AND @ForcePromotion_Indicator = 'N'
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 'Deployables with dependencies (including imported deployables) cannot be promoted to their final environment until all referenced dependencies are promoted to their respective final environments. If forced, this may cause deployment failures or invalid dependencies.'
			RETURN
		END	
		
	/* Validates */
	SET @Eligible_Indicator = 'Y'
	SET @IneligibleReason_Text = ''
	RETURN
	
END
GO

GRANT EXECUTE ON [Builds_ValidateForPromotion] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetAllGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_AllGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the test result groups in the system.'
GO


IF OBJECT_ID('[BuildTestResults_GetAllGroups]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetAllGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetAllGroups]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildTestResults_AllGroups]
	 ORDER BY [Group_Name]

END
GO

GRANT EXECUTE ON [BuildTestResults_GetAllGroups] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestGroupResults',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_TestGroupResults',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the unit test results for a specified build execution'
GO


IF OBJECT_ID('[BuildTestResults_GetTestGroupResults]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestGroupResults]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestGroupResults]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SELECT * 
      FROM [BuildTestResults_TestGroupResults]
	 WHERE [Execution_Id] = @Execution_Id

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestGroupResults] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestGroupResultsForBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_TestGroupResults',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the unit test results for a specified build execution'
GO


IF OBJECT_ID('[BuildTestResults_GetTestGroupResultsForBuild]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestGroupResultsForBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestGroupResultsForBuild]
(
	@Application_Id INT,
    @Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Max_Execution_Id INT

    SELECT TOP 1 @Max_Execution_Id = [Execution_Id]
      FROM [BuildTestResults_TestGroupResults]
	 WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number
     ORDER BY [Execution_Id] DESC

    SELECT *
      FROM [BuildTestResults_TestGroupResults]
	 WHERE [Execution_Id] = @Max_Execution_Id

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestGroupResultsForBuild] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestHistory',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets this history of a test by its name, and optionally filtered by test group and application.'
GO


IF OBJECT_ID('[BuildTestResults_GetTestHistory]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestHistory]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestHistory]
(
	@Test_Name VARCHAR(255),
	@Group_Name VARCHAR(50),
	@Application_Id INT,
	@TestStatus_Code CHAR(1),
	@Row_Count INT
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Row_Count IS NOT NULL
		SET ROWCOUNT @Row_Count

	SELECT *
	  FROM [BuildTestResults_Extended]
	 WHERE (@Group_Name IS NULL OR [Group_Name] = @Group_Name)
	   AND (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@TestStatus_Code IS NULL OR [TestStatus_Code] = @TestStatus_Code)
	   AND (@Test_Name = [Test_Name])
	 ORDER BY [Application_Name], [Group_Name], [Execution_Id] DESC

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestHistory] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestLog',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the results of a specific unit test'
GO


IF OBJECT_ID('[BuildTestResults_GetTestLog]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestLog]
(
	@Execution_Id INT,
	@Group_Name VARCHAR(50),
	@Test_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildTestResults_Extended]
	 WHERE [Execution_Id] = @Execution_Id
	   AND [Group_Name] = @Group_Name
	   AND [Test_Sequence] = @Test_Sequence

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestResults',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the build test results based on the specified criteria.'
GO


IF OBJECT_ID('[BuildTestResults_GetTestResults]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestResults]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestResults]
(
	@Execution_Id INT,
	@Group_Name VARCHAR(50) = NULL,
	@Original_DeploymentPlanActionGroup_Sequence INT = NULL,
	@Original_Action_Sequence INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON


	SELECT *
	  FROM [BuildTestResults_Extended]
	 WHERE [Execution_Id] = @Execution_Id
       AND [Group_Name] = ISNULL(@Group_Name, [Group_Name])
	   AND [Original_DeploymentPlanActionGroup_Sequence] = ISNULL(@Original_DeploymentPlanActionGroup_Sequence, [Original_DeploymentPlanActionGroup_Sequence])
	   AND [Original_Action_Sequence] = ISNULL(@Original_Action_Sequence, [Original_Action_Sequence])
	ORDER BY [Group_Name], [Test_Sequence]

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestResults] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_RecordTestResult',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Logs the result of an individual unit test.'
GO

IF OBJECT_ID('[BuildTestResults_RecordTestResult]') IS NOT NULL DROP PROCEDURE [BuildTestResults_RecordTestResult]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_RecordTestResult]
(
	@BuildExecution_PlanAction_Id INT,
	@Group_Name VARCHAR(50),
	@Test_Name VARCHAR(255),
	@TestPassed_Indicator YNINDICATOR,
	@TestResult_Text VARCHAR(MAX),
	@TestStarted_Date DATETIME,
	@TestEnded_Date DATETIME
)
AS
BEGIN

	SET NOCOUNT ON

	 DECLARE @Execution_Id INT, @Original_Action_Sequence INT, @Original_DeploymentPlanActionGroup_Sequence INT
      SELECT @Execution_Id = BEDP.[Execution_Id]
		      ,@Original_DeploymentPlanActionGroup_Sequence = BEPA.[Original_DeploymentPlanActionGroup_Sequence]
	          ,@Original_Action_Sequence = BEPA.[Original_Action_Sequence]
	    FROM [BuildExecution_ActionGroupActions] BEPA
  INNER JOIN [BuildExecution_DeploymentPlans] BEDP ON BEDP.[BuildExecution_DeploymentPlan_Id] = [BEPA].[BuildExecution_DeploymentPlan_Id]
       WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_PlanAction_Id

	INSERT INTO [BuildTestResults]
			   ([Execution_Id]
			   ,[Test_Sequence]
			   ,[Group_Name]
			   ,[Test_Name]
			   ,[TestStatus_Code]
			   ,[TestResult_Text]
			   ,[TestStarted_Date]
			   ,[TestEnded_Date]
			   ,[BuildExecution_PlanAction_Id])
		 SELECT
				@Execution_Id,
				ISNULL(MAX([Test_Sequence]), 0) + 1,
				@Group_Name,
				@Test_Name,
				CASE @TestPassed_Indicator WHEN 'Y' THEN 'P' WHEN 'N' THEN 'F' ELSE 'I' END,
				@TestResult_Text,
				@TestStarted_Date,
				@TestEnded_Date,
                @BuildExecution_PlanAction_Id
		   FROM [BuildTestResults]
           WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

END
GO

GRANT EXECUTE ON [BuildTestResults_RecordTestResult] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControlPerformances_GetReleaseSummary',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ChangeControlPerformances_ReleaseSummary',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a summary of the change control performances for a specified change control.'
GO

IF OBJECT_ID('[ChangeControlPerformances_GetReleaseSummary]') IS NOT NULL DROP PROCEDURE [ChangeControlPerformances_GetReleaseSummary]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControlPerformances_GetReleaseSummary]
(
	@ChangeControl_Id INT
)
AS
BEGIN

SET NOCOUNT ON

    SELECT RS.*
	  FROM [ChangeControlPerformances_ReleaseSummary] RS
	       INNER JOIN [Environments] E
		           ON E.[Environment_Id] = RS.[Environment_Id]
     WHERE RS.[ChangeControl_Id] = @ChangeControl_Id
	 ORDER BY E.[Display_Sequence]

END
GO

GRANT EXECUTE ON [ChangeControlPerformances_GetReleaseSummary] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_CreateOrUpdateChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ChangeControl_Id',
    /* Description_Text        */ 'Creates a new change control, or updates an existing one'
GO

IF OBJECT_ID('[ChangeControls_CreateOrUpdateChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_CreateOrUpdateChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_CreateOrUpdateChangeControl]
(
	@ChangeControl_Id INT = NULL OUT,
	@ChangeControl_Name VARCHAR(100),
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Group_Name VARCHAR(100),
	@Description_Text VARCHAR(MAX),
	@Principal_Name USERNAME,
	@PrincipalType_Code CHAR(1),
	@PerformanceType_Code CHAR(1),
	@Active_Indicator YNINDICATOR,
	@Environments_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
BEGIN TRANSACTION

    IF @ChangeControl_Id IS NULL
    BEGIN
    
        INSERT INTO [ChangeControls]
        (
            [Application_Id]
	       ,[Release_Number]
	       ,[ChangeControl_Name]
	       ,[Group_Name]
	       ,[Description_Text]
	       ,[Principal_Name]
	       ,[PrincipalType_Code]
	       ,[PerformanceType_Code]
	       ,[CreatedBy_User_Name]
	       ,[CreatedOn_Date]
	       ,[ModifiedBy_User_Name]
	       ,[ModifiedOn_Date]
	       ,[Active_Indicator]
        )
        VALUES
        (
	         @Application_Id
	        ,@Release_Number
	        ,@ChangeControl_Name
	        ,@Group_Name
	        ,@Description_Text
	        ,@Principal_Name
	        ,@PrincipalType_Code
	        ,@PerformanceType_Code
	        ,dbo.Session_GetUserName()
	        ,GETUTCDATE()
	        ,dbo.Session_GetUserName()
	        ,GETUTCDATE()
	        ,@Active_Indicator
        )
        
        SET @ChangeControl_Id = SCOPE_IDENTITY()
        
        INSERT INTO [ChangeControlEnvironments]
			(
				[ChangeControl_Id],
				[Environment_Id]
			)
		SELECT
				@ChangeControl_Id,
				CAST(CSV.[Value_Text] AS INT)
		  FROM [dbo].[CsvToTable](@Environments_Csv, ',') CSV
        
        EXEC [Events_RaiseEvent] 'CCTNEW', @Application_Id, @ChangeControl_Id
    
    END
    ELSE
    BEGIN
    
        UPDATE [ChangeControls]
           SET  [ChangeControl_Name] = @ChangeControl_Name
	           ,[Group_Name] = @Group_Name
	           ,[Description_Text] = @Description_Text
	           ,[Principal_Name] = @Principal_Name
	           ,[PrincipalType_Code] = @PrincipalType_Code
	           ,[PerformanceType_Code] = @PerformanceType_Code
	           ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
	           ,[ModifiedOn_Date] = GETUTCDATE()
	           ,[Active_Indicator] = @Active_Indicator
	     WHERE [ChangeControl_Id] = @ChangeControl_Id
	     
	     DELETE [ChangeControlEnvironments]
		 WHERE [ChangeControl_Id] = @ChangeControl_Id
        
         INSERT INTO [ChangeControlEnvironments] ([ChangeControl_Id], [Environment_Id])
	     SELECT
			    @ChangeControl_Id,
			    CAST(CSV.[Value_Text] AS INT)
	      FROM [dbo].[CsvToTable](@Environments_Csv, ',') CSV
	     
	     EXEC [Events_RaiseEvent] 'CCTMOD', @Application_Id, @ChangeControl_Id
    
    END

COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ChangeControls_CreateOrUpdateChangeControl] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_DeleteChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified change control'
GO

IF OBJECT_ID('[ChangeControls_DeleteChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_DeleteChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_DeleteChangeControl]
(
	@ChangeControl_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Application_Id INT
    
    SELECT @Application_Id = [Application_Id]
      FROM [ChangeControls]
     WHERE [ChangeControl_Id] = @ChangeControl_Id

    DELETE 
      FROM [ChangeControlEnvironments]
     WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    DELETE
      FROM [ChangeControlPerformances]
     WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    DELETE 
      FROM [ChangeControls]
     WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    EXEC [Events_RaiseEvent] 'CCTDEL', @Application_Id
    
COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ChangeControls_DeleteChangeControl] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_GetChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ChangeControls,ChangeControlEnvironments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specified change control'
GO

IF OBJECT_ID('[ChangeControls_GetChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_GetChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_GetChangeControl]
(
	@ChangeControl_Id INT
)
AS
BEGIN

SET NOCOUNT ON
    
    SELECT * FROM [ChangeControls]
    WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    SELECT * FROM [ChangeControlEnvironments_Extended]
    WHERE [ChangeControl_Id] = @ChangeControl_Id

END
GO

GRANT EXECUTE ON [ChangeControls_GetChangeControl] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_GetChangeControlPerformances',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ChangeControlPerformances_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of change control performances for a specified change control or all change controls if none is specified'
GO

IF OBJECT_ID('[ChangeControls_GetChangeControlPerformances]') IS NOT NULL DROP PROCEDURE [ChangeControls_GetChangeControlPerformances]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_GetChangeControlPerformances]
(
	@Application_Id INT,
	@ChangeControl_Id INT,
	@Release_Number RELEASENUMBER,
	@IncludeInactive_Indicator YNINDICATOR,
	@Environment_Id INT,
	@Row_Count INT
)
AS
BEGIN

SET NOCOUNT ON

    IF @Row_Count IS NOT NULL
        SET ROWCOUNT @Row_Count
    
    SELECT *
      FROM [ChangeControlPerformances_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
       AND [ChangeControl_Id] = COALESCE(@ChangeControl_Id, [ChangeControl_Id])
       AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
       AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])

END
GO

GRANT EXECUTE ON [ChangeControls_GetChangeControlPerformances] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_GetChangeControls',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ChangeControls,ChangeControlEnvironments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of change controls based on the specified criteria'
GO

IF OBJECT_ID('[ChangeControls_GetChangeControls]') IS NOT NULL DROP PROCEDURE [ChangeControls_GetChangeControls]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_GetChangeControls]
(
    @Application_Id INT,
    @Release_Number RELEASENUMBER,
    @IncludeInactive_Indicator YNINDICATOR,
    @ChangeControl_Count INT
)
AS
BEGIN

SET NOCOUNT ON

    IF @ChangeControl_Count IS NOT NULL
        SET ROWCOUNT @ChangeControl_Count

    SELECT *
	  FROM [ChangeControls]
     WHERE [Application_Id] = COALESCE(@Application_Id, [Application_Id])
       AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
       AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')

    SELECT CCE.*
	  FROM [ChangeControlEnvironments_Extended] CCE
	       INNER JOIN [ChangeControls] CC
		           ON CC.[ChangeControl_Id] = CCE.ChangeControl_Id
     WHERE CC.[Application_Id] = COALESCE(@Application_Id, CC.[Application_Id])
       AND CC.[Release_Number] = COALESCE(@Release_Number, CC.[Release_Number])
       AND (CC.[Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')

END
GO

GRANT EXECUTE ON [ChangeControls_GetChangeControls] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_PerformChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a change control performance for a specified environment'
GO

IF OBJECT_ID('[ChangeControls_PerformChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_PerformChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_PerformChangeControl]
(
	@ChangeControl_Id INT,
	@Environment_Id INT,
	@Notes_Text VARCHAR(MAX),
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON
    
BEGIN TRY
	BEGIN TRANSACTION
    
    DECLARE @Application_Id INT
    DECLARE @ChangeControlPerformance_Id INT
    
    SELECT @Application_Id = [Application_Id]
    FROM [ChangeControls]
    WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    INSERT INTO [ChangeControlPerformances]
    (
        [ChangeControl_Id]
       ,[Environment_Id]
       ,[PerformedBy_User_Name]
       ,[Performed_Date]
       ,[Notes_Text]
       ,[Execution_Id]
    )
    VALUES
    (
        @ChangeControl_Id,
        @Environment_Id,
        dbo.Session_GetUserName(),
        GETUTCDATE(),
        @Notes_Text,
        @Execution_Id
    )
    
    SET @ChangeControlPerformance_Id = SCOPE_IDENTITY()
    
    EXEC [Events_RaiseEvent] 'CCTPRF', @Application_Id, @ChangeControl_Id, @ChangeControlPerformance_Id
    
COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ChangeControls_PerformChangeControl] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ConfigurationFile_Id',
    /* Description_Text        */ 'Creates a configuration file in the system.'
GO


IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFile]
(
	@ConfigurationFile_Id INT = NULL OUT,
	@Deployable_Id INT,
	@FilePath_Text VARCHAR(500),
	@ConfigurationFile_Name NVARCHAR(100),
	@Description_Text NVARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [ConfigurationFiles]
			   WHERE [Deployable_Id] = @Deployable_Id
			     AND [FilePath_Text] = @FilePath_Text
				 AND [ConfigurationFile_Name] = @ConfigurationFile_Name)
	BEGIN

		RAISERROR('There is already a configuration file for the specified deployable that has the same alias and file path.', 16, 1)

	END

	INSERT INTO [ConfigurationFiles] 
	(
		[Deployable_Id],
		[FilePath_Text],
		[Active_Indicator],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[ModifiedBy_User_Name],
		[ModifiedOn_Date],
		[ConfigurationFile_Name],
		[Description_Text]
	)
	VALUES 
	(
		@Deployable_Id, 
		@FilePath_Text, 
		'Y',
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		NULLIF(@ConfigurationFile_Name, ''),
		NULLIF(@Description_Text, '')
	)

	SET @ConfigurationFile_Id = SCOPE_IDENTITY()
	
	DECLARE @Application_Id INT
	 SELECT @Application_Id = [Application_Id]
	   FROM [Deployables]
	  WHERE [Deployable_Id] = @Deployable_Id

	EXEC [Events_RaiseEvent] 'CFGNEW', @Application_Id, @ConfigurationFile_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFile] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFileInstance',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new instance of a configuration file.',
	/* Remarks_Text            */ 'If an instance with the specified name already exists but has been deactivated, the old instance will be reactivated and a blank version will be inserted as the latest.'
GO


IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFileInstance]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFileInstance]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFileInstance]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
	@Environment_Id INT,
	@Template_Indicator YNINDICATOR,
	@Template_Instance_Name VARCHAR(100),
	@TransformType_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT *
                    FROM [ConfigurationFileInstances]
				   WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
				     AND [Instance_Name] = @Instance_Name)
	BEGIN

		INSERT INTO [ConfigurationFileInstances]
        (
			[ConfigurationFile_Id],
			[Instance_Name],
			[Environment_Id],
			[Active_Indicator],
			[Template_Indicator],
			[Template_Instance_Name],
			[TransformType_Code]
		)
	    VALUES
	    (
			@ConfigurationFile_Id,
			@Instance_Name,
			@Environment_Id,
			'Y',
			@Template_Indicator,
			@Template_Instance_Name,
			@TransformType_Code
		)

	END
	ELSE IF (SELECT [Active_Indicator]
	           FROM [ConfigurationFileInstances]
			  WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
			    AND [Instance_Name] = @Instance_Name) = 'N'
	BEGIN

		UPDATE [ConfigurationFileInstances]
		   SET [Active_Indicator] = 'Y',
		       [Environment_Id] = @Environment_Id,
			   [Template_Indicator] = @Template_Indicator,
			   [Template_Instance_Name] = @Template_Instance_Name,
			   [TransformType_Code] = @TransformType_Code
		 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
		   AND [Instance_Name] = @Instance_Name

	END
	ELSE
		RAISERROR('Configuration file instance already exists.', 16, 1)

	-- If there are any versions at all
	IF EXISTS(SELECT *
	            FROM [ConfigurationFileVersions]
			   WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)
	BEGIN

		-- Find latest version with actual data
		DECLARE @Latest_NonNull_Version INT
		SET @Latest_NonNull_Version = (SELECT MAX([Version_Number])
		                                 FROM [ConfigurationFileVersions]
										WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
										  AND [Instance_Name] = @Instance_Name
										  AND [Null_Indicator] = 'N')

		-- If there isn't a latest version
		IF @Latest_NonNull_Version IS NULL
		-- or there is, but it is not empty
		OR (SELECT DATALENGTH([File_Bytes])
		      FROM [ConfigurationFileVersions]
			 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
			   AND [Instance_Name] = @Instance_Name
			   AND [Version_Number] = @Latest_NonNull_Version) <> 0
		BEGIN

			DECLARE @Rubbish INT
			SET @Rubbish = COALESCE((SELECT MAX([Version_Number])
				                       FROM [ConfigurationFileVersions]
				                      WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
				                        AND [Instance_Name] = @Instance_Name), 0) + 1

			-- Insert a blank version as the latest for this instance
			INSERT INTO [ConfigurationFileVersions]
			(
				[ConfigurationFile_Id],
				[Instance_Name],
				[Version_Number],
				[VersionNotes_Text],
				[File_Date],
				[File_Bytes],
				[CreatedBy_User_Name],
				[CreatedOn_Date]
			)
			VALUES
			(
				@ConfigurationFile_Id,
				@Instance_Name,
				@Rubbish,
				NULL,
				GETUTCDATE(),
				0x,
				dbo.Session_GetUserName(),
				GETUTCDATE()
			)

			-- Make sure there are no gaps
			EXEC [ConfigurationFiles_FillVersionGaps] @ConfigurationFile_Id

		END

	END


	DECLARE @Application_Id INT
	 SELECT @Application_Id = D.[Application_Id]
	   FROM [ConfigurationFiles] CF
	        INNER JOIN [Deployables] D
		            ON CF.[Deployable_Id] = D.[Deployable_Id]
	  WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	EXEC [Events_RaiseEvent] 'CFGNWI', @Application_Id, @ConfigurationFile_Id,@Instance_Name
		
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFileInstance] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFileVersions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates new configuration file versions for each instance specified in the ConfigurationFiles_Xml for the given configuration file.'
GO

IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFileVersions]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFileVersions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFileVersions]
(
	@ConfigurationFile_Id INT,
	@ConfigurationFiles_Xml XML,
	@ReleaseNumbers_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	-- Do nothing if there are no instances specified
	IF COALESCE(@ConfigurationFiles_Xml.exist('/ConfigFiles/Version/@Instance_Name'), 0) = 0
		RETURN

	EXEC [ConfigurationFiles_FillVersionGaps] @ConfigurationFile_Id


	/*******************************************************************
	 * Now versions should be consistent, so figure out next version
	 ******************************************************************/
	DECLARE @Version_Number INT
	    SET @Version_Number = COALESCE((SELECT MAX([Version_Number])
		                                  FROM [ConfigurationFileVersions]
								         WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id), 0) + 1

	/* XML format example
		<?xml version="1.0" encoding="utf-16"?>
		<ConfigFiles>
			<Version
				Instance_Name="Default"
				VersionNotes_Text="Notes..."
				File_Bytes="::base64::"
			/>
		</ConfigFiles>
	*/

	INSERT INTO [ConfigurationFileVersions]
	(
		[ConfigurationFile_Id],
		[Instance_Name],
		[Version_Number],
		[VersionNotes_Text],
		[File_Date],
		[File_Bytes],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	SELECT CFI.[ConfigurationFile_Id],
	       CFI.[Instance_Name],
		   @Version_Number,
		   CFV_Input.[VersionNotes_Text],
		   CASE WHEN CFV_Input.[File_Bytes] IS NOT NULL THEN GETUTCDATE() ELSE NULL END,
		   CFV_Input.[File_Bytes],
		   CASE WHEN CFV_Input.[File_Bytes] IS NOT NULL THEN dbo.Session_GetUserName() ELSE NULL END,
		   CASE WHEN CFV_Input.[File_Bytes] IS NOT NULL THEN GETUTCDATE() ELSE NULL END
	  FROM [ConfigurationFileInstances] CFI
	       LEFT JOIN (SELECT [Instance_Name] = CFVX.[Version].[value]('@Instance_Name', 'VARCHAR(100)'),
		                     [VersionNotes_Text] = CFVX.[Version].[value]('@VersionNotes_Text', 'VARCHAR(500)'),
							 [File_Bytes] = COALESCE(CFVX.[Version].[value]('@File_Bytes', 'VARBINARY(MAX)'), CAST('' AS VARBINARY(MAX)))
		                FROM @ConfigurationFiles_Xml.[nodes]('/ConfigFiles/Version') AS CFVX(Version)) CFV_Input
				  ON CFV_Input.[Instance_Name] = CFI.[Instance_Name]
	 WHERE CFI.[ConfigurationFile_Id] = @ConfigurationFile_Id

	 DECLARE @Application_Id INT
	 SET @Application_Id = (SELECT D.[Application_Id]
	                          FROM [ConfigurationFiles] CF
							       INNER JOIN [Deployables] D
								           ON D.[Deployable_Id] = CF.[Deployable_Id]
							 WHERE CF.[ConfigurationFile_Id] = @ConfigurationFile_Id)

	 DECLARE @ReleaseNumbers TABLE ([Release_Number] RELEASENUMBER NOT NULL)
	 INSERT INTO @ReleaseNumbers
	 SELECT [Value_Text] FROM dbo.CsvToTable(@ReleaseNumbers_Csv, ',')

	 UPDATE [ReleaseConfigurationFiles]
	    SET [Version_Number] = @Version_Number
	  WHERE [Application_Id] = @Application_Id
	    AND [Release_Number] IN (SELECT [Release_Number] FROM @ReleaseNumbers)
		AND [ConfigurationFile_Id] = @ConfigurationFile_Id

	 INSERT INTO [ReleaseConfigurationFiles]
	 (
		[Application_Id],
		[Release_Number],
		[ConfigurationFile_Id],
		[Version_Number]
	 )
	 SELECT @Application_Id,
	        R.[Release_Number],
			@ConfigurationFile_Id,
			@Version_Number
	   FROM @ReleaseNumbers R
	  WHERE NOT EXISTS(SELECT *
	                     FROM [ReleaseConfigurationFiles] RCF
						WHERE RCF.[Application_Id] = @Application_Id
						  AND RCF.[Release_Number] = R.[Release_Number]
						  AND RCF.[ConfigurationFile_Id] = @ConfigurationFile_Id)

	 EXEC [Events_RaiseEvent] 'CFGNWV', @Application_Id, @ConfigurationFile_Id, @Version_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFileVersions] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_DeployConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ConfigurationFileDeployment_Id',
    /* Description_Text        */ 'Saves the result of a deployed configuration file (this will not physically deploy the configuration file).'
GO

IF OBJECT_ID('[ConfigurationFiles_DeployConfigurationFile]') IS NOT NULL
	DROP PROCEDURE [ConfigurationFiles_DeployConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_DeployConfigurationFile]
(
	@ConfigurationFileDeployment_Id INT = NULL OUT,
	@Execution_Id INT,
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
	@Version_Number INT,
	@Server_Id INT,
	@DeployedTo_Path VARCHAR(500),
	@DeployedBy_User_Name USERNAME
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Template_Indicator YNINDICATOR
	SET @Template_Indicator = (SELECT [Template_Indicator]
	                             FROM [ConfigurationFileInstances]
								WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
								  AND [Instance_Name] = @Instance_Name)

	IF @Template_Indicator IS NULL
		RAISERROR('Configuration file does not exist.', 16, 1)

	IF @Template_Indicator = 'Y'
		RAISERROR('Cannot deploy configuration template.', 16, 1)

	INSERT INTO [ConfigurationFileDeployments]
	(
		[ConfigurationFile_Id],
		[Execution_Id],
		[Instance_Name],
		[Version_Number],
		[Server_Id],
		[DeployedTo_Path],
		[DeployedBy_User_Name],
		[DeployedOn_Date]
	)
	VALUES
	(
		@ConfigurationFile_Id,
		@Execution_Id,
		@Instance_Name,
		@Version_Number,
		@Server_Id,
		@DeployedTo_Path,
		COALESCE(@DeployedBy_User_Name, dbo.Session_GetUserName()),
		GETUTCDATE()
	)

	SET @ConfigurationFileDeployment_Id = SCOPE_IDENTITY()

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT D.[Application_Id] 
	                             FROM [ConfigurationFiles] CF
 	                                  INNER JOIN [Deployables] D
 	                                          ON CF.[Deployable_Id] = D.[Deployable_Id]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)

	EXEC [Events_RaiseEvent] 'CFGDPL', @Application_Id, @ConfigurationFileDeployment_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_DeployConfigurationFile] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ConfigurationFiles_Extended,ConfigurationFileInstances_Extended,ConfigurationFileVersions_Combined,ConfigurationFileVersions_ModifiedInstances',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a configuration file and all of its instances.'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFile]
(
	@ConfigurationFile_Id INT,
	@IncludeInactiveInstances_Indicator YNINDICATOR = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [ConfigurationFiles_Extended]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	
	IF (@@ROWCOUNT = 0) BEGIN
		RAISERROR('Configuration file %i does not exist.', 16, 1, @ConfigurationFile_Id)
		RETURN
	END

	SELECT CFI.*
	  FROM [ConfigurationFileInstances_Extended] CFI
	       LEFT JOIN [Environments] E
		           ON E.[Environment_Id] = CFI.[Environment_Id]
	 WHERE CFI.[ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND (CFI.[Active_Indicator] = 'Y' OR @IncludeInactiveInstances_Indicator = 'Y')
	 ORDER BY CFI.[Template_Indicator], E.[Display_Sequence], CFI.[Instance_Name]

	SELECT *
	  FROM [ConfigurationFileVersions_Combined]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	SELECT *
	  FROM [ConfigurationFileVersions_ModifiedInstances] CFVMI
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFile] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileDeployments',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileDeployments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all configuration file deployments based on the specified criteria.'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileDeployments]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileDeployments]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileDeployments]
(
	@Application_Id INT,
    @ConfigurationFile_Id INT = NULL,
    @Deployable_Id INT = NULL,
    @Instance_Name VARCHAR(100) = NULL,
    @Server_Id INT = NULL,
	@File_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	--Use RowCount to limit results 
	IF (@File_Count IS NOT NULL AND @File_Count > 0)
	BEGIN
		SET ROWCOUNT @File_Count
	END

	SELECT *
	  FROM [ConfigurationFileDeployments_Extended]
     WHERE [Deployable_Id] IN (SELECT [Deployable_Id] 
                                 FROM [Deployables] 
                                WHERE [Application_Id] = @Application_Id)	 
       AND [ConfigurationFile_Id] = COALESCE(@ConfigurationFile_Id,[ConfigurationFile_Id])
       AND [Deployable_Id] = COALESCE(@Deployable_Id,[Deployable_Id])
       AND [Instance_Name] = COALESCE(@Instance_Name,[Instance_Name])
       AND [Server_Id] = COALESCE(@Server_Id,[Server_Id])
	  ORDER BY [DeployedOn_Date] DESC

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileDeployments] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileInstances',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileInstances_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all configuration file instances that match the specified criteria.'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileInstances]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileInstances]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileInstances]
(
	@Application_Id INT = NULL,
	@Deployable_Id INT = NULL,
	@Active_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT CFI.*
	  FROM [ConfigurationFileInstances_Extended] CFI
		   LEFT JOIN [Environments] E
				 ON E.[Environment_Id] = CFI.[Environment_Id]
	 WHERE (@Application_Id IS NULL OR CFI.[Application_Id] = @Application_Id)
	   AND (@Deployable_Id IS NULL OR CFI.[Deployable_Id] = @Deployable_Id)
	   AND (@Active_Indicator IS NULL OR CFI.[Active_Indicator] = @Active_Indicator)
	ORDER BY E.[Display_Sequence], CFI.[Instance_Name]
END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileInstances] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFiles',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ConfigurationFiles_Extended,ConfigurationFileInstances_Extended,ConfigurationFileVersions_Combined,ConfigurationFileVersions_ModifiedInstances',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all configuration files that match the specified criteria.'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFiles]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFiles]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFiles]
(
	@Application_Id INT = NULL,
	@Deployable_Id INT = NULL,
	@IncludeInactive_Indicator YNINDICATOR = NULL,
	@IncludeInstances_Indicator YNINDICATOR = NULL,
	@IncludeVersions_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SET @IncludeInactive_Indicator = ISNULL(@IncludeInactive_Indicator, 'N')

	SELECT *
	  FROM [ConfigurationFiles_Extended]
	 WHERE [Application_Id] = ISNULL(@Application_Id, [Application_Id])
	   AND [Deployable_Id] = ISNULL(@Deployable_Id, [Deployable_Id])
	   AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	 ORDER BY [Deployable_Name], [Active_Indicator] DESC, [FilePath_Text]

	SELECT *
	  FROM [ConfigurationFileInstances_Extended]
	 WHERE @IncludeInstances_Indicator = 'Y'
	   AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	   AND [Application_Id] = ISNULL(@Application_Id, [Application_Id])
	   AND [Deployable_Id] = ISNULL(@Deployable_Id, [Deployable_Id])

	SELECT CFV.*
	  FROM [ConfigurationFileVersions_Combined] CFV
	       INNER JOIN [ConfigurationFiles] CF
		           ON CF.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
		   INNER JOIN [Deployables] D
		           ON D.[Deployable_Id] = CF.[Deployable_Id]
	 WHERE @IncludeVersions_Indicator = 'Y'
	   AND (CF.[Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	   AND CF.[Deployable_Id] = ISNULL(@Deployable_Id, CF.[Deployable_Id])
	   AND D.[Application_Id] = ISNULL(@Application_Id, D.[Application_Id])

	SELECT CFVMI.*
	  FROM [ConfigurationFileVersions_ModifiedInstances] CFVMI
	       INNER JOIN [ConfigurationFiles] CF
		           ON CF.[ConfigurationFile_Id] = CFVMI.[ConfigurationFile_Id]
		   INNER JOIN [Deployables] D
		           ON D.[Deployable_Id] = CF.[Deployable_Id]
	 WHERE @IncludeVersions_Indicator = 'Y'
	   AND (CF.[Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	   AND CF.[Deployable_Id] = ISNULL(@Deployable_Id, CF.[Deployable_Id])
	   AND D.[Application_Id] = ISNULL(@Application_Id, D.[Application_Id])

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFiles] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileVersion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileVersions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of configuration file versions for a specified configuration file.'
GO

IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileVersion]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileVersion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileVersion]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100) = NULL,
	@Version_Number INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Version_Number IS NULL
	BEGIN

		SELECT @Version_Number = MAX([Version_Number])
		  FROM [ConfigurationFileVersions_Extended]
		 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
		   AND [Instance_Name] = COALESCE(@Instance_Name, [Instance_Name])

	END	

	SELECT *
	  FROM [ConfigurationFileVersions_Extended]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = COALESCE(@Instance_Name, [Instance_Name])
	   AND [Version_Number] = @Version_Number

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileVersion] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileVersions]') IS NOT NULL 
   DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileVersions]

GO
IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileVersionsByRelease]') IS NOT NULL 
   DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileVersionsByRelease]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_PurgeConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges a configuration file, including all previous versions of all instances and deployment history.'
GO


IF OBJECT_ID('[ConfigurationFiles_PurgeConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_PurgeConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_PurgeConfigurationFile]
(
	@ConfigurationFile_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT
	DECLARE @ConfigurationFile_Name NVARCHAR(500)
	SELECT @Application_Id = D.[Application_Id],
	       @ConfigurationFile_Name = COALESCE(CF.[ConfigurationFile_Name], CF.[FilePath_Text])
	  FROM [ConfigurationFiles] CF
		   INNER JOIN [Deployables] D
		           ON D.[Deployable_Id] = CF.[Deployable_Id]
	 WHERE CF.[ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ConfigurationFileDeployments]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ReleaseConfigurationFiles]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ConfigurationFileVersions]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ConfigurationFileInstances]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DELETE [ConfigurationFiles]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	EXEC [Events_RaiseEvent] 'CFGDEL', @Application_Id, @ConfigurationFile_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_PurgeConfigurationFile] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_PurgeConfigurationFileInstance',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges a configuration file instance, including all previous versions and deployment history.'
GO


IF OBJECT_ID('[ConfigurationFiles_PurgeConfigurationFileInstance]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_PurgeConfigurationFileInstance]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_PurgeConfigurationFileInstance]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
	@UnlinkTemplate_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT
	SELECT @Application_Id = D.[Application_Id]
	  FROM [ConfigurationFiles] CF
		   INNER JOIN [Deployables] D
		           ON D.[Deployable_Id] = CF.[Deployable_Id]
	 WHERE CF.[ConfigurationFile_Id] = @ConfigurationFile_Id

	IF @UnlinkTemplate_Indicator = 'Y'
	BEGIN
		UPDATE [ConfigurationFileInstances]
		   SET [Template_Instance_Name] = NULL
		 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
		   AND [Template_Instance_Name] = @Instance_Name
	END

	DELETE [ConfigurationFileDeployments]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = @Instance_Name

	DELETE [ConfigurationFileVersions]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = @Instance_Name

    DELETE [ConfigurationFileInstances]
     WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
       AND [Instance_Name] = @Instance_Name

	EXEC [Events_RaiseEvent] 'CFGDLI', @Application_Id, @ConfigurationFile_Id, @Instance_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_PurgeConfigurationFileInstance] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[ConfigurationFiles_ReactivateConfigurationFile]') IS NOT NULL
	DROP PROCEDURE [ConfigurationFiles_ReactivateConfigurationFile]
GO

GO
IF OBJECT_ID('[ConfigurationFiles_RenameConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_RenameConfigurationFile]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_UpdateConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates metadata for an existing configuration file.'
GO


IF OBJECT_ID('[ConfigurationFiles_UpdateConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_UpdateConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_UpdateConfigurationFile]
(
	@ConfigurationFile_Id INT,
	@Active_Indicator YNINDICATOR,
	@ConfigurationFile_Name NVARCHAR(100),
	@FilePath_Text VARCHAR(500),
	@Description_Text NVARCHAR(MAX),
	@Deployable_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [ConfigurationFiles]
			   WHERE [ConfigurationFile_Id] <> @ConfigurationFile_Id
			     AND [Deployable_Id] = @Deployable_Id
			     AND [FilePath_Text] = @FilePath_Text
				 AND [ConfigurationFile_Name] = @ConfigurationFile_Name)
	BEGIN

		RAISERROR('There is already a configuration file for the specified deployable that has the same alias and file path.', 16, 1)

	END


	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT D.[Application_Id] 
	                             FROM [ConfigurationFiles] CF
 	                                  INNER JOIN [Deployables] D
 	                                          ON CF.[Deployable_Id] = D.[Deployable_Id]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)

	DECLARE @Other_Application_Id INT
	SET @Other_Application_Id = (SELECT [Application_Id] 
	                               FROM [Deployables]
 	                              WHERE [Deployable_Id] = @Deployable_Id)

	IF @Other_Application_Id <> @Application_Id
		RAISERROR('Cannot change configuration file deployable to a deployable of a different application.', 16, 1)

	UPDATE [ConfigurationFiles]
	   SET [Active_Indicator] = @Active_Indicator,
	       [ConfigurationFile_Name] = NULLIF(@ConfigurationFile_Name, ''),
		   [FilePath_Text] = @FilePath_Text,
		   [Description_Text] = NULLIF(@Description_Text, ''),
		   [Deployable_Id] = @Deployable_Id
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	EXEC [Events_RaiseEvent] 'CFGMOD', @Application_Id, @ConfigurationFile_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_UpdateConfigurationFile] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_UpdateConfigurationFileInstance',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates metadata for an existing configuration file instance.'
GO


IF OBJECT_ID('[ConfigurationFiles_UpdateConfigurationFileInstance]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_UpdateConfigurationFileInstance]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_UpdateConfigurationFileInstance]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
    @Environment_Id INT,
	@Active_Indicator YNINDICATOR,
	@Template_Instance_Name VARCHAR(100),
	@TransformType_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Template_Instance_Name IS NOT NULL
	BEGIN
		IF (SELECT [Template_Indicator] FROM [ConfigurationFileInstances] WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id AND [Instance_Name] = @Template_Instance_Name) = 'N'
			RAISERROR('Specified template instance name must actually refer to a template instance.', 16, 1)
	END

	UPDATE [ConfigurationFileInstances]
	   SET [Environment_Id] = @Environment_Id,
	       [Active_Indicator] = @Active_Indicator,
		   [Template_Instance_Name] = @Template_Instance_Name,
		   [TransformType_Code] = @TransformType_Code
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = @Instance_Name

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT D.[Application_Id] 
	                             FROM [ConfigurationFiles] CF
 	                                  INNER JOIN [Deployables] D
 	                                          ON CF.[Deployable_Id] = D.[Deployable_Id]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)

	EXEC [Events_RaiseEvent] 'CFGMDI', @Application_Id, @ConfigurationFile_Id, @Instance_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_UpdateConfigurationFileInstance] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Configuration',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a complete list of BuildMaster configuration settings.'
GO

IF OBJECT_ID('[Configuration_GetConfiguration]') IS NOT NULL DROP PROCEDURE [Configuration_GetConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Configuration_GetConfiguration] AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Configuration]
	 WHERE [Scope_Name] <> 'Internal'
	 ORDER BY [Scope_Name], [Key_Name]

END
GO

GRANT EXECUTE ON [Configuration_GetConfiguration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetInternalValue',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified BuildMaster internal configuration value.'
GO


IF OBJECT_ID('[dbo].[Configuration_GetInternalValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_GetInternalValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_GetInternalValue]
(
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [dbo].[Configuration]
	 WHERE [Scope_Name] = 'Internal'
	   AND [Key_Name] = @Key_Name

END
GO

GRANT EXECUTE ON [dbo].[Configuration_GetInternalValue] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified BuildMaster configuration value.'
GO


IF OBJECT_ID('[dbo].[Configuration_GetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_GetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_GetValue]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [dbo].[Configuration]
	 WHERE [Scope_Name] = @Scope_Name
	   AND [Key_Name] = @Key_Name
	   AND [Scope_Name] <> 'Internal'

END
GO

GRANT EXECUTE ON [dbo].[Configuration_GetValue] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_SetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets a BuildMaster configuration value.'
GO


IF OBJECT_ID('[dbo].[Configuration_SetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_SetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_SetValue]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Scope_Name <> 'Internal'
	BEGIN

		IF EXISTS(SELECT * FROM [Configuration] WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name) BEGIN
			UPDATE [Configuration]
	           SET [Value_Text] = @Value_Text
	         WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name

	    END ELSE BEGIN

			INSERT INTO [Configuration]
			([Scope_Name], [Key_Name], [Value_Text])
			VALUES
			(@Scope_Name, @Key_Name, @Value_Text)
	    END

	EXEC [Events_RaiseEvent] 'SYSCFG',NULL,@Key_Name

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Configuration_SetValue] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_SetValueWithoutEvent',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[dbo].[Configuration_SetValueWithoutEvent]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_SetValueWithoutEvent]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_SetValueWithoutEvent]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT * FROM [Configuration] WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name) BEGIN
		UPDATE [Configuration]
           SET [Value_Text] = @Value_Text
         WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name

    END ELSE BEGIN

		INSERT INTO [Configuration]
			([Scope_Name], [Key_Name], [Value_Text])
		VALUES
			(@Scope_Name, @Key_Name, @Value_Text)
    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Configuration_SetValueWithoutEvent] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_CreateDashboard',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a dashboard'
GO


IF OBJECT_ID('[Dashboards_CreateDashboard]') IS NOT NULL DROP PROCEDURE [Dashboards_CreateDashboard]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_CreateDashboard]
(
	@Application_Id INT,
	@User_Name USERNAME,
	@DashboardScope_Code CHAR(1)	
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	
	INSERT INTO [Dashboards] ([Application_Id], [User_Name], [DashboardScope_Code])
	VALUES (@Application_Id, @User_Name, @DashboardScope_Code)

END TRY BEGIN CATCH
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Dashboards_CreateDashboard] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_CreateOrUpdateGadget',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Display_Sequence',
    /* Description_Text        */ 'Creates or updates a gadget in an application or system-level dashboard'
GO


IF OBJECT_ID('[Dashboards_CreateOrUpdateGadget]') IS NOT NULL DROP PROCEDURE [Dashboards_CreateOrUpdateGadget]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_CreateOrUpdateGadget]
(
	@Gadget_Id INT = NULL OUT,
	@Dashboard_Id INT,
	@Dashboard_Zone INT,
	@Display_Sequence INT = NULL OUT,
	@DashboardGadget_Name VARCHAR(50),
	@DashboardGadget_Description VARCHAR(MAX),
	@DashboardGadget_Configuration VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		--Create
		IF @Gadget_Id IS NULL
		BEGIN
			SELECT @Display_Sequence = COALESCE(MAX([Display_Sequence]), 0) + 1 FROM [DashboardGadgets]
			
			INSERT INTO [DashboardGadgets]
			(
				[Dashboard_Id],
				[Dashboard_Zone],
				[Display_Sequence],
				[DashboardGadget_Name],
				[DashboardGadget_Description],
				[DashboardGadget_Configuration],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date]
			)
			VALUES
			(
				@Dashboard_Id,
				@Dashboard_Zone,
				@Display_Sequence,
				@DashboardGadget_Name,
				@DashboardGadget_Description,
				@DashboardGadget_Configuration,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE()
			)
			
			SET @Gadget_Id = SCOPE_IDENTITY()
		END
		--Update
		ELSE
		BEGIN
			UPDATE [DashboardGadgets]		
			   SET [DashboardGadget_Name] = @DashboardGadget_Name,
			       [DashboardGadget_Description] = @DashboardGadget_Description,
			       [DashboardGadget_Configuration] = @DashboardGadget_Configuration,
			       [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			       [ModifiedOn_Date] = GETUTCDATE()
			 WHERE [Gadget_Id] = @Gadget_Id
		END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Dashboards_CreateOrUpdateGadget] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_DeleteGadget',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a gadget in an application or system-level dashboard'
GO


IF OBJECT_ID('[Dashboards_DeleteGadget]') IS NOT NULL DROP PROCEDURE [Dashboards_DeleteGadget]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_DeleteGadget]
(
	@Gadget_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [DashboardGadgets]
	 WHERE [Gadget_Id] = @Gadget_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Dashboards_DeleteGadget] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_GetApplications',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Applications_Extended,Applications_LatestDeployedReleases',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an application summary for all applications in BuildMaster or a specific group of applications'
GO


IF OBJECT_ID('[Dashboards_GetApplications]') IS NOT NULL DROP PROCEDURE [Dashboards_GetApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_GetApplications]
(
	@Application_Id INT = NULL,
    @ApplicationGroup_Id INT = NULL
)
AS
BEGIN

	SELECT *
	  FROM [Applications_Extended]
	 WHERE ([ApplicationGroup_Id] = @ApplicationGroup_Id OR @ApplicationGroup_Id IS NULL)
	   AND ([Application_Id] = @Application_Id OR @Application_Id IS NULL)
	 ORDER BY   
	          CASE WHEN [ApplicationGroup_Id] IS NULL THEN 1 ELSE 0 END
	         ,[ApplicationGroup_Name]
	         ,[Application_Name]

	SELECT *
	  FROM [Applications_LatestDeployedReleases]
	 WHERE ([ApplicationGroup_Id] = @ApplicationGroup_Id OR @ApplicationGroup_Id IS NULL)
	   AND ([Application_Id] = @Application_Id OR @Application_Id IS NULL)
	 ORDER BY [Application_Id], [Display_Sequence]

END
GO

GRANT EXECUTE ON [Dashboards_GetApplications] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_GetDashboard',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Dashboards',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a dashboard.'
GO


IF OBJECT_ID('[Dashboards_GetDashboard]') IS NOT NULL DROP PROCEDURE [Dashboards_GetDashboard]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_GetDashboard]
(
	@Application_Id INT,
	@DashboardScope_Code CHAR(1)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT TOP 1 *
	  FROM [Dashboards]
	 WHERE [DashboardScope_Code] = @DashboardScope_Code
	  AND ([User_Name] = dbo.Session_GetUserName() OR [User_Name] IS NULL)
	  AND ([Application_Id] = @Application_Id OR [Application_Id] IS NULL)
	ORDER BY CASE WHEN [User_Name] IS NOT NULL THEN 0 ELSE 1 END, /* User Specific Comes First */
	         CASE WHEN [Application_Id] IS NOT NULL THEN 0 ELSE 1 END /* App Specific Comes Second */


END
GO

GRANT EXECUTE ON [Dashboards_GetDashboard] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_GetGadget',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a gadget in an application or system-level dashboard'
GO


IF OBJECT_ID('[Dashboards_GetGadget]') IS NOT NULL DROP PROCEDURE [Dashboards_GetGadget]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_GetGadget]
(
	@Gadget_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [DashboardGadgets_Extended]
	 WHERE [Gadget_Id] = @Gadget_Id

END
GO

GRANT EXECUTE ON [Dashboards_GetGadget] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_GetGadgets',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DashboardGadgets_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the gadgets in a dashboard.'
GO


IF OBJECT_ID('[Dashboards_GetGadgets]') IS NOT NULL DROP PROCEDURE [Dashboards_GetGadgets]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_GetGadgets]
(
	@Dashboard_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [DashboardGadgets_Extended]
	 WHERE [Dashboard_Id] = @Dashboard_Id
	ORDER BY [Dashboard_Zone], [Display_Sequence]

END
GO

GRANT EXECUTE ON [Dashboards_GetGadgets] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_ResequenceGadget',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the sequence of a gadget within a dashboard'
GO


IF OBJECT_ID('[Dashboards_ResequenceGadget]') IS NOT NULL DROP PROCEDURE [Dashboards_ResequenceGadget]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_ResequenceGadget]
(
	@Gadget_Id INT,
	@New_Display_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	DECLARE @Dashboard_Id INT
	DECLARE @Display_Sequence INT
	SELECT @Dashboard_Id = [Dashboard_Id],
	       @Display_Sequence = [Display_Sequence]
	  FROM [DashboardGadgets]
	 WHERE [Gadget_Id] = @Gadget_Id

	BEGIN TRANSACTION

	UPDATE [DashboardGadgets]
	   SET [Display_Sequence] = CASE
	          WHEN [DashboardGadgets].[Display_Sequence] = CUR.[Display_Sequence] 
	               THEN NEW.[Display_Sequence]
	          WHEN [DashboardGadgets].[Display_Sequence] BETWEEN CUR.[Display_Sequence] AND NEW.[Display_Sequence]
	               THEN [DashboardGadgets].[Display_Sequence] - 1
	          WHEN [DashboardGadgets].[Display_Sequence] BETWEEN NEW.[Display_Sequence] AND CUR.[Display_Sequence]
	               THEN [DashboardGadgets].[Display_Sequence] + 1
	          ELSE [DashboardGadgets].[Display_Sequence]
	          END
	  FROM [DashboardGadgets],
	       [DashboardGadgets] CUR,
	       [DashboardGadgets] NEW
	 WHERE CUR.[Display_Sequence] = @Display_Sequence
	   AND NEW.[Display_Sequence] = @New_Display_Sequence
	   AND @Display_Sequence <> @New_Display_Sequence
	   AND [DashboardGadgets].[Dashboard_Id] = @Dashboard_Id
	   AND CUR.[Dashboard_Id] = @Dashboard_Id
	   AND NEW.[Dashboard_Id] = @Dashboard_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Dashboards_ResequenceGadget] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_AddDatabaseProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a database change script provider if one does not exist, or updates an existing one.'
GO

IF OBJECT_ID('[DatabaseChangeScripts_AddDatabaseProvider]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_AddDatabaseProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_AddDatabaseProvider]
(
	@Provider_Id INT,
	@Application_Id INT,
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF EXISTS(SELECT * FROM [DatabaseChangeScriptProviders]
	           WHERE [Provider_Id] = @Provider_Id)
	BEGIN

		UPDATE [DatabaseChangeScriptProviders]
		   SET [Environment_Id] = @Environment_Id
		 WHERE [Provider_Id] = @Provider_Id
		IF @@ROWCOUNT <> 1 RAISERROR('UPDATE did not update exactly 1 row.', 16, 1)
		
		EXEC [Events_RaiseEvent] 'DBSMOD', @Application_Id, @Provider_Id
	
	END ELSE BEGIN

		INSERT INTO [DatabaseChangeScriptProviders]
			([Provider_Id], [Application_Id], [Environment_Id])
		VALUES
			(@Provider_Id, @Application_Id, @Environment_Id)
			
        EXEC [Events_RaiseEvent] 'DBSNEW', @Application_Id, @Provider_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_AddDatabaseProvider] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_CreateNewChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Script_Id',
    /* Description_Text        */ 'Creates a new database change script'
GO


IF OBJECT_ID('[DatabaseChangeScripts_CreateNewChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_CreateNewChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_CreateNewChangeScript]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Deployable_Id INT,
	@Script_Name VARCHAR(50),
	@Script_Text NVARCHAR(MAX),
	@Active_Indicator YNINDICATOR,

	@Script_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [DatabaseChangeScripts]
			   ([Application_Id]
			   ,[Release_Number]
			   ,[Deployable_Id]
			   ,[Script_Name]
			   ,[Script_Text]
			   ,[Active_Indicator]
			   ,[CreatedBy_User_Name]
			   ,[CreatedOn_Date]
			   ,[ModifiedBy_User_Name]
			   ,[ModifiedOn_Date])
		 VALUES
			   (@Application_Id
			   ,@Release_Number
			   ,@Deployable_Id
			   ,@Script_Name
			   ,@Script_Text
			   ,@Active_Indicator
			   ,[dbo].[Session_GetUserName]()
			   ,GETUTCDATE()
			   ,[dbo].[Session_GetUserName]()
			   ,GETUTCDATE())

	    SET @Script_Id = SCOPE_IDENTITY()

	EXEC [Events_RaiseEvent] 'DBSNWS', @Application_Id, @Script_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_CreateNewChangeScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_DeleteChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a database change script'
GO


IF OBJECT_ID('[DatabaseChangeScripts_DeleteChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_DeleteChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_DeleteChangeScript]
(
	@Script_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT, @Script_Name SQL_VARIANT
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [DatabaseChangeScripts] 
	                            WHERE [Script_Id] = @Script_Id)
	    SET @Script_Name    = (SELECT [Script_Name] 
	                             FROM [DatabaseChangeScripts] 
	                            WHERE [Script_Id] = @Script_Id)

	DELETE [DatabaseChangeScriptExecutions]
	 WHERE [Script_Id] = @Script_Id

	DELETE [DatabaseChangeScripts]
	 WHERE [Script_Id] = @Script_Id	

	IF @@ROWCOUNT > 0
	EXEC [Events_RaiseEvent] 'DBSDLS', @Application_Id, @Script_Name


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_DeleteChangeScript] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_DeleteDatabaseProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a database provider'
GO

IF OBJECT_ID('[DatabaseChangeScripts_DeleteDatabaseProvider]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_DeleteDatabaseProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_DeleteDatabaseProvider]
(
	@Provider_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT, @Provider_Name SQL_VARIANT
    SET @Application_Id = (SELECT [Application_Id] 
                             FROM [DatabaseChangeScriptProviders] 
                            WHERE [Provider_Id] = @Provider_Id)
	SET @Provider_Name =  (SELECT [Provider_Name] 
                             FROM [Providers] 
                            WHERE [Provider_Id] = @Provider_Id)

    DELETE [DatabaseChangeScriptExecutions]
	 WHERE [Provider_Id] = @Provider_Id
	 
	DELETE [DatabaseChangeScriptProviders]
	 WHERE [Provider_Id] = @Provider_Id

	DELETE [Providers]
	 WHERE [Provider_Id] = @Provider_Id

	EXEC [Events_RaiseEvent] 'DBSDEL', @Application_Id, @Provider_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_DeleteDatabaseProvider] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_EditChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates an existing database change script'
GO


IF OBJECT_ID('[DatabaseChangeScripts_EditChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_EditChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_EditChangeScript]
(
	@Application_Id INT,
	@Script_Id INT,
	@Release_Number RELEASENUMBER,
	@Deployable_Id INT,
	@Script_Name VARCHAR(50),
	@Active_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [DatabaseChangeScripts]
	   SET [Application_Id] = @Application_Id
		  ,[Release_Number] = @Release_Number
		  ,[Deployable_Id] = @Deployable_Id
		  ,[Script_Name] = @Script_Name
		  ,[Active_Indicator] = @Active_Indicator
		  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		  ,[ModifiedOn_Date] = GETUTCDATE()
	WHERE [Script_Id] = @Script_Id	
	IF @@ROWCOUNT = 0 RAISERROR('DbChangeScript #%i does not exist.',16,1,@Script_Id)

	EXEC [Events_RaiseEvent] 'DBSMDS', @Application_Id, @Script_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_EditChangeScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DatabaseChangeScripts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified database change script'
GO


IF OBJECT_ID('[DatabaseChangeScripts_GetChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_GetChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetChangeScript]
(
	@Application_Id INT,
	@Script_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
      FROM [DatabaseChangeScripts_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Script_Id] = @Script_Id


END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetChangeScript] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetChangeScripts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DatabaseChangeScripts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the database change scripts that match the specified criteria'
GO


IF OBJECT_ID('[DatabaseChangeScripts_GetChangeScripts]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_GetChangeScripts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetChangeScripts]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER = NULL,
	@IncludeInactive_Indicator YNINDICATOR = NULL,
	@DatabaseChangeScript_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SET @IncludeInactive_Indicator = ISNULL(@IncludeInactive_Indicator, 'N')

	IF ISNULL(@DatabaseChangeScript_Count, 0) > 0
		SET ROWCOUNT @DatabaseChangeScript_Count

	SELECT * 
      FROM [DatabaseChangeScripts_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
       AND (	[Active_Indicator] = 'Y'
			 OR @IncludeInactive_Indicator = 'Y' )
  ORDER BY [Numeric_Release_Number] DESC

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetChangeScripts] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetDatabaseChangeScriptExecution',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DatabaseChangeScriptExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified database change script execution details'
GO

IF OBJECT_ID('[DatabaseChangeScripts_GetDatabaseChangeScriptExecution]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_GetDatabaseChangeScriptExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetDatabaseChangeScriptExecution]
(
	@ScriptExecution_Id INT
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [DatabaseChangeScriptExecutions_Extended]
	 WHERE [ScriptExecution_Id] = @ScriptExecution_Id
	IF @@ROWCOUNT=0 RAISERROR('ScriptExecution #%i was not found.',16,1,@ScriptExecution_Id)

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetDatabaseChangeScriptExecution] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetDatabaseChangeScriptExecutions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DatabaseChangeScriptExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the database change script executions that have occurred matching the specified criteria'
GO

IF OBJECT_ID('[DatabaseChangeScripts_GetDatabaseChangeScriptExecutions]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_GetDatabaseChangeScriptExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetDatabaseChangeScriptExecutions]
(
	@Script_Id INT,
	@Application_Id INT,
	@IncludeInactive_Indicator YNINDICATOR = NULL,
	@Release_Number RELEASENUMBER,
	@Success_Indicator YNINDICATOR,
	@RowsToReturn_Count INT,
	@Environment_Id INT
) AS
BEGIN

	SET NOCOUNT ON
	
	IF @RowsToReturn_Count IS NOT NULL
		SET ROWCOUNT @RowsToReturn_Count

	SELECT *
	  FROM [DatabaseChangeScriptExecutions_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Script_Id] = COALESCE(@Script_Id, [Script_Id])
	   AND ([Script_Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	   AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
	   AND [Success_Indicator] = COALESCE(@Success_Indicator, [Success_Indicator])
	   AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])
	ORDER BY [ExecutedOn_Date] DESC

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetDatabaseChangeScriptExecutions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetDatabaseProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DatabaseChangeScriptProviders_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified database provider'
GO

IF OBJECT_ID('[DatabaseChangeScripts_GetDatabaseProvider]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_GetDatabaseProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetDatabaseProvider]
(
	@Provider_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [DatabaseChangeScriptProviders_Extended]
	 WHERE [Provider_Id] = @Provider_Id

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetDatabaseProvider] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetDatabaseProviders',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DatabaseChangeScriptProviders_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the database providers that match the specified criteria.'
GO


IF OBJECT_ID('[DatabaseChangeScripts_GetDatabaseProviders]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_GetDatabaseProviders]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetDatabaseProviders]
(
	@Application_Id INT,
	@Environment_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
      FROM [DatabaseChangeScriptProviders_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetDatabaseProviders] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_LogExecution',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Logs the execution of a database change script'
GO

IF OBJECT_ID('[DatabaseChangeScripts_LogExecution]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_LogExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_LogExecution]
(
	@Script_Id INT,
	@Success_Indicator YNINDICATOR,
	@Log_Text VARCHAR(MAX),
	@Execution_Id INT,
	@Provider_Id INT,

	@ScriptExecution_Id INT = NULL OUT

) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [DatabaseChangeScriptExecutions]
           ([Script_Id]
           ,[ExecutedOn_Date]
           ,[ExecutedBy_User_Name]
           ,[Success_Indicator]
           ,[Log_Text]
           ,[Execution_Id]
           ,[Provider_Id])
	VALUES
           (@Script_Id
           ,GETUTCDATE()
           ,dbo.[Session_GetUserName]()
           ,@Success_Indicator
           ,@Log_Text
           ,@Execution_Id
           ,@Provider_Id)

	    SET @ScriptExecution_Id = SCOPE_IDENTITY()
	DECLARE  @Application_Id INT
		SET @Application_Id = (SELECT [Application_Id] FROM [DatabaseChangeScripts] 
	                            WHERE [Script_Id] = @Script_Id)

	EXEC [Events_RaiseEvent] 'DBSEXC', @Application_Id, @ScriptExecution_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_LogExecution] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_AddOrRemoveServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Associates or disassociates a server from a specified environment'
GO


IF OBJECT_ID('[Environments_AddOrRemoveServer]') IS NOT NULL DROP PROCEDURE [Environments_AddOrRemoveServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_AddOrRemoveServer]
(
	@Server_Id INT,
	@Environment_Id INT,
	@AddOrRemove_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @AddOrRemove_Code = 'R'
		DELETE [EnvironmentServers]
		 WHERE [Server_Id] = @Server_Id
		   AND [Environment_Id] = @Environment_Id

	ELSE IF @AddOrRemove_Code = 'A' 
		BEGIN
			IF NOT EXISTS
				( SELECT * FROM [EnvironmentServers]
				   WHERE [Server_Id] = @Server_Id
					 AND [Environment_Id] = @Environment_Id )
			INSERT INTO [EnvironmentServers] ([Server_Id], [Environment_Id])
			VALUES (@Server_Id, @Environment_Id)
		END
	ELSE
		RAISERROR('"%s" is an invalid AddOrRemove_Code, must be "A" or "R"', 16, 1, @AddOrRemove_Code)

	EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_AddOrRemoveServer] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_AddOrRemoveServerFromGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds or removes a server from a specified server group'
GO


IF OBJECT_ID('[Environments_AddOrRemoveServerFromGroup]') IS NOT NULL DROP PROCEDURE [Environments_AddOrRemoveServerFromGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_AddOrRemoveServerFromGroup]
(
	@Server_Id INT,
	@ServerGroup_Server_Id INT,
	@AddOrRemove_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @AddOrRemove_Code = 'R'
		DELETE [ServerGroups]
		 WHERE [Server_Id] = @Server_Id
		   AND [ServerGroup_Server_Id] = @ServerGroup_Server_Id

	ELSE IF @AddOrRemove_Code = 'A' 
		BEGIN
			IF NOT EXISTS
				( SELECT * FROM [ServerGroups]
				   WHERE [Server_Id] = @Server_Id
				     AND [ServerGroup_Server_Id] = @ServerGroup_Server_Id)
			INSERT INTO [ServerGroups] ([ServerGroup_Server_Id], [Server_Id])
			VALUES (@ServerGroup_Server_Id, @Server_Id)
		END
	ELSE
		RAISERROR('"%s" is an invalid AddOrRemove_Code, must be "A" or "R"', 16, 1, @AddOrRemove_Code)

	EXEC [Events_RaiseEvent] 'ENVMDS', NULL, @ServerGroup_Server_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_AddOrRemoveServerFromGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_CreateOrUpdateAutomaticRequirement',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Requirement_Sequence',
    /* Description_Text        */ 'Creates a new automatic environment promotion requirement or updates an existing one.'
GO

IF OBJECT_ID('[Environments_CreateOrUpdateAutomaticRequirement]') IS NOT NULL 
	DROP PROCEDURE [Environments_CreateOrUpdateAutomaticRequirement]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_CreateOrUpdateAutomaticRequirement]
(
	@Environment_Id INT,
	@Requirement_Description NVARCHAR(1000),
	@Requirement_Configuration XML,
	@Requirement_Sequence INT = NULL OUT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Requirement_Sequence IS NULL
	BEGIN

		SET @Requirement_Sequence = COALESCE((SELECT MAX([Requirement_Sequence])
		                                        FROM [EnvironmentAutomaticRequirements]
									           WHERE [Environment_Id] = @Environment_Id), 0) + 1

		INSERT INTO [EnvironmentAutomaticRequirements]
		(
			[Environment_Id],
			[Requirement_Sequence],
			[Requirement_Description],
			[Requirement_Configuration],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@Environment_Id,
			@Requirement_Sequence,
			@Requirement_Description,
			@Requirement_Configuration,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

	END
	ELSE
	BEGIN

		UPDATE [EnvironmentAutomaticRequirements]
		   SET [Requirement_Description] = @Requirement_Description,
		       [Requirement_Configuration] = @Requirement_Configuration,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Environment_Id] = @Environment_Id
		   AND [Requirement_Sequence] = @Requirement_Sequence

	END

	EXEC [Promotions_InvalidateApprovals] NULL, @Environment_Id

	EXEC [Events_RaiseEvent] 'ENVMOD', @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_CreateOrUpdateAutomaticRequirement] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_CreateOrUpdateEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Environment_Id',
    /* Description_Text        */ 'Creates a new environment or updates an existing one'
GO


IF OBJECT_ID('[Environments_CreateOrUpdateEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_CreateOrUpdateEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_CreateOrUpdateEnvironment]
(
	@Environment_Id INT = NULL OUT,
	@Environment_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@Environment_Id IS NULL) BEGIN
		DECLARE @Display_Sequence INT
		 SELECT @Display_Sequence = COALESCE(MAX([Display_Sequence]),0)+1
		   FROM [Environments]
		BEGIN
			INSERT INTO [Environments] 
			(
				[Environment_Name],
				[Display_Sequence],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date]
			)
			VALUES 
			(
				@Environment_Name,
				@Display_Sequence,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE()
			)
			SET @Environment_Id = SCOPE_IDENTITY()
			INSERT INTO [EnvironmentServers] ([Environment_Id],[Server_Id]) VALUES(@Environment_Id,1)

			EXEC [Events_RaiseEvent] 'ENVNEW', NULL, @Environment_Id
		END

	END ELSE
		BEGIN
			UPDATE [Environments]
			   SET [Environment_Name] = @Environment_Name,
				   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
				   [ModifiedOn_Date] = GETUTCDATE()
			 WHERE [Environment_Id] = @Environment_Id

			EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id
		END



	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_CreateOrUpdateEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_CreateOrUpdateManualRequirement',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Requirement_Sequence',
    /* Description_Text        */ 'Creates a new manual promotion requirement or updates an existing one.'
GO

IF OBJECT_ID('[Environments_CreateOrUpdateManualRequirement]') IS NOT NULL 
	DROP PROCEDURE [Environments_CreateOrUpdateManualRequirement]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_CreateOrUpdateManualRequirement]
(
	@Environment_Id INT,
	@Principal_Name USERNAME,
	@RequirementType_Code CHAR(1),
	@Requirement_Description NVARCHAR(1000),
	@RequiredInGroup_Count INT = NULL,
	@Requirement_Sequence INT = NULL OUT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Requirement_Sequence IS NULL
	BEGIN

		SET @Requirement_Sequence = COALESCE((SELECT MAX([Requirement_Sequence])
		                                        FROM [EnvironmentManualRequirements]
									           WHERE [Environment_Id] = @Environment_Id), 0) + 1

		INSERT INTO [EnvironmentManualRequirements]
		(
			[Environment_Id],
			[Requirement_Sequence],
			[Principal_Name],
			[RequiredInGroup_Count],
			[Requirement_Description],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@Environment_Id,
			@Requirement_Sequence,
			@Principal_Name,
			CASE WHEN @RequirementType_Code = 'G' THEN @RequiredInGroup_Count ELSE NULL END,
			@Requirement_Description,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

	END
	ELSE
	BEGIN

		UPDATE [EnvironmentManualRequirements]
		   SET [Principal_Name] = @Principal_Name,
		       [Requirement_Description] = @Requirement_Description,
		       [RequiredInGroup_Count] = CASE WHEN @RequirementType_Code = 'G' THEN @RequiredInGroup_Count ELSE NULL END,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Environment_Id] = @Environment_Id
		   AND [Requirement_Sequence] = @Requirement_Sequence

	END

	EXEC [Promotions_InvalidateApprovals] NULL, @Environment_Id

	EXEC [Events_RaiseEvent] 'ENVMOD', @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_CreateOrUpdateManualRequirement] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_CreateOrUpdateServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Server_Id',
    /* Description_Text        */ 'Creates a new server, or updates an existing one.'
GO


IF OBJECT_ID('[Environments_CreateOrUpdateServer]') IS NOT NULL DROP PROCEDURE [Environments_CreateOrUpdateServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_CreateOrUpdateServer]
(
	@Server_Id INT = NULL OUT,
	@Server_Name VARCHAR(50),
	@ServerType_Code CHAR(1),
	@Agent_Configuration VARCHAR(MAX),
	@Active_Indicator YNINDICATOR,
	@EnvironmentIds_Csv VARCHAR(MAX) = NULL,
	@ServerIdsInGroup_Csv VARCHAR(MAX) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@Server_Id IS NULL)
		BEGIN
			INSERT INTO [Servers] 
				(
					[Server_Name],
					[CreatedBy_User_Name],
					[CreatedOn_Date],
					[ModifiedBy_User_Name],
					[ModifiedOn_Date],
					[ServerType_Code],
					[Agent_Configuration],
					[ServerStatus_Code],
					[Active_Indicator]
				)
			VALUES
				(
					@Server_Name,
					dbo.Session_GetUserName(),
					GETUTCDATE(),
					dbo.Session_GetUserName(),
					GETUTCDATE(),
					@ServerType_Code,
					@Agent_Configuration,
					'UNKWN',
					@Active_Indicator
				)
			SET @Server_Id = SCOPE_IDENTITY()

			EXEC [Events_RaiseEvent] 'ENVNWS', NULL, @Server_Id
		END

	ELSE
		BEGIN
			UPDATE [Servers]
			   SET [Server_Name] = @Server_Name,
				   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
				   [ModifiedOn_Date] = GETUTCDATE(),
				   [ServerType_Code] = @ServerType_Code,
				   [Agent_Configuration] = @Agent_Configuration,
				   [Active_Indicator] = @Active_Indicator
			 WHERE [Server_Id] = @Server_Id

			EXEC [Events_RaiseEvent] 'ENVMDS', NULL, @Server_Id
		END

	IF (@EnvironmentIds_Csv IS NOT NULL) BEGIN
		DELETE [EnvironmentServers] WHERE [Server_Id] = @Server_Id
		INSERT [EnvironmentServers] ([Server_Id], [Environment_Id])
		SELECT @Server_Id, [Value_Text] FROM dbo.CsvToTable(@EnvironmentIds_Csv, ',')
	END

	IF (@ServerIdsInGroup_Csv IS NOT NULL) BEGIN
		DELETE [ServerGroups] WHERE [ServerGroup_Server_Id] = @Server_Id
		INSERT [ServerGroups] ([ServerGroup_Server_Id], [Server_Id])
		SELECT @Server_Id, [Value_Text] FROM dbo.CsvToTable(@ServerIdsInGroup_Csv, ',')
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_CreateOrUpdateServer] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_DeleteEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Soft deletes an environment'
GO

IF OBJECT_ID('[Environments_DeleteEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_DeleteEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_DeleteEnvironment]
(
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	IF EXISTS(SELECT * FROM WorkflowSteps WHERE [Environment_Id] = @Environment_Id)
	BEGIN
		RAISERROR('Environment %i is currently associated with a workflow and cannot be removed.', 16, 1, @Environment_Id)
	END

	UPDATE [Environments] 
	   SET [Active_Indicator] = 'N'
	 WHERE [Environment_Id] = @Environment_Id
	 
	EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_DeleteEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_DeleteServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a server.'
GO


IF OBJECT_ID('[Environments_DeleteServer]') IS NOT NULL DROP PROCEDURE [Environments_DeleteServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_DeleteServer]
(
	@Server_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Server_Name SQL_VARIANT
	    SET @Server_Name = (SELECT [Server_Name] FROM [Servers] WHERE [Server_Id] = @Server_Id)

    UPDATE [ActionGroups]
       SET [Server_Id] = NULL
     WHERE [Server_Id] = @Server_Id

	UPDATE [DeploymentPlanActionGroups]
	   SET [Server_Id] = NULL
	 WHERE [Server_Id] = @Server_Id

	DELETE [EnvironmentServers]
	 WHERE [Server_Id] = @Server_Id

	DELETE [ServerGroups]
	 WHERE [ServerGroup_Server_Id] = @Server_Id
	    OR [Server_Id] = @Server_Id

	UPDATE [ConfigurationFileDeployments]
	   SET [Server_Id] = NULL
	 WHERE [Server_Id] = @Server_Id

	UPDATE [ActionGroupActions]
	   SET [Server_Id] = NULL
	 WHERE [Server_Id] = @Server_Id

    UPDATE [ActionGroupActions]
	   SET [Target_Server_Id] = NULL
	 WHERE [Target_Server_Id] = @Server_Id

	DELETE [Servers]
	 WHERE [Server_Id] = @Server_Id

	IF @@ROWCOUNT>0
	EXEC [Events_RaiseEvent] 'ENVDLS', NULL, @Server_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_DeleteServer] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Environments,Servers,BuildPromotionRequirements_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an environment'
GO


IF OBJECT_ID('[Environments_GetEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_GetEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetEnvironment]
(
	@Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT OFF 

	SELECT * 
	  FROM [Environments]
     WHERE [Environment_Id] = @Environment_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Environment %i does not exist.', 16, 1, @Environment_Id)

	SELECT *
	  FROM [Servers]
	 WHERE [Server_Id] IN (SELECT [Server_Id] FROM [EnvironmentServers]
	                        WHERE [Environment_Id] = @Environment_Id)

    SELECT * 
      FROM [BuildPromotionRequirements_Extended]
     WHERE [Environment_Id] = @Environment_Id
       AND [Workflow_Id] IS NULL

END
GO

GRANT EXECUTE ON [Environments_GetEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetEnvironments',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Environments',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the environments for a specified application, or all environments if no application is provided.'
GO

IF OBJECT_ID('[Environments_GetEnvironments]') IS NOT NULL DROP PROCEDURE [Environments_GetEnvironments]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetEnvironments]
(
	@Application_Id INT = NULL
)
AS
BEGIN

	IF @Application_Id IS NULL
	BEGIN

		/* All environments */
		SELECT E.*
		  FROM [Environments] E
		 WHERE E.[Active_Indicator] = 'Y'
		 ORDER BY [Display_Sequence]

	END
	ELSE
	BEGIN

		/* Only environments for the specified application */
		SELECT E.*
		  FROM [Environments] E
		 WHERE E.[Active_Indicator] = 'Y'
		   AND E.[Environment_Id] IN (SELECT WS.[Environment_Id]
		                                FROM [WorkflowSteps] WS
									         INNER JOIN [ApplicationWorkflows] AW
											         ON AW.[Workflow_Id] = WS.[Workflow_Id]
									   WHERE AW.[Application_Id] = @Application_Id)
		 ORDER BY E.[Display_Sequence]

	END

END
GO

GRANT EXECUTE ON [Environments_GetEnvironments] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetEnvironmentsAndServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Environments,EnvironmentServers_Extended,Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a dataset containing all environments, all environment/server relations, and servers'
GO


IF OBJECT_ID('[Environments_GetEnvironmentsAndServers]') IS NOT NULL DROP PROCEDURE [Environments_GetEnvironmentsAndServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetEnvironmentsAndServers]
(
	@IncludeInactive_Environments_Indicator YNINDICATOR,
	@IncludeInactive_Servers_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [Environments] 
	 WHERE [Active_Indicator] = 'Y' OR @IncludeInactive_Environments_Indicator = 'Y'
	 ORDER BY [Display_Sequence]

	SELECT * FROM [EnvironmentServers_Extended] 
	 WHERE [Environment_Id] IS NULL OR
	       (([Environment_Active_Indicator] = 'Y' OR @IncludeInactive_Environments_Indicator = 'Y') AND
	       ([Server_Active_Indicator] = 'Y' OR @IncludeInactive_Servers_Indicator = 'Y'))
	 ORDER BY [Server_Name]

	SELECT *
	  FROM [Servers]
     WHERE @IncludeInactive_Servers_Indicator = 'Y' OR [Active_Indicator] = 'Y'
  ORDER BY CASE WHEN [Server_Id] = 1 THEN 1 ELSE 2 END,
	       [Server_Name]

END
GO

GRANT EXECUTE ON [Environments_GetEnvironmentsAndServers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Servers,Environments,LogMessages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specific server including its environments and any log messages.'
GO


IF OBJECT_ID('[Environments_GetServer]') IS NOT NULL DROP PROCEDURE [Environments_GetServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetServer]
(
	@Server_Id INT,
	@LogMessage_Count INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Servers]
	 WHERE [Server_Id] = @Server_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Server %i does not exist.', 16, 1, @Server_Id)

	SELECT *
	  FROM [Environments]
	 WHERE [Environment_Id] IN (SELECT [Environment_Id] FROM [EnvironmentServers]
	                             WHERE [Server_Id] = @Server_Id)

	IF @LogMessage_Count IS NOT NULL
		SET ROWCOUNT @LogMessage_Count

	SELECT *
	  FROM [LogMessages_Extended]
	 WHERE [Server_Id] = @Server_Id

END
GO

GRANT EXECUTE ON [Environments_GetServer] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the servers in the system.'
GO


IF OBJECT_ID('[Environments_GetServers]') IS NOT NULL DROP PROCEDURE [Environments_GetServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetServers]
(
	@IncludeInactive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Servers]
	 WHERE @IncludeInactive_Indicator = 'Y' OR [Active_Indicator] = 'Y'
  ORDER BY CASE WHEN [Server_Id] = 1 THEN 1 ELSE 2 END,
	       [Server_Name] 

END
GO

GRANT EXECUTE ON [Environments_GetServers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetServersInGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the servers in a specified server group.'
GO


IF OBJECT_ID('[Environments_GetServersInGroup]') IS NOT NULL DROP PROCEDURE [Environments_GetServersInGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetServersInGroup]
(
	@ServerGroup_Server_Id INT,
	@IncludeInactive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT S.*
	  FROM [Servers] S
	  INNER JOIN [ServerGroups] SG
	          ON SG.[Server_Id] = S.[Server_Id]
	 WHERE (SG.[ServerGroup_Server_Id] = @ServerGroup_Server_Id) AND
	       (@IncludeInactive_Indicator = 'Y' OR S.[Active_Indicator] = 'Y')
	 ORDER BY S.[Server_Name]

END
GO

GRANT EXECUTE ON [Environments_GetServersInGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_ResequenceEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Resequences the specified environment by placing it where another environment is sequenced'
GO


IF OBJECT_ID('[Environments_ResequenceEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_ResequenceEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_ResequenceEnvironment]
(
	@Environment_Id INT,
	@New_Environment_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Environment_Sequence INT
	SELECT @Environment_Sequence = [Display_Sequence]
	  FROM [Environments]
	 WHERE [Environment_Id] = @Environment_Id

	IF @Environment_Sequence IS NULL BEGIN
	   RAISERROR('Environment (ID=%i) was not found.', 16, 1, @Environment_Id)
	   RETURN
    END

	UPDATE [Environments] 
	   SET [Display_Sequence] = CASE
	       WHEN /* id */ [Environment_Id] = @Environment_Id THEN @New_Environment_Sequence

		   WHEN /* up */ @New_Environment_Sequence > @Environment_Sequence THEN CASE
		        WHEN [Display_Sequence] BETWEEN @Environment_Sequence AND @New_Environment_Sequence THEN [Display_Sequence] - 1
				ELSE [Display_Sequence] END
		   WHEN /* dn */ @New_Environment_Sequence < @Environment_Sequence THEN CASE
		        WHEN [Display_Sequence] BETWEEN @New_Environment_Sequence AND @Environment_Sequence THEN [Display_Sequence] + 1
				ELSE [Display_Sequence] END
		   ELSE @New_Environment_Sequence END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_ResequenceEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_UndeleteEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Undeletes the specified environment'
GO


IF OBJECT_ID('[Environments_UndeleteEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_UndeleteEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_UndeleteEnvironment]
(
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Environments] 
	   SET [Active_Indicator] = 'Y'
	 WHERE [Environment_Id] = @Environment_Id

    EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_UndeleteEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_UpdateEnvironmentServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Assigns all server IDs passed in as a CSV to the specified environment'
GO


IF OBJECT_ID('[Environments_UpdateEnvironmentServers]') IS NOT NULL DROP PROCEDURE [Environments_UpdateEnvironmentServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_UpdateEnvironmentServers]
(
	@Environment_Id INT,
	@ServerIdsInEnvironment_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION       


        DELETE [EnvironmentServers] WHERE [Environment_Id] = @Environment_Id

        INSERT [EnvironmentServers] ([Environment_Id], [Server_Id])
        SELECT @Environment_Id, [Value_Text] 
          FROM [dbo].[CsvToTable] (@ServerIdsInEnvironment_Csv, ',')


	    
        EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id
        
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_UpdateEnvironmentServers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_UpdateServerGroupServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Assigns all server IDs passed in as a CSV to the specified server group'
GO


IF OBJECT_ID('[Environments_UpdateServerGroupServers]') IS NOT NULL DROP PROCEDURE [Environments_UpdateServerGroupServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_UpdateServerGroupServers]
(
	@ServerGroup_Server_Id INT,
	@ServerIdsInGroup_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

    

BEGIN TRY
	BEGIN TRANSACTION       


        DELETE [ServerGroups] WHERE [ServerGroup_Server_Id] = @ServerGroup_Server_Id

        INSERT [ServerGroups] ([ServerGroup_Server_Id], [Server_Id])
        SELECT @ServerGroup_Server_Id, [Value_Text] 
          FROM [dbo].[CsvToTable] (@ServerIdsInGroup_Csv, ',')


	    
        EXEC [Events_RaiseEvent] 'ENVMDS', NULL, @ServerGroup_Server_Id
        
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_UpdateServerGroupServers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_UpdateServerStatus',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Environments_UpdateServerStatus]') IS NOT NULL DROP PROCEDURE [Environments_UpdateServerStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_UpdateServerStatus]
(
	@Server_Id INT,
	@ServerStatus_Code CHAR(5)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

        DECLARE @Old_ServerStatus_Code CHAR(5)
        
        SELECT @Old_ServerStatus_Code = [ServerStatus_Code]
	      FROM [Servers]
	     WHERE [Server_Id] = @Server_Id

	    UPDATE [Servers]
	       SET [ServerStatus_Code] = @ServerStatus_Code
	     WHERE [Server_Id] = @Server_Id
	     
	     -- raise server error event if the status changes to ERROR from something else
	     IF (@ServerStatus_Code = 'ERROR' AND @Old_ServerStatus_Code <> 'ERROR')
	        EXEC [Events_RaiseEvent] 'SRVERR', NULL, @Server_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_UpdateServerStatus] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'EventListeners_CreateOrUpdateEventListeners',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new event listener, or updates an existing one'
GO

IF OBJECT_ID('[EventListeners_CreateOrUpdateEventListeners]') IS NOT NULL
	DROP PROCEDURE [EventListeners_CreateOrUpdateEventListeners]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [EventListeners_CreateOrUpdateEventListeners]
(
	@EventListener_Description NVARCHAR(MAX),
	@EventListener_Configuration XML,
	@Owner_User_Name USERNAME,
	@Active_Indicator YNINDICATOR,

	@WorkflowStep_Workflow_Id INT,
	@WorkflowStep_Environment_Id INT,

    @BuildStep_Workflow_Id INT,

	@EventCodeList_Csv VARCHAR(MAX),
	@ApplicationIdList_Csv VARCHAR(MAX),
	@ApplicationGroupIdList_Csv VARCHAR(MAX),
	@EnvironmentIdList_Csv VARCHAR(MAX),

	@EventListener_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @EventListener_Id IS NULL BEGIN

		INSERT INTO [EventListeners]
			([EventListener_Description],[EventListener_Configuration],[Owner_User_Name],[Active_Indicator],
			 [WorkflowStep_Workflow_Id],[WorkflowStep_Environment_Id],[BuildStep_Workflow_Id])
		 VALUES
			(@EventListener_Description,@EventListener_Configuration,@Owner_User_Name,@Active_Indicator,
			 @WorkflowStep_Workflow_Id,@WorkflowStep_Environment_Id, @BuildStep_Workflow_Id)
		
		SET @EventListener_Id = SCOPE_IDENTITY()

	END ELSE BEGIN

		UPDATE [EventListeners]
		   SET [EventListener_Description] = @EventListener_Description,
		       [EventListener_Configuration] = @EventListener_Configuration,
		       [Owner_User_Name] = @Owner_User_Name,
		       [Active_Indicator] = @Active_Indicator
		 WHERE [EventListener_Id] = @EventListener_Id
	END

	DELETE [EventListenerApplications] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerApplicationGroups] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerEnvironments] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerEvents] WHERE [EventListener_Id] = @EventListener_Id

	INSERT INTO [EventListenerApplications] ([EventListener_Id],[Application_Id])
	SELECT @EventListener_Id, V.[Value_Text] 
	 FROM dbo.CsvToTable(@ApplicationIdList_Csv,',') V

	INSERT INTO [EventListenerApplicationGroups] ([EventListener_Id],[ApplicationGroup_Id])
	SELECT @EventListener_Id, V.[Value_Text] 
	 FROM dbo.CsvToTable(@ApplicationGroupIdList_Csv,',') V

	INSERT INTO [EventListenerEnvironments] ([EventListener_Id],[Environment_Id])
	SELECT @EventListener_Id, V.[Value_Text] 
	 FROM dbo.CsvToTable(@EnvironmentIdList_Csv,',') V

	INSERT INTO [EventListenerEvents] ([EventListener_Id],[Event_Code])
	SELECT @EventListener_Id, V.[Value_Text] 
	 FROM dbo.CsvToTable(@EventCodeList_Csv,',') V


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [EventListeners_CreateOrUpdateEventListeners] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'EventListeners_DeleteEventListener',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a EventListener from the system'
GO

IF OBJECT_ID('[EventListeners_DeleteEventListener]') IS NOT NULL
	DROP PROCEDURE [EventListeners_DeleteEventListener]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [EventListeners_DeleteEventListener]
(
	@EventListener_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [EventListenerEvents] WHERE [EventListener_Id] = @EventListener_Id

	DELETE [EventListenerApplications] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerApplicationGroups] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerEnvironments] WHERE [EventListener_Id] = @EventListener_Id
	DELETE [EventListenerEvents] WHERE [EventListener_Id] = @EventListener_Id

	DELETE [EventListeners] WHERE [EventListener_Id] = @EventListener_Id
	

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [EventListeners_DeleteEventListener] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'EventListeners_GetEventListener',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventListeners_Extended,EventListenerApplications,EventListenerApplicationGroups,EventListenerEnvironments,EventListenerEvents',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a EventListener and a list of event codes that the EventListener is subscribed to'
GO

IF OBJECT_ID('[EventListeners_GetEventListener]') IS NOT NULL
	DROP PROCEDURE [EventListeners_GetEventListener]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [EventListeners_GetEventListener]
(
	@EventListener_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [EventListeners_Extended] WHERE [EventListener_Id] = @EventListener_Id
	SELECT * FROM [EventListenerApplications] WHERE [EventListener_Id] = @EventListener_Id
	SELECT * FROM [EventListenerApplicationGroups] WHERE [EventListener_Id] = @EventListener_Id
	SELECT * FROM [EventListenerEnvironments] WHERE [EventListener_Id] = @EventListener_Id
	SELECT * FROM [EventListenerEvents] WHERE [EventListener_Id] = @EventListener_Id

END
GO

GRANT EXECUTE ON [EventListeners_GetEventListener] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'EventListeners_GetEventListeners',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventListeners_Extended,EventListenerApplications,EventListenerApplicationGroups,EventListenerEnvironments,EventListenerEvents',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of EventListeners for the specified user, or all users if no user is specified'
GO

IF OBJECT_ID('[EventListeners_GetEventListeners]') IS NOT NULL
	DROP PROCEDURE [EventListeners_GetEventListeners]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [EventListeners_GetEventListeners]
(
	@Owner_User_Name USERNAME = NULL
)
AS
BEGIN

SET NOCOUNT ON

	DECLARE @IDS TABLE ([Id] INT)
	
	INSERT INTO @IDS
	SELECT [EventListener_Id] FROM [EventListeners] 
	 WHERE [Owner_User_Name] = COALESCE(@Owner_User_Name,[Owner_User_Name])
	    OR (@Owner_User_Name IS NULL AND [Owner_User_Name] IS NULL)

	SELECT * FROM [EventListeners_Extended] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)
	SELECT * FROM [EventListenerApplications] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)
	SELECT * FROM [EventListenerApplicationGroups] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)
	SELECT * FROM [EventListenerEnvironments] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)
	SELECT * FROM [EventListenerEvents] WHERE [EventListener_Id] IN (SELECT [Id] FROM @IDS)

END
GO

GRANT EXECUTE ON [EventListeners_GetEventListeners] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetEventListeners',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventListeners_ForEvent,EventOccurenceDetails,EventListenerEnvironments,EventListenerApplications,EventListenerApplicationGroups,ApplicationWorkflows',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the EventListeners that have occurred since the date specified.'
GO

IF OBJECT_ID('[Events_GetEventListeners]') IS NOT NULL
	DROP PROCEDURE [Events_GetEventListeners]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_GetEventListeners]
(
	@From_Date DATETIME
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  INTO #EN
	  FROM [EventListeners_ForEvent]
	 WHERE [Occurence_Date] >= @From_Date

	SELECT *
	  FROM #EN

	SELECT DISTINCT EOD.*
	  FROM [EventOccurenceDetails] EOD
     INNER JOIN #EN ON #EN.[EventOccurence_Id] = EOD.[EventOccurence_Id]

	SELECT ELE.*
	  FROM [EventListenerEnvironments] ELE
     INNER JOIN #EN ON #EN.[EventListener_Id] = ELE.[EventListener_Id]

    SELECT ELA.*
	  FROM [EventListenerApplications] ELA
     INNER JOIN #EN ON #EN.[EventListener_Id] = ELA.[EventListener_Id]

    SELECT ELAG.*
	  FROM [EventListenerApplicationGroups] ELAG
     INNER JOIN #EN ON #EN.[EventListener_Id] = ELAG.[EventListener_Id]

    SELECT AW.* 
      FROM [ApplicationWorkflows] AW
     INNER JOIN #EN 
             ON #EN.[WorkflowStep_Workflow_Id] = AW.[Workflow_Id]
             OR #EN.[BuildStep_Workflow_Id] = AW.[Workflow_Id]

END
GO

GRANT EXECUTE ON [Events_GetEventListeners] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetEventOccurence',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventOccurences_Extended,EventOccurenceDetails',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Get a specific event occurence along with its event details'
GO

IF OBJECT_ID('[Events_GetEventOccurence]') IS NOT NULL
	DROP PROCEDURE [Events_GetEventOccurence]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_GetEventOccurence]
(
	@EventOccurence_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [EventOccurences_Extended]
	 WHERE [EventOccurence_Id] = @EventOccurence_Id
	IF @@ROWCOUNT=0 RAISERROR('Event #%i does not exist.', 16,1,@EventOccurence_Id)

	SELECT *
	  FROM [EventOccurenceDetails]
	 WHERE [EventOccurence_Id] = @EventOccurence_Id

END
GO

GRANT EXECUTE ON [Events_GetEventOccurence] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetEventOccurences',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'EventOccurences_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the event occurences in the system for a specific application, or all applications, limited to a specified count if provided'
GO

IF OBJECT_ID('[Events_GetEventOccurences]') IS NOT NULL
	DROP PROCEDURE [Events_GetEventOccurences]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_GetEventOccurences]
(
	@Application_Id INT = NULL,
	@Event_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Event_Count IS NOT NULL
		SET ROWCOUNT @Event_Count

	SELECT *
	  FROM [EventOccurences_Extended]
	 WHERE ([Application_Id] = @Application_Id OR @Application_Id IS NULL)
	 ORDER BY [Occurence_Date] DESC

END
GO

GRANT EXECUTE ON [Events_GetEventOccurences] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetNotifiers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventNotifiers,EventOccurenceDetails',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the notifiers that have arisen since the date specified'
GO

IF OBJECT_ID('[Events_GetNotifiers]') IS NOT NULL
	DROP PROCEDURE [Events_GetNotifiers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_GetNotifiers]
(
	@From_Date DATETIME
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  INTO #EN
	  FROM [EventNotifiers]
	 WHERE [Occurence_Date] >= @From_Date


	SELECT * FROM #EN

	SELECT *
	  FROM [EventOccurenceDetails]
	  WHERE [EventOccurence_Id] IN (SELECT [EventOccurence_Id] FROM #EN)

END
GO

GRANT EXECUTE ON [Events_GetNotifiers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_AppendActionLog',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Writes a log message to an action log.'
GO

IF OBJECT_ID('[Executions_AppendActionLog]') IS NOT NULL DROP PROCEDURE [Executions_AppendActionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_AppendActionLog]
(
	@BuildExecution_PlanAction_Id INT,
	@LogEntry_Level INT,
	@LogEntry_Text VARCHAR(MAX)
)
AS
BEGIN

	INSERT INTO [BuildExecution_ActionGroupActionLogEntries]
	(
		[BuildExecution_ActionGroupAction_Id],
		[LogEntry_Sequence],
		[LogEntry_Level],
		[LogEntry_Text]
	)
	SELECT @BuildExecution_PlanAction_Id,
		   COALESCE(MAX([LogEntry_Sequence]),0) + 1,
		   @LogEntry_Level,
		   @LogEntry_Text
	  FROM [BuildExecution_ActionGroupActionLogEntries] 
	  WITH (TABLOCKX) -- added table lock to resolve issue BM-897
	 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_PlanAction_Id

END
GO

GRANT EXECUTE ON [Executions_AppendActionLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_BeginAction',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'BuildExecution_ActionGroupAction_Id',
    /* Description_Text        */ 'Initiates execution of a specific action.'
GO


IF OBJECT_ID('[Executions_BeginAction]') IS NOT NULL DROP PROCEDURE [Executions_BeginAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_BeginAction]
(
	@BuildExecution_DeploymentPlan_Id INT,
	@DeploymentPlanActionGroup_Sequence INT,
	@Action_Sequence INT,
	@Server_Name VARCHAR(50),
	@Attempt_Sequence INT,
	@VariableValues_Xml XML,
	@ExecutionAction_Description VARCHAR(500),
	@BuildExecution_ActionGroupAction_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [BuildExecution_ActionGroupActions]
	(
		[ExecutionStarted_Date],
		[ExecutionStatus_Name],
		[ExecutionAction_Description],
		[Original_Server_Name],
		[Original_Action_Sequence],
		[Attempt_Sequence],
		[WarningLogEntry_Indicator],
		[BuildExecution_DeploymentPlan_Id],
		[Original_DeploymentPlanActionGroup_Sequence]
	)
	VALUES
	(
		GETUTCDATE(),
		'Executing',
		@ExecutionAction_Description,
		@Server_Name,
		@Action_Sequence,
		@Attempt_Sequence,
		'N',
		@BuildExecution_DeploymentPlan_Id,
		@DeploymentPlanActionGroup_Sequence
	)

	SET @BuildExecution_ActionGroupAction_Id = SCOPE_IDENTITY()

	INSERT INTO [BuildExecution_ActionGroupActionVariableValues]
	(
		[BuildExecution_ActionGroupAction_Id],
		[Variable_Name],
		[Value_Text]
	)
	SELECT [BuildExecution_PlanAction_Id] = @BuildExecution_ActionGroupAction_Id,
	       [Variable_Name] = V.value('@N', 'VARCHAR(50)'),
	       V.value('@V', 'VARCHAR(MAX)')
	  FROM @VariableValues_Xml.nodes('/Vars/Var') AS _V(V)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_BeginAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_BeginDeploymentPlan',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'BuildExecution_DeploymentPlan_Id',
    /* Description_Text        */ 'Initiates execution of a specific deployment plan.'
GO


IF OBJECT_ID('[Executions_BeginDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Executions_BeginDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_BeginDeploymentPlan]
(
	@Execution_Id INT,
	@DeploymentPlan_Id INT,
	@InitiatedBy_BuildExecution_ActionGroupAction_Id INT = NULL,
	@BuildExecution_DeploymentPlan_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @DeploymentPlan_Name VARCHAR(50)
	SELECT @DeploymentPlan_Name = [DeploymentPlan_Name]
	  FROM [DeploymentPlans]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	INSERT INTO [BuildExecution_DeploymentPlans]
	(
		[Execution_Id],
		[Original_DeploymentPlan_Id],
		[Original_DeploymentPlan_Name]
	)
	VALUES
	(
		@Execution_Id,
		@DeploymentPlan_Id,
		@DeploymentPlan_Name
	)

	SET @BuildExecution_DeploymentPlan_Id = SCOPE_IDENTITY()
	
	IF @InitiatedBy_BuildExecution_ActionGroupAction_Id IS NOT NULL
	BEGIN

		UPDATE [BuildExecution_ActionGroupActions]
		   SET [Nested_BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		 WHERE [BuildExecution_ActionGroupAction_Id] = @InitiatedBy_BuildExecution_ActionGroupAction_Id

	END
	ELSE IF @DeploymentPlan_Id > 0
	BEGIN

		UPDATE [BuildExecutions]
		   SET [Initial_BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
		 WHERE [Execution_Id] = @Execution_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_BeginDeploymentPlan] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_BeginDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Initiates execution of a specific deployment plan action group.'
GO


IF OBJECT_ID('[Executions_BeginDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Executions_BeginDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_BeginDeploymentPlanActionGroup]
(
	@BuildExecution_DeploymentPlan_Id INT,
	@DeploymentPlanActionGroup_Id INT,
	@Deployable_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
		IF @DeploymentPlanActionGroup_Id IS NOT NULL
		BEGIN

			INSERT INTO [BuildExecution_DeploymentPlanActionGroups]
			(
				[BuildExecution_DeploymentPlan_Id],
				[Original_DeploymentPlanActionGroup_Sequence],
				[Original_Deployable_Name],
				[Original_Deployable_Application_Name],
				[Original_DeploymentPlanActionGroup_Id],
				[Original_DeploymentPlanActionGroup_Name]
			)
			SELECT @BuildExecution_DeploymentPlan_Id,
			       DPAG.[DeploymentPlanActionGroup_Sequence],
			       D.[Deployable_Name],
				   D_A.[Application_Name],
			       @DeploymentPlanActionGroup_Id,
			       COALESCE(DPAG.[ActionGroup_Name], AG.[ActionGroup_Name])
			  FROM [DeploymentPlanActionGroups] DPAG
			       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
			               ON BEDP.[BuildExecution_DeploymentPlan_Id] = @BuildExecution_DeploymentPlan_Id
			       INNER JOIN [BuildExecutions] BE
				           ON BE.[Execution_Id] = BEDP.[Execution_Id]
					LEFT JOIN [Deployables] D
			               ON D.[Deployable_Id] = @Deployable_Id
			        LEFT JOIN [Applications] D_A
			               ON D_A.[Application_Id] = D.[Application_Id]
				          AND D_A.[Application_Id] <> BE.[Application_Id]
				   INNER JOIN [ActionGroups] AG
						   ON AG.[ActionGroup_Id] = DPAG.[ActionGroup_Id]
			 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

		 END
		 ELSE
		 BEGIN

			INSERT INTO [BuildExecution_DeploymentPlanActionGroups]
			(
				[BuildExecution_DeploymentPlan_Id],
				[Original_DeploymentPlanActionGroup_Sequence],
				[Original_DeploymentPlanActionGroup_Id]
			)
			VALUES
			(
				@BuildExecution_DeploymentPlan_Id,
				0,
				0
			)

		 END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_BeginDeploymentPlanActionGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_BeginExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the state of an execution from Pending to Executing.'
GO


IF OBJECT_ID('[Executions_BeginExecution]') IS NOT NULL DROP PROCEDURE [Executions_BeginExecution]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_BeginExecution]
(
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [BuildExecutions]
	   SET [ExecutionStatus_Name] = 'Executing',
	       [ExecutionStart_Date] = GETUTCDATE()
	 WHERE [Execution_Id] = @Execution_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Executions_BeginExecution] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CancelExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[dbo].[Executions_CancelExecution]') IS NOT NULL
	DROP PROCEDURE [dbo].[Executions_CancelExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Executions_CancelExecution]
(
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ExecutionStatus_Name VARCHAR(10)
	SET @ExecutionStatus_Name =
		(SELECT [ExecutionStatus_Name]
		   FROM [BuildExecutions]
		  WHERE [Execution_Id] = @Execution_Id)

	IF @ExecutionStatus_Name NOT IN ('Pending', 'Executing')
	BEGIN
		RAISERROR('This execution is not currently running; only executions in progress may be canceled.', 16, 1)
		RETURN
	END

	/* Pending executions immediately get failed; ones that are currently
	   executing will be canceled and picked up by the Plan Executer */
	UPDATE [BuildExecutions]
	   SET [ExecutionStatus_Name] = CASE WHEN @ExecutionStatus_Name = 'Executing' THEN 'Canceled' ELSE 'Failed' END
		  ,[CanceledBy_User_Name] = dbo.Session_GetUserName()
		  ,[CanceledOn_Date] = GETUTCDATE()
		  ,[ExecutionEnded_Date] = GETUTCDATE()
	 WHERE [Execution_Id] = @Execution_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Executions_CancelExecution] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_FailExecutingPlans',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_FailExecutingPlans]') IS NOT NULL DROP PROCEDURE [Executions_FailExecutingPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_FailExecutingPlans]
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	EXEC [Events_RaiseEvent] 'EXEFAL', NULL

	/* Fail all promotions */
	UPDATE [BuildPromotions]
	   SET [PromotionStatus_Name] = 'Failed'
	  FROM [BuildPromotions] BP
	       INNER JOIN [BuildExecutions] BE
	               ON BE.Application_Id = BP.Application_Id
	              AND BE.Release_Number = BP.Release_Number
	              AND BE.Build_Number = BP.Build_Number
	              AND BE.Environment_Id = BP.Environment_Id
	 WHERE BE.[ExecutionStatus_Name] IN ('Executing', 'Canceled')

	/* Fail all actions */
	UPDATE [BuildExecution_ActionGroupActions]
	   SET [ExecutionStatus_Name] = 'Failed',
	       [ExecutionEnded_Date] = GETUTCDATE()
	 WHERE [ExecutionStatus_Name] IN ('Executing', 'Canceled')

	/* Fail all executions */
	DECLARE @CUR CURSOR , @Application_Id INT, @Execution_Id INT
	    SET @CUR = CURSOR FOR (SELECT [Application_Id],[Execution_Id] 
	                             FROM [BuildExecutions] WHERE [ExecutionStatus_Name] IN ('Executing', 'Canceled'))
	   OPEN @CUR FETCH @CUR INTO @Application_Id, @Execution_Id
	WHILE @@FETCH_STATUS=0 BEGIN

		UPDATE [BuildExecutions]
		   SET [ExecutionStatus_Name] = 'Failed'
		 WHERE [Execution_Id] = @Execution_Id
	
		EXEC [Events_RaiseEvent] 'EXECPT', @Application_Id, @Execution_Id

		FETCH @CUR INTO @Application_Id, @Execution_Id
	END


	/* Kill all manual actions */
	DELETE [ManualActionsInProgress]


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH		 


END
GO

GRANT EXECUTE ON [Executions_FailExecutingPlans] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetExecutionsPendingCancellation',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_PendingCancellation',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns executions which are awaiting cancellation.'
GO


IF OBJECT_ID('[Executions_GetExecutionsPendingCancellation]') IS NOT NULL DROP PROCEDURE [Executions_GetExecutionsPendingCancellation]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetExecutionsPendingCancellation]
AS
BEGIN

	SET NOCOUNT ON
	
	SELECT *
	  FROM [BuildExecutions_PendingCancellation]

END
GO

GRANT EXECUTE ON [Executions_GetExecutionsPendingCancellation] TO [BuildMasterUser_Role]

GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetExecutionsToExecute',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns executions which are ready to be started by the executer.'
GO


IF OBJECT_ID('[Executions_GetExecutionsToExecute]') IS NOT NULL DROP PROCEDURE [Executions_GetExecutionsToExecute]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetExecutionsToExecute]
AS
BEGIN

	SET NOCOUNT ON
	
	SELECT BEX.*
	  FROM [BuildExecutions_Extended] BEX
	       INNER JOIN (SELECT BE.[Execution_Id],
	                          BE.[ExecutionStatus_Name],
	                          [Row_Number] = ROW_NUMBER() OVER (PARTITION BY BE.[Application_Id] ORDER BY CASE WHEN BE.[ExecutionStatus_Name] <> 'Pending' THEN 0 ELSE 1 END, BE.[ExecutionStart_Date])
	                     FROM [BuildExecutions] BE
	                    WHERE BE.[ExecutionStatus_Name] IN ('Pending', 'Executing', 'Canceled')
	                      AND BE.[ExecutionStart_Date] <= GETUTCDATE()) Active_BE
	               ON Active_BE.[Execution_Id] = BEX.[Execution_Id]
	 WHERE Active_BE.[Row_Number] = 1
	   AND Active_BE.[ExecutionStatus_Name] = 'Pending'

END
GO

GRANT EXECUTE ON [Executions_GetExecutionsToExecute] TO [BuildMasterUser_Role]

GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetManualActionsInProgress',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_GetManualActionsInProgress]') IS NOT NULL DROP PROCEDURE [Executions_GetManualActionsInProgress]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetManualActionsInProgress]
(
	@Application_Id INT = NULL,
	@AssignedTo_Principal_Name VARCHAR(255) = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [ManualActionsInProgress_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@AssignedTo_Principal_Name IS NULL OR [AssignedTo_Principal_Name] = @AssignedTo_Principal_Name)
	   AND [ActionComplete_Indicator] = 'N'
	
END
GO

GRANT EXECUTE ON [Executions_GetManualActionsInProgress] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetManualActionsInProgressForUser',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_GetManualActionsInProgressForUser]') IS NOT NULL DROP PROCEDURE [Executions_GetManualActionsInProgressForUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetManualActionsInProgressForUser]
(
	@Current_User_Name VARCHAR(255),
	@Current_User_Groups_Csv VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [ManualActionsInProgress_Extended]
	 WHERE [ActionComplete_Indicator] = 'N'
	   AND [AssignedTo_Principal_Name] IN 
	        (SELECT [Value_text] FROM [dbo].[CsvToTable](@Current_User_Groups_Csv, ',')
             UNION ALL 
             SELECT @Current_User_Name)
	       
	
END
GO

GRANT EXECUTE ON [Executions_GetManualActionsInProgressForUser] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetOrSetManualActionComplete',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Executions_GetOrSetManualActionComplete]') IS NOT NULL DROP PROCEDURE [Executions_GetOrSetManualActionComplete]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetOrSetManualActionComplete]
(
	@BuildExecution_PlanAction_Id INT,
	@ActionComplete_Indicator YNINDICATOR = NULL OUT,
	@CompletedBy_User_Name USERNAME = NULL OUT,
	@AssignedTo_Principal_Name USERNAME = NULL,
	@Task_Name VARCHAR(50),
	@Task_Description TEXT = NULL
)
AS
BEGIN


SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [ManualActionsInProgress]
	               WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id)
	BEGIN
		INSERT INTO [ManualActionsInProgress]
		(
			[BuildExecution_PlanAction_Id],
			[ActionComplete_Indicator],
			[AssignedTo_Principal_Name],
			[Task_Name],
			[Task_Description]
		)
		VALUES
		(
			@BuildExecution_PlanAction_Id,
			'N',
			@AssignedTo_Principal_Name,
			@Task_Name,
			@Task_Description
		)
	END

	IF @ActionComplete_Indicator IS NULL
	BEGIN
		SELECT @ActionComplete_Indicator = [ActionComplete_Indicator],
		       @CompletedBy_User_Name = [CompletedBy_User_Name]
		  FROM [ManualActionsInProgress]
		 WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id
	END
	ELSE
	BEGIN
		UPDATE [ManualActionsInProgress]
		   SET [ActionComplete_Indicator] = @ActionComplete_Indicator,
		       [CompletedBy_User_Name] = @CompletedBy_User_Name
	     WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_GetOrSetManualActionComplete] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetVariableValues',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecution_ActionGroupActionVariableValues',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets execution time variable values.'
GO

IF OBJECT_ID('[Executions_GetVariableValues]') IS NOT NULL DROP PROCEDURE [Executions_GetVariableValues]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetVariableValues]
(
	@BuildExecution_ActionGroupAction_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecution_ActionGroupActionVariableValues]
	 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
	 ORDER BY [Variable_Name]

END
GO

GRANT EXECUTE ON [Executions_GetVariableValues] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_SetVariableValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates, updates, or deletes an execution time variable value.'
GO

IF OBJECT_ID('[Executions_SetVariableValue]') IS NOT NULL DROP PROCEDURE [Executions_SetVariableValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_SetVariableValue]
(
	@BuildExecution_ActionGroupAction_Id INT,
	@Variable_Name VARCHAR(50),
	@Value_Text VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON
	
BEGIN TRY
	BEGIN TRANSACTION
	
	DECLARE @ExecutionStatus_Name VARCHAR(10)
	
	SELECT @ExecutionStatus_Name = BE.[ExecutionStatus_Name]
	  FROM [BuildExecution_ActionGroupActions] BEAGA
	       INNER JOIN [BuildExecution_DeploymentPlans] BEDP
	               ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
	       INNER JOIN [BuildExecutions] BE
	               ON BE.[Execution_Id] = BEDP.[Execution_Id]
	 WHERE BEAGA.[BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

	IF @ExecutionStatus_Name <> 'Pending' AND @ExecutionStatus_Name <> 'Executing'
		RAISERROR('Variables cannot be set on a %s execution.', 16, 1, @ExecutionStatus_Name)

	IF EXISTS(SELECT *
	            FROM [BuildExecution_ActionGroupActionVariableValues]
	           WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
	             AND [Variable_Name] = @Variable_Name)
	BEGIN
		IF @Value_Text IS NULL
		BEGIN
			DELETE [BuildExecution_ActionGroupActionVariableValues]
             WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
               AND [Variable_Name] = @Variable_Name
		END
		ELSE
		BEGIN
			UPDATE [BuildExecution_ActionGroupActionVariableValues]
			   SET [Value_Text] = @Value_Text
	         WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
	           AND [Variable_Name] = @Variable_Name
	    END
	END
	ELSE IF @Value_Text IS NOT NULL
	BEGIN
		INSERT INTO [BuildExecution_ActionGroupActionVariableValues]
		(
			[BuildExecution_ActionGroupAction_Id],
			[Variable_Name],
			[Value_Text]
		)
		VALUES
		(
			@BuildExecution_ActionGroupAction_Id,
			@Variable_Name,
			@Value_Text
		)
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_SetVariableValue] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_DeleteConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an extension configuration'
GO

IF OBJECT_ID('[ExtensionConfiguration_DeleteConfiguration]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_DeleteConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_DeleteConfiguration] 
(
	@ExtensionConfiguration_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DECLARE @QualifiedType_Name VARCHAR(500)
		    SET @QualifiedType_Name = (SELECT [QualifiedType_Name]
		                             FROM [ExtensionConfigurations]
		                            WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id)
		IF @@ROWCOUNT = 0 RETURN

        UPDATE [ActionGroupActions]
           SET [ExtensionConfiguration_Id] = NULL
         WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id

		DELETE [ExtensionConfigurations]
		 WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id

		EXEC [Events_RaiseEvent] 'EXTCFG', NULL, @QualifiedType_Name 

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ExtensionConfiguration_DeleteConfiguration] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_GetConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ExtensionConfigurations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of an extension configurer for extension with the specified name'
GO

IF OBJECT_ID('[ExtensionConfiguration_GetConfiguration]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_GetConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_GetConfiguration] 
(
	@QualifiedType_Name VARCHAR(500),
	@Profile_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [ExtensionConfigurations]
	 WHERE [QualifiedType_Name] = @QualifiedType_Name
	   AND ([Profile_Name] = @Profile_Name OR [Default_Indicator] = 'Y')


END
GO

GRANT EXECUTE ON [ExtensionConfiguration_GetConfiguration] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_GetConfigurationById',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ExtensionConfigurations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of an extension configurer for extension with the specified ID'
GO

IF OBJECT_ID('[ExtensionConfiguration_GetConfigurationById]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_GetConfigurationById]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_GetConfigurationById] 
(
	@ExtensionConfiguration_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [ExtensionConfigurations]
	 WHERE [ExtensionConfiguration_Id]  = @ExtensionConfiguration_Id 

END
GO

GRANT EXECUTE ON [ExtensionConfiguration_GetConfigurationById] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_GetConfigurations',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ExtensionConfigurations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for all extension configurers in the system, or a specific one if specified'
GO

IF OBJECT_ID('[ExtensionConfiguration_GetConfigurations]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_GetConfigurations]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_GetConfigurations] 
(
	@QualifiedType_Name VARCHAR(500) = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [ExtensionConfigurations]
	 WHERE @QualifiedType_Name IS NULL OR [QualifiedType_Name] = @QualifiedType_Name
  ORDER BY [Default_Indicator] DESC, [Profile_Name] ASC


END
GO

GRANT EXECUTE ON [ExtensionConfiguration_GetConfigurations] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_SaveConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ExtensionConfiguration_Id',
    /* Description_Text        */ 'Creates an extension configurer in the system, or updates an existing one'
GO

IF OBJECT_ID('[ExtensionConfiguration_SaveConfiguration]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_SaveConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_SaveConfiguration] 
(
	@ExtensionConfiguration_Id INT OUT,

	@QualifiedType_Name VARCHAR(500),
	@Profile_Name VARCHAR(50),

	@Extension_Configuration VARCHAR(MAX),
	@Default_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
		
		IF (@ExtensionConfiguration_Id IS NULL)
		BEGIN

			INSERT INTO [ExtensionConfigurations]
					   ([QualifiedType_Name]
					   ,[Profile_Name]
					   ,[Extension_Configuration]
					   ,[CreatedBy_User_Name]
					   ,[CreatedOn_Date]
					   ,[ModifiedBy_User_Name]
					   ,[ModifiedOn_Date]
					   ,[Default_Indicator])
				 VALUES
					   (@QualifiedType_Name
					   ,@Profile_Name
					   ,@Extension_Configuration
					   ,dbo.Session_GetUserName()
					   ,GETUTCDATE()
					   ,dbo.Session_GetUserName()
					   ,GETUTCDATE()
					   ,@Default_Indicator)
			SET @ExtensionConfiguration_Id = SCOPE_IDENTITY()

		END ELSE BEGIN

			UPDATE [ExtensionConfigurations]
			   SET [QualifiedType_Name] = @QualifiedType_Name
			      ,[Profile_Name] = @Profile_Name
			      ,[Extension_Configuration] = @Extension_Configuration
			      ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			      ,[ModifiedOn_Date] = GETUTCDATE()
			      ,[Default_Indicator] = @Default_Indicator
			 WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id

		END

        IF (@Default_Indicator = 'Y')
        BEGIN 
                
        UPDATE [ExtensionConfigurations]
            SET [Default_Indicator] = 'N'
            WHERE [QualifiedType_Name] = @QualifiedType_Name
            AND [ExtensionConfiguration_Id] <> @ExtensionConfiguration_Id

        END

		/* Raise Extension Configured event */
		EXEC [Events_RaiseEvent] 'EXTCFG', NULL, @QualifiedType_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ExtensionConfiguration_SaveConfiguration] TO [BuildMasterUser_Role]
GO

GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_CloseIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Closes a BuildMaster issue.'
GO

IF OBJECT_ID('[Issues_CloseIssue]') IS NOT NULL DROP PROCEDURE [Issues_CloseIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_CloseIssue]
(
	@Application_Id INT,
	@Issue_Id INT,
	@Build_Number BUILDNUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Issues]
	   SET [Closed_Indicator] = 'Y',
	       [ClosedOn_Date] = GETUTCDATE(),
		   [ClosedOn_Original_Build_Number] = @Build_Number
	 WHERE [Application_Id] = @Application_Id
	   AND [Issue_Id] = @Issue_Id
       AND [IssueTracker_IssueId_Text] IS NULL

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_CloseIssue] TO [BuildMasterUser_Role]


GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_CreateIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Issue_Id',
    /* Description_Text        */ 'Creates an issue for a release of an application.'
GO

IF OBJECT_ID('[Issues_CreateIssue]') IS NOT NULL DROP PROCEDURE [Issues_CreateIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_CreateIssue]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Title_Text NVARCHAR(200),
	@Description_Text NVARCHAR(MAX),
    @Closed_Indicator YNINDICATOR, 
    @Build_Number BUILDNUMBER = NULL,
	@CreatedBy_User_Name USERNAME = NULL,
	@CreatedOn_Date DATETIME = NULL,
	@IssueTracker_IssueId_Text NVARCHAR(100) = NULL,
	@IssueTracker_IssueStatus_Text NVARCHAR(100) = NULL,
	@IssueTracker_LastSync_Date DATETIME = NULL,
    @IssueTracker_IssueUrl_Text NVARCHAR(255) = NULL,
	@Issue_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SET @Issue_Id = COALESCE((SELECT MAX([Issue_Id])
	                            FROM [Issues]
							   WHERE [Application_Id] = @Application_Id), 0) + 1

	INSERT INTO [Issues]
	(
		[Application_Id],
		[Issue_Id],
		[Release_Number],
		[IssueTracker_IssueId_Text],
		[IssueTracker_IssueStatus_Text],
        [IssueTracker_IssueUrl_Text],
		[IssueTracker_LastSync_Date],
		[Title_Text],
		[Description_Text],
		[Closed_Indicator],
        [ClosedOn_Date],
        [OpenedOn_Original_Build_Number],
        [ClosedOn_Original_Build_Number],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	VALUES
	(
		@Application_Id,
		@Issue_Id,
		NULLIF(@Release_Number, ''),
		@IssueTracker_IssueId_Text,
		@IssueTracker_IssueStatus_Text,
        @IssueTracker_IssueUrl_Text,
		@IssueTracker_LastSync_Date,
		@Title_Text,
		NULLIF(@Description_Text, ''),
		@Closed_Indicator,
        CASE WHEN @Closed_Indicator = 'Y' THEN GETUTCDATE() ELSE NULL END,
        @Build_Number,
        CASE WHEN @Closed_Indicator = 'Y' THEN @Build_Number ELSE NULL END,
		COALESCE(@CreatedBy_User_Name, dbo.Session_GetUserName()),
		COALESCE(@CreatedOn_Date, GETUTCDATE())
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_CreateIssue] TO [BuildMasterUser_Role]


GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_DeleteIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an issue.'
GO

IF OBJECT_ID('[Issues_DeleteIssue]') IS NOT NULL DROP PROCEDURE [Issues_DeleteIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_DeleteIssue]
(
	@Application_Id INT,
	@Issue_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [Issues]
		 WHERE [Application_Id] = @Application_Id
		   AND [Issue_Id] = @Issue_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_DeleteIssue] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Issues_GetBuildStatusChanges]') IS NOT NULL DROP PROCEDURE [Issues_GetBuildStatusChanges]
GO


GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_GetIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Issues_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified.'
GO


IF OBJECT_ID('[Issues_GetIssue]') IS NOT NULL DROP PROCEDURE [Issues_GetIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_GetIssue]
(
    @Application_Id INT,
	@Issue_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Issues_Extended]
	 WHERE [Application_Id] = @Application_Id
       AND [Issue_Id] = @Issue_Id

END
GO

GRANT EXECUTE ON [Issues_GetIssue] TO [BuildMasterUser_Role]


GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_GetIssues',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Issues_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the issues for an application, optionally filtered by release or build.'
GO


IF OBJECT_ID('[Issues_GetIssues]') IS NOT NULL DROP PROCEDURE [Issues_GetIssues]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_GetIssues]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER = NULL,
    @Build_Number BUILDNUMBER = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT IE.*
	  FROM [Issues_Extended] IE
 LEFT JOIN [Builds] B 
        ON B.[Application_Id] = @Application_Id
       AND B.[Release_Number] = @Release_Number
       AND B.[Build_Number] = @Build_Number
	 WHERE IE.[Application_Id] = @Application_Id
	   AND (@Release_Number IS NULL OR IE.[Release_Number] = @Release_Number)
       AND (@Build_Number IS NULL OR (IE.[OpenedOn_Original_Build_Sequence] IS NULL AND IE.[ClosedOn_Original_Build_Sequence] IS NOT NULL) OR B.[Build_Sequence] >= IE.[OpenedOn_Original_Build_Sequence])
       AND (@Build_Number IS NULL OR IE.[ClosedOn_Original_Build_Sequence] IS NULL OR B.[Build_Sequence] = IE.[ClosedOn_Original_Build_Sequence])
  ORDER BY [Release_Sequence] DESC, [Closed_Indicator], [Issue_Id] DESC

END
GO

GRANT EXECUTE ON [Issues_GetIssues] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Issues_RecordStatusChange]') IS NOT NULL DROP PROCEDURE [Issues_RecordStatusChange]
GO

GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_ReopenIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Re-opens a BuildMaster issue.'
GO

IF OBJECT_ID('[Issues_ReopenIssue]') IS NOT NULL DROP PROCEDURE [Issues_ReopenIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_ReopenIssue]
(
	@Application_Id INT,
	@Issue_Id INT,
	@Build_Number BUILDNUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Issues]
	   SET [Closed_Indicator] = 'N',
	       [ClosedOn_Date] = NULL,
		   [ClosedOn_Original_Build_Number] = NULL,
           [OpenedOn_Original_Build_Number] = @Build_Number
	 WHERE [Application_Id] = @Application_Id
	   AND [Issue_Id] = @Issue_Id
       AND [IssueTracker_IssueId_Text] IS NULL

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_ReopenIssue] TO [BuildMasterUser_Role]


GO
EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Issues_UpdateIssue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates one or more properties of an issue.'
GO

IF OBJECT_ID('[Issues_UpdateIssue]') IS NOT NULL DROP PROCEDURE [Issues_UpdateIssue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Issues_UpdateIssue]
(
	@Application_Id INT,
	@Issue_Id INT,
	@Release_Number RELEASENUMBER,
	@Title_Text NVARCHAR(200) = NULL,
	@Description_Text NVARCHAR(MAX) = NULL,
    @Closed_Indicator YNINDICATOR = NULL,
	@IssueTracker_IssueStatus_Text NVARCHAR(100) = NULL,
    @IssueTracker_IssueUrl_Text NVARCHAR(255) = NULL,
	@IssueTracker_LastSync_Date DATETIME = NULL,
    @OpenedOn_Original_Build_Number BUILDNUMBER = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Issues]
	   SET [Release_Number] = NULLIF(@Release_Number, ''),
	       [Title_Text] = COALESCE(@Title_Text, [Title_Text]),
		   [Description_Text] = COALESCE(@Description_Text, [Description_Text]),
           [Closed_Indicator] = COALESCE(@Closed_Indicator, [Closed_Indicator]),
		   [IssueTracker_IssueStatus_Text] = COALESCE(@IssueTracker_IssueStatus_Text, [IssueTracker_IssueStatus_Text]),
           [IssueTracker_IssueUrl_Text] = COALESCE(@IssueTracker_IssueUrl_Text, [IssueTracker_IssueUrl_Text]),
		   [IssueTracker_LastSync_Date] = COALESCE(@IssueTracker_LastSync_Date, [IssueTracker_LastSync_Date]),
           [OpenedOn_Original_Build_Number] = NULLIF(COALESCE(@OpenedOn_Original_Build_Number, [OpenedOn_Original_Build_Number]), '')
	 WHERE [Application_Id] = @Application_Id
	   AND [Issue_Id] = @Issue_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Issues_UpdateIssue] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_ActivateLicenseKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the activation code for a license key.'
GO

IF OBJECT_ID('[LicenseKeys_ActivateLicenseKey]') IS NOT NULL DROP PROCEDURE [LicenseKeys_ActivateLicenseKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_ActivateLicenseKey]
(
	@LicenseKey_Text CHAR(36),
	@ActivationCode_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

IF @ActivationCode_Text IS NULL
BEGIN
	UPDATE [LicenseKeys]
	   SET [ActivationCode_Text] = NULL,
	       [ActivatedBy_User_Name] = NULL,
	       [ActivatedOn_Date] = NULL
	 WHERE [LicenseKey_Text] = @LicenseKey_Text
END
ELSE
BEGIN
	UPDATE [LicenseKeys]
	   SET [ActivationCode_Text] = @ActivationCode_Text,
	       [ActivatedBy_User_Name] = dbo.Session_GetUserName(),
	       [ActivatedOn_Date] = GETUTCDATE()
	 WHERE [LicenseKey_Text] = @LicenseKey_Text
END

END
GO

GRANT EXECUTE ON [LicenseKeys_ActivateLicenseKey] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_AddLicenseKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a new license key.'
GO

IF OBJECT_ID('[LicenseKeys_AddLicenseKey]') IS NOT NULL DROP PROCEDURE [LicenseKeys_AddLicenseKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_AddLicenseKey]
(
	@LicenseKey_Text CHAR(36)
)
AS
BEGIN

SET NOCOUNT ON

INSERT INTO [LicenseKeys]
(
	[LicenseKey_Text],
	[AddedBy_User_Name],
	[AddedOn_Date]
)
VALUES
(
	@LicenseKey_Text,
	dbo.Session_GetUserName(),
	GETUTCDATE()
)

END
GO

GRANT EXECUTE ON [LicenseKeys_AddLicenseKey] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_DeleteLicenseKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a license key.'
GO

IF OBJECT_ID('[LicenseKeys_DeleteLicenseKey]') IS NOT NULL DROP PROCEDURE [LicenseKeys_DeleteLicenseKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_DeleteLicenseKey]
(
	@LicenseKey_Text CHAR(36)
)
AS
BEGIN

SET NOCOUNT ON

DELETE [LicenseKeys]
 WHERE [LicenseKey_Text] = @LicenseKey_Text

END
GO

GRANT EXECUTE ON [LicenseKeys_DeleteLicenseKey] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_GetLicenseKeys',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LicenseKeys',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the license keys.'
GO

IF OBJECT_ID('[LicenseKeys_GetLicenseKeys]') IS NOT NULL DROP PROCEDURE [LicenseKeys_GetLicenseKeys]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_GetLicenseKeys]
AS
BEGIN

SET NOCOUNT ON
	SELECT *
	  FROM [LicenseKeys]
	 ORDER BY [AddedOn_Date]

END
GO

GRANT EXECUTE ON [LicenseKeys_GetLicenseKeys] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_AddMessage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[LogMessages_AddMessage]') IS NOT NULL DROP PROCEDURE [LogMessages_AddMessage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_AddMessage]
(
	@Message_Text VARCHAR(MAX),
    @ExceptionStackTrace_Text VARCHAR(MAX) = NULL,
    @Server_Id INT = NULL
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

INSERT INTO [LogMessages]
    ([Read_Indicator] ,[Logged_Date] ,[Message_Text] ,[ExceptionStackTrace_Text] ,[Server_Id])
VALUES
    ('N', GETUTCDATE(), @Message_Text, @ExceptionStackTrace_Text, @Server_Id)

END TRY BEGIN CATCH
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LogMessages_AddMessage] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_DeleteMessages',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes one or all message from the error log'
GO

IF OBJECT_ID('[LogMessages_DeleteMessages]') IS NOT NULL DROP PROCEDURE [LogMessages_DeleteMessages]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_DeleteMessages]
(
	@LogMessage_Id INT = NULL
) AS
BEGIN

    SET NOCOUNT ON

    BEGIN TRY

	    DELETE [LogMessages]
	     WHERE @LogMessage_Id IS NULL
            OR [LogMessage_Id] = @LogMessage_Id

    END TRY BEGIN CATCH
	    EXEC [HandleError]
    END CATCH

END
GO

GRANT EXECUTE ON [LogMessages_DeleteMessages] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_GetMessage',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LogMessages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a specified message from the error log'
GO

IF OBJECT_ID('[LogMessages_GetMessage]') IS NOT NULL DROP PROCEDURE [LogMessages_GetMessage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_GetMessage]
(
	@LogMessage_Id INT
) AS
BEGIN

    SET NOCOUNT ON

    SELECT * FROM [LogMessages_Extended] WHERE @LogMessage_Id = [LogMessage_Id]
	
END
GO

GRANT EXECUTE ON [LogMessages_GetMessage] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_GetMessages',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LogMessages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the messages from the error log that match the specified criteria'
GO

IF OBJECT_ID('[LogMessages_GetMessages]') IS NOT NULL DROP PROCEDURE [LogMessages_GetMessages]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_GetMessages]
(
	@Row_Count INT,
	@From_Date DATETIME = NULL,
    @IncludeReadMessages_Indicator YNINDICATOR = 'N'
) AS
BEGIN

    SET NOCOUNT ON

	IF @Row_Count IS NOT NULL
	    SET ROWCOUNT @Row_Count

    SELECT * FROM [LogMessages_Extended]
     WHERE (@From_Date IS NULL OR [Logged_Date] > @From_Date)
       AND (COALESCE(@IncludeReadMessages_Indicator, 'N') = 'Y' OR [Read_Indicator] = 'N')
     ORDER BY [Logged_Date] DESC

END
GO

GRANT EXECUTE ON [LogMessages_GetMessages] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_SetReadIndicator',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[LogMessages_SetReadIndicator]') IS NOT NULL DROP PROCEDURE [LogMessages_SetReadIndicator]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_SetReadIndicator]
(
	@LogMessage_Id INT,
    @Read_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

UPDATE [LogMessages] SET [Read_Indicator] = @Read_Indicator
 WHERE [LogMessage_Id] = @LogMessage_Id

END TRY BEGIN CATCH
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LogMessages_SetReadIndicator] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_CreateOrUpdateNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new notifier, or updates an existing one'
GO

IF OBJECT_ID('[Notifiers_CreateOrUpdateNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_CreateOrUpdateNotifier]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_CreateOrUpdateNotifier]
(
	@Notifier_Id INT = NULL,
	@Notifier_Name VARCHAR(50),
	@Notifier_Description VARCHAR(MAX),
	@Notifier_Configuration VARCHAR(MAX),
	@Owner_User_Name USERNAME,
	@Active_Indicator YNINDICATOR,
	@EventCodeList_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Notifier_Id IS NULL BEGIN
		INSERT INTO [Notifiers]
				   ([Notifier_Name]
				   ,[Notifier_Description]
				   ,[Notifier_Configuration]
				   ,[Owner_User_Name]
				   ,[Active_Indicator])
			 VALUES
				   (@Notifier_Name
				   ,@Notifier_Description
				   ,@Notifier_Configuration
				   ,@Owner_User_Name
				   ,@Active_Indicator)
		SET @Notifier_Id = SCOPE_IDENTITY()
	END ELSE BEGIN
		UPDATE [Notifiers] SET
		   [Notifier_Name]=@Notifier_Name
		   ,[Notifier_Description] = @Notifier_Description
		   ,[Notifier_Configuration] = @Notifier_Configuration
		   ,[Active_Indicator] = @Active_Indicator
		WHERE [Notifier_Id] = @Notifier_Id
		IF @@ROWCOUNT=0 RAISERROR('Notifier #%i does not exist.',16,1,@Notifier_Id)

		DELETE [NotifierEventCodes] 
		 WHERE [Notifier_Id] = @Notifier_Id
	END

	INSERT INTO [NotifierEventCodes] ([Notifier_Id],[Event_Code])
	SELECT @Notifier_Id, V.[Value_Text]
	  FROM dbo.CsvToTable(@EventCodeList_Csv,',') V

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Notifiers_CreateOrUpdateNotifier] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_DeleteNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a notifier from the system'
GO

IF OBJECT_ID('[Notifiers_DeleteNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_DeleteNotifier]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_DeleteNotifier]
(
	@Notifier_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [NotifierEventCodes] WHERE [Notifier_Id] = @Notifier_Id

	DELETE [Notifiers] WHERE [Notifier_Id] = @Notifier_Id

 

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Notifiers_DeleteNotifier] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_GetNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Notifiers,NotifierEventCodes',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a notifier and a list of event codes that the notifier is subscribed to'
GO

IF OBJECT_ID('[Notifiers_GetNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_GetNotifier]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_GetNotifier]
(
	@Notifier_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [Notifiers]
	 WHERE [Notifier_Id] = @Notifier_Id
	IF @@ROWCOUNT=0 RAISERROR('Notifier %i does not exist.', 16,1,@Notifier_Id)

	SELECT * FROM [NotifierEventCodes]
	 WHERE [Notifier_Id] = @Notifier_Id

END
GO

GRANT EXECUTE ON [Notifiers_GetNotifier] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_GetNotifiers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Notifiers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of notifiers for the specified user, or all users if no user is specified'
GO

IF OBJECT_ID('[Notifiers_GetNotifiers]') IS NOT NULL
	DROP PROCEDURE [Notifiers_GetNotifiers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_GetNotifiers]
(
	@Owner_User_Name USERNAME = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [Notifiers]
	 WHERE [Owner_User_Name] = COALESCE(@Owner_User_Name,[Owner_User_Name])
	   OR ([Owner_User_Name] IS NULL AND @Owner_User_Name IS NULL)

END
GO

GRANT EXECUTE ON [Notifiers_GetNotifiers] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ChangeActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves an action into a different action group.'
GO


IF OBJECT_ID('[Plans_ChangeActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_ChangeActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ChangeActionGroup]
(
	@DeploymentPlanActionGroup_Id INT,
	@Action_Sequence INT,
	@New_DeploymentPlanActionGroup_Id INT,
	@New_Action_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	DECLARE @ActionGroup_Id INT
	DECLARE @DeploymentPlan_Id INT
	 SELECT @ActionGroup_Id = [ActionGroup_Id],
	        @DeploymentPlan_Id = [DeploymentPlan_Id]
	   FROM [DeploymentPlanActionGroups]
	  WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	DECLARE @New_ActionGroup_Id INT
	 SELECT @New_ActionGroup_Id = [ActionGroup_Id]
	   FROM [DeploymentPlanActionGroups]
	  WHERE [DeploymentPlanActionGroup_Id] = @New_DeploymentPlanActionGroup_Id

	BEGIN TRANSACTION

	IF (@ActionGroup_Id = @New_ActionGroup_Id)
	    RAISERROR('Actions cannot be moved to the same plan; use Plans_ResequenceAction instead.', 16, 1)

   DECLARE @Last_Action_Sequence INT
	SELECT @Last_Action_Sequence = COALESCE(MAX([Action_Sequence]), 1)
	  FROM [ActionGroupActions]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

   DECLARE @Target_Last_Action_Sequence INT
	SELECT @Target_Last_Action_Sequence = COALESCE(MAX([Action_Sequence]), 0) + 1
      FROM [ActionGroupActions]
     WHERE [ActionGroup_Id] = @New_ActionGroup_Id


	-- move action to end of list to appease trigger
	EXEC [Plans_ResequenceAction] @DeploymentPlanActionGroup_Id, @Action_Sequence, @Last_Action_Sequence

    -- move the action to the end of its new action group 
     UPDATE [ActionGroupActions]
        SET [ActionGroup_Id] = @New_ActionGroup_Id,
            [Action_Sequence] = @Target_Last_Action_Sequence
      WHERE [ActionGroup_Id] = @ActionGroup_Id
	    AND [Action_Sequence] = @Last_Action_Sequence

	-- move the action from the end of its action group to its new sequence
	EXEC [Plans_ResequenceAction] @New_DeploymentPlanActionGroup_Id, @Target_Last_Action_Sequence, @New_Action_Sequence

	-- disable reexecution for anything that used the containing plan of either group
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                                 INNER JOIN [DeploymentPlanActionGroups] DPAG
	                                         ON BEDP.[Original_DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
	                           WHERE DPAG.[DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id
	                              OR DPAG.[DeploymentPlanActionGroup_Id] = @New_DeploymentPlanActionGroup_Id)

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ChangeActionGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ChangeActionGroupDeploymentPlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves a plan action group to the final position of a different deployment plan.'
GO


IF OBJECT_ID('[Plans_ChangeActionGroupDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Plans_ChangeActionGroupDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ChangeActionGroupDeploymentPlan]
(
	@DeploymentPlanActionGroup_Id INT,
	@New_DeploymentPlan_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

    DECLARE @DeploymentPlanActionGroup_Sequence INT
    DECLARE @ActionGroup_Id INT
    DECLARE @DeploymentPlan_Id INT
	
	SELECT @ActionGroup_Id = [ActionGroup_Id],
	       @DeploymentPlanActionGroup_Sequence = [DeploymentPlanActionGroup_Sequence],
	       @DeploymentPlan_Id = [DeploymentPlan_Id]
	  FROM [DeploymentPlanActionGroups]
	 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

    DECLARE @Last_DeploymentPlanActionGroup_Sequence INT
    SELECT @Last_DeploymentPlanActionGroup_Sequence = MAX([DeploymentPlanActionGroup_Sequence])
      FROM [DeploymentPlanActionGroups]
     WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

    -- move action group to the end of the sequence in the current environment to appease trigger
    EXEC [Plans_ResequenceActionGroup] @DeploymentPlanActionGroup_Id, @Last_DeploymentPlanActionGroup_Sequence

    -- move action group to the end of the sequence of action groups in the new environment
    UPDATE [DeploymentPlanActionGroups]
	   SET [DeploymentPlanActionGroup_Sequence] = (SELECT COALESCE(MAX([DeploymentPlanActionGroup_Sequence]), 0) 
	                                                 FROM [DeploymentPlanActionGroups] 
	                                                WHERE [DeploymentPlan_Id] = @New_DeploymentPlan_Id) + 1,
	       [DeploymentPlan_Id] = @New_DeploymentPlan_Id
	 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

 	-- disable reexecution for anything that used either plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id
	                              OR BEDP.[Original_DeploymentPlan_Id] = @New_DeploymentPlan_Id)

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id
	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @New_DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ChangeActionGroupDeploymentPlan] TO [BuildMasterUser_Role]

GO
IF OBJECT_ID('[Plans_ChangeApplicationDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Plans_ChangeApplicationDeploymentPlan]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CloneDeploymentPlans',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'NewDeploymentPlanIdsCsv',
    /* Description_Text        */ 'Clones one or more deployment plans'
GO

IF OBJECT_ID('[Plans_CloneDeploymentPlans]') IS NOT NULL
	DROP PROCEDURE [Plans_CloneDeploymentPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CloneDeploymentPlans]
(
	@DeploymentPlanCloneData_Xml XML,
	@NewDeploymentPlanIdsCsv VARCHAR(MAX) = NULL OUT
) AS
BEGIN
	/*<DeploymentPlanCloneData LinkAllActionGroups_Indicator="Y">
        <DeploymentPlan>
          <New_DeploymentPlan_Name>Standard</New_DeploymentPlan_Name>
          <DeploymentPlan_Id>3</DeploymentPlan_Id>
        </DeploymentPlan>
	  </DeploymentPlanCloneData>*/

SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

	--@DeploymentPlanIdsCsv VARCHAR(MAX),
	--@LinkAllActionGroups_Indicator YNINDICATOR = 'N',

		DECLARE @LinkAllActionGroups_Indicator YNINDICATOR
		    SET @LinkAllActionGroups_Indicator =
		       (SELECT DPCD.[value]('@LinkAllActionGroups_Indicator', 'CHAR(1)')
	              FROM @DeploymentPlanCloneData_Xml.[nodes]('/DeploymentPlanCloneData') AS DeploymentPlanCloneData(DPCD))

		DECLARE @DeploymentPlans TABLE ([Old_DeploymentPlan_Id] INT, [New_DeploymentPlan_Id] INT, [AllowLocalChanges_Indicator] CHAR(1), [New_DeploymentPlan_Name] VARCHAR(50), Seq INT IDENTITY(1,1))
		INSERT INTO @DeploymentPlans ([Old_DeploymentPlan_Id], [New_DeploymentPlan_Name], [AllowLocalChanges_Indicator])
		SELECT DISTINCT [Old_DeploymentPlan_Id], [New_DeploymentPlan_Name], [AllowLocalChanges_Indicator]
		  FROM (SELECT DPCD.[value]('New_DeploymentPlan_Name[1]', 'VARCHAR(20)') [New_DeploymentPlan_Name],
					   DPCD.[value]('DeploymentPlan_Id[1]', 'INT') [Old_DeploymentPlan_Id]
				  FROM @DeploymentPlanCloneData_Xml.[nodes]('/DeploymentPlanCloneData/DeploymentPlan') AS DeploymentPlanCloneData(DPCD)) DPCD
			   INNER JOIN [DeploymentPlans] DP ON DP.[DeploymentPlan_Id] = DPCD.[Old_DeploymentPlan_Id]

		DECLARE @ActionGroups TABLE ([Old_ActionGroup_Id] INT, [New_ActionGroup_Id] INT, [Old_DeploymentPlan_Id] INT, [Old_DeploymentPlanActionGroup_Sequence] INT)

		IF @LinkAllActionGroups_Indicator = 'Y' BEGIN
			
			INSERT INTO @ActionGroups ([Old_ActionGroup_Id], [New_ActionGroup_Id], [Old_DeploymentPlan_Id], [Old_DeploymentPlanActionGroup_Sequence])
			SELECT  AG.[ActionGroup_Id], 
					AG.[ActionGroup_Id], 
					DPAG.[DeploymentPlan_Id],
                    DPAG.[DeploymentPlanActionGroup_Sequence]
			  FROM [ActionGroups] AG
				   INNER JOIN [DeploymentPlanActionGroups] DPAG
						   ON AG.[ActionGroup_Id] = DPAG.[ActionGroup_Id]
				   INNER JOIN @DeploymentPlans DP
						   ON DP.[Old_DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]

		END ELSE BEGIN 

			INSERT INTO @ActionGroups ([Old_ActionGroup_Id], [Old_DeploymentPlan_Id], [Old_DeploymentPlanActionGroup_Sequence])
		    SELECT  AG.[ActionGroup_Id],
                    DPAG.[DeploymentPlan_Id],
                    DPAG.[DeploymentPlanActionGroup_Sequence]
			  FROM [ActionGroups] AG
				   INNER JOIN [DeploymentPlanActionGroups] DPAG
						   ON AG.[ActionGroup_Id] = DPAG.[ActionGroup_Id]
				   INNER JOIN @DeploymentPlans DP
						   ON DP.[Old_DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
		
			DECLARE @ActionGroupsOldId INT
			DECLARE @ActionGroupsInsertCur CURSOR
			   SET @ActionGroupsInsertCur = CURSOR FOR 
				SELECT [Old_ActionGroup_Id] FROM @ActionGroups
					FOR UPDATE OF [New_ActionGroup_Id]
			OPEN @ActionGroupsInsertCur
			FETCH NEXT FROM @ActionGroupsInsertCur INTO @ActionGroupsOldId
			WHILE @@FETCH_STATUS = 0 
			BEGIN

				INSERT INTO [ActionGroups] 
				(
					[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],
					[ActionGroup_Name],[Predicate_Configuration],[ActionGroup_Description],
					[Server_Id],[Server_Variable_Name]
				)
				SELECT	
					dbo.Session_GetUserName(), GETUTCDATE(), [ModifiedBy_User_Name],[ModifiedOn_Date],
					[ActionGroup_Name],[Predicate_Configuration],[ActionGroup_Description],
					[Server_Id],[Server_Variable_Name]   
				FROM [ActionGroups] 
				WHERE [ActionGroup_Id] = @ActionGroupsOldId

				UPDATE @ActionGroups SET [New_ActionGroup_Id] = SCOPE_IDENTITY()
				 WHERE CURRENT OF @ActionGroupsInsertCur
			
				FETCH NEXT FROM @ActionGroupsInsertCur INTO @ActionGroupsOldId
			END
			CLOSE @ActionGroupsInsertCur


			--==[ActionGroupActions]==--
			INSERT INTO [ActionGroupActions] (
				   [ActionGroup_Id]
				  ,[Action_Sequence]
				  ,[Server_Id]
				  ,[Long_Description]
				  ,[ResumeNextOnFailure_Indicator]
				  ,[Action_Configuration]
				  ,[Short_Description]
				  ,[Active_Indicator]
				  ,[Retry_Count]
				  ,[LogFailureAsWarning_Indicator]
				  ,[Server_Variable_Name]
				  ,[Target_Server_Id]
				  ,[Target_Server_Variable_Name]
				  ,[ExtensionConfiguration_Id]
			) SELECT 
				   [New_ActionGroup_Id]
				  ,[Action_Sequence]
				  ,[Server_Id]
				  ,[Long_Description]
				  ,[ResumeNextOnFailure_Indicator]
				  ,[Action_Configuration]
				  ,[Short_Description]
				  ,[Active_Indicator]
				  ,[Retry_Count]
				  ,[LogFailureAsWarning_Indicator]
				  ,[Server_Variable_Name]
				  ,[Target_Server_Id]
				  ,[Target_Server_Variable_Name]
				  ,[ExtensionConfiguration_Id]
			  FROM [ActionGroupActions] JOIN @ActionGroups ON [ActionGroup_Id] = [Old_ActionGroup_Id]
		END

		--==[DeploymentPlans]==--
		DECLARE @DeploymentPlansNewId INT, @DeploymentPlansNewName VARCHAR(50), @AllowLocalChanges_Indicator CHAR(1)
		DECLARE @DeploymentPlansInsertCur CURSOR
			SET @DeploymentPlansInsertCur = CURSOR FOR 
			SELECT [New_DeploymentPlan_Id], [New_DeploymentPlan_Name], [AllowLocalChanges_Indicator] FROM @DeploymentPlans
				FOR UPDATE OF [New_DeploymentPlan_Id]
		OPEN @DeploymentPlansInsertCur
		FETCH NEXT FROM @DeploymentPlansInsertCur INTO @DeploymentPlansNewId, @DeploymentPlansNewName, @AllowLocalChanges_Indicator
		WHILE @@FETCH_STATUS = 0 
		BEGIN

			INSERT INTO [DeploymentPlans] ([DeploymentPlan_Name], [AllowLocalChanges_Indicator]) VALUES (@DeploymentPlansNewName, @AllowLocalChanges_Indicator)
			
			UPDATE @DeploymentPlans SET [New_DeploymentPlan_Id] = SCOPE_IDENTITY()
				WHERE CURRENT OF @DeploymentPlansInsertCur

			FETCH NEXT FROM @DeploymentPlansInsertCur INTO @DeploymentPlansNewId, @DeploymentPlansNewName, @AllowLocalChanges_Indicator

		END
		CLOSE @DeploymentPlansInsertCur

		--==[DeploymentPlanActionGroups]==--
		INSERT INTO [DeploymentPlanActionGroups] (
			  /*[DeploymentPlanActionGroup_Id]*/
			  [DeploymentPlan_Id]
			  ,[DeploymentPlanActionGroup_Sequence]
			  ,[ActionGroup_Id]
			  ,[Deployable_Name]
			  ,[Deployable_Application_Id]
			  ,[Active_Indicator]
			  ,[ActionGroup_Name]
			  ,[ActionGroup_Description]
			  ,[Predicate_Configuration]
			  ,[Server_Id]
			  ,[Server_Variable_Name]
			  ,[Iterate_Servers_Indicator]
			  ,[Iterate_Deployables_Indicator]
			  ,[Parallel_Indicator]
			  ,[OnSuccess_DeploymentPlanActionGroup_Id]
			  ,[OnFailure_DeploymentPlanActionGroup_Id]
			  ,[OnFailure_Continue_Indicator]
			  ,[ModifiedBy_User_Name]
			  ,[ModifiedOn_Date]
		) SELECT 
			  [New_DeploymentPlan_Id]
			  ,[DeploymentPlanActionGroup_Sequence]
			  ,[New_ActionGroup_Id]
			  ,[Deployable_Name]
			  ,[Deployable_Application_Id]
			  ,[Active_Indicator]
			  ,[ActionGroup_Name]
			  ,[ActionGroup_Description]
			  ,[Predicate_Configuration]
			  ,[Server_Id]
			  ,[Server_Variable_Name]
			  ,[Iterate_Servers_Indicator]
			  ,[Iterate_Deployables_Indicator]
			  ,[Parallel_Indicator]
			  ,[OnSuccess_DeploymentPlanActionGroup_Id]
			  ,[OnFailure_DeploymentPlanActionGroup_Id]
			  ,[OnFailure_Continue_Indicator]
			  ,[ModifiedBy_User_Name]
			  ,[ModifiedOn_Date]
		  FROM [DeploymentPlanActionGroups] DPAG
         INNER JOIN @ActionGroups AG
                 ON DPAG.[DeploymentPlan_Id] = AG.[Old_DeploymentPlan_Id] 
                AND DPAG.[ActionGroup_Id] = AG.[Old_ActionGroup_Id] 
                AND DPAG.[DeploymentPlanActionGroup_Sequence] = AG.[Old_DeploymentPlanActionGroup_Sequence]
         INNER JOIN @DeploymentPlans DP 
                 ON [DeploymentPlan_Id] = DP.[Old_DeploymentPlan_Id] 
                AND [DeploymentPlanActionGroup_Sequence] = AG.[Old_DeploymentPlanActionGroup_Sequence]

		SELECT @NewDeploymentPlanIdsCsv = ISNULL(@NewDeploymentPlanIdsCsv,'') + CAST([New_DeploymentPlan_Id] AS VARCHAR(MAX)) + ','
		  FROM @DeploymentPlans ORDER BY [Seq]
		SET @NewDeploymentPlanIdsCsv = SUBSTRING(@NewDeploymentPlanIdsCsv, 1, LEN(@NewDeploymentPlanIdsCsv)-1)

	COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CloneDeploymentPlans] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CreateDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'PlanActionGroup_Id,ActionGroup_Id',
    /* Description_Text        */ 'Creates a new action group in the specified deployment plan.'
GO


IF OBJECT_ID('[Plans_CreateDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_CreateDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CreateDeploymentPlanActionGroup]
(
	@DeploymentPlan_Id INT = NULL,
	@Deployable_Name VARCHAR(50) = NULL,
	@Deployable_Application_Id INT = NULL,
	@DeploymentPlanActionGroup_Sequence INT = NULL,
	@Server_Id INT = NULL,
	@Server_Variable_Name VARCHAR(50) = NULL,

	@Active_Indicator YNINDICATOR = NULL,
	@ActionGroup_Name VARCHAR(50) = NULL,
	@ActionGroup_Description VARCHAR(MAX) = NULL,
	@Predicate_Configuration XML = NULL,

	@DeploymentPlanActionGroup_Id INT = NULL OUT,
	@ActionGroup_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	--If sequence is not specified, create at the end	
	IF @DeploymentPlanActionGroup_Sequence IS NULL
	BEGIN
		SET @DeploymentPlanActionGroup_Sequence = COALESCE(
			(SELECT MAX([DeploymentPlanActionGroup_Sequence])
			   FROM [DeploymentPlanActionGroups]
			  WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id), 0) + 1
	END

	IF @ActionGroup_Id IS NULL
	BEGIN
		INSERT INTO [ActionGroups]
		(
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
			[ActionGroup_Name],
			[Predicate_Configuration],
			[ActionGroup_Description],
			[Server_Id],
			[Server_Variable_Name]
		)
		VALUES
		(
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			@ActionGroup_Name,
			@Predicate_Configuration,
			@ActionGroup_Description,
			@Server_Id,
			NULLIF(@Server_Variable_Name, '')
		)

		SET @ActionGroup_Id = SCOPE_IDENTITY()

		INSERT INTO [DeploymentPlanActionGroups]
		(
			[DeploymentPlan_Id],
			[DeploymentPlanActionGroup_Sequence],
			[ActionGroup_Id],
			[Deployable_Name],
			[Deployable_Application_Id],
			[Active_Indicator],
			[ActionGroup_Name],
			[ActionGroup_Description],
			[Predicate_Configuration],
			[Server_Id],
			[Server_Variable_Name],
			[Iterate_Servers_Indicator],
			[Iterate_Deployables_Indicator],
			[Parallel_Indicator],
			[OnFailure_Continue_Indicator],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@DeploymentPlan_Id,
			@DeploymentPlanActionGroup_Sequence,
			@ActionGroup_Id,
			@Deployable_Name,
			@Deployable_Application_Id,
			COALESCE(@Active_Indicator, 'Y'),
			NULL,
			NULL,
			NULL,
			NULL,
			NULL,
			'N',
			'N',
			'N',
			'N',
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		SET @DeploymentPlanActionGroup_Id = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		INSERT INTO [DeploymentPlanActionGroups]
		(
			[DeploymentPlan_Id],
			[DeploymentPlanActionGroup_Sequence],
			[ActionGroup_Id],
			[Deployable_Name],
			[Deployable_Application_Id],
			[Active_Indicator],
			[ActionGroup_Name],
			[ActionGroup_Description],
			[Predicate_Configuration],
			[Server_Id],
			[Server_Variable_Name],
			[Iterate_Servers_Indicator],
			[Iterate_Deployables_Indicator],
			[Parallel_Indicator],
			[OnFailure_Continue_Indicator],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@DeploymentPlan_Id,
			@DeploymentPlanActionGroup_Sequence,
			@ActionGroup_Id,
			@Deployable_Name,
			@Deployable_Application_Id,
			@Active_Indicator,
			NULLIF(@ActionGroup_Name, ''),
			NULLIF(@ActionGroup_Description, ''),
			@Predicate_Configuration,
			@Server_Id,
			NULLIF(@Server_Variable_Name, ''),
			'N',
			'N',
			'N',
			'N',
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		
		SET @DeploymentPlanActionGroup_Id = SCOPE_IDENTITY()
	END
	
 	-- disable reexecution for anything that used the modified plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id)

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CreateDeploymentPlanActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CreateOrUpdateAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Action_Sequence',
    /* Description_Text        */ 'Creates an action in the specified action group or updates an existing one.'
GO


IF OBJECT_ID('[Plans_CreateOrUpdateAction]') IS NOT NULL DROP PROCEDURE [Plans_CreateOrUpdateAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CreateOrUpdateAction]
(
	@ActionGroup_Id INT = NULL,
	@Action_Sequence INT = NULL OUT,
	@Server_Id INT = NULL,
	@Server_Variable_Name VARCHAR(50) = NULL,
	@Long_Description VARCHAR(500) = NULL,
	@ResumeNextOnFailure_Indicator YNINDICATOR = NULL,
	@Action_Configuration XML = NULL,
	@Short_Description VARCHAR(100) = NULL,
	@Active_Indicator YNINDICATOR = NULL,
	@Retry_Count INT = NULL,
	@LogFailureAsWarning_Indicator YNINDICATOR = NULL,
	@Target_Server_Id INT = NULL,
	@Target_Server_Variable_Name VARCHAR(50) = NULL,
	@ExtensionConfiguration_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @ActionGroup_Id IS NULL
	BEGIN
		RAISERROR('ActionGroup_Id must be specified.', 16, 1)
	END

	IF (@Action_Sequence IS NULL)
	BEGIN
		SET @Action_Sequence =
			(SELECT COALESCE(MAX([Action_Sequence]), 0) + 1
		       FROM [ActionGroupActions] WHERE [ActionGroup_Id] = @ActionGroup_Id)

		INSERT INTO [ActionGroupActions]
		(
			[ActionGroup_Id],
			[Action_Sequence],
			[Server_Id],
			[Server_Variable_Name],
			[Long_Description],
			[ResumeNextOnFailure_Indicator],
			[Action_Configuration],
			[Short_Description],
			[Active_Indicator],
			[Retry_Count],
			[LogFailureAsWarning_Indicator],
			[ExtensionConfiguration_Id],
			[Target_Server_Id],
			[Target_Server_Variable_Name]
		)
		VALUES
		(
			@ActionGroup_Id,
			@Action_Sequence,
			@Server_Id,
			@Server_Variable_Name,
			@Long_Description,
			COALESCE(@ResumeNextOnFailure_Indicator, 'N'),
			@Action_Configuration,
			@Short_Description,
			COALESCE(@Active_Indicator, 'Y'),
			COALESCE(@Retry_Count, 0),
			COALESCE(@LogFailureAsWarning_Indicator, 'N'),
			@ExtensionConfiguration_Id,
			@Target_Server_Id,
			@Target_Server_Variable_Name
		)

		-- disable reexecution for anything that used the plan containing the action group
		UPDATE [BuildExecutions]
		   SET [ReexecutionState_Xml] = NULL
		 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
			                        FROM [BuildExecution_DeploymentPlans] BEDP
			                             INNER JOIN [DeploymentPlanActionGroups] DPAG
			                                     ON DPAG.[DeploymentPlan_Id] = BEDP.[Original_DeploymentPlan_Id]
			                       WHERE DPAG.[ActionGroup_Id] = @ActionGroup_Id)

	END
	ELSE
	BEGIN
		DECLARE @Previous_Active_Indicator YNINDICATOR
		DECLARE @Previous_ResumeNextOnFailure_Indicator YNINDICATOR
		
		SELECT @Previous_Active_Indicator = [Active_Indicator]
		  FROM [ActionGroupActions]
		 WHERE [ActionGroup_Id] = @ActionGroup_Id
		   AND [Action_Sequence] = @Action_Sequence

		UPDATE [ActionGroupActions]
		   SET [Server_Id] = @Server_Id
		      ,[Server_Variable_Name] = @Server_Variable_Name
			  ,[Long_Description] = COALESCE(@Long_Description, [Long_Description])
		      ,[Action_Configuration] = COALESCE(@Action_Configuration, [Action_Configuration])
		      ,[ResumeNextOnFailure_Indicator] = COALESCE(@ResumeNextOnFailure_Indicator, [ResumeNextOnFailure_Indicator])
			  ,[Short_Description] = COALESCE(@Short_Description, [Short_Description])
			  ,[Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator])
			  ,[Retry_Count] = COALESCE(@Retry_Count, [Retry_Count])
			  ,[LogFailureAsWarning_Indicator] = COALESCE(@LogFailureAsWarning_Indicator, [LogFailureAsWarning_Indicator])
			  ,[Target_Server_Id] = @Target_Server_Id
			  ,[Target_Server_Variable_Name] = @Target_Server_Variable_Name
			  ,[ExtensionConfiguration_Id] = @ExtensionConfiguration_Id
		 WHERE [ActionGroup_Id] = @ActionGroup_Id
		   AND [Action_Sequence] = @Action_Sequence
	END

	DECLARE @Plans TABLE ([DeploymentPlan_Id] INT)
	INSERT INTO @Plans
	SELECT DISTINCT([DeploymentPlan_Id])
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id
 
	WHILE ((SELECT COUNT(*) FROM @Plans) > 0)
	BEGIN

		DECLARE @DeploymentPlan_Id INT
		SET @DeploymentPlan_Id = (SELECT TOP 1 [DeploymentPlan_Id] FROM @Plans)
		DELETE @Plans WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	END

	UPDATE [ActionGroups]
	   SET [ModifiedOn_Date] = GETUTCDATE(),
	       [ModifiedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CreateOrUpdateAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CreateOrUpdateDeploymentPlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'DeploymentPlan_Id',
    /* Description_Text        */ 'Creates a new deployment plan or changes the name of an existing deployment plan.'
GO


IF OBJECT_ID('[Plans_CreateOrUpdateDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Plans_CreateOrUpdateDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CreateOrUpdateDeploymentPlan]
(
	@DeploymentPlan_Name NVARCHAR(50),
	@AllowLocalChanges_Indicator YNINDICATOR = NULL,
	@DeploymentPlan_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @DeploymentPlan_Id IS NULL
	BEGIN

		INSERT INTO [DeploymentPlans]
		(
			[DeploymentPlan_Name],
			[AllowLocalChanges_Indicator]
		)
		VALUES
		(
			@DeploymentPlan_Name,
			COALESCE(@AllowLocalChanges_Indicator, 'N')
		)

		SET @DeploymentPlan_Id = SCOPE_IDENTITY()

	END
	ELSE
	BEGIN

		UPDATE [DeploymentPlans]
		   SET [DeploymentPlan_Name] = @DeploymentPlan_Name,
		       [AllowLocalChanges_Indicator] = COALESCE(@AllowLocalChanges_Indicator, 'N')
		 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	END

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CreateOrUpdateDeploymentPlan] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_DeleteAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an action from an action group.'
GO


IF OBJECT_ID('[Plans_DeleteAction]') IS NOT NULL DROP PROCEDURE [Plans_DeleteAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_DeleteAction]
(
	@DeploymentPlanActionGroup_Id INT,
	@Action_Sequence INT
)
AS
BEGIN

	DECLARE @ActionGroup_Id INT
	    SET @ActionGroup_Id = (SELECT [ActionGroup_Id]
	                             FROM [DeploymentPlanActionGroups]
	                            WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id)

	IF NOT EXISTS(SELECT * FROM [ActionGroupActions] WHERE [ActionGroup_Id] = @ActionGroup_Id AND [Action_Sequence] = @Action_Sequence)
	BEGIN
		/* Silently Return */
		RETURN;
	END

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	-- move action to the end of the sequence before deletion to appease trigger
    DECLARE @New_Action_Sequence INT
    SELECT @New_Action_Sequence = MAX([Action_Sequence])
      FROM [ActionGroupActions]
     WHERE [ActionGroup_Id] = @ActionGroup_Id

	UPDATE [ActionGroupActions]
	   SET [Action_Sequence] = CASE
	          WHEN [ActionGroupActions].[Action_Sequence] = CUR.[Action_Sequence] 
	               THEN NEW.[Action_Sequence]
	          WHEN [ActionGroupActions].[Action_Sequence] BETWEEN CUR.[Action_Sequence] AND NEW.[Action_Sequence]
	               THEN [ActionGroupActions].[Action_Sequence] - 1
	          WHEN [ActionGroupActions].[Action_Sequence] BETWEEN NEW.[Action_Sequence] AND CUR.[Action_Sequence]
	               THEN [ActionGroupActions].[Action_Sequence] + 1
	          ELSE [ActionGroupActions].[Action_Sequence]
	          END
	  FROM [ActionGroupActions],
	       [ActionGroupActions] CUR,
	       [ActionGroupActions] NEW
	 WHERE CUR.[Action_Sequence] = @Action_Sequence
	   AND NEW.[Action_Sequence] = @New_Action_Sequence
	   AND @Action_Sequence <> @New_Action_Sequence
	   AND [ActionGroupActions].[ActionGroup_Id] = @ActionGroup_Id
	   AND CUR.[ActionGroup_Id] = @ActionGroup_Id
	   AND NEW.[ActionGroup_Id] = @ActionGroup_Id

	-- delete the action now that it's at the end
	DELETE [ActionGroupActions]
	WHERE [ActionGroup_Id] = @ActionGroup_Id
	  AND [Action_Sequence] = @New_Action_Sequence
	  
	DECLARE @Plans TABLE ([DeploymentPlan_Id] INT)
	INSERT INTO @Plans
	SELECT DISTINCT([DeploymentPlan_Id])
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id
 
	WHILE ((SELECT COUNT(*) FROM @Plans) > 0)
	BEGIN

		DECLARE @DeploymentPlan_Id INT
		SET @DeploymentPlan_Id = (SELECT TOP 1 [DeploymentPlan_Id] FROM @Plans)
		DELETE @Plans WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	END

	UPDATE [ActionGroups]
	   SET [ModifiedOn_Date] = GETUTCDATE(),
	       [ModifiedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	-- disable reexecution for anything that used the plan containing the action group
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
		                        FROM [BuildExecution_DeploymentPlans] BEDP
		                             INNER JOIN [DeploymentPlanActionGroups] DPAG
		                                     ON DPAG.[DeploymentPlan_Id] = BEDP.[Original_DeploymentPlan_Id]
		                       WHERE DPAG.[ActionGroup_Id] = @ActionGroup_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Plans_DeleteAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_DeleteDeploymentPlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a deployment plan.'
GO


IF OBJECT_ID('[Plans_DeleteDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Plans_DeleteDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_DeleteDeploymentPlan]
(
	@DeploymentPlan_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

 	DECLARE @TmpDeploymentPlanActionGroups TABLE ([DeploymentPlanActionGroup_Id] INT)
	INSERT @TmpDeploymentPlanActionGroups
	SELECT [DeploymentPlanActionGroup_Id] 
	  FROM [DeploymentPlanActionGroups]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	 WHILE ((SELECT COUNT(*) FROM @TmpDeploymentPlanActionGroups) > 0 )
	 BEGIN
		DECLARE @DeploymentPlanActionGroup_Id INT
		SET @DeploymentPlanActionGroup_Id = (SELECT TOP 1 [DeploymentPlanActionGroup_Id] FROM @TmpDeploymentPlanActionGroups)

		EXEC [Plans_DeleteDeploymentPlanActionGroup] @DeploymentPlanActionGroup_Id

		DELETE @TmpDeploymentPlanActionGroups WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id
	 END

    UPDATE [Workflows] 
       SET [Build_DeploymentPlan_Id] = NULL 
     WHERE [Build_DeploymentPlan_Id] = @DeploymentPlan_Id

    UPDATE [WorkflowSteps] 
       SET [DeploymentPlan_Id] = NULL
     WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	DELETE [DeploymentPlans]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_DeleteDeploymentPlan] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_DeleteDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified action group.'
GO

IF OBJECT_ID('[Plans_DeleteDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_DeleteDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_DeleteDeploymentPlanActionGroup]
(
	@DeploymentPlanActionGroup_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ActionGroup_Id INT
	DECLARE @DeploymentPlanActionGroup_Sequence INT
	DECLARE @DeploymentPlan_Id INT

	 SELECT @DeploymentPlan_Id = [DeploymentPlan_Id],
	        @ActionGroup_Id = [ActionGroup_Id],
	        @DeploymentPlanActionGroup_Sequence = [DeploymentPlanActionGroup_Sequence]
	   FROM [DeploymentPlanActionGroups]
	  WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	DECLARE @Inst_Count INT
	SELECT @Inst_Count = COUNT(*)
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	-- move plan action group to the end of the sequence before deletion to appease triggers
	DECLARE @New_DeploymentPlanActionGroup_Sequence INT
	 SELECT @New_DeploymentPlanActionGroup_Sequence = MAX([DeploymentPlanActionGroup_Sequence])
	   FROM [DeploymentPlanActionGroups]
	  WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	UPDATE [DeploymentPlanActionGroups]
	   SET [DeploymentPlanActionGroup_Sequence] = CASE
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] = CUR.[DeploymentPlanActionGroup_Sequence] 
	               THEN NEW.[DeploymentPlanActionGroup_Sequence]
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] BETWEEN CUR.[DeploymentPlanActionGroup_Sequence] AND NEW.[DeploymentPlanActionGroup_Sequence]
	               THEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] - 1
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] BETWEEN NEW.[DeploymentPlanActionGroup_Sequence] AND CUR.[DeploymentPlanActionGroup_Sequence]
	               THEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] + 1
	          ELSE [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence]
	          END
	  FROM [DeploymentPlanActionGroups],
	       [DeploymentPlanActionGroups] CUR,
	       [DeploymentPlanActionGroups] NEW
	 WHERE CUR.[DeploymentPlanActionGroup_Sequence] = @DeploymentPlanActionGroup_Sequence
	   AND NEW.[DeploymentPlanActionGroup_Sequence] = @New_DeploymentPlanActionGroup_Sequence
	   AND @DeploymentPlanActionGroup_Sequence <> @New_DeploymentPlanActionGroup_Sequence
	   AND [DeploymentPlanActionGroups].[DeploymentPlan_Id] = @DeploymentPlan_Id
	   AND CUR.[DeploymentPlan_Id] = @DeploymentPlan_Id
	   AND NEW.[DeploymentPlan_Id] = @DeploymentPlan_Id

	-- is this the last PlanActionGroup for the ActionGroup?
	IF @Inst_Count = 1
	BEGIN

		DELETE [ActionGroupActions]
		 WHERE [ActionGroup_Id] = @ActionGroup_Id

		-- delete the PlanActionGroup
		DELETE [DeploymentPlanActionGroups]
		 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

		-- delete the ActionGroup
		DELETE [ActionGroups]
		 WHERE [ActionGroup_Id] = @ActionGroup_Id

	END
	ELSE
	BEGIN

		-- if there is only one other instance left, unshare it
		IF @Inst_Count = 2
		BEGIN
			DECLARE @Other_DeploymentPlanActionGroup_Id INT
			SELECT @Other_DeploymentPlanActionGroup_Id = [DeploymentPlanActionGroup_Id]
			  FROM [DeploymentPlanActionGroups]
			 WHERE [DeploymentPlanActionGroup_Id] <> @DeploymentPlanActionGroup_Id
			   AND [ActionGroup_Id] = @ActionGroup_Id

			UPDATE [ActionGroups]
			   SET [ActionGroup_Name] = COALESCE(DPAG.[ActionGroup_Name], AG.[ActionGroup_Name]),
			       [Predicate_Configuration] = COALESCE(DPAG.[Predicate_Configuration], AG.[Predicate_Configuration]),
			       [ActionGroup_Description] = COALESCE(DPAG.[ActionGroup_Description], AG.[ActionGroup_Description]),
			       [Server_Id] = COALESCE(DPAG.[Server_Id], AG.[Server_Id]),
			       [Server_Variable_Name] = COALESCE(DPAG.[Server_Variable_Name], AG.[Server_Variable_Name])
			  FROM [ActionGroups] AG
			       INNER JOIN [DeploymentPlanActionGroups] DPAG
			               ON DPAG.[DeploymentPlanActionGroup_Id] = @Other_DeploymentPlanActionGroup_Id
			 WHERE AG.[ActionGroup_Id] = @ActionGroup_Id

			 UPDATE [DeploymentPlanActionGroups]
			    SET [ActionGroup_Name] = NULL,
			        [ActionGroup_Description] = NULL,
			        [Predicate_Configuration] = NULL,
                    [Server_Id] = NULL,
                    [Server_Variable_Name] = NULL
			  WHERE [DeploymentPlanActionGroup_Id] = @Other_DeploymentPlanActionGroup_Id
		END

		-- delete the PlanActionGroup
		DELETE [DeploymentPlanActionGroups]
		 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	END

 	-- disable reexecution for anything that used the modified plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id)

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_DeleteDeploymentPlanActionGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ActionGroupActions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified action.'
GO


IF OBJECT_ID('[Plans_GetAction]') IS NOT NULL DROP PROCEDURE [Plans_GetAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetAction]
(
	@ActionGroup_Id INT,
	@Action_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [ActionGroupActions_Extended]
     WHERE [ActionGroup_Id] = @ActionGroup_Id
	   AND [Action_Sequence] = @Action_Sequence
	IF @@ROWCOUNT=0 RAISERROR('Action %i (action group #%i) does not exist.',16,1,@Action_Sequence,@ActionGroup_Id)

END
GO

GRANT EXECUTE ON [Plans_GetAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ActionGroups,ActionGroupActions_Extended,ActionGroupUsage_Slim',
    /* OutputPropertyNames_Csv */  NULL,
    /* Description_Text        */ 'Gets the details of the specified plan, and all the actions associated with the plan.'
GO


IF OBJECT_ID('[Plans_GetActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_GetActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetActionGroup]
(
	@ActionGroup_Id INT
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [ActionGroups]
     WHERE [ActionGroup_Id] = @ActionGroup_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Action group %i does not exist.', 16, 1, @ActionGroup_Id)

	SELECT *
	  FROM [ActionGroupActions_Extended]
     WHERE [ActionGroup_Id] = @ActionGroup_Id
	 ORDER BY [Action_Sequence]

	SELECT *
	  FROM [ActionGroupUsage_Slim]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

END
GO

GRANT EXECUTE ON [Plans_GetActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetActionGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ActionGroups',
    /* OutputPropertyNames_Csv */  NULL,
    /* Description_Text        */ 'Gets all action groups.'
GO

IF OBJECT_ID('[Plans_GetActionGroups]') IS NOT NULL DROP PROCEDURE [Plans_GetActionGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetActionGroups]
AS
BEGIN
	
    SET NOCOUNT ON

    SELECT * FROM [ActionGroups]

END
GO

GRANT EXECUTE ON [Plans_GetActionGroups] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Plans_GetActionsInProgress]') IS NOT NULL DROP PROCEDURE [Plans_GetActionsInProgress]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetDeploymentPlan',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'DeploymentPlans,ApplicationDeploymentPlans,DeploymentPlanActionGroups_Extended,ActionGroupActions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns action groups and actions for a deployment plan.'
GO


IF OBJECT_ID('[Plans_GetDeploymentPlan]') IS NOT NULL DROP PROCEDURE [Plans_GetDeploymentPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetDeploymentPlan]
(
	@DeploymentPlan_Id INT,
	@IncludeActionUsage_Indicator YNINDICATOR = 'N'
)
AS
BEGIN
	SET NOCOUNT ON
	
	SELECT *
	  FROM [DeploymentPlans]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	SELECT *
	  FROM [ApplicationDeploymentPlans]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

	SELECT *
	  FROM [DeploymentPlanActionGroups_Extended]
	 WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id
	 ORDER BY [DeploymentPlanActionGroup_Sequence]

	SELECT AGA.*
	  FROM [ActionGroupActions_Extended] AGA
	 WHERE AGA.[ActionGroup_Id] IN (SELECT DPAG.[ActionGroup_Id]
	                                  FROM [DeploymentPlanActionGroups] DPAG
									 WHERE DPAG.[DeploymentPlan_Id] = @DeploymentPlan_Id)
	 ORDER BY AGA.[ActionGroup_Id], AGA.[Action_Sequence]

	IF @IncludeActionUsage_Indicator = 'Y'
	BEGIN
		SELECT *
		  FROM [DeploymentPlans_ActionUsage]
		 WHERE [Uses_DeploymentPlan_Id] = @DeploymentPlan_Id
	END
	ELSE
	BEGIN
		SELECT *
		  FROM [DeploymentPlans_ActionUsage]
		 WHERE 0=1
	END

END
GO

GRANT EXECUTE ON [Plans_GetDeploymentPlan] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'DeploymentPlanActionGroups_Extended,ActionGroupActions_Extended,ActionGroupUsage_Slim,ApplicationDeploymentPlans',
    /* OutputPropertyNames_Csv */  NULL,
    /* Description_Text        */ 'Gets the details of the specified plan, and all the actions associated with the plan.'
GO


IF OBJECT_ID('[Plans_GetDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_GetDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetDeploymentPlanActionGroup]
(
	@DeploymentPlanActionGroup_Id INT
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [DeploymentPlanActionGroups_Extended]
     WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id
	
	DECLARE @ActionGroup_Id INT
	SET @ActionGroup_Id = (SELECT [ActionGroup_Id]
	                  FROM [DeploymentPlanActionGroups]
	                 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id)

	SELECT *
	  FROM [ActionGroupActions_Extended]
     WHERE [ActionGroup_Id] = @ActionGroup_Id
	 ORDER BY [Action_Sequence]

	SELECT *
	  FROM [ActionGroupUsage_Slim]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	SELECT ADP.*
	  FROM [ApplicationDeploymentPlans] ADP
	       INNER JOIN [DeploymentPlanActionGroups_Extended] DPAG
	               ON ADP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
	 WHERE DPAG.[DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

END
GO

GRANT EXECUTE ON [Plans_GetDeploymentPlanActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetDeploymentPlans',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'DeploymentPlans,ApplicationDeploymentPlans',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns the deployment plans in the system.'
GO


IF OBJECT_ID('[Plans_GetDeploymentPlans]') IS NOT NULL DROP PROCEDURE [Plans_GetDeploymentPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetDeploymentPlans]
(
	@IncludeAnonymous_Indicator YNINDICATOR
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [DeploymentPlans]
	 WHERE @IncludeAnonymous_Indicator = 'Y' OR [DeploymentPlan_Name] IS NOT NULL
	 ORDER BY [DeploymentPlan_Name]

	SELECT ADP.*
	  FROM [ApplicationDeploymentPlans] ADP
	       INNER JOIN [DeploymentPlans] DP
	               ON DP.[DeploymentPlan_Id] = ADP.[DeploymentPlan_Id]
	 WHERE @IncludeAnonymous_Indicator = 'Y' OR DP.[DeploymentPlan_Name] IS NOT NULL

END
GO

GRANT EXECUTE ON [Plans_GetDeploymentPlans] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetGlobalDeploymentPlans',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'DeploymentPlans,Workflows_Extended,WorkflowSteps_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns named deployment plans and optionally plans orphaned.'
GO


IF OBJECT_ID('[Plans_GetGlobalDeploymentPlans]') IS NOT NULL DROP PROCEDURE [Plans_GetGlobalDeploymentPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetGlobalDeploymentPlans]  AS
BEGIN
	SET NOCOUNT ON
	
	SELECT * INTO #DeploymentPlans
	  FROM [DeploymentPlans]
	 WHERE [DeploymentPlan_Name] IS NOT NULL
	    OR (    [DeploymentPlan_Id] NOT IN (SELECT [DeploymentPlan_Id] FROM [WorkflowSteps] WHERE [DeploymentPlan_Id] IS NOT NULL)
	        AND [DeploymentPlan_Id] NOT IN (SELECT [Build_DeploymentPlan_Id] FROM [Workflows] WHERE [Build_DeploymentPlan_Id] IS NOT NULL))

	SELECT * FROM #DeploymentPlans
         ORDER BY [DeploymentPlan_Name]
	
	SELECT *
	  FROM [Workflows_Extended]
	 WHERE [Build_DeploymentPlan_Id] IN (SELECT [DeploymentPlan_Id] FROM #DeploymentPlans)
	    OR [Workflow_Id] IN (SELECT [Workflow_Id]
		                       FROM [WorkflowSteps] _WS
		                            INNER JOIN #DeploymentPlans _DP 
									        ON _WS.[DeploymentPlan_Id] = _DP.[DeploymentPlan_Id])

	SELECT *
	  FROM [WorkflowSteps_Extended]
	 WHERE [Workflow_Id] IN (SELECT [Workflow_Id]
		                       FROM [WorkflowSteps] _WS
		                            INNER JOIN #DeploymentPlans _DP 
									        ON _WS.[DeploymentPlan_Id] = _DP.[DeploymentPlan_Id])
									 
END
GO

GRANT EXECUTE ON [Plans_GetGlobalDeploymentPlans] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetMostCommonActions',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ 'Plans_MostCommonActions',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns the most commonly used actions in the system.'
GO


IF OBJECT_ID('[Plans_GetMostCommonActions]') IS NOT NULL DROP PROCEDURE [Plans_GetMostCommonActions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetMostCommonActions]
(
    @MostRecent_Count INT
)
AS
BEGIN

    SET NOCOUNT ON

    IF (@MostRecent_Count IS NOT NULL)
        SET ROWCOUNT @MostRecent_Count

	SELECT *
      FROM [Plans_MostCommonActions]
     ORDER BY [Usage_Count] DESC

END
GO

GRANT EXECUTE ON [Plans_GetMostCommonActions] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ResequenceAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the sequence of an action within a plan.'
GO


IF OBJECT_ID('[Plans_ResequenceAction]') IS NOT NULL DROP PROCEDURE [Plans_ResequenceAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ResequenceAction]
(
	@DeploymentPlanActionGroup_Id INT,
	@Action_Sequence INT,
	@New_Action_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	DECLARE @ActionGroup_Id INT
	SELECT @ActionGroup_Id = [ActionGroup_Id]
	  FROM [DeploymentPlanActionGroups]
	 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	BEGIN TRANSACTION

	UPDATE [ActionGroupActions]
	   SET [Action_Sequence] = CASE
	          WHEN [ActionGroupActions].[Action_Sequence] = CUR.[Action_Sequence] 
	               THEN NEW.[Action_Sequence]
	          WHEN [ActionGroupActions].[Action_Sequence] BETWEEN CUR.[Action_Sequence] AND NEW.[Action_Sequence]
	               THEN [ActionGroupActions].[Action_Sequence] - 1
	          WHEN [ActionGroupActions].[Action_Sequence] BETWEEN NEW.[Action_Sequence] AND CUR.[Action_Sequence]
	               THEN [ActionGroupActions].[Action_Sequence] + 1
	          ELSE [ActionGroupActions].[Action_Sequence]
	          END
	  FROM [ActionGroupActions],
	       [ActionGroupActions] CUR,
	       [ActionGroupActions] NEW
	 WHERE CUR.[Action_Sequence] = @Action_Sequence
	   AND NEW.[Action_Sequence] = @New_Action_Sequence
	   AND @Action_Sequence <> @New_Action_Sequence
	   AND [ActionGroupActions].[ActionGroup_Id] = @ActionGroup_Id
	   AND CUR.[ActionGroup_Id] = @ActionGroup_Id
	   AND NEW.[ActionGroup_Id] = @ActionGroup_Id


	DECLARE @Plans TABLE ([DeploymentPlan_Id] INT)
	INSERT INTO @Plans
	SELECT DISTINCT([DeploymentPlan_Id])
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id
 
	WHILE ((SELECT COUNT(*) FROM @Plans) > 0)
	BEGIN

		DECLARE @DeploymentPlan_Id INT
		SET @DeploymentPlan_Id = (SELECT TOP 1 [DeploymentPlan_Id] FROM @Plans)
		DELETE @Plans WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	END

	UPDATE [ActionGroups]
	   SET [ModifiedOn_Date] = GETUTCDATE(),
	       [ModifiedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

 	-- disable reexecution for anything that used the plan containing the action group
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
		                        FROM [BuildExecution_DeploymentPlans] BEDP
		                             INNER JOIN [DeploymentPlanActionGroups] DPAG
		                                     ON DPAG.[DeploymentPlan_Id] = BEDP.[Original_DeploymentPlan_Id]
		                       WHERE DPAG.[ActionGroup_Id] = @ActionGroup_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ResequenceAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ResequenceActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the sequence of a deployment plan action group.'
GO


IF OBJECT_ID('[Plans_ResequenceActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_ResequenceActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ResequenceActionGroup]
(
	@DeploymentPlanActionGroup_Id INT,
	@New_DeploymentPlanActionGroup_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

    DECLARE @DeploymentPlanActionGroup_Sequence INT
    DECLARE @ActionGroup_Id INT
	DECLARE @DeploymentPlan_Id INT

	SELECT @DeploymentPlan_Id = [DeploymentPlan_Id],
	       @ActionGroup_Id = [ActionGroup_Id],
	       @DeploymentPlanActionGroup_Sequence = [DeploymentPlanActionGroup_Sequence]
	  FROM [DeploymentPlanActionGroups]
	 WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	BEGIN TRANSACTION

	UPDATE [DeploymentPlanActionGroups]
	   SET [DeploymentPlanActionGroup_Sequence] = CASE
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] = CUR.[DeploymentPlanActionGroup_Sequence] 
	               THEN NEW.[DeploymentPlanActionGroup_Sequence]
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] BETWEEN CUR.[DeploymentPlanActionGroup_Sequence] AND NEW.[DeploymentPlanActionGroup_Sequence]
	               THEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] - 1
	          WHEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] BETWEEN NEW.[DeploymentPlanActionGroup_Sequence] AND CUR.[DeploymentPlanActionGroup_Sequence]
	               THEN [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence] + 1
	          ELSE [DeploymentPlanActionGroups].[DeploymentPlanActionGroup_Sequence]
	          END
	  FROM [DeploymentPlanActionGroups],
	       [DeploymentPlanActionGroups] CUR,
	       [DeploymentPlanActionGroups] NEW
	 WHERE CUR.[DeploymentPlanActionGroup_Sequence] = @DeploymentPlanActionGroup_Sequence
	   AND NEW.[DeploymentPlanActionGroup_Sequence] = @New_DeploymentPlanActionGroup_Sequence
	   AND @DeploymentPlanActionGroup_Sequence <> @New_DeploymentPlanActionGroup_Sequence
	   AND [DeploymentPlanActionGroups].[DeploymentPlan_Id] = @DeploymentPlan_Id
	   AND CUR.[DeploymentPlan_Id] = @DeploymentPlan_Id
	   AND NEW.[DeploymentPlan_Id] = @DeploymentPlan_Id

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

 	-- disable reexecution for anything that used the modified plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ResequenceActionGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_UpdateActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the specified action group.'
GO


IF OBJECT_ID('[Plans_UpdateActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_UpdateActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_UpdateActionGroup]
(
	@ActionGroup_Id INT,
	@ActionGroup_Name VARCHAR(50),
	@ActionGroup_Description VARCHAR(MAX),
	@Predicate_Configuration XML,
	@Server_Id INT,
	@Server_Variable_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Old_Predicate_Configuration XML
	SELECT @Old_Predicate_Configuration = [Predicate_Configuration]
	  FROM [ActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	UPDATE [ActionGroups]
	   SET [ActionGroup_Name] = NULLIF(@ActionGroup_Name, '')
		  ,[ActionGroup_Description] = NULLIF(@ActionGroup_Description, '')
	      ,[Predicate_Configuration] = @Predicate_Configuration
	      ,[ModifiedOn_Date] = GETUTCDATE()
	      ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
	      ,[Server_Id] = @Server_Id
	      ,[Server_Variable_Name] = NULLIF(@Server_Variable_Name, '')
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	DECLARE @Plans TABLE ([DeploymentPlan_Id] INT)
	INSERT INTO @Plans
	SELECT DISTINCT([DeploymentPlan_Id])
	  FROM [DeploymentPlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id
 
	WHILE ((SELECT COUNT(*) FROM @Plans) > 0)
	BEGIN

		DECLARE @DeploymentPlan_Id INT
		SET @DeploymentPlan_Id = (SELECT TOP 1 [DeploymentPlan_Id] FROM @Plans)
		DELETE @Plans WHERE [DeploymentPlan_Id] = @DeploymentPlan_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

	END

 	-- disable reexecution for anything that used the plan containing the action group
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
		                        FROM [BuildExecution_DeploymentPlans] BEDP
		                             INNER JOIN [DeploymentPlanActionGroups] DPAG
		                                     ON DPAG.[DeploymentPlan_Id] = BEDP.[Original_DeploymentPlan_Id]
		                       WHERE DPAG.[ActionGroup_Id] = @ActionGroup_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_UpdateActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_UpdateDeploymentPlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
     /* Description_Text        */ 'Updates the specified deployment plan action group.'
GO

IF OBJECT_ID('[Plans_UpdateDeploymentPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_UpdateDeploymentPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_UpdateDeploymentPlanActionGroup]
(
	@DeploymentPlanActionGroup_Id INT,
	@Active_Indicator YNINDICATOR,
	@ActionGroup_Name VARCHAR(50),
	@ActionGroup_Description VARCHAR(MAX),
	@Predicate_Configuration XML,
	@Server_Id INT,
	@Server_Variable_Name VARCHAR(50),
	@Deployable_Name VARCHAR(50),
	@Deployable_Application_Id INT,
	@Iterate_Servers_Indicator YNINDICATOR,
	@Iterate_Deployables_Indicator YNINDICATOR,
	@Parallel_Indicator YNINDICATOR,
	@OnSuccess_DeploymentPlanActionGroup_Id INT,
	@OnFailure_DeploymentPlanActionGroup_Id INT,
	@OnFailure_Continue_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DECLARE @DeploymentPlan_Id INT
	SET @DeploymentPlan_Id = (SELECT [DeploymentPlan_Id]
	                            FROM [DeploymentPlanActionGroups]
	                           WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id)

    UPDATE [DeploymentPlanActionGroups]
       SET [Active_Indicator] = @Active_Indicator
          ,[ActionGroup_Name] = NULLIF(@ActionGroup_Name, '')
          ,[ActionGroup_Description] = NULLIF(@ActionGroup_Description, '')
          ,[Predicate_Configuration] = @Predicate_Configuration
          ,[Server_Id] = @Server_Id
          ,[Server_Variable_Name] = NULLIF(@Server_Variable_Name, '')
          ,[Deployable_Name] = NULLIF(@Deployable_Name, '')
          ,[Deployable_Application_Id] = @Deployable_Application_Id
          ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
          ,[ModifiedOn_Date] = GETUTCDATE()
          ,[Iterate_Servers_Indicator] = @Iterate_Servers_Indicator
          ,[Iterate_Deployables_Indicator] = @Iterate_Deployables_Indicator
          ,[Parallel_Indicator] = @Parallel_Indicator
          ,[OnSuccess_DeploymentPlanActionGroup_Id] = @OnSuccess_DeploymentPlanActionGroup_Id
          ,[OnFailure_DeploymentPlanActionGroup_Id] = @OnFailure_DeploymentPlanActionGroup_Id
          ,[OnFailure_Continue_Indicator] = @OnFailure_Continue_Indicator
     WHERE [DeploymentPlanActionGroup_Id] = @DeploymentPlanActionGroup_Id

	EXEC [Events_RaiseEvent] 'PLNMOD', NULL, @DeploymentPlan_Id

 	-- disable reexecution for anything that used the modified plan
	UPDATE [BuildExecutions]
	   SET [ReexecutionState_Xml] = NULL
	 WHERE [Execution_Id] IN (SELECT BEDP.[Execution_Id]
	                            FROM [BuildExecution_DeploymentPlans] BEDP
	                           WHERE BEDP.[Original_DeploymentPlan_Id] = @DeploymentPlan_Id)
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_UpdateDeploymentPlanActionGroup] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[PromotionRequirements_CreateOrUpdatePromotionRequirement]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_CreateOrUpdatePromotionRequirement]
GO

GO
IF OBJECT_ID('[PromotionRequirements_DeletePromotionRequirement]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_DeletePromotionRequirement]
GO

GO
IF OBJECT_ID('[PromotionRequirements_GetPromotionRequirement]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_GetPromotionRequirement]
GO

GO
IF OBJECT_ID('[PromotionRequirements_GetPromotionRequirements]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_GetPromotionRequirements]
GO

GO
IF OBJECT_ID('[PromotionRequirements_GetPromotionRequirementStatus]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_GetPromotionRequirementStatus]
GO

GO
IF OBJECT_ID('[PromotionRequirements_SetPromotionRequirementStatus]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_SetPromotionRequirementStatus]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_AddApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Approval_Id',
    /* Description_Text        */ 'Adds an approval for a promotion.'
GO

IF OBJECT_ID('[Promotions_AddApproval]') IS NOT NULL 
	DROP PROCEDURE [Promotions_AddApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_AddApproval]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@Requirement_Id INT,
	@User_Name USERNAME = NULL,
	@Comments_Text NVARCHAR(1000) = NULL,
	@Approval_Id INT = NULL OUT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [BuildPromotions]
			   WHERE [Application_Id] = @Application_Id
			     AND [Release_Number] = @Release_Number
				 AND [Build_Number] = @Build_Number
				 AND [Environment_Id] = @Environment_Id)
	BEGIN
		RAISERROR('Build has already been promoted. Cannot add approval.', 16, 1)
	END

	DECLARE @Requirement_Description NVARCHAR(1000)
	DECLARE @RequirementType_Code1 CHAR(1)
	DECLARE @RequirementType_Code2 CHAR(2)

	SELECT @Requirement_Description = [Requirement_Description],
	       @RequirementType_Code1 = CASE
		                               WHEN [Requirement_Configuration] IS NOT NULL THEN 'A'
									   ELSE 'M'
									END,
		   @RequirementType_Code2 = CASE
		                               WHEN [Workflow_Id] IS NOT NULL THEN 'W'
									   ELSE 'E'
									END
	  FROM [BuildPromotionRequirements]
	 WHERE [Requirement_Id] = @Requirement_Id

	INSERT INTO [BuildApprovalsReceived]
	(
		[Application_Id],
		[Release_Number],
		[Build_Number],
		[Environment_Id],
		[Original_Requirement_Id],
		[RequirementType_Code],
		[User_Name],
		[Comments_Text],
		[Received_Date],
		[Requirement_Description]
	)
	VALUES
	(
		@Application_Id,
		@Release_Number,
		@Build_Number,
		@Environment_Id,
		@Requirement_Id,
		@RequirementType_Code1 + @RequirementType_Code2,
		@User_Name,
		NULLIF(@Comments_Text, ''),
		GETUTCDATE(),
		@Requirement_Description
	)

	SET @Approval_Id = SCOPE_IDENTITY()

	EXEC [Events_RaiseEvent] 'BLDAPR', @Application_Id, @Approval_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_AddApproval] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_CreateOrUpdateRequirement',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Requirement_Id',
    /* Description_Text        */ 'Adds a requirement for a promotion.'
GO

IF OBJECT_ID('[Promotions_CreateOrUpdateRequirement]') IS NOT NULL 
	DROP PROCEDURE [Promotions_CreateOrUpdateRequirement]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_CreateOrUpdateRequirement]
(
	@Environment_Id INT,
	@Requirement_Description NVARCHAR(1000),
	@Workflow_Id INT = NULL,
	@Principal_Name USERNAME = NULL,
	@RequiredInGroup_Count INT = NULL,
	@Requirement_Configuration XML = NULL,
	@Requirement_Id INT = NULL OUT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Requirement_Id IS NULL
	BEGIN

		INSERT INTO [BuildPromotionRequirements]
		(
			[Workflow_Id],
			[Environment_Id],
			[Principal_Name],
			[RequiredInGroup_Count],
			[Requirement_Description],
			[Requirement_Configuration],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@Workflow_Id,
			@Environment_Id,
			@Principal_Name,
			@RequiredInGroup_Count,
			@Requirement_Description,
			@Requirement_Configuration,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		SET @Requirement_Id = SCOPE_IDENTITY()

	END
	ELSE
	BEGIN

		UPDATE [BuildPromotionRequirements]
		   SET [Principal_Name] = @Principal_Name,
		       [RequiredInGroup_Count] = @RequiredInGroup_Count,
			   [Requirement_Description] = @Requirement_Description,
			   [Requirement_Configuration] = @Requirement_Configuration
		 WHERE [Requirement_Id] = @Requirement_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_CreateOrUpdateRequirement] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_DeleteApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an approval for a promotion.'
GO

IF OBJECT_ID('[Promotions_DeleteApproval]') IS NOT NULL 
	DROP PROCEDURE [Promotions_DeleteApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_DeleteApproval]
(
	@Approval_Id INT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [BuildPromotions] BP
				     INNER JOIN [BuildApprovalsReceived] BAR
					         ON BAR.[Application_Id] = BP.[Application_Id]
							AND BAR.[Release_Number] = BP.[Release_Number]
							AND BAR.[Build_Number] = BP.[Build_Number]
							AND BAR.[Environment_Id] = BP.[Environment_Id]
			   WHERE BAR.[Approval_Id] = @Approval_Id)
	BEGIN
		RAISERROR('Build has already been promoted. Cannot delete approval.', 16, 1)
	END

	DELETE [BuildApprovalsReceived]
	 WHERE [Approval_Id] = @Approval_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_DeleteApproval] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_DeleteRequirement',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a promotion requirement.'
GO

IF OBJECT_ID('[Promotions_DeleteRequirement]') IS NOT NULL 
	DROP PROCEDURE [Promotions_DeleteRequirement]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_DeleteRequirement]
(
	@Requirement_Id INT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [BuildPromotionRequirements]
	 WHERE [Requirement_Id] = @Requirement_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_DeleteRequirement] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_GetApprovalsReceived',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotionRequirements_Extended,BuildApprovalsReceived_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all requirements and approvals received for the specified promotion.'
GO

IF OBJECT_ID('[Promotions_GetApprovalsReceived]') IS NOT NULL 
	DROP PROCEDURE [Promotions_GetApprovalsReceived]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_GetApprovalsReceived]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@Approval_Id INT = NULL
) AS BEGIN

SET NOCOUNT ON

	IF @Approval_Id IS NULL
	BEGIN

		SELECT BPR.*
		  FROM [Builds] B
		       INNER JOIN [Releases] R
			           ON R.[Application_Id] = B.[Application_Id]
					  AND R.[Release_Number] = B.[Release_Number]
			   INNER JOIN [WorkflowSteps] WS
			           ON WS.[Workflow_Id] = R.[Workflow_Id]
			   INNER JOIN [BuildPromotionRequirements_Extended] BPR
			           ON BPR.[Environment_Id] = WS.[Environment_Id]
					  AND (BPR.[Workflow_Id] IS NULL OR BPR.[Workflow_Id] = R.[Workflow_Id])
	     WHERE B.[Application_Id] = @Application_Id
		   AND B.[Release_Number] = @Release_Number
		   AND B.[Build_Number] = @Build_Number
		   AND (@Environment_Id IS NULL OR WS.[Environment_Id] = @Environment_Id)

		SELECT *
		  FROM [BuildApprovalsReceived_Extended]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND (@Environment_Id IS NULL OR [Environment_Id] = @Environment_Id)

	END
	ELSE
	BEGIN

		SELECT BPR.*
		  FROM [BuildApprovalsReceived_Extended] BAR
		       INNER JOIN [BuildPromotionRequirements_Extended] BPR
			           ON BPR.[Requirement_Id] = BAR.[Original_Requirement_Id]
	     WHERE BAR.[Approval_Id] = @Approval_Id

		SELECT *
		  FROM [BuildApprovalsReceived_Extended]
		 WHERE [Approval_Id] = @Approval_Id

	END

END
GO

GRANT EXECUTE ON [Promotions_GetApprovalsReceived] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_GetRequirement',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildPromotionRequirements_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified promotion requirement.'
GO

IF OBJECT_ID('[Promotions_GetRequirement]') IS NOT NULL 
	DROP PROCEDURE [Promotions_GetRequirement]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_GetRequirement]
(
	@Requirement_Id INT
) AS BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [BuildPromotionRequirements_Extended]
	 WHERE [Requirement_Id] = @Requirement_Id

END
GO

GRANT EXECUTE ON [Promotions_GetRequirement] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_GetRequirements',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotionRequirements_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all requirements for the specified promotion.'
GO

IF OBJECT_ID('[Promotions_GetRequirements]') IS NOT NULL 
	DROP PROCEDURE [Promotions_GetRequirements]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_GetRequirements]
(
	@Environment_Id INT,
	@Workflow_Id INT
) AS BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [BuildPromotionRequirements_Extended]
	 WHERE [Environment_Id] = @Environment_Id
	   AND ([Workflow_Id] IS NULL OR [Workflow_Id] = @Workflow_Id)

END
GO

GRANT EXECUTE ON [Promotions_GetRequirements] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_GetRequirementsToVerify',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotionRequirements_VerificationRequired',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all requirements that need to be verified.'
GO

IF OBJECT_ID('[Promotions_GetRequirementsToVerify]') IS NOT NULL 
	DROP PROCEDURE [Promotions_GetRequirementsToVerify]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_GetRequirementsToVerify]
(
	@Application_Id INT = NULL
)
AS BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [BuildPromotionRequirements_VerificationRequired]
	 WHERE @Application_Id IS NULL OR [Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [Promotions_GetRequirementsToVerify] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_GetScheduled',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotionsScheduled',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets scheduled promotions, optionally filtered by promotion and date.'
GO

IF OBJECT_ID('[Promotions_GetScheduled]') IS NOT NULL 
	DROP PROCEDURE [Promotions_GetScheduled]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_GetScheduled]
(
	@Application_Id INT = NULL,
	@Release_Number RELEASENUMBER = NULL,
	@Build_Number BUILDNUMBER = NULL,
	@Environment_Id INT = NULL,
	@Before_Date DATETIME = NULL
) AS BEGIN

SET NOCOUNT ON

	IF @Application_Id IS NULL
	BEGIN

		SELECT BPS.*
		  FROM [BuildPromotionsScheduled] BPS
		       INNER JOIN [Builds] B
			           ON B.[Application_Id] = BPS.[Application_Id]
					  AND B.[Release_Number] = BPS.[Release_Number]
					  AND B.[Build_Number] = BPS.[Build_Number]
		       LEFT JOIN [BuildPromotions] BP
			          ON BP.[Application_Id] = BPS.[Application_Id]
					 AND BP.[Release_Number] = BPS.[Release_Number]
					 AND BP.[Build_Number] = BPS.[Build_Number]
					 AND BP.[Environment_Id] = BPS.[Environment_Id]
		 WHERE @Before_Date IS NULL OR BPS.[PromoteOn_Date] < @Before_Date
		   AND BP.[Application_Id] IS NULL
		   AND B.[BuildStatus_Name] = 'Active'
		 ORDER BY BPS.[PromoteOn_Date] DESC

	END
	ELSE
	BEGIN

		SELECT BPS.*
		  FROM [BuildPromotionsScheduled] BPS
		       INNER JOIN [Builds] B
			           ON B.[Application_Id] = BPS.[Application_Id]
					  AND B.[Release_Number] = BPS.[Release_Number]
					  AND B.[Build_Number] = BPS.[Build_Number]
		       LEFT JOIN [BuildPromotions] BP
			          ON BP.[Application_Id] = BPS.[Application_Id]
					 AND BP.[Release_Number] = BPS.[Release_Number]
					 AND BP.[Build_Number] = BPS.[Build_Number]
					 AND BP.[Environment_Id] = BPS.[Environment_Id]
		 WHERE BPS.[Application_Id] = @Application_Id
		   AND BPS.[Release_Number] = COALESCE(@Release_Number, BPS.[Release_Number])
		   AND BPS.[Build_Number] = COALESCE(@Build_Number, BPS.[Build_Number])
		   AND BPS.[Environment_Id] = COALESCE(@Environment_Id, BPS.[Environment_Id])
		   AND (@Before_Date IS NULL OR BPS.[PromoteOn_Date] < @Before_Date)
		   AND BP.[Application_Id] IS NULL
		   AND B.[BuildStatus_Name] = 'Active'
		 ORDER BY BPS.[PromoteOn_Date] DESC

	END

END
GO

GRANT EXECUTE ON [Promotions_GetScheduled] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_SchedulePromotion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Schedules a promotion or removes a previously scheduled promotion.'
GO

IF OBJECT_ID('[Promotions_SchedulePromotion]') IS NOT NULL 
	DROP PROCEDURE [Promotions_SchedulePromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_SchedulePromotion]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@PromoteOn_Date DATETIME
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [BuildPromotions]
			   WHERE [Application_Id] = @Application_Id
			     AND [Release_Number] = @Release_Number
				 AND [Build_Number] = @Build_Number
				 AND [Environment_Id] = @Environment_Id)
	BEGIN
		RAISERROR('Build has already been promoted. Cannot schedule promotion.', 16, 1)
	END

	IF @PromoteOn_Date IS NULL
	BEGIN

		DELETE [BuildPromotionsScheduled]
	     WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] = @Environment_Id

	END
	ELSE IF EXISTS(SELECT *
	            FROM [BuildPromotionsScheduled]
			   WHERE [Application_Id] = @Application_Id
			     AND [Release_Number] = @Release_Number
				 AND [Build_Number] = @Build_Number
				 AND [Environment_Id] = @Environment_Id)
	BEGIN

		UPDATE [BuildPromotionsScheduled]
		   SET [PromoteOn_Date] = @PromoteOn_Date
	     WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] = @Environment_Id

	END
	ELSE
	BEGIN

		INSERT INTO [BuildPromotionsScheduled]
		(
			[Application_Id],
			[Release_Number],
			[Build_Number],
			[Environment_Id],
			[PromoteOn_Date]
		)
		VALUES
		(
			@Application_Id,
			@Release_Number,
			@Build_Number,
			@Environment_Id,
			@PromoteOn_Date
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_SchedulePromotion] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Promotions_SetAutomaticRequirementStatus',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Records the status of an automatic promotion requirement.'
GO

IF OBJECT_ID('[Promotions_SetAutomaticRequirementStatus]') IS NOT NULL 
	DROP PROCEDURE [Promotions_SetAutomaticRequirementStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Promotions_SetAutomaticRequirementStatus]
(
	@Requirement_Id INT,
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@Applicable_Indicator YNINDICATOR,
	@Met_Indicator YNINDICATOR
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Met_Indicator = 'N' AND @Applicable_Indicator = 'Y'
	BEGIN

		DELETE [BuildApprovalsReceived]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] = @Environment_Id
		   AND [Original_Requirement_Id] = @Requirement_Id
		   AND [RequirementType_Code] IN ('AW', 'AE')

	END
	ELSE IF EXISTS(SELECT *
	            FROM [BuildApprovalsReceived]
			   WHERE [Application_Id] = @Application_Id
			     AND [Release_Number] = @Release_Number
				 AND [Build_Number] = @Build_Number
				 AND [Environment_Id] = @Environment_Id
				 AND [Original_Requirement_Id] = @Requirement_Id
				 AND [RequirementType_Code] IN ('AW', 'AE'))
	BEGIN


		UPDATE [BuildApprovalsReceived]
		   SET [Received_Date] = GETUTCDATE(),
		       [Comments_Text] = CASE WHEN @Applicable_Indicator = 'N' THEN 'not applicable' ELSE 'met' END
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] = @Environment_Id
		   AND [Original_Requirement_Id] = @Requirement_Id
		   AND [RequirementType_Code] IN ('AW', 'AE')

	END
	ELSE
	BEGIN

		INSERT INTO [BuildApprovalsReceived]
		(
			[Application_Id],
			[Release_Number],
			[Build_Number],
			[Environment_Id],
			[Original_Requirement_Id],
			[RequirementType_Code],
			[Comments_Text],
			[Received_Date],
			[Requirement_Description]
		)
		SELECT @Application_Id,
			   @Release_Number,
			   @Build_Number,
			   @Environment_Id,
			   @Requirement_Id,
			   CASE WHEN [Workflow_Id] IS NULL THEN 'AE' ELSE 'AW' END,
			   CASE WHEN @Applicable_Indicator = 'N' THEN 'not applicable' ELSE 'met' END,
			   GETUTCDATE(),
			   [Requirement_Description]
		  FROM [BuildPromotionRequirements]
		 WHERE [Requirement_Id] = @Requirement_Id
		   AND [Requirement_Configuration] IS NOT NULL

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Promotions_SetAutomaticRequirementStatus] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_CreateOrUpdateProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Provider_Id',
    /* Description_Text        */ 'Creates a new provider or updates an existing one.'
GO


IF OBJECT_ID('[Providers_CreateOrUpdateProvider]') IS NOT NULL DROP PROCEDURE [Providers_CreateOrUpdateProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_CreateOrUpdateProvider]
(
	@Provider_Id INT = NULL OUT,
	@ProviderType_Code CHAR(1),
	@ExecuteOn_Server_Id INT,
	@Provider_Name VARCHAR(50),
	@Provider_Description VARCHAR(100),
	@Provider_Configuration VARCHAR(MAX),
	@Internal_Indicator YNINDICATOR	 = 'N'
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    -- BitChecker recipe checks for the existence of a provider before attempting to create one, but
    -- Providers_GetProviders doesn't return internal providers - so without a new SDK it can never know
    -- if an example provider was created already. This hack will ensure the recipe can be run more than once.
    IF (@Provider_Id IS NULL AND @Internal_Indicator = 'Y')
    BEGIN

        SELECT @Provider_Id = [Provider_Id]
          FROM [Providers]
         WHERE [Provider_Name] = @Provider_Name
           AND [ProviderType_Code] = @ProviderType_Code

            IF (@Provider_Id IS NOT NULL)
            BEGIN
                COMMIT
                RETURN
            END

    END

	IF (@Provider_Id IS NULL) 
	BEGIN

		IF @ProviderType_Code <> 'B' AND EXISTS(SELECT * FROM [Providers] WHERE [Provider_Name] = @Provider_Name AND [ProviderType_Code] = @ProviderType_Code)
			RAISERROR('A provider with that name already exists.', 16, 1)

		INSERT INTO [Providers]
            (
				[ProviderType_Code],
				[Provider_Name],
				[ExecuteOn_Server_Id],
				[Provider_Description],
				[Provider_Configuration],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date],
				[Internal_Indicator]
			)
		VALUES
            (
				@ProviderType_Code,
				@Provider_Name,
				@ExecuteOn_Server_Id,
				@Provider_Description,
				@Provider_Configuration,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				@Internal_Indicator
			)
		SET @Provider_Id = SCOPE_IDENTITY()

		IF @ProviderType_Code = 'B' /*DataBase */
			EXEC [Events_RaiseEvent] 'DBSNEW', NULL, @Provider_Id
		ELSE
			EXEC [Events_RaiseEvent] 'PVDNEW', NULL, @Provider_Id
	END
	ELSE
	BEGIN
		IF EXISTS(SELECT * FROM [Providers]
		           WHERE [Provider_Id] = @Provider_Id
		             AND [Internal_Indicator] = 'Y')
		    RAISERROR('A provider marked as internal may not be edited.', 16, 1)

		IF @ProviderType_Code <> 'B' AND EXISTS(SELECT * FROM [Providers] WHERE [Provider_Name] = @Provider_Name AND [Provider_Id] <> @Provider_Id AND [ProviderType_Code] = @ProviderType_Code)
			RAISERROR('A provider with that name already exists.', 16, 1)

		UPDATE [Providers]
		   SET [ProviderType_Code] = @ProviderType_Code,
		       [Provider_Name] = @Provider_Name,
			   [ExecuteOn_Server_Id] = @ExecuteOn_Server_Id,
			   [Provider_Description] = @Provider_Description,
			   [Provider_Configuration] = @Provider_Configuration,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Provider_Id] = @Provider_Id

		IF @ProviderType_Code = 'B' /*DataBase */
			EXEC [Events_RaiseEvent] 'DBSMOD', NULL, @Provider_Id
		ELSE
			EXEC [Events_RaiseEvent] 'PVDMOD', NULL, @Provider_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Providers_CreateOrUpdateProvider] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_DeleteProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified provider'
GO


IF OBJECT_ID('[Providers_DeleteProvider]') IS NOT NULL DROP PROCEDURE [Providers_DeleteProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_DeleteProvider]
(
	@Provider_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ProviderType_Code CHAR(1), @Provider_Name SQL_VARIANT
		SELECT @Provider_Name = [Provider_Name],
			   @ProviderType_Code = [ProviderType_Code]
		  FROM [Providers]
		 WHERE [Provider_Id] = @Provider_Id

	IF @ProviderType_Code = 'B' BEGIN
		RAISERROR('Database providers may not be deleted with this procedure.', 16, 1)
		END

	ELSE IF @ProviderType_Code = 'I' BEGIN
		UPDATE [Applications] 
		   SET [IssueTracking_Provider_Id] = NULL
		 WHERE [IssueTracking_Provider_Id] = @Provider_Id
		END
	
	ELSE IF @ProviderType_Code = 'S' BEGIN
		DELETE [BuildSchedules]
		 WHERE [SourceControl_Provider_Id] = @Provider_Id
		END

	ELSE IF @ProviderType_Code IS NOT NULL BEGIN
		RAISERROR('Deleting Provider type %s is not be supported.', 16, 1, @ProviderType_Code)
		END

	DELETE [Providers]
	 WHERE [Provider_Id] = @Provider_Id

	IF @@ROWCOUNT>0
	EXEC [Events_RaiseEvent] 'PVDDEL', NULL, @Provider_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Providers_DeleteProvider] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_GetProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Providers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified provider'
GO


IF OBJECT_ID('[Providers_GetProvider]') IS NOT NULL DROP PROCEDURE [Providers_GetProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_GetProvider]
(
	@Provider_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Providers]
	 WHERE [Provider_Id] = @Provider_Id

END
GO

GRANT EXECUTE ON [Providers_GetProvider] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_GetProviders',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Providers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of providers in the system based on the specified criteria'
GO

IF OBJECT_ID('[Providers_GetProviders]') IS NOT NULL DROP PROCEDURE [Providers_GetProviders]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_GetProviders]
(
	@ProviderType_Code CHAR(1) = NULL,
	@Application_Id INT = NULL,
	@Environment_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Providers]
	 WHERE [ProviderType_Code] = COALESCE(@ProviderType_Code, [ProviderType_Code])
	   AND [Internal_Indicator] = 'N'	 
	   AND (
	          @Application_Id IS NULL  
		  OR (@ProviderType_Code = 'B' AND [Provider_Id]
						  IN (SELECT _CSP.[Provider_Id]
								FROM [DatabaseChangeScriptProviders] _CSP
							   WHERE [Application_Id] = COALESCE(@Application_Id, [Application_Id])
								 AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])))
	      OR (@Application_Id IS NOT NULL AND [Provider_Id] 
	                       = (SELECT [IssueTracking_Provider_Id] FROM [Applications]
	                           WHERE [Application_Id] = @Application_Id))
	       )
	 ORDER BY [Provider_Name]
END
GO

GRANT EXECUTE ON [Providers_GetProviders] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CancelRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Cancels the specified release'
GO


IF OBJECT_ID('[Releases_CancelRelease]') IS NOT NULL DROP PROCEDURE [Releases_CancelRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CancelRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@CancelledReason_Text VARCHAR(500)
)
AS
BEGIN

SET NOCOUNT ON

	IF EXISTS(SELECT * 
	            FROM [Releases] 
			   WHERE [Application_Id] = @Application_Id
                 AND [Release_Number] = @Release_Number
			     AND [ReleaseStatus_Name] IN ('Deployed', 'Cancelled'))
	BEGIN
		RETURN
	END



BEGIN TRY
	BEGIN TRANSACTION


    UPDATE [Builds]
       SET [BuildStatus_Name] = 'Rejected'
          ,[RejectedBy_User_Name] = dbo.Session_GetUserName()
          ,[Rejected_Date] = GETUTCDATE()
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
	 
    UPDATE [Releases]
       SET [ReleaseStatus_Name] = 'Cancelled',
		   [CancelledBy_User_Name] = dbo.Session_GetUserName(),
		   [CancelledReason_Text] = @CancelledReason_Text,
           [CancelledOn_Date] = GETUTCDATE()
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number

	EXEC [Events_RaiseEvent] 'RELCNL', @Application_Id, @Release_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_CancelRelease] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CreateOrUpdateReleaseNote',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ReleaseNote_Sequence',
    /* Description_Text        */ 'Creates a new release note, or updates an existing one'
GO

IF OBJECT_ID('[Releases_CreateOrUpdateReleaseNote]') IS NOT NULL DROP PROCEDURE [Releases_CreateOrUpdateReleaseNote]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CreateOrUpdateReleaseNote]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@ReleaseNote_Sequence INT = NULL OUT,
	@Build_Number BUILDNUMBER = NULL,
	@Environment_Id INT= NULL,
	@Notes_Text VARCHAR(MAX)
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

	IF @ReleaseNote_Sequence IS NOT NULL

		UPDATE [ReleaseNotes]
		   SET [Notes_Text] = @Notes_Text
		      ,[Build_Number] = @Build_Number
		      ,[Environment_Id] = @Environment_Id
		      ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
		      ,[ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Application_ID] = @Application_ID
		   AND [Release_Number] = @Release_Number
		   AND [ReleaseNote_Sequence] = @ReleaseNote_Sequence
	
	ELSE BEGIN

		SET @ReleaseNote_Sequence = COALESCE(
		            (SELECT MAX([ReleaseNote_Sequence])+1
		               FROM [ReleaseNotes]
		              WHERE [Application_ID] = @Application_ID
		                AND [Release_Number] = @Release_Number) , 1)
		INSERT INTO [ReleaseNotes]
			 ([Application_Id],[Release_Number],[ReleaseNote_Sequence], [Build_Number],[Environment_Id],[Notes_Text]
			 ,[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date])
		VALUES
			 (@Application_Id,@Release_Number,@ReleaseNote_Sequence,@Build_Number,@Environment_Id,@Notes_Text
			 ,dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE())
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH
		 

END
GO

GRANT EXECUTE ON [Releases_CreateOrUpdateReleaseNote] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_DeleteReleaseNote',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a release note from the system'
GO

IF OBJECT_ID('[Releases_DeleteReleaseNote]') IS NOT NULL DROP PROCEDURE [Releases_DeleteReleaseNote]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_DeleteReleaseNote]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@ReleaseNote_Sequence INT
) AS BEGIN

	DELETE [ReleaseNotes]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND [ReleaseNote_Sequence] = @ReleaseNote_Sequence

END
GO

GRANT EXECUTE ON [Releases_DeleteReleaseNote] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetBuildExecutions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the list of build executions for the specified release'
GO

IF OBJECT_ID('[Releases_GetBuildExecutions]') IS NOT NULL
	DROP PROCEDURE [Releases_GetBuildExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetBuildExecutions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

END
GO

GRANT EXECUTE ON [Releases_GetBuildExecutions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetDependentReleases',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all releases with referenced deployables from a specified release'
GO

IF OBJECT_ID('[Releases_GetDependentReleases]') IS NOT NULL DROP PROCEDURE [Releases_GetDependentReleases]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetDependentReleases]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

 
	SELECT RE.*
	  FROM [Releases_Extended] RE
	 WHERE EXISTS 
           (
	         SELECT * FROM [ReleaseDeployables] RD
	          WHERE RE.[Application_Id] = RD.[Application_Id]
	            AND RE.[Release_Number] = RD.[Release_Number]
	            AND RD.[Referenced_Application_Id] = @Application_Id
	            AND RD.[Referenced_Release_Number] = @Release_Number
	            
	       )
	   AND NOT ([Application_Id] = @Application_Id AND 
	            [Release_Number] = @Release_Number)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_GetDependentReleases] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetNextReleaseNumber',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Next_Release_Number',
    /* Description_Text        */ 'Gets the next release number.'
GO

IF OBJECT_ID('[Releases_GetNextReleaseNumber]') IS NOT NULL DROP PROCEDURE [Releases_GetNextReleaseNumber]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetNextReleaseNumber]
(
	@Application_Id INT,
	@Next_Release_Number RELEASENUMBER = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Highest_Release_Number RELEASENUMBER

	SELECT TOP 1 @Highest_Release_Number = [Release_Number] 
      FROM [Releases]
     WHERE [Application_Id] = @Application_Id
     ORDER BY [Release_Sequence] DESC    

    
    SET @Next_Release_Number = dbo.GetNextReleaseNumber(@Application_Id, @Highest_Release_Number)

END
GO

GRANT EXECUTE ON [Releases_GetNextReleaseNumber] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Releases_Extended,ReleaseDeployables_Extended,ReleaseConfigurationFiles',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specified release, the deployables associated with the release, and the configuration file versions associated with the release.'
GO


IF OBJECT_ID('[Releases_GetRelease]') IS NOT NULL DROP PROCEDURE [Releases_GetRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Releases_Extended]
     WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	SELECT *
	  FROM [ReleaseDeployables_Extended]
	 WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
	ORDER BY [Deployable_Name]

	SELECT *
	  FROM [ReleaseConfigurationFiles]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

END
GO

GRANT EXECUTE ON [Releases_GetRelease] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ReleaseDeployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the set of deployables associated with any release for an application'
GO


IF OBJECT_ID('[Releases_GetReleaseDeployables]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseDeployables]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [ReleaseDeployables_Extended]
	 WHERE @Application_Id IS NULL OR [Application_Id] = @Application_Id
END
GO

GRANT EXECUTE ON [Releases_GetReleaseDeployables] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseNote',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ReleaseNotes_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a release note from the system'
GO

IF OBJECT_ID('[Releases_GetReleaseNote]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseNote]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseNote]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@ReleaseNote_Sequence INT
) AS BEGIN

	SELECT * 
	  FROM [ReleaseNotes_Extended]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND [ReleaseNote_Sequence] = @ReleaseNote_Sequence
	IF @@ROWCOUNT = 0 RAISERROR('Release Note %i not found', 16, 1, @ReleaseNote_Sequence)
END
GO

GRANT EXECUTE ON [Releases_GetReleaseNote] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseNotes',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ReleaseNotes_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the release notes that match the specified criteria'
GO

IF OBJECT_ID('[Releases_GetReleaseNotes]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseNotes]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseNotes]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER = NULL,
	@Environment_Id INT= NULL
) AS BEGIN

	SELECT * 
	  FROM [ReleaseNotes_Extended]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND ([Build_Number] = COALESCE(@Build_Number,[Build_Number]) OR COALESCE(@Build_Number,[Build_Number]) IS NULL)
	   AND ([Environment_Id] = COALESCE(@Environment_Id,[Environment_Id]) OR COALESCE(@Environment_Id,[Environment_Id]) IS NULL)

END
GO

GRANT EXECUTE ON [Releases_GetReleaseNotes] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleases',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of releases that match the specified criteria'
GO


IF OBJECT_ID('[Releases_GetReleases]') IS NOT NULL DROP PROCEDURE [Releases_GetReleases]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleases]
(
	@Application_Id INT,
	@ReleaseStatus_Name VARCHAR(9) = NULL,
	@Release_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Release_Count IS NOT NULL
		SET ROWCOUNT @Release_Count

	SELECT * 
	  FROM [Releases_Extended]
     WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@ReleaseStatus_Name IS NULL OR [ReleaseStatus_Name] = @ReleaseStatus_Name)
      ORDER BY [Release_Sequence] DESC
               

END
GO

GRANT EXECUTE ON [Releases_GetReleases] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetRequiredPromotions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_RequiredPromotions',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of the promotions that are required for the specified release'
GO

IF OBJECT_ID('Releases_GetRequiredPromotions') IS NOT NULL DROP PROCEDURE Releases_GetRequiredPromotions
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetRequiredPromotions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS BEGIN

	SET NOCOUNT ON


	SELECT * FROM [Releases_RequiredPromotions]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	ORDER BY [Step_Sequence]

END
GO

GRANT EXECUTE ON Releases_GetRequiredPromotions TO BuildMasterUser_Role

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetSortableReleaseNumber',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Release_Sequence',
    /* Description_Text        */ 'Gets the sortable release number for a release.'
GO


IF OBJECT_ID('[Releases_GetSortableReleaseNumber]') IS NOT NULL DROP PROCEDURE [Releases_GetSortableReleaseNumber]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetSortableReleaseNumber]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Release_Sequence DECIMAL(38,0) OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SET @Release_Sequence = COALESCE(
	                               (SELECT [Release_Sequence]
	                                  FROM [Releases]
	                                 WHERE [Release_Number] = @Release_Number AND [Application_Id] = @Application_Id),
	                               dbo.GetSortableReleaseNumber(
	                                                            @Release_Number,
	                                                            (SELECT [ReleaseNumber_Scheme_Name]
	                                                               FROM [Applications]
	                                                              WHERE [Application_Id] = @Application_Id)))
END
GO

GRANT EXECUTE ON [Releases_GetSortableReleaseNumber] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_PurgeReleaseData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire release and all associated data from the system.'
GO

IF OBJECT_ID('[Releases_PurgeReleaseData]') IS NOT NULL
	DROP PROCEDURE [Releases_PurgeReleaseData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_PurgeReleaseData]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [Applications] WHERE [Application_Id] = @Application_Id)
		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)
	
	IF NOT EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number)
		RAISERROR('Release %s of application %i does not exist.', 16, 1, @Release_Number, @Application_Id)
		
	DECLARE @Application_Name VARCHAR(50)
	SELECT @Application_Name = [Application_Name]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

    DECLARE @Release_Sequence INT
    SELECT @Release_Sequence = [Release_Sequence] 
      FROM [Releases] 
     WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number

	/* 1) Delete data without sub-release dependencies */
	DELETE [ReleaseNotes]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [ReleaseDeployables]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [BuildArtifacts]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [BuildPromotionContingencies]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [BuildApprovalsReceived]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

    DELETE [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	/* 2) Delete data with sub-release dependencies */
	/* Dependencies */
	DECLARE @Builds TABLE ([Build_Number] BUILDNUMBER)
	INSERT INTO @Builds ([Build_Number])
	SELECT [Build_Number]
	  FROM [Builds]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DECLARE @Executions TABLE ([Execution_Id] INT NOT NULL)
	INSERT INTO @Executions ([Execution_Id])
	SELECT [Execution_Id]
	  FROM [BuildExecutions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DECLARE @BuildExecution_DeploymentPlans TABLE ([BuildExecution_DeploymentPlan_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_DeploymentPlans ([BuildExecution_DeploymentPlan_Id])
	SELECT [BuildExecution_DeploymentPlan_Id]
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DECLARE @BuildExecution_ActionGroupActions TABLE ([BuildExecution_ActionGroupAction_Id] INT NOT NULL)
	INSERT INTO @BuildExecution_ActionGroupActions ([BuildExecution_ActionGroupAction_Id])
	SELECT [BuildExecution_ActionGroupAction_Id]
	  FROM [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	DECLARE @DatabaseChangeScripts TABLE ([Script_Id] INT NOT NULL)
	INSERT INTO @DatabaseChangeScripts ([Script_Id])
	SELECT [Script_Id]
	  FROM [DatabaseChangeScripts]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DECLARE @ChangeControls TABLE ([ChangeControl_Id] INT NOT NULL)
	INSERT INTO @ChangeControls ([ChangeControl_Id])
	SELECT [ChangeControl_Id]
	  FROM [ChangeControls]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	/* 2a) Builds */
	DELETE
	  FROM [BuildTestResults]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildOutputs]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildExecution_ActionGroupActionLogEntries]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

    DELETE
	  FROM [BuildExecution_ActionGroupActionVariableValues]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE
	  FROM [ManualActionsInProgress]
	 WHERE [BuildExecution_PlanAction_Id] IN (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE
	  FROM [BuildExecution_ActionGroupActions]
	 WHERE [BuildExecution_ActionGroupAction_Id] IN  (SELECT [BuildExecution_ActionGroupAction_Id] FROM @BuildExecution_ActionGroupActions)

	DELETE
	  FROM [BuildExecution_DeploymentPlanActionGroups]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	UPDATE [BuildExecutions]
	   SET [Initial_BuildExecution_DeploymentPlan_Id] = NULL
	 WHERE [Execution_Id] IN (SELECT [Execution_Id] FROM @Executions)

	DELETE
	  FROM [BuildExecution_DeploymentPlans]
	 WHERE [BuildExecution_DeploymentPlan_Id] IN (SELECT [BuildExecution_DeploymentPlan_Id] FROM @BuildExecution_DeploymentPlans)

	/* Have to delete change script executions before Build Executions... */
	DELETE [DatabaseChangeScriptExecutions]
	 WHERE [Script_Id] IN ( SELECT [Script_Id] FROM @DatabaseChangeScripts )
	    OR [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE [ConfigurationFileDeployments]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )


	DELETE
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildPromotions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE
	  FROM [Issues_BuildStatusChanges]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	DELETE
	  FROM [Builds]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	/* 2b) Change Scripts */
	DELETE
	  FROM [DatabaseChangeScripts]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	/* 2c) Configuration */
	DELETE [ChangeControls]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [Builds]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [BuildExecutions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [ReleaseConfigurationFiles]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	 /* 2d) Change Controls */
	DELETE 
	  FROM [ChangeControlEnvironments]
	 WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)

	DELETE
	  FROM [ChangeControlPerformances]
	 WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)

	DELETE 
	  FROM [ChangeControls]
	 WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)


	 /* 2e) Release Deployables */
	DELETE [ReleaseDeployables]
	 WHERE ([Application_Id] = @Application_Id AND [Release_Number] = @Release_Number)
		OR ([Referenced_Application_Id] = @Application_Id AND [Referenced_Release_Number] = @Release_Number)


	/* 3) Release */
	DELETE [Releases]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

    UPDATE [Releases]
       SET [Release_Sequence] = [Release_Sequence] - 1
     WHERE [Application_Id] = @Application_Id
       AND [Release_Sequence] > @Release_Sequence


	 /* Raise Event */
	EXEC [Events_RaiseEvent] 'RELDEL',NULL,@Application_Name,@Release_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_PurgeReleaseData] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_ReleaseExists',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ReleaseExists_Indicator,NewerDeployedReleaseExists_Indicator',
    /* Description_Text        */ 'Determines whether a release already exists, and also whether a newer release of an application exists'
GO

IF OBJECT_ID('[Releases_ReleaseExists]') IS NOT NULL DROP PROCEDURE [Releases_ReleaseExists]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_ReleaseExists]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@ReleaseExists_Indicator YNINDICATOR = NULL OUT,
	@NewerDeployedReleaseExists_Indicator YNINDICATOR = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON


	SET @ReleaseExists_Indicator =
		CASE WHEN EXISTS(SELECT *
						   FROM [Releases_Extended]
						  WHERE [Release_Number] = @Release_Number
							AND [Application_Id] = @Application_Id)
			 THEN 'Y'
			 ELSE 'N' END

    DECLARE @Release_Sequence BIGINT

    SELECT @Release_Sequence = [Release_Sequence]
     FROM [Releases]
    WHERE [Application_Id] = @Application_Id 
      AND [Release_Number] = @Release_Number

	SET @NewerDeployedReleaseExists_Indicator =
		CASE WHEN EXISTS(SELECT *
						   FROM [Releases_Extended]
						  WHERE [Release_Sequence] > @Release_Sequence
							AND [Application_Id] = @Application_Id
							AND [ReleaseStatus_Name] = 'Deployed')
			 THEN 'Y'
			 ELSE 'N' END

END
GO

GRANT EXECUTE ON [Releases_ReleaseExists] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_ResequenceRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Resequences the specified release by placing it where another release is sequenced'
GO


IF OBJECT_ID('[Releases_ResequenceRelease]') IS NOT NULL DROP PROCEDURE [Releases_ResequenceRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_ResequenceRelease]
(
	@Application_Id INT,
    @Release_Sequence INT,
	@Replaced_Release_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON
	
	UPDATE [Releases]
	   SET [Release_Sequence] = CASE
	          WHEN [Releases].[Release_Sequence] = RTAR.[Release_Sequence] 
	               THEN RPOS.[Release_Sequence]
	          WHEN [Releases].[Release_Sequence] BETWEEN RTAR.[Release_Sequence] AND RPOS.[Release_Sequence]
	               THEN [Releases].[Release_Sequence] - 1
	          WHEN [Releases].[Release_Sequence] BETWEEN RPOS.[Release_Sequence] AND RTAR.[Release_Sequence]
	               THEN [Releases].[Release_Sequence] + 1
	          ELSE [Releases].[Release_Sequence]
	          END
	  FROM [Releases],
	       [Releases] RTAR,
	       [Releases] RPOS
	 WHERE [Releases].[Application_Id] = @Application_Id
       AND RTAR.[Application_Id] = @Application_Id
       AND RTAR.[Release_Sequence] = @Release_Sequence
	   AND RPOS.[Application_Id] = @Application_Id
       AND RPOS.[Release_Sequence] = @Replaced_Release_Sequence
	   AND @Release_Sequence <> @Replaced_Release_Sequence


END
GO

GRANT EXECUTE ON [Releases_ResequenceRelease] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_SetConfigurationFileVersion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the configuration file version for the specified release.'
GO


IF OBJECT_ID('[Releases_SetConfigurationFileVersion]') IS NOT NULL DROP PROCEDURE [Releases_SetConfigurationFileVersion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_SetConfigurationFileVersion]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@ConfigurationFile_Id INT,
    @Version_Number INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DELETE [ReleaseConfigurationFiles]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
       AND [ConfigurationFile_Id] = @ConfigurationFile_Id

    IF (@Version_Number IS NOT NULL) 
    BEGIN

        INSERT INTO [ReleaseConfigurationFiles]
        (
            [Application_Id],
            [Release_Number],
            [ConfigurationFile_Id],
            [Version_Number]
        )
        VALUES
        (
            @Application_Id,
	        @Release_Number,
	        @ConfigurationFile_Id,
            @Version_Number
        )

    END

	COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_SetConfigurationFileVersion] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_UnCancelRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Uncancels a previously cancelled release'
GO

IF OBJECT_ID('[Releases_UnCancelRelease]') IS NOT NULL
	DROP PROCEDURE [Releases_UnCancelRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_UnCancelRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS
BEGIN

	SET NOCOUNT ON

	IF NOT EXISTS ( SELECT *
					  FROM [Releases]
					 WHERE [Application_Id] = @Application_Id
					   AND [Release_Number] = @Release_Number
					   AND [ReleaseStatus_Name] = 'Cancelled' )
	BEGIN
		RAISERROR('Release ''%s'' does not exist or its status is not ''Cancelled''', 16, 1, @Release_Number)
		RETURN
	END


BEGIN TRY
	BEGIN TRANSACTION


		/* Set the release as Active */
		UPDATE [Releases]
		   SET [ReleaseStatus_Name] = 'Active',
		       [Original_Workflow_Name] = COALESCE(W.[Workflow_Name],'{unknown}')
		  FROM [Releases] R 
			   LEFT JOIN [Workflows] W
					  ON R.[Workflow_Id] = W.[Workflow_Id]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number

		/* Cancel other active releases if applicable */
		IF 'N' = ( SELECT [AllowMultipleActiveReleases_Indicator]
					 FROM [Applications]
					WHERE [Application_Id] = @Application_Id )
		BEGIN

			/* Cancel existing active releases */
			UPDATE [Releases]
			   SET [ReleaseStatus_Name] = 'Cancelled'
			 WHERE [Application_Id] = @Application_Id
			   AND [Release_Number] <> @Release_Number
			   AND [ReleaseStatus_Name] = 'Active'
		END

	EXEC [Events_RaiseEvent] 'RELMOD', @Application_Id, @Release_Number


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_UnCancelRelease] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_BeginPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'RetentionPoliciesPurgeHistoryEntry_Id',
    /* Description_Text        */ 'Marks the start of a new purge.'
GO


IF OBJECT_ID('[RetentionPolicies_BeginPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_BeginPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_BeginPurge]
(
	@RetentionPolicy_Id INT,
	@Original_RetentionPolicy_Name NVARCHAR(50),
	@RetentionPoliciesPurgeHistoryEntry_Id INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	--Make sure this always gets reset
	UPDATE [RetentionPolicies]
	   SET [PurgeNow_Indicator] = 'N'
	 WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

	DECLARE @History_Count INT
	SET @History_Count = (SELECT COUNT(*) FROM [RetentionPoliciesPurgeHistory] WITH (TABLOCKX))

	IF @History_Count > 1000
	BEGIN

		DECLARE @RowsToDelete TABLE ([RetentionPolicyPurgeHistoryEntry_Id] INT NOT NULL)

		INSERT INTO @RowsToDelete
		SELECT [RetentionPoliciesPurgeHistoryEntry_Id]
		  FROM (SELECT *,
		               [Rank] = ROW_NUMBER() OVER(ORDER BY [Start_Date] DESC)
				  FROM [RetentionPoliciesPurgeHistory]) _TMP
		 WHERE _TMP.[Rank] > 1000

		DELETE [RetentionPoliciesPurgeHistory]
		 WHERE [RetentionPoliciesPurgeHistoryEntry_Id] IN (SELECT [RetentionPolicyPurgeHistoryEntry_Id] FROM @RowsToDelete)

	END

	INSERT INTO [RetentionPoliciesPurgeHistory]
	(
		[Original_RetentionPolicy_Id],
		[Original_RetentionPolicy_Name],
		[Start_Date],
		[End_Date],
		[LogText_Bytes]
	)
	VALUES
	(
		@RetentionPolicy_Id,
		@Original_RetentionPolicy_Name,
		GETUTCDATE(),
		NULL,
		NULL
	)

	SET @RetentionPoliciesPurgeHistoryEntry_Id = SCOPE_IDENTITY()

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_BeginPurge] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_CreateOrUpdateRetentionPolicy',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'RetentionPolicy_Id',
    /* Description_Text        */ 'Creates or updates a retention policy.'
GO


IF OBJECT_ID('[RetentionPolicies_CreateOrUpdateRetentionPolicy]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_CreateOrUpdateRetentionPolicy]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_CreateOrUpdateRetentionPolicy]
(
	@RetentionPolicy_Name NVARCHAR(50),
	@PurgeType_Code CHAR(1),
	@PurgeIn_Application_Id INT,
	@PurgeIn_ApplicationGroup_Id INT,
	@PurgeIn_OnlyDeployedReleases_Indicator YNINDICATOR,
	@PurgeIn_OnlyRejectedBuilds_Indicator YNINDICATOR,
	@PurgeIn_Workflow_Id INT,
	@PurgeIn_Environment_Id INT,
	@PurgeIn_BuildStep_Indicator YNINDICATOR,
	@Retention_Count INT,
	@Retention_Size BIGINT,
    @Retention_Days_Count INT,
	@PurgeWindow_Start_Minutes INT,
    @PurgeWindow_Duration_Minutes INT,
    @Active_Indicator YNINDICATOR,
	@RetentionPolicy_Id INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [RetentionPolicies]
			    WITH (UPDLOCK)
			   WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id)
	BEGIN

		UPDATE [RetentionPolicies]
		   SET [RetentionPolicy_Name] = NULLIF(RTRIM(LTRIM(@RetentionPolicy_Name)), ''),
		       [PurgeType_Code] = @PurgeType_Code,
			   [PurgeIn_Application_Id] = @PurgeIn_Application_Id,
			   [PurgeIn_ApplicationGroup_Id] = @PurgeIn_ApplicationGroup_Id,
			   [PurgeIn_OnlyDeployedReleases_Indicator] = @PurgeIn_OnlyDeployedReleases_Indicator,
			   [PurgeIn_OnlyRejectedBuilds_Indicator] = @PurgeIn_OnlyRejectedBuilds_Indicator,
			   [PurgeIn_Workflow_Id] = @PurgeIn_Workflow_Id,
			   [PurgeIn_Environment_Id] = @PurgeIn_Environment_Id,
			   [PurgeIn_BuildStep_Indicator] = @PurgeIn_BuildStep_Indicator,
			   [Retention_Count] = @Retention_Count,
			   [Retention_Size] = @Retention_Size,
               [Retention_Days_Count] = @Retention_Days_Count,
			   [PurgeWindow_Start_Minutes] = @PurgeWindow_Start_Minutes,
               [PurgeWindow_Duration_Minutes] = @PurgeWindow_Duration_Minutes,
               [Active_Indicator] = @Active_Indicator,
			   [ModifiedOn_Date] = GETUTCDATE(),
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName()
		 WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

	END
	ELSE
	BEGIN

		INSERT INTO [RetentionPolicies]
		(
			[RetentionPolicy_Name],
			[PurgeType_Code],
			[PurgeIn_Application_Id],
			[PurgeIn_ApplicationGroup_Id],
			[PurgeIn_OnlyDeployedReleases_Indicator],
			[PurgeIn_OnlyRejectedBuilds_Indicator],
			[PurgeIn_Workflow_Id],
			[PurgeIn_Environment_Id],
			[PurgeIn_BuildStep_Indicator],
			[Retention_Count],
			[Retention_Size],
            [Retention_Days_Count],
			[PurgeWindow_Start_Minutes],
            [PurgeWindow_Duration_Minutes],
			[PurgeNow_Indicator],
            [Active_Indicator],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@RetentionPolicy_Name,
			@PurgeType_Code,
			@PurgeIn_Application_Id,
			@PurgeIn_ApplicationGroup_Id,
			@PurgeIn_OnlyDeployedReleases_Indicator,
			@PurgeIn_OnlyRejectedBuilds_Indicator,
			@PurgeIn_Workflow_Id,
			@PurgeIn_Environment_Id,
			@PurgeIn_BuildStep_Indicator,
			@Retention_Count,
			@Retention_Size,
            @Retention_Days_Count,
			@PurgeWindow_Start_Minutes,
            @PurgeWindow_Duration_Minutes,
			'N',
            @Active_Indicator,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		SET @RetentionPolicy_Id = SCOPE_IDENTITY()

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_CreateOrUpdateRetentionPolicy] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_DeleteRetentionPolicy',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a retention policy.'
GO


IF OBJECT_ID('[RetentionPolicies_DeleteRetentionPolicy]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_DeleteRetentionPolicy]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_DeleteRetentionPolicy]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [RetentionPolicies]
	 WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_DeleteRetentionPolicy] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_EndPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks the end of a purge.'
GO


IF OBJECT_ID('[RetentionPolicies_EndPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_EndPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_EndPurge]
(
	@RetentionPoliciesPurgeHistoryEntry_Id INT,
	@LogText_Bytes VARBINARY(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    IF (@LogText_Bytes <> 0)
    BEGIN

	    UPDATE [RetentionPoliciesPurgeHistory]
	       SET [End_Date] = GETUTCDATE(),
	           [LogText_Bytes] = @LogText_Bytes
	     WHERE [RetentionPoliciesPurgeHistoryEntry_Id] = @RetentionPoliciesPurgeHistoryEntry_Id

    END
    ELSE
    BEGIN

        DELETE [RetentionPoliciesPurgeHistory]
         WHERE [RetentionPoliciesPurgeHistoryEntry_Id] = @RetentionPoliciesPurgeHistoryEntry_Id

    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_EndPurge] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetArtifactsToPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Slim',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the artifacts for the specified retention policy that can be purged.'
GO

IF OBJECT_ID('[RetentionPolicies_GetArtifactsToPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetArtifactsToPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetArtifactsToPurge]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @ArtifactsMatchingFilter_Count INT
    DECLARE @ArtifactsToDelete_Count INT
	DECLARE @DeployedReleasesOnly_Indicator YNINDICATOR
    DECLARE @RejectedBuildsOnly_Indicator YNINDICATOR
    DECLARE @Retention_Count INT
    DECLARE @AlwaysRetainAfter_Date DATETIME
	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
    DECLARE @Workflow_Id INT
    
    SELECT @DeployedReleasesOnly_Indicator = [PurgeIn_OnlyDeployedReleases_Indicator],
           @RejectedBuildsOnly_Indicator = [PurgeIn_OnlyRejectedBuilds_Indicator],
           @Retention_Count = [Retention_Count],
           @Application_Id = [PurgeIn_Application_Id],
           @ApplicationGroup_Id = [PurgeIn_ApplicationGroup_Id],
           @Workflow_Id = [PurgeIn_Workflow_Id],
           @AlwaysRetainAfter_Date = DATEADD(DAY, -1 * [Retention_Days_Count], GETUTCDATE())
      FROM [RetentionPolicies]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

    IF @Retention_Count IS NOT NULL
    BEGIN

        SELECT @ArtifactsMatchingFilter_Count = COUNT(*) 
                                      FROM [BuildArtifacts_Slim] BA
	                            INNER JOIN [Applications] A
		                                ON A.[Application_Id] = BA.[Application_Id]
		                        INNER JOIN [Releases] R
		                                ON R.[Application_Id] = BA.[Application_Id]
				                        AND R.[Release_Number] = BA.[Release_Number]
	                        WHERE (@Application_Id IS NULL OR BA.[Application_Id] = @Application_Id)
	                          AND (@ApplicationGroup_Id IS NULL OR A.[ApplicationGroup_Id] = @ApplicationGroup_Id)
                              AND (@Workflow_Id IS NULL OR R.[Workflow_Id] = @Workflow_Id)
	                          AND (@RejectedBuildsOnly_Indicator = 'N' OR BA.[BuildStatus_Name] = 'Rejected')
	                          AND (@DeployedReleasesOnly_Indicator = 'N' OR R.[ReleaseStatus_Name] = 'Deployed')

        SET @ArtifactsToDelete_Count = @ArtifactsMatchingFilter_Count - @Retention_Count
        
        IF (@ArtifactsToDelete_Count < 1)
            RETURN

        SET ROWCOUNT @ArtifactsToDelete_Count

    END

	SELECT *
	    FROM [BuildArtifacts_Slim] BA
	        INNER JOIN [Applications] A
		            ON A.[Application_Id] = BA.[Application_Id]
		    INNER JOIN [Releases] R
		            ON R.[Application_Id] = BA.[Application_Id]
				    AND R.[Release_Number] = BA.[Release_Number]
	    WHERE (@Application_Id IS NULL OR BA.[Application_Id] = @Application_Id)
	    AND (@ApplicationGroup_Id IS NULL OR A.[ApplicationGroup_Id] = @ApplicationGroup_Id)
        AND (@Workflow_Id IS NULL OR R.[Workflow_Id] = @Workflow_Id)
	    AND (@RejectedBuildsOnly_Indicator = 'N' OR BA.[BuildStatus_Name] = 'Rejected')
	    AND (@DeployedReleasesOnly_Indicator = 'N' OR R.[ReleaseStatus_Name] = 'Deployed')
	    AND (@AlwaysRetainAfter_Date IS NULL OR BA.[Created_Date] < @AlwaysRetainAfter_Date)
        ORDER BY [Created_Date]

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetArtifactsToPurge] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetBuildCount',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Build_Count',
    /* Description_Text        */ 'Gets the total number of builds in the specified scope.'
GO


IF OBJECT_ID('[RetentionPolicies_GetBuildCount]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetBuildCount]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetBuildCount]
(
	@Application_Id INT = NULL,
	@ApplicationGroup_Id INT = NULL,
	@Build_Count INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Application_Id IS NOT NULL
	BEGIN

		SELECT @Build_Count = COUNT(*)
		  FROM [Builds]
		 WHERE [Application_Id] = @Application_Id

	END
	ELSE IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

		SELECT @Build_Count = COUNT(*)
		  FROM [Builds] B
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = B.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id

	END
	ELSE
	BEGIN

		SELECT @Build_Count = COUNT(*)
		  FROM [Builds]

	END

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetBuildCount] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetBuildsToPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the builds for the specified retention policy that can be purged.'
GO


IF OBJECT_ID('[RetentionPolicies_GetBuildsToPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetBuildsToPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetBuildsToPurge]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @BuildsMatchingFilter_Count INT
    DECLARE @BuildsToDelete_Count INT
	DECLARE @DeployedReleasesOnly_Indicator YNINDICATOR
    DECLARE @RejectedOnly_Indicator YNINDICATOR
    DECLARE @Retention_Count INT
    DECLARE @AlwaysRetainAfter_Date DATETIME
	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
    DECLARE @Workflow_Id INT
    
    SELECT @DeployedReleasesOnly_Indicator = [PurgeIn_OnlyDeployedReleases_Indicator],
           @RejectedOnly_Indicator = [PurgeIn_OnlyRejectedBuilds_Indicator],
           @Retention_Count = [Retention_Count],
           @Application_Id = [PurgeIn_Application_Id],
           @ApplicationGroup_Id = [PurgeIn_ApplicationGroup_Id],
           @Workflow_Id = [PurgeIn_Workflow_Id],
           @AlwaysRetainAfter_Date = DATEADD(DAY, -1 * [Retention_Days_Count], GETUTCDATE())
      FROM [RetentionPolicies]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

    IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @BuildsMatchingFilter_Count = COUNT(*) 
                         FROM [Builds_Extended] B
		           INNER JOIN [Applications] A
			               ON A.[Application_Id] = B.[Application_Id]
		                WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
                          AND (@Workflow_Id IS NULL OR B.[Workflow_Id] = @Workflow_Id)
		                  AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		                  AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')

            SET @BuildsToDelete_Count = @BuildsMatchingFilter_Count - @Retention_Count
        
            IF (@BuildsToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @BuildsToDelete_Count

        END

		SELECT B.*
		  FROM [Builds_Extended] B
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = B.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
           AND (@Workflow_Id IS NULL OR B.[Workflow_Id] = @Workflow_Id)
		   AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		   AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
		   AND (@AlwaysRetainAfter_Date IS NULL OR B.[CreatedOn_Date] < @AlwaysRetainAfter_Date)
		 ORDER BY [CreatedOn_Date]

	END
	ELSE
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @BuildsMatchingFilter_Count = COUNT(*) 
                         FROM [Builds_Extended]
		                WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
                          AND (@Workflow_Id IS NULL OR [Workflow_Id] = @Workflow_Id)
		                  AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		                  AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')

            SET @BuildsToDelete_Count = @BuildsMatchingFilter_Count - @Retention_Count
        
            IF (@BuildsToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @BuildsToDelete_Count

        END

		SELECT *
		  FROM [Builds_Extended]
		 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
           AND (@Workflow_Id IS NULL OR [Workflow_Id] = @Workflow_Id)
		   AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		   AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
		   AND (@AlwaysRetainAfter_Date IS NULL OR [CreatedOn_Date] < @AlwaysRetainAfter_Date)
		 ORDER BY [CreatedOn_Date]

	END

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetBuildsToPurge] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetBuildsWithLogsToPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets only the builds that have logs remaining for the specified retention policy that can be purged.'
GO


IF OBJECT_ID('[RetentionPolicies_GetBuildsWithLogsToPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetBuildsWithLogsToPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetBuildsWithLogsToPurge]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @BuildsMatchingFilter_Count INT
    DECLARE @BuildsToDelete_Count INT
	DECLARE @DeployedReleasesOnly_Indicator YNINDICATOR
    DECLARE @RejectedOnly_Indicator YNINDICATOR
    DECLARE @Retention_Count INT
    DECLARE @AlwaysRetainAfter_Date DATETIME
	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
    DECLARE @Workflow_Id INT
    
    SELECT @DeployedReleasesOnly_Indicator = [PurgeIn_OnlyDeployedReleases_Indicator],
           @RejectedOnly_Indicator = [PurgeIn_OnlyRejectedBuilds_Indicator],
           @Retention_Count = [Retention_Count],
           @Application_Id = [PurgeIn_Application_Id],
           @ApplicationGroup_Id = [PurgeIn_ApplicationGroup_Id],
           @Workflow_Id = [PurgeIn_Workflow_Id],
           @AlwaysRetainAfter_Date = DATEADD(DAY, -1 * [Retention_Days_Count], GETUTCDATE())
      FROM [RetentionPolicies]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

    IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @BuildsMatchingFilter_Count = COUNT(*) 
                         FROM [Builds_Extended] B
		           INNER JOIN [Applications] A
			               ON A.[Application_Id] = B.[Application_Id]
		                WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
                          AND (@Workflow_Id IS NULL OR B.[Workflow_Id] = @Workflow_Id)
		                  AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		                  AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
                          AND EXISTS(
                                SELECT TOP 1 BEAGALE.[LogEntry_Sequence] 
                                  FROM [BuildExecutions] BEX
                            INNER JOIN [BuildExecution_DeploymentPlans] BEDP
                                    ON BEDP.[Execution_Id] = BEX.[Execution_Id]
                            INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
                                    ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
                            INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
                                    ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
                                 WHERE B.[Application_Id] = BEX.[Application_Id]
                                   AND B.[Release_Number] = BEX.[Release_Number]
                                   AND B.[Build_Number] = BEX.[Build_Number]
                          )

            SET @BuildsToDelete_Count = @BuildsMatchingFilter_Count - @Retention_Count
        
            IF (@BuildsToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @BuildsToDelete_Count

        END

		SELECT B.*
		  FROM [Builds_Extended] B
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = B.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
           AND (@Workflow_Id IS NULL OR B.[Workflow_Id] = @Workflow_Id)
		   AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		   AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
		   AND (@AlwaysRetainAfter_Date IS NULL OR B.[CreatedOn_Date] < @AlwaysRetainAfter_Date)
           AND EXISTS(
                SELECT TOP 1 BEAGALE.[LogEntry_Sequence] 
                    FROM [BuildExecutions] BEX
            INNER JOIN [BuildExecution_DeploymentPlans] BEDP
                    ON BEDP.[Execution_Id] = BEX.[Execution_Id]
            INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
                    ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
            INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
                    ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
                    WHERE B.[Application_Id] = BEX.[Application_Id]
                    AND B.[Release_Number] = BEX.[Release_Number]
                    AND B.[Build_Number] = BEX.[Build_Number]
            )
		 ORDER BY [CreatedOn_Date]

	END
	ELSE
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @BuildsMatchingFilter_Count = COUNT(*) 
                         FROM [Builds_Extended] BE
		                WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
                          AND (@Workflow_Id IS NULL OR [Workflow_Id] = @Workflow_Id)
		                  AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		                  AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
                          AND EXISTS(
                                SELECT TOP 1 BEAGALE.[LogEntry_Sequence] 
                                  FROM [BuildExecutions] BEX
                            INNER JOIN [BuildExecution_DeploymentPlans] BEDP
                                    ON BEDP.[Execution_Id] = BEX.[Execution_Id]
                            INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
                                    ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
                            INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
                                    ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
                                 WHERE BE.[Application_Id] = BEX.[Application_Id]
                                   AND BE.[Release_Number] = BEX.[Release_Number]
                                   AND BE.[Build_Number] = BEX.[Build_Number]
                          ) 

            SET @BuildsToDelete_Count = @BuildsMatchingFilter_Count - @Retention_Count
        
            IF (@BuildsToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @BuildsToDelete_Count

        END

		SELECT *
		  FROM [Builds_Extended] B
		 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
           AND (@Workflow_Id IS NULL OR [Workflow_Id] = @Workflow_Id)
		   AND (@DeployedReleasesOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
		   AND (@RejectedOnly_Indicator = 'N' OR [BuildStatus_Name] = 'Rejected')
		   AND (@AlwaysRetainAfter_Date IS NULL OR [CreatedOn_Date] < @AlwaysRetainAfter_Date)
           AND EXISTS(
                SELECT TOP 1 BEAGALE.[LogEntry_Sequence] 
                    FROM [BuildExecutions] BEX
            INNER JOIN [BuildExecution_DeploymentPlans] BEDP
                    ON BEDP.[Execution_Id] = BEX.[Execution_Id]
            INNER JOIN [BuildExecution_ActionGroupActions] BEAGA
                    ON BEAGA.[BuildExecution_DeploymentPlan_Id] = BEDP.[BuildExecution_DeploymentPlan_Id]
            INNER JOIN [BuildExecution_ActionGroupActionLogEntries] BEAGALE
                    ON BEAGALE.[BuildExecution_ActionGroupAction_Id] = BEAGA.[BuildExecution_ActionGroupAction_Id]
                    WHERE B.[Application_Id] = BEX.[Application_Id]
                    AND B.[Release_Number] = BEX.[Release_Number]
                    AND B.[Build_Number] = BEX.[Build_Number]
            )
		 ORDER BY [CreatedOn_Date]

	END

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetBuildsWithLogsToPurge] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetPurgeHistory',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'RetentionPoliciesPurgeHistory',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the purge history for one or all retention policies.'
GO


IF OBJECT_ID('[RetentionPolicies_GetPurgeHistory]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetPurgeHistory]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetPurgeHistory]
(
	@RetentionPolicy_Id INT = NULL,
	@Max_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Max_Count IS NOT NULL
		SET ROWCOUNT @Max_Count

	SELECT *
	  FROM [RetentionPoliciesPurgeHistory]
	 WHERE @RetentionPolicy_Id IS NULL OR [Original_RetentionPolicy_Id] = @RetentionPolicy_Id
	 ORDER BY [Start_Date] DESC

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetPurgeHistory] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetPurgeLog',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'RetentionPoliciesPurgeHistory',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified purge history log.'
GO


IF OBJECT_ID('[RetentionPolicies_GetPurgeLog]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetPurgeLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetPurgeLog]
(
	@RetentionPoliciesPurgeHistoryEntry_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [RetentionPoliciesPurgeHistory]
	 WHERE [RetentionPoliciesPurgeHistoryEntry_Id] = @RetentionPoliciesPurgeHistoryEntry_Id

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetPurgeLog] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetReleaseCount',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Release_Count',
    /* Description_Text        */ 'Gets the total number of releases in the specified scope.'
GO


IF OBJECT_ID('[RetentionPolicies_GetReleaseCount]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetReleaseCount]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetReleaseCount]
(
	@Application_Id INT = NULL,
	@ApplicationGroup_Id INT = NULL,
	@Release_Count INT = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Application_Id IS NOT NULL
	BEGIN

		SELECT @Release_Count = COUNT(*)
		  FROM [Releases]
		 WHERE [Application_Id] = @Application_Id

	END
	ELSE IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

		SELECT @Release_Count = COUNT(*)
		  FROM [Releases] R
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = R.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id

	END
	ELSE
	BEGIN

		SELECT @Release_Count = COUNT(*)
		  FROM [Releases]

	END

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetReleaseCount] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetReleasesToPurge',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the releases for the specified retention policy that can be purged.'
GO


IF OBJECT_ID('[RetentionPolicies_GetReleasesToPurge]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetReleasesToPurge]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetReleasesToPurge]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @ReleasesMatchingFilter_Count INT
    DECLARE @ReleasesToDelete_Count INT
	DECLARE @DeployedOnly_Indicator YNINDICATOR
    DECLARE @Retention_Count INT
    DECLARE @AlwaysRetainAfter_Date DATETIME
	DECLARE @Application_Id INT
	DECLARE @ApplicationGroup_Id INT
    DECLARE @Workflow_Id INT
    
    SELECT @DeployedOnly_Indicator = [PurgeIn_OnlyDeployedReleases_Indicator],
           @Retention_Count = [Retention_Count],
           @Application_Id = [PurgeIn_Application_Id],
           @ApplicationGroup_Id = [PurgeIn_ApplicationGroup_Id],
           @Workflow_Id = [PurgeIn_Workflow_Id],
           @AlwaysRetainAfter_Date = DATEADD(DAY, -1 * [Retention_Days_Count], GETUTCDATE())
      FROM [RetentionPolicies]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

    IF @ApplicationGroup_Id IS NOT NULL
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @ReleasesMatchingFilter_Count = COUNT(*) 
                          FROM [Releases_Extended] R
		            INNER JOIN [Applications] A
			                ON A.[Application_Id] = R.[Application_Id]
		                 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
                           AND (@Workflow_Id IS NULL OR R.[Workflow_Id] = @Workflow_Id)
		                   AND (@DeployedOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')

            SET @ReleasesToDelete_Count = @ReleasesMatchingFilter_Count - @Retention_Count
        
            IF (@ReleasesToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @ReleasesToDelete_Count

        END

		SELECT R.*
		  FROM [Releases_Extended] R
		       INNER JOIN [Applications] A
			           ON A.[Application_Id] = R.[Application_Id]
		 WHERE A.[ApplicationGroup_Id] = @ApplicationGroup_Id
           AND (@Workflow_Id IS NULL OR R.[Workflow_Id] = @Workflow_Id)
		   AND (@DeployedOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
           AND (@AlwaysRetainAfter_Date IS NULL OR R.[CreatedOn_Date] < @AlwaysRetainAfter_Date)
		 ORDER BY [CreatedOn_Date]

	END
	ELSE
	BEGIN

        IF @Retention_Count IS NOT NULL
        BEGIN        
        
            SELECT @ReleasesMatchingFilter_Count = COUNT(*) 
                          FROM [Releases_Extended]
		                 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
                           AND (@Workflow_Id IS NULL OR [Workflow_Id] = @Workflow_Id)
		                   AND (@DeployedOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')

            SET @ReleasesToDelete_Count = @ReleasesMatchingFilter_Count - @Retention_Count
        
            IF (@ReleasesToDelete_Count < 1)
                RETURN

            SET ROWCOUNT @ReleasesToDelete_Count

        END

		SELECT *
		  FROM [Releases_Extended]
		 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
           AND (@Workflow_Id IS NULL OR [Workflow_Id] = @Workflow_Id)
		   AND (@DeployedOnly_Indicator = 'N' OR [ReleaseStatus_Name] = 'Deployed')
           AND (@AlwaysRetainAfter_Date IS NULL OR [CreatedOn_Date] < @AlwaysRetainAfter_Date)
		 ORDER BY [CreatedOn_Date]

	END 

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetReleasesToPurge] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetRetentionPolicies',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'RetentionPolicies_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the retention policies.'
GO


IF OBJECT_ID('[RetentionPolicies_GetRetentionPolicies]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetRetentionPolicies]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetRetentionPolicies]
(
    @Include_Inactive YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [RetentionPolicies_Extended]
     WHERE @Include_Inactive = 'Y' OR [Active_Indicator] = 'Y'

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetRetentionPolicies] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_GetRetentionPolicy',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'RetentionPolicies_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified retention policy.'
GO


IF OBJECT_ID('[RetentionPolicies_GetRetentionPolicy]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_GetRetentionPolicy]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_GetRetentionPolicy]
(
    @RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [RetentionPolicies_Extended]
     WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

END
GO

GRANT EXECUTE ON [RetentionPolicies_GetRetentionPolicy] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'RetentionPolicies_PurgeNow',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks a retention policy as ready to run immediately.'
GO


IF OBJECT_ID('[RetentionPolicies_PurgeNow]') IS NOT NULL DROP PROCEDURE [RetentionPolicies_PurgeNow]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [RetentionPolicies_PurgeNow]
(
	@RetentionPolicy_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [RetentionPolicies]
	   SET [PurgeNow_Indicator] = 'Y'
	 WHERE [RetentionPolicy_Id] = @RetentionPolicy_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [RetentionPolicies_PurgeNow] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateOrUpdateParameter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new script parameter or updates an existing one.'
GO


IF OBJECT_ID('[Scripts_CreateOrUpdateParameter]') IS NOT NULL DROP PROCEDURE [Scripts_CreateOrUpdateParameter]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateOrUpdateParameter]
(
	@Script_Id INT,
	@Parameter_Name NVARCHAR(50),
	@ParameterType_Code CHAR(1),
	@DefaultValue_Text NVARCHAR(MAX),
	@Description_Text NVARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [ScriptParameters]
	           WHERE [Script_Id] = @Script_Id
	             AND [Parameter_Name] = @Parameter_Name)
	BEGIN

		UPDATE [ScriptParameters]
		   SET [ParameterType_Code] = @ParameterType_Code,
		       [DefaultValue_Text] = @DefaultValue_Text,
		       [Description_Text] = @Description_Text
		 WHERE [Script_Id] = @Script_Id
		   AND [Parameter_Name] = @Parameter_Name

	END
	ELSE
	BEGIN

		INSERT INTO [ScriptParameters]
		(
			[Script_Id],
			[Parameter_Name],
			[ParameterType_Code],
			[DefaultValue_Text],
			[Description_Text]
		)
		VALUES
		(
			@Script_Id,
			@Parameter_Name,
			@ParameterType_Code,
			@DefaultValue_Text,
			@Description_Text
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateOrUpdateParameter] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateOrUpdateScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Script_Id',
    /* Description_Text        */ 'Creates a new script or updates an existing one.'
GO


IF OBJECT_ID('[Scripts_CreateOrUpdateScript]') IS NOT NULL DROP PROCEDURE [Scripts_CreateOrUpdateScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateOrUpdateScript]
(
	@Script_Id INT OUT,
	@Script_Name NVARCHAR(50),
	@Description_Text NVARCHAR(MAX),
	@ScriptType_Code VARCHAR(20),
	@Active_Indicator YNINDICATOR,
	@AutoSync_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Script_Id IS NULL
	BEGIN

		INSERT INTO [Scripts]
		(
			[Script_Name],
			[Description_Text],
			[Active_Indicator],
			[AutoSync_Indicator],
			[ScriptType_Code],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@Script_Name,
			@Description_Text,
			@Active_Indicator,
			@AutoSync_Indicator,
			@ScriptType_Code,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		SET @Script_Id = SCOPE_IDENTITY()

	END
	ELSE
	BEGIN

		UPDATE [Scripts]
		   SET [Script_Name] = @Script_Name,
		       [Description_Text] = @Description_Text,
		       [ScriptType_Code] = @ScriptType_Code,
		       [Active_Indicator] = @Active_Indicator,
		       [AutoSync_Indicator] = @AutoSync_Indicator,
		       [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
		       [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Script_Id] = @Script_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateOrUpdateScript] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateVersion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Version_Number',
    /* Description_Text        */ 'Creates a new version of a script.'
GO


IF OBJECT_ID('[Scripts_CreateVersion]') IS NOT NULL DROP PROCEDURE [Scripts_CreateVersion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateVersion]
(
	@Version_Number INT OUT,
	@Script_Id INT,
	@Script_Bytes VARBINARY(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SET @Version_Number = (SELECT MAX([Version_Number])
	                         FROM [ScriptVersions]
	                        WHERE [Script_Id] = @Script_Id)

	DECLARE @Latest_Bytes VARBINARY(MAX)
	SET @Latest_Bytes = (SELECT [Script_Bytes]
	                       FROM [ScriptVersions]
	                      WHERE [Script_Id] = @Script_Id
	                        AND [Version_Number] = @Version_Number)

	--Do not insert a copy of the most recent version
	IF @Latest_Bytes IS NULL OR @Latest_Bytes <> @Script_Bytes
	BEGIN

		SET @Version_Number = COALESCE(@Version_Number, 0) + 1

		INSERT INTO [ScriptVersions]
		(
			[Script_Id],
			[Version_Number],
			[Script_Bytes],
			[CreatedBy_User_Name],
			[CreatedOn_Date]
		)
		VALUES
		(
			@Script_Id,
			@Version_Number,
			@Script_Bytes,
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		UPDATE [Scripts]
		   SET [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Script_Id] = @Script_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateVersion] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_DeleteParameter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new script parameter or updates an existing one.'
GO


IF OBJECT_ID('[Scripts_DeleteParameter]') IS NOT NULL DROP PROCEDURE [Scripts_DeleteParameter]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_DeleteParameter]
(
	@Script_Id INT,
	@Parameter_Name NVARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [ScriptParameters]
	 WHERE [Script_Id] = @Script_Id
	   AND [Parameter_Name] = @Parameter_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_DeleteParameter] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_DeleteScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Permanently deletes a script.'
GO


IF OBJECT_ID('[Scripts_DeleteScript]') IS NOT NULL DROP PROCEDURE [Scripts_DeleteScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_DeleteScript]
(
	@Script_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [ScriptParameters]
	 WHERE [Script_Id] = @Script_Id

	DELETE [ScriptVersions]
	 WHERE [Script_Id] = @Script_Id

	DELETE [Scripts]
	 WHERE [Script_Id] = @Script_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_DeleteScript] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_GetScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Scripts_Extended,ScriptParameters',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a script and all of its associated information.'
GO


IF OBJECT_ID('[Scripts_GetScript]') IS NOT NULL DROP PROCEDURE [Scripts_GetScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_GetScript]
(
	@Script_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [Scripts_Extended]
	 WHERE [Script_Id] = @Script_Id

	SELECT *
	  FROM [ScriptParameters]
	 WHERE [Script_Id] = @Script_Id
	 ORDER BY [Parameter_Name]

END
GO

GRANT EXECUTE ON [Scripts_GetScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_GetScripts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Scripts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the scripts in the system, optionally filtered by type code.'
GO


IF OBJECT_ID('[Scripts_GetScripts]') IS NOT NULL DROP PROCEDURE [Scripts_GetScripts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_GetScripts]
(
	@ScriptType_Code VARCHAR(20),
	@IncludeInactive_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [Scripts_Extended]
	 WHERE (@IncludeInactive_Indicator = 'Y' OR [Active_Indicator] = 'Y')
	   AND (@ScriptType_Code IS NULL OR [ScriptType_Code] = @ScriptType_Code)
	 ORDER BY [Script_Name]

END
GO

GRANT EXECUTE ON [Scripts_GetScripts] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_GetScriptTypeUsage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ScriptTypes_Usage',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets usage data for the script types in the database.'
GO


IF OBJECT_ID('[Scripts_GetScriptTypeUsage]') IS NOT NULL DROP PROCEDURE [Scripts_GetScriptTypeUsage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_GetScriptTypeUsage]
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [ScriptTypes_Usage]

END
GO

GRANT EXECUTE ON [Scripts_GetScriptTypeUsage] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_AddNamedUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Assigns the specified user as a named user with the specified user class type'
GO

IF OBJECT_ID('[Security_AddNamedUser]') IS NOT NULL DROP PROCEDURE [Security_AddNamedUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_AddNamedUser]
(
	@User_Name USERNAME,
	@UserClassType_Code CHAR(1)
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [NamedUsers] ([User_Name], [UserClassType_Code])
	VALUES (@User_Name, @UserClassType_Code)

	EXEC [Events_RaiseEvent] 'SECNWU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_AddNamedUser] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_AddPrivilege',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Privilege_Id',
    /* Description_Text        */ 'Grants or denies the specified principal the ability to perform a certain role'
GO

IF OBJECT_ID('[Security_AddPrivilege]') IS NOT NULL
	DROP PROCEDURE [Security_AddPrivilege]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_AddPrivilege]
(
	@Privilege_Id INT = NULL OUT,
	@Directory_Provider_Id INT,
	@Principal_Name USERNAME,
	@PrincipalType_Code CHAR(1),
	@Role_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Environment_Id INT,
	@PrivilegeType_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	/* Catch duplicate data */
	IF EXISTS(SELECT *
				FROM [Privileges]
			   WHERE [Principal_Name] = @Principal_Name
				 AND [Directory_Provider_Id] = @Directory_Provider_Id
				 AND [PrincipalType_Code] = @PrincipalType_Code
				 AND [Role_Id] = @Role_Id
				 AND (	[Application_Id] = @Application_Id
					 OR COALESCE([Application_Id], @Application_Id) IS NULL )
				 AND (	[Environment_Id] = @Environment_Id
					 OR COALESCE([Environment_Id], @Environment_Id) IS NULL )
				 AND (  [ApplicationGroup_Id] = @ApplicationGroup_Id
				     OR COALESCE([ApplicationGroup_Id], @ApplicationGroup_Id) IS NULL )
				 AND [PrivilegeType_Code] = @PrivilegeType_Code )
	BEGIN
	    ROLLBACK
		RETURN
    END

	INSERT INTO [Privileges]
		([Directory_Provider_Id]
		,[Principal_Name]
		,[PrincipalType_Code]
		,[Role_Id]
		,[ApplicationGroup_Id]
		,[Application_Id]
		,[Environment_Id]
		,[PrivilegeType_Code])
	VALUES
		(@Directory_Provider_Id
		,@Principal_Name
		,@PrincipalType_Code
		,@Role_Id
		,@ApplicationGroup_Id
		,@Application_Id
		,@Environment_Id
		,@PrivilegeType_Code)
	SET @Privilege_Id = SCOPE_IDENTITY()

	EXEC [Events_RaiseEvent] 'SECNWP', NULL, @Privilege_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_AddPrivilege] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_CreateOrUpdateRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Role_Id',
    /* Description_Text        */ 'Creates a new role defined by the specified tasks'
GO

IF OBJECT_ID('[dbo].[Security_CreateOrUpdateRole]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_CreateOrUpdateRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_CreateOrUpdateRole]
(
	@Role_Id INT = NULL OUT,
	@Role_Name VARCHAR(50),
	@Tasks_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Role_Id IS NULL
	BEGIN
		/* Add new role */
		INSERT INTO [dbo].[Roles]
			([Role_Name])
		VALUES
			(@Role_Name)
		SET @Role_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'SECNWR', NULL, @Role_Id

	END
	ELSE
	BEGIN
		/* Update existing role */
		UPDATE [Roles]
		   SET [Role_Name] = @Role_Name
		 WHERE [Role_Id] = @Role_Id

		/* Remove existing tasks for the role */
		DELETE [RoleTasks]
		 WHERE [Role_Id] = @Role_Id

		EXEC [Events_RaiseEvent] 'SECMDR', NULL, @Role_Id
	END

	/* Add the appropriate role tasks */
	INSERT INTO [dbo].[RoleTasks]
		([Role_Id]
		,[Task_Id])
	SELECT @Role_Id,[Value_Text]
	  FROM [dbo].[CsvToTable](@Tasks_Csv, ',')

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Security_CreateOrUpdateRole] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_DeleteRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified role from the system'
GO

IF OBJECT_ID('[dbo].[Security_DeleteRole]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_DeleteRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_DeleteRole]
(
	@Role_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Role_Name SQL_VARIANT
	    SET @Role_Name = (SELECT CAST([Role_Name] AS VARCHAR(50)) FROM [Roles] WHERE [Role_Id] = @Role_Id)

	DELETE [Privileges]
	 WHERE [Role_Id] = @Role_Id
	
	DELETE [RoleTasks]
	 WHERE [Role_Id] = @Role_Id
	
	DELETE [Roles]
	 WHERE [Role_Id] = @Role_Id

	IF @@ROWCOUNT>0 EXEC [Events_RaiseEvent] 'SECDLR', NULL, @Role_Name
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [dbo].[Security_DeleteRole] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetNamedUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'NamedUsers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified named user'
GO

IF OBJECT_ID('[Security_GetNamedUser]') IS NOT NULL DROP PROCEDURE [Security_GetNamedUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetNamedUser]
(
    @User_Name USERNAME
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [NamedUsers]
	WHERE [User_Name] = @User_Name


END
GO

GRANT EXECUTE ON [Security_GetNamedUser] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetNamedUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'NamedUsers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of the named users in the system'
GO

IF OBJECT_ID('[Security_GetNamedUsers]') IS NOT NULL DROP PROCEDURE [Security_GetNamedUsers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetNamedUsers]  AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [NamedUsers]


END
GO

GRANT EXECUTE ON [Security_GetNamedUsers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetOrAddAnonymousRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Role_Id',
    /* Description_Text        */ 'Gets an anonymous role containing the specified tasks, creating one if necessary.'
GO

IF OBJECT_ID('[Security_GetOrAddAnonymousRole]') IS NOT NULL
	DROP PROCEDURE [Security_GetOrAddAnonymousRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetOrAddAnonymousRole]
(
	@Tasks_Csv VARCHAR(MAX),
	@Role_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Tasks TABLE([Task_Id] INT NOT NULL)

	INSERT INTO @Tasks
	SELECT DISTINCT CAST([Value_Text] AS INT)
	  FROM dbo.CsvToTable(@Tasks_Csv, ',')

	DECLARE @Task_Count INT
	SET @Task_Count = (SELECT COUNT(*) FROM @Tasks)

	SET @Role_Id = (SELECT TOP 1 RT.[Role_Id]
	                  FROM @Tasks T
	                       INNER JOIN [RoleTasks] RT
		                           ON RT.[Task_Id] = T.[Task_Id]
		                   INNER JOIN [Roles] R
		                           ON R.[Role_Id] = RT.[Role_Id]
	                 WHERE R.[Role_Name] IS NULL
	                 GROUP BY RT.[Role_Id]
	                HAVING COUNT(*) = @Task_Count)

	IF @Role_Id IS NULL
	BEGIN

		INSERT INTO [Roles] ([Role_Name]) VALUES (NULL)
		SET @Role_Id = SCOPE_IDENTITY()

		INSERT INTO [RoleTasks]
		(
			[Role_Id],
			[Task_Id]
		)
		SELECT [Role_Id] = @Role_Id,
		       [Task_Id]
		  FROM @Tasks

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_GetOrAddAnonymousRole] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetPrivileges',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Privileges_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of privileges assigned in the system'
GO

IF OBJECT_ID('[dbo].[Security_GetPrivileges]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_GetPrivileges]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_GetPrivileges]
(
	@Directory_Provider_Id INT,
	@PrincipalType_Code CHAR(1) = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Privileges_Extended]
	 WHERE [PrincipalType_Code] = COALESCE(@PrincipalType_Code, [PrincipalType_Code])
	   AND [Directory_Provider_Id] = @Directory_Provider_Id
END
GO

GRANT EXECUTE ON [dbo].[Security_GetPrivileges] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Roles,RoleTasks_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified role, along with all the tasks that comprise the role.'
GO

IF OBJECT_ID('[Security_GetRole]') IS NOT NULL
	DROP PROCEDURE [Security_GetRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetRole]
(
	@Role_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Roles]
	 WHERE [Role_Id] = @Role_Id
	IF @@ROWCOUNT=0 RAISERROR('Role #%i does not exist.',16,1,@Role_Id)

	SELECT *
	  FROM [RoleTasks_Extended]
	 WHERE [Role_Id] = @Role_Id

END
GO

GRANT EXECUTE ON [Security_GetRole] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetRoles',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Roles,RoleTasks_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the roles in the system, along with all the tasks comprising each role.'
GO

IF OBJECT_ID('[dbo].[Security_GetRoles]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_GetRoles]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_GetRoles]
(
	@IncludeAnonymous_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Roles]
	 WHERE @IncludeAnonymous_Indicator = 'Y' OR [Role_Name] IS NOT NULL
	 ORDER BY [Role_Name]

	SELECT RT.*
	  FROM [RoleTasks_Extended] RT
	       INNER JOIN [Roles] R
		           ON R.[Role_Id] = RT.[Role_Id]
	 WHERE @IncludeAnonymous_Indicator = 'Y' OR R.[Role_Name] IS NOT NULL
	 ORDER BY RT.[Task_Name]

END
GO

GRANT EXECUTE ON [dbo].[Security_GetRoles] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetTasks',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Tasks',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the tasks in the system.'
GO

IF OBJECT_ID('[Security_GetTasks]') IS NOT NULL
	DROP PROCEDURE [Security_GetTasks]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetTasks]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Tasks]

END
GO

GRANT EXECUTE ON [Security_GetTasks] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetUserClassCounts',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Security_GetUserClassCounts]') IS NOT NULL DROP PROCEDURE [Security_GetUserClassCounts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetUserClassCounts]  AS
BEGIN

	SET NOCOUNT ON

    DECLARE @FullUsers INT
    DECLARE @LimitedUsers INT
    DECLARE @ApprovalUsers INT
    DECLARE @NamedUsers INT

	SELECT @LimitedUsers = 0 --(SELECT COUNT(*) FROM [NamedUsers] WHERE [UserClassType_Code] = 'L')
	SELECT @ApprovalUsers = (SELECT COUNT(*) FROM [NamedUsers] WHERE [UserClassType_Code] = 'A')
    SELECT @FullUsers = (SELECT COUNT(*) FROM [NamedUsers] WHERE [UserClassType_Code] = 'F')
    SELECT @NamedUsers = (SELECT COUNT(*) FROM [NamedUsers])
    
    SELECT 
        [NamedUsers_Count] = @NamedUsers,
        [FullUsers_Count] = @FullUsers,
        [LimitedUsers_Count] = @LimitedUsers,
        [ApprovalUsers_Count] = @ApprovalUsers

END
GO

GRANT EXECUTE ON [Security_GetUserClassCounts] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetUserPrivileges',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Privileges_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the list of privileges specifically assigned to a user or any specified groups'
GO

IF OBJECT_ID('[Security_GetUserPrivileges]') IS NOT NULL
	DROP PROCEDURE [Security_GetUserPrivileges]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetUserPrivileges]
(
	@Directory_Provider_Id INT,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

	/* Privileges */
	SELECT *
	  FROM [Privileges_Extended]
	 WHERE [Directory_Provider_Id] = @Directory_Provider_Id
	   AND (   ([Principal_Name] = @User_Name AND [PrincipalType_Code] = 'U') 
		    OR ([Principal_Name] IN (SELECT [Value_Text] FROM [CsvToTable](@GroupNames_Csv, ',') )
			    AND [PrincipalType_Code] = 'G' ) )

END
GO

GRANT EXECUTE ON [Security_GetUserPrivileges] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_IsAuthorizedForAnyTask',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Authorized_Indicator',
    /* Description_Text        */ 'Determines whether or not the specified user with specified groups has authorized to perform a task'
GO


IF OBJECT_ID('Security_IsAuthorizedForAnyTask') IS NOT NULL DROP PROCEDURE [Security_IsAuthorizedForAnyTask]
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_IsAuthorizedForAnyTask] (
	@Directory_Provider_Id INT,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX),
	@Authorized_Indicator YNINDICATOR OUT
) AS
	SET NOCOUNT ON

	IF @Directory_Provider_Id IS NULL
	SET @Directory_Provider_Id = CAST((SELECT [Value_Text]
	                                     FROM [Configuration]
										WHERE [Scope_Name] = 'CoreEx'
										  AND [Key_Name] = 'DirectoryProvider') AS INT)

	IF EXISTS(
		SELECT *
		  FROM [Privileges] P
			   INNER JOIN [RoleTasks] RT
					   ON RT.[Role_Id] = P.[Role_Id]
			   INNER JOIN [Tasks] T
					   ON T.[Task_Id] = RT.[Task_Id]
		 WHERE (( /* User */
				   P.[PrincipalType_Code] = 'U'
				   AND P.[Principal_Name] = @User_Name
				 ) OR (
				   /* Group */ 
				   P.[PrincipalType_Code] = 'G'
				   AND P.[Principal_Name] IN 
					 (SELECT [Value_Text]
						FROM [dbo].[CsvToTable](@GroupNames_Csv, ',') ) 
				))
			AND P.[Directory_Provider_Id] = @Directory_Provider_Id
			AND P.[PrivilegeType_Code] <> 'D'
	) SET @Authorized_Indicator = 'Y'
	  ELSE SET @Authorized_Indicator = 'N'

GO

GRANT EXECUTE ON [Security_IsAuthorizedForAnyTask] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_IsAuthorizedForTask',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Authorized_Indicator',
    /* Description_Text        */ 'Determines whether or not the specified user with specified groups has authorized to perform a task.'
GO


IF OBJECT_ID('Security_IsAuthorizedForTask') IS NOT NULL DROP PROCEDURE [Security_IsAuthorizedForTask]
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Security_IsAuthorizedForTask (
	@Directory_Provider_Id INT,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX),
	@Task_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Environment_Id INT,
	@Server_Id INT,
	@Authorized_Indicator YNINDICATOR OUT
) AS
	SET NOCOUNT ON

	IF @Directory_Provider_Id IS NULL
		SET @Directory_Provider_Id = CAST((SELECT [Value_Text]
		                                     FROM [Configuration]
											WHERE [Scope_Name] = 'CoreEx'
											  AND [Key_Name] = 'DirectoryProvider') AS INT)

	IF dbo.IsAuthorizedForTask(@Directory_Provider_Id, @User_Name, @GroupNames_Csv, @Task_Id, @ApplicationGroup_Id, @Application_Id, @Environment_Id) = 1
		SET @Authorized_Indicator = 'Y'
	ELSE
		SET @Authorized_Indicator = 'N'

	IF @Server_Id IS NOT NULL AND EXISTS(SELECT * FROM [EnvironmentServers] WHERE [Server_Id] = @Server_Id)
	BEGIN
	
		IF EXISTS(
			SELECT *
			  FROM [EnvironmentServers] ES
			 WHERE ES.[Server_Id] = @Server_Id
			   AND dbo.IsAuthorizedForTask(@Directory_Provider_Id, @User_Name, @GroupNames_Csv, @Task_Id, @ApplicationGroup_Id, @Application_Id, ES.[Environment_Id]) = 1)
		   SET @Authorized_Indicator = 'Y'
		ELSE
		   SET @Authorized_Indicator = 'N'

	END
GO

GRANT EXECUTE ON [Security_IsAuthorizedForTask] TO BuildMasterUser_Role
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_IsAuthorizedForTaskBatch',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'AuthorizedTasks_Xml',
    /* Description_Text        */ 'Determines whether or not the specified user with specified groups has authorized to perform the specified tasks.'
GO


IF OBJECT_ID('Security_IsAuthorizedForTaskBatch') IS NOT NULL DROP PROCEDURE [Security_IsAuthorizedForTaskBatch]
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Security_IsAuthorizedForTaskBatch(
	@Directory_Provider_Id INT = NULL,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX),
	@RequiredTasks_Xml XML,
	@AuthorizedTasks_Xml XML OUT
)
AS
	SET NOCOUNT ON

	IF @Directory_Provider_Id IS NULL
		SET @Directory_Provider_Id = CAST((SELECT [Value_Text]
		                                     FROM [Configuration]
											WHERE [Scope_Name] = 'CoreEx'
											  AND [Key_Name] = 'DirectoryProvider') AS INT)

SET @AuthorizedTasks_Xml =
	(SELECT [Task] = CASE WHEN dbo.IsAuthorizedForTask(
					       @Directory_Provider_Id,
					       @User_Name,
					       @GroupNames_Csv,
					       T.c.value('(@Id)[1]', 'int'),
					       T.c.value('(@ApplicationGroupId)[1]', 'int'),
					       T.c.value('(@ApplicationId)[1]', 'int'),
					       T.c.value('(@EnvironmentId)[1]', 'int')) = 0
					     THEN 'N' ELSE 'Y' END
	  FROM @RequiredTasks_Xml.nodes('/RequiredTasks/Task') T(c)
	   FOR XML PATH(''),ROOT('AuthorizedTasks'))
GO

GRANT EXECUTE ON [Security_IsAuthorizedForTaskBatch] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_RemoveNamedUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Removes the specified user as a named user'
GO

IF OBJECT_ID('[Security_RemoveNamedUser]') IS NOT NULL DROP PROCEDURE [Security_RemoveNamedUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_RemoveNamedUser]
(
	@User_Name USERNAME
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [NamedUsers]
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'SECDLU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_RemoveNamedUser] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_RemovePrivilege',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a privilege from the system.'
GO

IF OBJECT_ID('[Security_RemovePrivilege]') IS NOT NULL
	DROP PROCEDURE ..[Security_RemovePrivilege]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_RemovePrivilege]
(
	@Privilege_Id INT
)
AS
BEGIN
SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [Privileges]
	 WHERE [Privilege_Id] = @Privilege_Id

	-- Clean up orphaned anonymous roles
	DELETE RT
	  FROM [RoleTasks] RT
	       INNER JOIN [Roles] R
		           ON R.[Role_Id] = RT.[Role_Id]
	 WHERE R.[Role_Name] IS NULL
	   AND RT.[Role_Id] NOT IN (SELECT [Role_Id] FROM [Privileges])

	DELETE R
	  FROM [Roles] R
	 WHERE R.[Role_Name] IS NULL
	   AND R.[Role_Id] NOT IN (SELECT [Role_Id] FROM [Privileges])

	EXEC [Events_RaiseEvent] 'SECDLP', NULL, @Privilege_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Security_RemovePrivilege] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_UpdateUserClassType',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the user class type of the specified user'
GO

IF OBJECT_ID('[Security_UpdateUserClassType]') IS NOT NULL DROP PROCEDURE [Security_UpdateUserClassType]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_UpdateUserClassType]
(
	@User_Name USERNAME,
	@UserClassType_Code CHAR(1)
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	
	UPDATE [NamedUsers]
	   SET [UserClassType_Code] = @UserClassType_Code
	 WHERE [User_Name] = @User_Name
	
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_UpdateUserClassType] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_UserIsNamed',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'UserIsNamed_Indicator',
    /* Description_Text        */ 'Determines whether or not a user is a named user in the system'
GO

IF OBJECT_ID('[Security_UserIsNamed]') IS NOT NULL DROP PROCEDURE [Security_UserIsNamed]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_UserIsNamed]
(
	@User_Name USERNAME,
	@UserIsNamed_Indicator YNINDICATOR = NULL OUT
) AS
BEGIN

	SET NOCOUNT ON

	SET @UserIsNamed_Indicator = 'N'

	IF EXISTS ( SELECT *
				  FROM [NamedUsers]
				 WHERE [User_Name] = @User_Name )
		SET @UserIsNamed_Indicator = 'Y'

END
GO

GRANT EXECUTE ON [Security_UserIsNamed] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserConfiguration_GetUserConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'UserConfiguration',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a complete list of user configuration settings'
GO

IF OBJECT_ID('[UserConfiguration_GetUserConfiguration]') IS NOT NULL DROP PROCEDURE [UserConfiguration_GetUserConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [UserConfiguration_GetUserConfiguration] 
(
	@User_Name USERNAME = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [UserConfiguration]
	 WHERE (@User_Name IS NULL OR [User_Name] = @User_Name)
	ORDER BY [User_Name], [Key_Name]

END
GO

GRANT EXECUTE ON [UserConfiguration_GetUserConfiguration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserConfiguration_GetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified user configuration value'
GO


IF OBJECT_ID('[dbo].[UserConfiguration_GetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[UserConfiguration_GetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[UserConfiguration_GetValue]
(
	@User_Name USERNAME,
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [UserConfiguration]
	 WHERE [User_Name] = @User_Name
	   AND [Key_Name] = @Key_Name

END
GO

GRANT EXECUTE ON [dbo].[UserConfiguration_GetValue] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserConfiguration_SetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets a user configuration value'
GO


IF OBJECT_ID('[dbo].[UserConfiguration_SetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[UserConfiguration_SetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[UserConfiguration_SetValue]
(
	@User_Name USERNAME,
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [UserConfiguration]
	 WHERE [User_Name] = @User_Name
	   AND [Key_Name] = @Key_Name

	IF (@Value_Text IS NOT NULL) 
		INSERT INTO [UserConfiguration]
			([User_Name], [Key_Name], [Value_Text])
		VALUES
			(@User_Name, @Key_Name, @Value_Text)

	--EXEC [Events_RaiseEvent] 'SYSCFG',NULL,@Key_Name


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[UserConfiguration_SetValue] TO [BuildMasterUser_Role]
GO


GO
IF OBJECT_ID('[dbo].[Users_CreateGroup]') IS NOT NULL DROP PROCEDURE [dbo].[Users_CreateGroup]
GO


EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateOrUpdateGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new user group in the system'
GO

IF OBJECT_ID('[dbo].[Users_CreateOrUpdateGroup]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_CreateOrUpdateGroup]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_CreateOrUpdateGroup]
(
	@Group_Name VARCHAR(50),
	@Users_Xml XML
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Users]
			   WHERE [User_Name] = @Group_Name)
		RAISERROR('Cannot create group "%s"; a user of the same name exists', 16, 1)

	IF NOT EXISTS(SELECT * FROM [Groups] WHERE [Group_Name] = @Group_Name)
    BEGIN
        
        INSERT INTO [Groups] ([Group_Name]) VALUES (@Group_Name)
        EXEC [Events_RaiseEvent] 'USRNWG', NULL, @Group_Name

    END

    IF @Users_XML IS NOT NULL 
    BEGIN

	    DELETE [UserGroups] WHERE [Group_Name] = @Group_Name

	    INSERT INTO [UserGroups] ([User_Name] ,[Group_Name])
	    SELECT UX.[value]('.', 'VARCHAR(250)') [User_Name], @Group_Name
	      FROM @Users_Xml.[nodes]('/users/username') AS UsersXml(UX)

    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_CreateOrUpdateGroup] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateOrUpdateUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new user in the system or updates an existing one.'
GO

IF OBJECT_ID('[dbo].[Users_CreateOrUpdateUser]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_CreateOrUpdateUser]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_CreateOrUpdateUser]
(
	@User_Name USERNAME,
	@Display_Name VARCHAR(100),
	@Email_Address VARCHAR(250) = NULL,
	@Groups_Xml XML,
	@ChangePasswordOnLogin_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Groups]
			   WHERE [Group_Name] = @User_Name)
		RAISERROR('Cannot create user %s; a group of the same name exists', 16, 1)

	IF EXISTS(SELECT *
				FROM [Users]
	            WITH (UPDLOCK)
			   WHERE [User_Name] = @User_Name)
	BEGIN

		UPDATE [Users]
		   SET [Display_Name] = @Display_Name,
			   [Email_Address] = @Email_Address,
			   [ChangePasswordOnLogin_Indicator] = @ChangePasswordOnLogin_Indicator
		 WHERE [User_Name] = @User_Name

		EXEC [Events_RaiseEvent] 'USRMDU', NULL, @User_Name

	END
	ELSE
	BEGIN

		INSERT INTO [Users]
		(
			[User_Name],
			[Display_Name],
			[Email_Address],
			[ChangePasswordOnLogin_Indicator]
		)
		VALUES
		(
			@User_Name,
			@Display_Name,
			@Email_Address,
			@ChangePasswordOnLogin_Indicator
		)

		EXEC [Events_RaiseEvent] 'USRNWU', NULL, @User_Name

	END

	DELETE
	  FROM [UserGroups]
	 WHERE [User_Name] = @User_Name

	INSERT INTO [UserGroups]
	(
		[User_Name],
		[Group_Name]
	)
	SELECT @User_Name
		  ,GX.[value]('.', 'VARCHAR(50)') [Group_Name]
	  FROM @Groups_Xml.[nodes]('/groups/group') AS GroupsXml(GX)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_CreateOrUpdateUser] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_DeleteGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a user group from the system, but does not delete the users who belong to the group'
GO

IF OBJECT_ID('[dbo].[Users_DeleteGroup]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_DeleteGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_DeleteGroup]
(
	@Group_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	/* Remove group privileges */
	DELETE [Privileges]
	 WHERE [Principal_Name] = @Group_Name
	   AND [PrincipalType_Code] = 'G'

	/* Remove group users */
	DELETE [UserGroups]
	 WHERE [Group_Name] = @Group_Name

	/* Remove group */
	DELETE [Groups]
	 WHERE [Group_Name] = @Group_Name

	EXEC [Events_RaiseEvent] 'USRDLG', NULL, @Group_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_DeleteGroup] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_DeleteUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a user from the system'
GO

IF OBJECT_ID('[dbo].[Users_DeleteUser]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_DeleteUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_DeleteUser]
(
	@User_Name VARCHAR(255)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	DELETE [Privileges]
	 WHERE [Principal_Name] = @User_Name
	   AND [PrincipalType_Code] = 'U'

	DELETE [UserGroups]
	 WHERE [User_Name] = @User_Name

	DELETE [Users]
	 WHERE [User_Name] = @User_Name

	DELETE [NamedUsers]
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'USRDLU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_DeleteUser] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Groups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of user groups in the system'
GO

IF OBJECT_ID('[dbo].[Users_GetGroups]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_GetGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_GetGroups]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Groups]
	ORDER BY [Group_Name]

END
GO

GRANT EXECUTE ON [dbo].[Users_GetGroups] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Users,UserGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified user, and a list of all the groups the user belongs to'
GO

IF OBJECT_ID('[Users_GetUser]') IS NOT NULL DROP PROCEDURE [Users_GetUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_GetUser]
(
	@User_Name USERNAME
)
AS
BEGIN
	
	SET NOCOUNT ON

	SELECT *
	  FROM [Users]
	 WHERE [User_Name] = @User_Name

	SELECT *
	  FROM [UserGroups]
	 WHERE [User_Name] = @User_Name

END
GO

GRANT EXECUTE ON [Users_GetUser] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Users,UserGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the users in the system, along with all group membership information.'
GO

IF OBJECT_ID('[Users_GetUsers]') IS NOT NULL DROP PROCEDURE [Users_GetUsers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Users_GetUsers
 AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Users]

	SELECT *
	  FROM [UserGroups]

END
GO

GRANT EXECUTE ON [Users_GetUsers] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUsersInGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Users',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of users in the specified group'
GO

IF OBJECT_ID('[dbo].[Users_GetUsersInGroup]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_GetUsersInGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_GetUsersInGroup]
(
    @Group_Name VARCHAR(50)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT U.*
	  FROM [UserGroups] UG
	INNER JOIN [Users] U ON UG.[User_Name] = U.[User_Name]
	WHERE [Group_Name] = @Group_Name
	ORDER BY [User_Name]

END
GO

GRANT EXECUTE ON [dbo].[Users_GetUsersInGroup] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SearchGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Groups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a specific group by name, or if a wildcard search is performed, a group whose name starts with the specified name'
GO

IF OBJECT_ID('[dbo].[Users_SearchGroups]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_SearchGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_SearchGroups]
(
    @Group_Name VARCHAR(50),
    @PerformWildcardSearch_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON
    
    IF (@PerformWildcardSearch_Indicator = 'Y')
    BEGIN
    
        SELECT *
          FROM [Groups]
         WHERE [Group_Name] LIKE @Group_Name + '%'
      ORDER BY [Group_Name]
        
    END
    ELSE
    BEGIN
	
	    SELECT *
	      FROM [Groups]
	     WHERE [Group_Name] = @Group_Name
	  ORDER BY [Group_Name]
	
	END

END
GO

GRANT EXECUTE ON [dbo].[Users_SearchGroups] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SearchUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Users',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified user, or searchs for a user whose name starts with any of the specified criteria'
GO

IF OBJECT_ID('[Users_SearchUsers]') IS NOT NULL DROP PROCEDURE [Users_SearchUsers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_SearchUsers]
(
    @User_Name USERNAME,
    @Display_Name VARCHAR(100),
    @Email_Address VARCHAR(250),
    @PerformWildcardSearch_Indicator YNINDICATOR
)
 AS BEGIN

	SET NOCOUNT ON

    IF (@PerformWildcardSearch_Indicator = 'Y')
    BEGIN
    
        -- Add wildcards if specified and the arguments are not null
        IF (@User_Name IS NOT NULL)
            SET @User_Name = @User_Name + '%'
        IF (@Display_Name IS NOT NULL)
            SET @Display_Name = @Display_Name + '%'
        IF (@Email_Address IS NOT NULL)
            SET @Email_Address = @Email_Address + '%'
    
        SELECT *
	      FROM [Users]
	      WHERE [User_Name] LIKE @User_Name
	         OR [Display_Name] LIKE @Display_Name
	         OR [Email_Address] LIKE @Email_Address
    END
    ELSE
    BEGIN
        -- Perform an exact match
         SELECT *
	      FROM [Users]
	      WHERE [User_Name] = @User_Name
	        OR [Display_Name] = @Display_Name
	        OR [Email_Address] = @Email_Address
    END

	

END
GO

GRANT EXECUTE ON [Users_SearchUsers] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SetPassword',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Users_SetPassword]') IS NOT NULL
	DROP PROCEDURE [Users_SetPassword]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_SetPassword]
(
	@User_Name USERNAME,
	@Password_Text VARCHAR(20),
	@ClearChangePasswordOnLogin_Indicator YNINDICATOR = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DECLARE @Salt_Bytes BINARY(10)
	DECLARE @Password_Bytes BINARY(20)
	
	SET @Salt_Bytes = CAST(NEWID() AS BINARY(10))
	SET @Password_Bytes = HASHBYTES('SHA1', @Salt_Bytes + CAST(@Password_Text AS VARBINARY))
	
	UPDATE [Users]
	   SET [Password_Bytes] = @Password_Bytes,
	       [Salt_Bytes] = @Salt_Bytes,
		   [ChangePasswordOnLogin_Indicator] = CASE WHEN @ClearChangePasswordOnLogin_Indicator = 'Y' THEN 'N' ELSE [ChangePasswordOnLogin_Indicator] END
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'USRMDU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_SetPassword] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_ValidateLogin',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Validated_Indicator',
    /* Description_Text        */ 'Validates the password of a specified user'
GO

IF OBJECT_ID('Users_ValidateLogin') IS NOT NULL DROP PROCEDURE Users_ValidateLogin
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Users_ValidateLogin
(
	@User_Name USERNAME,
	@Password_Text VARCHAR(20),
	@Validated_Indicator YNINDICATOR = NULL OUT
) AS BEGIN

	SET NOCOUNT ON

	/* Set defaults */
	SET @Validated_Indicator = 'N'

	IF EXISTS(SELECT * FROM [Users]
	           WHERE [User_Name] = @User_Name
	             AND HASHBYTES('SHA1', [Salt_Bytes] + CAST(@Password_Text AS VARBINARY)) = [Password_Bytes])
		SET @Validated_Indicator = 'Y'


END
GO

GRANT EXECUTE ON Users_ValidateLogin TO BuildMasterUser_Role

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_CreateOrUpdateVariableDeclaration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'VariableDeclaration_Id',
    /* Description_Text        */ 'Creates a new variable declaration or updates an existing one'
GO


IF OBJECT_ID('[Variables_CreateOrUpdateVariableDeclaration]') IS NOT NULL DROP PROCEDURE [Variables_CreateOrUpdateVariableDeclaration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_CreateOrUpdateVariableDeclaration]
(
	@VariableDeclaration_Id INT OUT,
	@Variable_Name VARCHAR(50),
	@Scope_Code CHAR(1),
	@Scoped_Application_Id INT,
	@Scoped_Environment_Id INT,
	@Required_Indicator YNINDICATOR,
	@Variable_Configuration VARCHAR(MAX),
	@DefaultValue_Text VARCHAR(MAX),
	@Sensitive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	IF @VariableDeclaration_Id IS NULL
	BEGIN
		INSERT INTO [VariableDeclarations]
		(
			[Variable_Name],
			[Scope_Code],
			[Scoped_Application_Id],
			[Scoped_Environment_Id],
			[Required_Indicator],
			[Variable_Configuration],
			[DefaultValue_Text],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
			[Sensitive_Indicator]
		)
		VALUES
		(
			@Variable_Name,
			@Scope_Code,
			@Scoped_Application_Id,
			@Scoped_Environment_Id,
			@Required_Indicator,
			@Variable_Configuration,
			@DefaultValue_Text,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			@Sensitive_Indicator
		)

		SET @VariableDeclaration_Id = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		UPDATE [VariableDeclarations]
		   SET [Required_Indicator] = COALESCE(@Required_Indicator, [Required_Indicator]),
			   [Variable_Configuration] = COALESCE(@Variable_Configuration, [Variable_Configuration]),
			   [Scoped_Environment_Id] = @Scoped_Environment_Id,
			   [DefaultValue_Text] = @DefaultValue_Text,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE(),
			   [Sensitive_Indicator] = @Sensitive_Indicator
		 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_CreateOrUpdateVariableDeclaration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_CreateOrUpdateVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new variable definition or updates an existing one'
GO


IF OBJECT_ID('[Variables_CreateOrUpdateVariableDefinition]') IS NOT NULL DROP PROCEDURE [Variables_CreateOrUpdateVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_CreateOrUpdateVariableDefinition]
(
	@Variable_Name VARCHAR(50),
	@Environment_Id INT,
	@Server_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Deployable_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Execution_Id INT,
	@Value_Text VARCHAR(MAX),
	@Sensitive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Inferred_Scope_Code CHAR(1)
        SET @Inferred_Scope_Code = CASE
            WHEN @Server_Id IS NOT NULL THEN 'V'
            WHEN @ApplicationGroup_Id IS NOT NULL THEN 'G'
            WHEN @Deployable_Id IS NOT NULL THEN 'D'
            WHEN @Environment_Id IS NOT NULL AND @Build_Number IS NULL THEN 'E'
            WHEN @Application_Id IS NOT NULL AND @Release_Number IS NULL THEN 'A'
            WHEN @Release_Number IS NOT NULL AND @Build_Number IS NULL THEN 'R'
            WHEN @Build_Number IS NOT NULL AND @Environment_Id IS NULL THEN 'B'
            WHEN @Environment_Id IS NOT NULL AND @Execution_Id IS NULL THEN 'P'
            WHEN @Execution_Id IS NOT NULL THEN 'X'              
            ELSE 'S' END
            
    DECLARE @VariableDeclaration_Id INT
		SET @VariableDeclaration_Id =
              (SELECT [VariableDeclaration_Id] 
                 FROM [VariableDeclarations]
                WHERE [Variable_Name] = @Variable_Name
                  AND [Scope_Code] = @Inferred_Scope_Code
                  AND ([Scope_Code] NOT IN ('R','B','P','X') OR [Scoped_Application_Id] = @Application_Id)
                  AND ([Scope_Code] NOT IN ('P','X') OR [Scoped_Environment_Id] = @Environment_Id))

    DECLARE @Proper_Variable_Name VARCHAR(50)
    DECLARE @Declaration_Sensitive_Indicator YNINDICATOR

    SELECT @Proper_Variable_Name = [Variable_Name],
           @Declaration_Sensitive_Indicator = [Sensitive_Indicator]
      FROM [VariableDeclarations]
     WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

    SET @Variable_Name = COALESCE(@Proper_Variable_Name, @Variable_Name)
    SET @Sensitive_Indicator = COALESCE(@Declaration_Sensitive_Indicator, @Sensitive_Indicator)

    IF @Inferred_Scope_Code = 'R' AND 
       EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number AND [ReleaseStatus_Name] <> 'Active')
       RAISERROR('Cannot set Release-Scoped Variable %s because Release %s of application id %d is not Active.', 16, 1, @Variable_Name, @Release_Number, @Application_Id)

    IF @Inferred_Scope_Code = 'B' AND 
       EXISTS(SELECT * FROM [Builds] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number AND [Build_Number] = @Build_Number AND [BuildStatus_Name] <> 'Active')
       RAISERROR('Cannot set Build-Scoped Variable %s Build %s of release %s in application id %d is not Active.', 16, 1, @Variable_Name, @Build_Number, @Release_Number, @Application_Id)

    IF @Inferred_Scope_Code = 'P' AND 
       EXISTS(SELECT * FROM [BuildPromotions] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number 
                                                AND [Build_Number] = @Build_Number AND [Environment_Id] = @Environment_Id AND [PromotionStatus_Name] ='Completed')
       RAISERROR('Cannot set Promotion-Scoped Variable %s Promotion to (EnvId=%d) for Build %s of release %s in application id %d is not Active.', 16, 1, @Variable_Name, @Environment_Id, @Build_Number, @Release_Number, @Application_Id)

    IF @Inferred_Scope_Code = 'X' AND 
       EXISTS(SELECT * FROM [BuildExecutions] WHERE [Execution_Id] = @Execution_Id AND [ExecutionStatus_Name] NOT IN ('Pending', 'Executing'))
       RAISERROR('Cannot set Execution-Scoped Variable %s Execution id %d is not executing or pending execution', 16, 1, @Variable_Name, @Execution_Id)

    DECLARE @VariableDefinition_Id INT
        SET @VariableDefinition_Id = (SELECT [VariableDefinition_Id] FROM [VariableDefinitions]
             WHERE [Variable_Name] = @Variable_Name
               AND ([Environment_Id] = @Environment_Id OR ([Environment_Id] IS NULL AND @Environment_Id IS NULL))
               AND ([Server_Id] = @Server_Id OR ([Server_Id] IS NULL AND @Server_Id IS NULL))
               AND ([ApplicationGroup_Id] = @ApplicationGroup_Id OR ([ApplicationGroup_Id] IS NULL AND @ApplicationGroup_Id IS NULL))
               AND ([Deployable_Id] = @Deployable_Id OR ([Deployable_Id] IS NULL AND @Deployable_Id IS NULL))

               AND ([Application_Id] = @Application_Id OR ([Application_Id] IS NULL AND @Application_Id IS NULL))
               AND ([Release_Number] = @Release_Number OR ([Release_Number] IS NULL AND @Release_Number IS NULL))
               AND ([Build_Number] = @Build_Number OR ([Build_Number] IS NULL AND @Build_Number IS NULL))

               AND ([Execution_Id] = @Execution_Id OR ([Execution_Id] IS NULL AND @Execution_Id IS NULL)) )

    IF @VariableDefinition_Id IS NULL
    BEGIN

		INSERT INTO [VariableDefinitions] 
        (
			[Variable_Name],
			[Environment_Id],
			[Server_Id],
			[ApplicationGroup_Id],
			[Application_Id],
			[Deployable_Id], 
			[Release_Number],
			[Build_Number],
			[Execution_Id],
			[Value_Text],
			[Sensitive_Indicator]
		)
		VALUES
		(
			@Variable_Name,
			@Environment_Id,
			@Server_Id,
			@ApplicationGroup_Id,
			@Application_Id,
			@Deployable_Id,
			@Release_Number,
			@Build_Number,
			@Execution_Id,
			@Value_Text,
			@Sensitive_Indicator
		)

	END
	ELSE
	BEGIN

		UPDATE [VariableDefinitions]
		   SET [Value_Text] = @Value_Text,
		       [Sensitive_Indicator] = @Sensitive_Indicator
		 WHERE [VariableDefinition_Id] = @VariableDefinition_Id

	END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_CreateOrUpdateVariableDefinition] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_DeleteVariableDeclaration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a variable declaration'
GO


IF OBJECT_ID('[Variables_DeleteVariableDeclaration]') IS NOT NULL DROP PROCEDURE [Variables_DeleteVariableDeclaration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_DeleteVariableDeclaration]
(
	@VariableDeclaration_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DELETE [VariableDefinitions]
      FROM [VariableDefinitions] VDEF
           INNER JOIN [VariableDeclarations] VDEC
                   ON VDEF.[Variable_Name] = VDEC.[Variable_Name]
                  AND (
                        (VDEC.[Scope_Code] = 'S' AND VDEF.[Environment_Id] IS NULL
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'E' AND VDEF.[Environment_Id] IS NOT NULL
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'V' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NOT NULL
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'G' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NOT NULL
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'A' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NOT NULL
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'D' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NOT NULL
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'R' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NOT NULL
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NOT NULL
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                      )               
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id
	
    DELETE [BuildScheduleVariableDefinitions]
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

    DELETE [VariableDeclarations]
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_DeleteVariableDeclaration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_DeleteVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a variable definition'
GO


IF OBJECT_ID('[Variables_DeleteVariableDefinition]') IS NOT NULL DROP PROCEDURE [Variables_DeleteVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_DeleteVariableDefinition]
(
	@VariableDefinition_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DELETE [VariableDefinitions]
	 WHERE [VariableDefinition_Id] = @VariableDefinition_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_DeleteVariableDefinition] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetSystemVariableDefinitions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDefinitions_System',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable definitions for system-scoped variables.'
GO


IF OBJECT_ID('[Variables_GetSystemVariableDefinitions]') IS NOT NULL DROP PROCEDURE [Variables_GetSystemVariableDefinitions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetSystemVariableDefinitions]
AS
BEGIN

    SET NOCOUNT ON

    SELECT * FROM [VariableDefinitions_System]


END
GO

GRANT EXECUTE ON [Variables_GetSystemVariableDefinitions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDeclaration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'VariableDeclarations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable declaration in the system that has the specified ID.'
GO


IF OBJECT_ID('[Variables_GetVariableDeclaration]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDeclaration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDeclaration]
(
	@VariableDeclaration_Id INT
)
AS
BEGIN

	SET NOCOUNT ON
	
	SELECT *
	  FROM [VariableDeclarations]
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

END
GO

GRANT EXECUTE ON [Variables_GetVariableDeclaration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDeclarations',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDeclarations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable declarations in the system that are defined in the specified scope'
GO


IF OBJECT_ID('[Variables_GetVariableDeclarations]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDeclarations]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDeclarations]
(
	@Scope_Code CHAR(1),
	@Scoped_Application_Id INT,
    @Scoped_Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SELECT * 
      FROM [VariableDeclarations]
     WHERE (@Scope_Code IS NULL OR [Scope_Code] = @Scope_Code)
       AND ([Scope_Code] NOT IN ('R','B','P','X') OR @Scoped_Application_Id IS NULL OR [Scoped_Application_Id] = @Scoped_Application_Id)
       AND ([Scope_Code] <> 'X' OR (@Scoped_Environment_Id IS NULL AND [Scoped_Environment_Id] IS NULL) OR [Scoped_Environment_Id] = @Scoped_Environment_Id OR @Scope_Code IS NULL)
       AND ([Scope_Code] <> 'P' OR @Scoped_Environment_Id IS NULL  OR [Scoped_Environment_Id] = @Scoped_Environment_Id)
     ORDER BY [Variable_Name]

END
GO

GRANT EXECUTE ON [Variables_GetVariableDeclarations] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'VariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified variable definition.'
GO


IF OBJECT_ID('[Variables_GetVariableDefinition]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDefinition]
(
    @VariableDefinition_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    SELECT *
      FROM [VariableDefinitions_Extended]
	 WHERE [VariableDefinition_Id] = @VariableDefinition_Id


END
GO

GRANT EXECUTE ON [Variables_GetVariableDefinition] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDefinitions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable definitions in the system that are defined in the specified scope'
GO


IF OBJECT_ID('[Variables_GetVariableDefinitions]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDefinitions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDefinitions]
(
    @Environment_Id INT,
    @Server_Id INT,
    @ApplicationGroup_Id INT,
    @Application_Id INT,
    @Deployable_Id INT,
    @Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER,
    @Execution_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    SELECT *
      FROM [VariableDefinitions_Extended]
     WHERE ([Environment_Id] = @Environment_Id OR ([Environment_Id] IS NULL AND @Environment_Id IS NULL))
       AND ([Server_Id] = @Server_Id OR ([Server_Id] IS NULL AND @Server_Id IS NULL))
       AND ([ApplicationGroup_Id] = @ApplicationGroup_Id OR ([ApplicationGroup_Id] IS NULL AND @ApplicationGroup_Id IS NULL))
       AND ([Deployable_Id] = @Deployable_Id OR ([Deployable_Id] IS NULL AND @Deployable_Id IS NULL))
       
       AND ([Application_Id] = @Application_Id OR ([Application_Id] IS NULL AND @Application_Id IS NULL))
       AND ([Release_Number] = @Release_Number OR ([Release_Number] IS NULL AND @Release_Number IS NULL))
       AND ([Build_Number] = @Build_Number OR ([Build_Number] IS NULL AND @Build_Number IS NULL))
       
       AND ([Execution_Id] = @Execution_Id OR ([Execution_Id] IS NULL AND @Execution_Id IS NULL))


END
GO

GRANT EXECUTE ON [Variables_GetVariableDefinitions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableValues',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Variable_Values',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable values for the specified scope.'
GO

IF OBJECT_ID('[Variables_GetVariableValues]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableValues]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableValues]
(
    @Environment_Id INT,
    @Server_Id INT,
    @ApplicationGroup_Id INT,
    @Application_Id INT,
    @Deployable_Id INT,
    @Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER,
    @Execution_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    IF @Execution_Id IS NOT NULL
    BEGIN
		-- Use standard function for execution-scoped variable cascading
		SELECT *
		  FROM dbo.Variables_GetValuesForExecution(@Execution_Id, @Server_Id, @Deployable_Id)
    END
    ELSE
    BEGIN

		-- If deployable is specified but not application, find the application
		IF @Deployable_Id IS NOT NULL AND @Application_Id IS NULL
		BEGIN
			SET @Application_Id = (SELECT [Application_Id]
			                         FROM [Deployables]
			                        WHERE [Deployable_Id] = @Deployable_Id)
		END

		-- If application is specified but not application group find the group
		IF @Application_Id IS NOT NULL AND @ApplicationGroup_Id IS NULL
		BEGIN
			SET @ApplicationGroup_Id = (SELECT [ApplicationGroup_Id]
			                              FROM [Applications]
			                             WHERE [Application_Id] = @Application_Id)
		END

		-- Make sure irrelevant release number is not considered
		IF @Release_Number IS NOT NULL AND @Application_Id IS NULL
			SET @Release_Number = NULL

		-- Make sure irrelevant build number is not considered
		IF @Build_Number IS NOT NULL AND @Release_Number IS NULL
			SET @Build_Number = NULL

		-- Determine the narrowest scope that will be considered			
		DECLARE @DesiredScope_Code CHAR(1)
		    SET @DesiredScope_Code = CASE
				WHEN @Environment_Id IS NOT NULL AND @Build_Number IS NOT NULL THEN 'P'
				WHEN @Build_Number IS NOT NULL THEN 'B'
				WHEN @Release_Number IS NOT NULL THEN 'R'
				WHEN @Deployable_Id IS NOT NULL THEN 'D'
				WHEN @Application_Id IS NOT NULL THEN 'A'
				WHEN @ApplicationGroup_Id IS NOT NULL THEN 'G'
				WHEN @Server_Id IS NOT NULL THEN 'V'
				WHEN @Environment_Id IS NOT NULL THEN 'E'
				ELSE 'S'
			END

		DECLARE @DesiredScope_Index INT
		    SET @DesiredScope_Index = CHARINDEX(@DesiredScope_Code, 'SEVGADRBPX')

		--Return the results by joining against all variable names in the system and filtering based on input criteria
		--The inner SELECT DISTINCTS required that the VDEF and VDEC tables need an index on the Variable_Name columns for decent performance
        SELECT [Variable_Name],
               [Value_Text],
               [Scope_Code],
               [Sensitive_Indicator]

          FROM (SELECT VN.[Variable_Name],
                       [Value_Text] = COALESCE(VDEF.[Value_Text], VDEC.[DefaultValue_Text]),
                       [Scope_Code] = COALESCE(VDEF.[Scope_Code], VDEC.[Scope_Code]),
                       [Sensitive_Indicator] = COALESCE(VDEF.[Sensitive_Indicator], VDEC.[Sensitive_Indicator]),
                       [Rank] = RANK() OVER (PARTITION BY VN.[Variable_Name] ORDER BY CHARINDEX(COALESCE(VDEF.[Scope_Code], VDEC.[Scope_Code]), 'SEVGADRBPX') DESC)
                  FROM (SELECT DISTINCT [Variable_Name]
						  FROM [VariableDeclarations]
                               UNION
                        SELECT DISTINCT [Variable_Name]
                          FROM [VariableDefinitions]) VN

                       LEFT JOIN [VariableDefinitions] VDEF
                              ON VN.[Variable_Name] = VDEF.[Variable_Name]
                             AND CHARINDEX(VDEF.[Scope_Code], 'SEVGADRBPX') <= @DesiredScope_Index
                             AND (VDEF.[Application_Id] IS NULL OR VDEF.[Application_Id] = @Application_Id)
                             AND (VDEF.[Release_Number] IS NULL OR (VDEF.[Application_Id] = @Application_Id AND VDEF.[Release_Number] = @Release_Number))
                             AND (VDEF.[Build_Number] IS NULL OR (VDEF.[Application_Id] = @Application_Id AND VDEF.[Release_Number] = @Release_Number AND VDEF.[Build_Number] = @Build_Number))
                             AND (VDEF.[ApplicationGroup_Id] IS NULL OR VDEF.[ApplicationGroup_Id] = @ApplicationGroup_Id)
                             AND (VDEF.[Deployable_Id] IS NULL OR VDEF.[Deployable_Id] = @Deployable_Id)
                             AND (VDEF.[Environment_Id] IS NULL OR VDEF.[Environment_Id] = @Environment_Id)
                             AND (VDEF.[Server_Id] IS NULL OR VDEF.[Server_Id] = @Server_Id)
                       LEFT JOIN [VariableDeclarations] VDEC
                              ON VN.[Variable_Name] = VDEC.[Variable_Name]
                             AND CHARINDEX(VDEC.[Scope_Code], 'SEVGADRBPX') <= @DesiredScope_Index
                             AND (VDEC.[Scoped_Application_Id] IS NULL OR VDEC.[Scoped_Application_Id] = @Application_Id)
                             AND (VDEC.[Scoped_Environment_Id] IS NULL OR (VDEC.[Scoped_Application_Id] = @Application_Id AND VDEC.[Scoped_Environment_Id] = @Environment_Id))
                 WHERE VDEF.[Scope_Code] = VDEC.[Scope_Code]
                    OR (VDEF.[Scope_Code] IS NULL AND VDEC.[Scope_Code] IS NOT NULL)
                    OR (VDEC.[Scope_Code] IS NULL AND VDEF.[Scope_Code] IS NOT NULL)) _
         WHERE [Rank] = 1

    END

END
GO

GRANT EXECUTE ON [Variables_GetVariableValues] TO [BuildMasterUser_Role]

GO

IF OBJECT_ID('[Workflows_AddOrRemoveApproval]') IS NOT NULL DROP PROCEDURE [Workflows_AddOrRemoveApproval]
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_AssignGlobalWorkflowToApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Assigns a global workflow to an application.'
GO

IF OBJECT_ID('[Workflows_AssignGlobalWorkflowToApplication]') IS NOT NULL DROP PROCEDURE [Workflows_AssignGlobalWorkflowToApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_AssignGlobalWorkflowToApplication]
(
	@Application_Id INT,
    @Workflow_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

        IF EXISTS(SELECT [Global_Indicator] 
                    FROM [Workflows] 
                   WHERE [Workflow_Id] = @Workflow_Id
                     AND [Global_Indicator] = 'N')
        BEGIN
            RAISERROR('Workflow %i must be promoted to a global workflow before it can be associated with an application.', 16, 1, @Workflow_Id)
        END

        DELETE [ApplicationWorkflows]
         WHERE [Application_Id] = @Application_Id
           AND [Workflow_Id] = @Workflow_Id

		INSERT INTO [ApplicationWorkflows]
		(
			[Application_Id],
			[Workflow_Id]
		)
		VALUES
		(
			@Application_Id,
			@Workflow_Id
		)

		EXEC [Events_RaiseEvent] 'WKFNEW',NULL, @Workflow_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_AssignGlobalWorkflowToApplication] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_CloneWorkflow',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'NewWorkflowIdsCsv',
    /* Description_Text        */ 'Clones one or more workflows.',
    /* Remarks_Text            */ 'To specify workflow data, the WorkflowCloneData_Xml argument should be a string in the format:

    <WorkflowCloneData CloneDeploymentPlans_Indicator="[Y/N]" LinkAllActionGroups_Indicator="[Y/N]">
        <Workflow>
            <New_Workflow_Name>Standard</New_Workflow_Name>
            <Workflow_Id>3</Workflow_Id>
            <New_Application_Id>1000</New_Application_Id> [optional if cloning into a single application instead of global]
        </Workflow>
    </WorkflowCloneData>'
GO

IF OBJECT_ID('[Workflows_CloneWorkflow]') IS NOT NULL
	DROP PROCEDURE [Workflows_CloneWorkflow]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_CloneWorkflow]
(
	@WorkflowCloneData_Xml XML,
	@NewWorkflowIdsCsv VARCHAR(MAX) = NULL OUT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

	DECLARE @DeploymentPlanCloneData_Xml XML, @NewDeploymentPlanIdsCsv VARCHAR(MAX)

	DECLARE @CloneDeploymentPlans_Indicator YNINDICATOR
	    SET @CloneDeploymentPlans_Indicator = 
		       (SELECT WCD.[value]('@CloneDeploymentPlans_Indicator', 'CHAR(1)') [CloneDeploymentPlans_Indicator]
	              FROM @WorkflowCloneData_Xml.[nodes]('/WorkflowCloneData') AS WorkflowCloneData(WCD))

    DECLARE @LinkAllActionGroups_Indicator YNINDICATOR
	    SET @LinkAllActionGroups_Indicator = COALESCE(
		       (SELECT WCD.[value]('@LinkAllActionGroups_Indicator', 'CHAR(1)') [LinkAllActionGroups_Indicator]
	              FROM @WorkflowCloneData_Xml.[nodes]('/WorkflowCloneData') AS WorkflowCloneData(WCD)), 'N')

	DECLARE @Workflows TABLE ([Seq] INT IDENTITY(1,1), [Old_Workflow_Id] INT, 
	                          [New_Workflow_Id] INT, [New_Workflow_Name] NVARCHAR(50), [New_Application_Id] INT)

	INSERT INTO @Workflows ([Old_Workflow_Id],[New_Workflow_Name], [New_Application_Id])
	SELECT DISTINCT [Old_Workflow_Id], [New_Workflow_Name], [New_Application_Id]
	  FROM (SELECT WCD.[value]('New_Workflow_Name[1]', 'NVARCHAR(50)') [New_Workflow_Name],
	               WCD.[value]('Workflow_Id[1]', 'INT') [Old_Workflow_Id],
                   WCD.[value]('New_Application_Id[1]', 'INT') [New_Application_Id]
	          FROM @WorkflowCloneData_Xml.[nodes]('/WorkflowCloneData/Workflow') AS WorkflowCloneData(WCD)) WCD
           INNER JOIN [Workflows] W ON WCD.[Old_Workflow_Id] = W.[Workflow_Id]

	WHILE EXISTS(SELECT * FROM @Workflows WHERE [New_Workflow_Id] IS NULL) BEGIN

		DECLARE @Old_Workflow_Id INT, @New_Workflow_Id INT, @New_Workflow_Name NVARCHAR(50), @New_Application_Id INT
		SELECT TOP 1 @Old_Workflow_Id = [Old_Workflow_Id], @New_Workflow_Name = [New_Workflow_Name], @New_Application_Id = [New_Application_Id]
		   FROM @Workflows WHERE [New_Workflow_Id] IS NULL ORDER BY [Old_Workflow_Id]

		INSERT INTO [Workflows]
			([Workflow_Name], [Active_Indicator],
			 [CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],
			 [AutoDeployRelease_Indicator],[AllowOutOfSequencePromotions_Indicator],[AutoCancelReleases_Indicator],[AutoIncrementRelease_Indicator],
			 [Build_DeploymentPlan_Id],[BuildImporterTemplate_Configuration],
			 [Color_Code],[Global_Indicator], [Build_AutoPromote_Indicator],[AllowLocalChanges_Indicator])
		SELECT 
			 @New_Workflow_Name, [Active_Indicator],
			 dbo.Session_GetUserName(), GETUTCDATE(),[ModifiedBy_User_Name],[ModifiedOn_Date],
			 [AutoDeployRelease_Indicator],[AllowOutOfSequencePromotions_Indicator],[AutoCancelReleases_Indicator],[AutoIncrementRelease_Indicator],
			 [Build_DeploymentPlan_Id],[BuildImporterTemplate_Configuration],
			 [Color_Code],
             CASE WHEN @New_Application_Id IS NOT NULL THEN 'N' ELSE [Global_Indicator] END,
             [Build_AutoPromote_Indicator],
             [AllowLocalChanges_Indicator]
		 FROM [Workflows] WHERE [Workflow_Id] = @Old_Workflow_Id

		SET @New_Workflow_Id = SCOPE_IDENTITY()

        IF (@New_Application_Id IS NOT NULL) 
        BEGIN
            INSERT INTO [ApplicationWorkflows] ([Application_Id], [Workflow_Id])
            VALUES (@New_Application_Id, @New_Workflow_Id)
        END

		UPDATE @Workflows SET [New_Workflow_Id] = @New_Workflow_Id
		 WHERE [Old_Workflow_Id] = @Old_Workflow_Id

		DECLARE @Build_DeploymentPlan_Id INT
		    SET @Build_DeploymentPlan_Id = (SELECT [Build_DeploymentPlan_Id]
		                                      FROM [Workflows] WHERE [Workflow_Id] = @New_Workflow_Id
		                                       AND @CloneDeploymentPlans_Indicator = 'Y')
		IF @Build_DeploymentPlan_Id IS NOT NULL BEGIN

			SET @DeploymentPlanCloneData_Xml = (
				SELECT [@LinkAllActionGroups_Indicator] = @LinkAllActionGroups_Indicator,
				       (SELECT [New_DeploymentPlan_Name] = CASE
					             WHEN DP.[DeploymentPlan_Name] IS NOT NULL
								 THEN W.[Workflow_Name] + ': Build' ELSE NULL END,
				               [DeploymentPlan_Id]
				          FROM [Workflows] W 
						       INNER JOIN [DeploymentPlans] DP
						               ON W.[Build_DeploymentPlan_Id] = DP.[DeploymentPlan_Id]
						 WHERE [Workflow_Id] = @Old_Workflow_Id
				        FOR XML PATH ('DeploymentPlan'), TYPE)
				FOR XML PATH ('DeploymentPlanCloneData'))

			EXEC [Plans_CloneDeploymentPlans]
			  @DeploymentPlanCloneData_Xml = @DeploymentPlanCloneData_Xml,
			  @NewDeploymentPlanIdsCsv = @NewDeploymentPlanIdsCsv OUT

			UPDATE [Workflows] SET [Build_DeploymentPlan_Id] = CAST(@NewDeploymentPlanIdsCsv AS INT)
			 WHERE [Workflow_Id] = @New_Workflow_Id
		END
	END

	INSERT INTO [WorkflowSteps] ([Workflow_Id],[Environment_Id],[Step_Sequence],[DeploymentPlan_Id],[AutoPromote_Indicator])
    SELECT W.[New_Workflow_Id], WS.[Environment_Id], WS.[Step_Sequence], WS.[DeploymentPlan_Id], WS.[AutoPromote_Indicator]
	  FROM [WorkflowSteps] WS INNER JOIN @Workflows W ON WS.[Workflow_Id] = W.[Old_Workflow_Id]

	IF @CloneDeploymentPlans_Indicator = 'Y' OR @LinkAllActionGroups_Indicator = 'Y' BEGIN
		SET @DeploymentPlanCloneData_Xml = NULL
        
		DECLARE @DeploymentPlans TABLE ([New_Workflow_Id] INT, [Step_Sequence] INT, [Old_DeploymentPlan_Id] INT, Seq INT IDENTITY(1,1))
        
        INSERT INTO @DeploymentPlans ([New_Workflow_Id], [Step_Sequence], [Old_DeploymentPlan_Id])
		SELECT WS.[Workflow_Id], WS.[Step_Sequence], WS.[DeploymentPlan_Id]
		  FROM [WorkflowSteps] WS INNER JOIN @Workflows W ON WS.[Workflow_Id] = W.[New_Workflow_Id]

		SET @DeploymentPlanCloneData_Xml = (
			SELECT [@LinkAllActionGroups_Indicator] = @LinkAllActionGroups_Indicator,
				    (SELECT [New_DeploymentPlan_Name] = CASE
					            WHEN DP.[DeploymentPlan_Name] IS NOT NULL
								THEN W.[Workflow_Name] + ': ' + E.[Environment_Name] ELSE NULL END,
				            [DeploymentPlan_Id] = DPX.[Old_DeploymentPlan_Id]
				        FROM @DeploymentPlans DPX
						    INNER JOIN [DeploymentPlans] DP
						            ON DPX.[Old_DeploymentPlan_Id] = DP.[DeploymentPlan_Id]
						    INNER JOIN [WorkflowSteps] WS
							        ON DPX.[New_Workflow_Id] = WS.[Workflow_Id]
								   AND DPX.[Step_Sequence] = WS.[Step_Sequence]
						    INNER JOIN [Workflows] W
							        ON DPX.[New_Workflow_Id] = W.[Workflow_Id]
						    INNER JOIN [Environments] E
							        ON WS.[Environment_Id] = E.[Environment_Id]
						ORDER BY [Seq]
				    FOR XML PATH ('DeploymentPlan'), TYPE)
			FOR XML PATH ('DeploymentPlanCloneData'))

        SET @NewDeploymentPlanIdsCsv = NULL

		EXEC [Plans_CloneDeploymentPlans]
			@DeploymentPlanCloneData_Xml = @DeploymentPlanCloneData_Xml,
			@NewDeploymentPlanIdsCsv = @NewDeploymentPlanIdsCsv OUT

		UPDATE [WorkflowSteps]
		   SET [DeploymentPlan_Id] = CAST(NEW.[Value_Text] AS INT)
		  FROM [WorkflowSteps] WS
		       INNER JOIN @DeploymentPlans DP
			           ON WS.[Workflow_Id] = DP.[New_Workflow_Id]
					  AND WS.[Step_Sequence] = DP.[Step_Sequence]
			   INNER JOIN dbo.CsvToTable(@NewDeploymentPlanIdsCsv,',') NEW
			           ON NEW.[Value_Sequence] = DP.[Seq]

	END

	INSERT INTO [BuildPromotionRequirements]
        ([Workflow_Id],[Environment_Id],[Principal_Name],[RequiredInGroup_Count],[Requirement_Description],[Requirement_Configuration],
		 [CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date])
    SELECT 
         [New_Workflow_Id],[Environment_Id],[Principal_Name],[RequiredInGroup_Count],[Requirement_Description],[Requirement_Configuration],
		 dbo.Session_GetUserName(), GETUTCDATE(),[ModifiedBy_User_Name],[ModifiedOn_Date]
	  FROM [BuildPromotionRequirements] R INNER JOIN @Workflows W ON R.[Workflow_Id] = W.[Old_Workflow_Id]

	INSERT INTO [EventListeners]
		([EventListener_Description],[EventListener_Configuration],[Owner_User_Name],
		[Active_Indicator],[WorkflowStep_Workflow_Id],[WorkflowStep_Environment_Id],[BuildStep_Workflow_Id])
	SELECT
		[EventListener_Description],
        [EventListener_Configuration],
        [Owner_User_Name],
		[Active_Indicator],
        CASE WHEN [BuildStep_Workflow_Id] IS NULL THEN [New_Workflow_Id] ELSE NULL END,
        [WorkflowStep_Environment_Id],
        CASE WHEN [BuildStep_Workflow_Id] IS NULL THEN NULL ELSE [New_Workflow_Id] END
	  FROM [EventListeners] EL INNER JOIN @Workflows W 
	     ON EL.[BuildStep_Workflow_Id] = W.[Old_Workflow_Id]
		 OR EL.[WorkflowStep_Workflow_Id] = W.[Old_Workflow_Id]

	SELECT @NewWorkflowIdsCsv = ISNULL(@NewWorkflowIdsCsv,'') + CAST([New_Workflow_Id] AS VARCHAR(MAX)) + ','
	  FROM @Workflows ORDER BY [Seq]
	SET @NewWorkflowIdsCsv = SUBSTRING(@NewWorkflowIdsCsv, 1, LEN(@NewWorkflowIdsCsv)-1)

	COMMIT

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_CloneWorkflow] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_CreateOrUpdateWorkflow',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Workflow_Id',
    /* Description_Text        */ 'Adds a new workflow or updates an existing one.'
GO

IF OBJECT_ID('[Workflows_CreateOrUpdateWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_CreateOrUpdateWorkflow]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_CreateOrUpdateWorkflow]
(
	@Workflow_Id INT = NULL OUT,
	@Application_Id INT,
	@Workflow_Name NVARCHAR(50),
	@Active_Indicator YNINDICATOR = NULL,
    @AutoIncrementRelease_Indicator YNINDICATOR = NULL,
    @AutoDeployRelease_Indicator YNINDICATOR = NULL,
    @AllowOutOfSequencePromotions_Indicator YNINDICATOR = NULL,
    @AutoCancelReleases_Indicator YNINDICATOR = NULL,
	@Color_Code CHAR(6) = NULL,
	@Build_AutoPromote_Indicator YNINDICATOR = NULL,
	@AllowLocalChanges_Indicator YNINDICATOR = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF @Workflow_Id IS NULL
	BEGIN
		SET @Active_Indicator = COALESCE(@Active_Indicator, 'Y')

		/* Create new workflow */
		INSERT INTO [Workflows]
			(
				[Workflow_Name], 
				[Active_Indicator],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date],
                [AutoIncrementRelease_Indicator],
				[AutoDeployRelease_Indicator],
                [AllowOutOfSequencePromotions_Indicator],
                [AutoCancelReleases_Indicator],
				[Color_Code],
				[Global_Indicator],
				[Build_AutoPromote_Indicator],
				[AllowLocalChanges_Indicator]
			)
		VALUES
			(
				@Workflow_Name, 
				@Active_Indicator,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE(),
                @AutoIncrementRelease_Indicator,
				@AutoDeployRelease_Indicator,
                @AllowOutOfSequencePromotions_Indicator,
                @AutoCancelReleases_Indicator,
				@Color_Code,
				'N',
				COALESCE(@Build_AutoPromote_Indicator, 'N'),
				COALESCE(@AllowLocalChanges_Indicator, 'N')
			)

		SET @Workflow_Id = SCOPE_IDENTITY()

		IF @Application_Id IS NOT NULL
			INSERT INTO [ApplicationWorkflows]
			(
				[Application_Id],
				[Workflow_Id]
			)
			VALUES
			(
				@Application_Id,
				@Workflow_Id
			)

		EXEC [Events_RaiseEvent] 'WKFNEW',NULL, @Workflow_Id
	END
	ELSE
	BEGIN

		/* Update Workflow */
		UPDATE [Workflows]
		   SET [Workflow_Name] = @Workflow_Name,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE(),
			   [Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator]),
               [AutoIncrementRelease_Indicator] = @AutoIncrementRelease_Indicator,
               [AutoDeployRelease_Indicator] = @AutoDeployRelease_Indicator,
               [AllowOutOfSequencePromotions_Indicator] = @AllowOutOfSequencePromotions_Indicator,
               [AutoCancelReleases_Indicator] = @AutoCancelReleases_Indicator,
			   [Color_Code] = @Color_Code,
			   [Build_AutoPromote_Indicator] = COALESCE(@Build_AutoPromote_Indicator, 'N'),
			   [AllowLocalChanges_Indicator] = COALESCE(@AllowLocalChanges_Indicator, 'N')
		 WHERE [Workflow_Id] = @Workflow_Id

		UPDATE [Releases]
		   SET [Original_Workflow_Name] = @Workflow_Name
		 WHERE [Workflow_Id] = @Workflow_Id
		   AND [ReleaseStatus_Name] = 'Active'

		EXEC [Events_RaiseEvent] 'WKFMOD', NULL, @Workflow_Id
	END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_CreateOrUpdateWorkflow] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_DeleteApplicationWorkflow',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Disassociates a global workflow from an application.'
GO

IF OBJECT_ID('[Workflows_DeleteApplicationWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_DeleteApplicationWorkflow]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_DeleteApplicationWorkflow]
(
	@Application_Id INT,
    @Workflow_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

        DELETE [ApplicationWorkflows]
         WHERE [Application_Id] = @Application_Id
           AND [Workflow_Id] = @Workflow_Id

		EXEC [Events_RaiseEvent] 'WKFDEL',NULL, @Workflow_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_DeleteApplicationWorkflow] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_DeleteStep',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a workflow step.'
GO


IF OBJECT_ID('[Workflows_DeleteStep]') IS NOT NULL DROP PROCEDURE [Workflows_DeleteStep]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_DeleteStep]
(
	@Workflow_Id INT,
	@Step_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Environment_Id INT, @Application_Id INT
	    SET @Environment_Id = (SELECT [Environment_Id] 
	                             FROM [WorkflowSteps]
	                            WHERE [Workflow_Id] = @Workflow_Id
	                              AND [Step_Sequence] = @Step_Sequence)

	DECLARE @EventListeners_ToDelete TABLE ([Id] INT)

	INSERT @EventListeners_ToDelete ([Id])
	SELECT [EventListener_Id] 
	  FROM [EventListeners]
	 WHERE [WorkflowStep_Workflow_Id] = @Workflow_Id
	   AND [WorkflowStep_Environment_Id] = @Environment_Id

	DELETE [EventListenerApplicationGroups] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListenerApplications] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListenerEnvironments] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListenerEvents] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListeners] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)

	DELETE FROM [BuildPromotionRequirements]
		  WHERE [Workflow_Id] = @Workflow_Id
			AND [Environment_Id] = @Environment_Id
	   
	-- move the workflow step to the end of the list to appease the sequencing trigger before deletion
	DECLARE @Last_Step_Sequence INT
	SELECT @Last_Step_Sequence = MAX([Step_Sequence])
	  FROM [WorkflowSteps]
	 WHERE [Workflow_Id] = @Workflow_Id
	  
	UPDATE [WorkflowSteps] SET [Step_Sequence] = CASE
	  WHEN [Step_Sequence] = @Step_Sequence THEN @Last_Step_Sequence  -- Move Target Item
      WHEN (@Last_Step_Sequence > @Step_Sequence) AND [Step_Sequence] > @Step_Sequence AND [Step_Sequence] <= @Last_Step_Sequence THEN [Step_Sequence] - 1  --If "Move Up (@New_Step_Sequence > @Step_Sequence)"
      WHEN (@Last_Step_Sequence < @Step_Sequence) AND [Step_Sequence] < @Step_Sequence AND [Step_Sequence] >= @Last_Step_Sequence THEN [Step_Sequence] + 1  --If "Move Down (@New_Step_Sequence < @Step_Sequence)"
      ELSE [Step_Sequence] END
	 WHERE [Workflow_Id] = @Workflow_Id   
	
	DELETE FROM [WorkflowSteps]
		  WHERE [Workflow_Id] = @Workflow_Id
			AND [Step_Sequence] = @Last_Step_Sequence

    EXEC [Events_RaiseEvent] 'WKFDLS', NULL, @Workflow_Id, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_DeleteStep] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_DeleteWorkflow',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a workflow step.'
GO


IF OBJECT_ID('[Workflows_DeleteWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_DeleteWorkflow]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_DeleteWorkflow]
(
	@Workflow_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @EventListeners_ToDelete TABLE ([Id] INT)

	INSERT @EventListeners_ToDelete ([Id])
	SELECT [EventListener_Id] FROM [EventListeners]
	 WHERE [WorkflowStep_Workflow_Id] = @Workflow_Id
        OR [BuildStep_Workflow_Id] = @Workflow_Id

	DECLARE @DeploymentPlans_ToDelete TABLE ([Id] INT)

	 INSERT @DeploymentPlans_ToDelete ([Id])
	 SELECT [DeploymentPlan_Id] FROM [DeploymentPlans]
	  WHERE [DeploymentPlan_Id] IN (SELECT [DeploymentPlan_Id] FROM [WorkflowSteps] WHERE [Workflow_Id] = @Workflow_Id)
	    AND [DeploymentPlan_Id] NOT IN (SELECT [DeploymentPlan_Id] FROM [WorkflowSteps] WHERE [Workflow_Id] <> @Workflow_Id)
	    AND [DeploymentPlan_Name] IS NULL

	 INSERT @DeploymentPlans_ToDelete ([Id])
	 SELECT [DeploymentPlan_Id] FROM [DeploymentPlans]
	  WHERE [DeploymentPlan_Id] IN (SELECT [Build_DeploymentPlan_Id] FROM [Workflows] WHERE [Workflow_Id] = @Workflow_Id)
	    AND [DeploymentPlan_Id] NOT IN (SELECT [Build_DeploymentPlan_Id] FROM [Workflows] WHERE [Workflow_Id] <> @Workflow_Id)
		AND [DeploymentPlan_Name] IS NULL

	DELETE [EventListenerApplicationGroups] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListenerApplications] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListenerEnvironments] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListenerEvents] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)
	DELETE [EventListeners] WHERE [EventListener_Id] IN (SELECT [Id] FROM @EventListeners_ToDelete)

	DELETE [BuildPromotionRequirements] WHERE [Workflow_Id] = @Workflow_Id
	DELETE [ApplicationWorkflows] WHERE [Workflow_Id] = @Workflow_Id
	DELETE [WorkflowSteps] WHERE [Workflow_Id] = @Workflow_Id

	UPDATE [Releases]
	   SET [Workflow_Id] = NULL
	 WHERE [Workflow_Id] = @Workflow_Id

    UPDATE [BuildSchedules]
       SET [Releasefilter_Workflow_Id] = NULL
     WHERE [Releasefilter_Workflow_Id] = @Workflow_Id

    DELETE [RetentionPolicies]
     WHERE [PurgeIn_Workflow_Id] = @Workflow_Id

	DELETE [Workflows]
	 WHERE [Workflow_Id] = @Workflow_Id

	WHILE EXISTS(SELECT * FROM @DeploymentPlans_ToDelete) BEGIN
	    DECLARE @DeploymentPlan_Id INT
		    SET @DeploymentPlan_Id = (SELECT TOP 1 [Id] FROM @DeploymentPlans_ToDelete)
		
		EXEC [Plans_DeleteDeploymentPlan] @DeploymentPlan_Id = @DeploymentPlan_Id
		DELETE @DeploymentPlans_ToDelete WHERE [Id] = @DeploymentPlan_Id
	END

    EXEC [Events_RaiseEvent] 'WKFDEL', NULL, @Workflow_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_DeleteWorkflow] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_GetGlobalWorkflows',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Workflows_Extended,WorkflowSteps_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of global workflows and their steps.'
GO

IF OBJECT_ID('[Workflows_GetGlobalWorkflows]') IS NOT NULL DROP PROCEDURE [Workflows_GetGlobalWorkflows]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_GetGlobalWorkflows]
(
	@IncludeInactive_Indicator YNINDICATOR = 'N'
)
AS
BEGIN
	SET NOCOUNT ON

	SET @IncludeInactive_Indicator = ISNULL(@IncludeInactive_Indicator, 'N')

	UPDATE [Workflows]
	   SET [Global_Indicator] = 'Y'
	 WHERE (SELECT COUNT(*) FROM [ApplicationWorkflows] _AW WHERE _AW.[Workflow_Id]  = [Workflows].[Workflow_Id]) <> 1
	   AND [Global_Indicator] = 'N'

	DECLARE @WorkflowIds TABLE ([Workflow_Id] INT)
	INSERT @WorkflowIds
	SELECT W.[Workflow_Id]
	  FROM [Workflows_Extended] W
	 WHERE (@IncludeInactive_Indicator = 'Y' OR [Active_Indicator] = 'Y')
	   AND [Global_Indicator] = 'Y'


	SELECT * FROM [Workflows_Extended]
	 WHERE [Workflow_Id] IN (SELECT [Workflow_Id] FROM @WorkflowIds)
	ORDER BY CASE [Active_Indicator] WHEN 'Y' THEN 1 ELSE 2 END
				,[Workflow_Name]

	SELECT * FROM [WorkflowSteps_Extended]
	 WHERE [Workflow_Id] IN (SELECT [Workflow_Id] FROM @WorkflowIds)

END
GO

GRANT EXECUTE ON [Workflows_GetGlobalWorkflows] TO [BuildMasterUser_Role]

GO
IF OBJECT_ID('[Workflows_GetPromotionApprovalRequirements]') IS NOT NULL DROP PROCEDURE [Workflows_GetPromotionApprovalRequirements]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_GetWorkflow',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Workflows_Extended,WorkflowSteps_Extended,BuildPromotionRequirements_Extended,EventListeners_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a specific workflow and all of its workflow steps.'
GO


IF OBJECT_ID('[Workflows_GetWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_GetWorkflow]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_GetWorkflow]
(
	@Workflow_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT *
	  FROM [Workflows_Extended]
     WHERE [Workflow_Id] = @Workflow_Id

	SELECT *
	  FROM [WorkflowSteps_Extended]
     WHERE [Workflow_Id] = @Workflow_Id
	ORDER BY [Step_Sequence]

	SELECT *
	  FROM [BuildPromotionRequirements_Extended]
     WHERE [Workflow_Id] IS NULL OR [Workflow_Id] = @Workflow_Id
     ORDER BY CASE WHEN [Workflow_Id] IS NULL THEN 1 ELSE 0 END

	SELECT * 
	  FROM [EventListeners_Extended]
	 WHERE [WorkflowStep_Workflow_Id] = @Workflow_Id
        OR [BuildStep_Workflow_Id] = @Workflow_Id

END
GO

GRANT EXECUTE ON [Workflows_GetWorkflow] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_GetWorkflows',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Workflows_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all workflows, optionally filtered by application.'
GO


IF OBJECT_ID('[Workflows_GetWorkflows]') IS NOT NULL DROP PROCEDURE [Workflows_GetWorkflows]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_GetWorkflows]
(
	@Application_Id INT = NULL,
	@IncludeGlobal_Indicator YNINDICATOR = 'N',
	@IncludeInactive_Indicator YNINDICATOR = 'N'
)
AS
BEGIN
	SET NOCOUNT ON

	SET @IncludeGlobal_Indicator = ISNULL(@IncludeGlobal_Indicator, 'N')
	SET @IncludeInactive_Indicator = ISNULL(@IncludeInactive_Indicator, 'N')

	SELECT W.*
	  FROM [Workflows_Extended] W
	 WHERE (@IncludeInactive_Indicator = 'Y' OR [Active_Indicator] = 'Y')
	   AND (@Application_Id IS NOT NULL OR @IncludeGlobal_Indicator = 'Y' OR [Global_Indicator] = 'N')
	   AND (@Application_Id IS NULL OR @Application_Id IN (
	             SELECT [Application_Id] FROM [ApplicationWorkflows] _AW
				  WHERE _AW.[Workflow_Id] = W.[Workflow_Id]))
	ORDER BY CASE [Workflow_Name] WHEN 'Standard' THEN 1 ELSE 2 END
		        ,CASE [Active_Indicator] WHEN 'Y' THEN 1 ELSE 2 END
				,[Workflow_Name]

END
GO

GRANT EXECUTE ON [Workflows_GetWorkflows] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_GetWorkflowStep',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'WorkflowSteps_Extended,BuildPromotionRequirements_Extended,EventListeners_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified workflow step.'
GO


IF OBJECT_ID('[Workflows_GetWorkflowStep]') IS NOT NULL DROP PROCEDURE [Workflows_GetWorkflowStep]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_GetWorkflowStep]
(
	@Workflow_Id INT,
	@Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [WorkflowSteps_Extended]
     WHERE [Workflow_Id] = @Workflow_Id
	   AND [Environment_Id] = @Environment_Id

	SELECT *
	  FROM [BuildPromotionRequirements_Extended]
	 WHERE ([Workflow_Id] IS NULL OR [Workflow_Id] = @Workflow_Id)
	   AND [Environment_Id] = @Environment_Id
     ORDER BY CASE WHEN [Workflow_Id] IS NULL THEN 1 ELSE 0 END

	SELECT *
	  FROM [EventListeners_Extended]
	 WHERE [WorkflowStep_Workflow_Id] = @Workflow_Id
	   AND [WorkflowStep_Environment_Id] = @Environment_Id

END
GO

GRANT EXECUTE ON [Workflows_GetWorkflowStep] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_MakeGlobal',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves a workflow step to a new sequence.'
GO


IF OBJECT_ID('[Workflows_MakeGlobal]') IS NOT NULL DROP PROCEDURE [Workflows_MakeGlobal]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_MakeGlobal]
(
	@Workflow_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT * FROM [Workflows] WHERE [Workflow_Id] = @Workflow_Id AND [Global_Indicator] = 'Y') RETURN

	UPDATE [Workflows]
	   SET [Global_Indicator] = 'Y'
	 WHERE [Workflow_Id] = @Workflow_Id

	 UPDATE [DeploymentPlans]
	    SET [DeploymentPlan_Name] = W.[Workflow_Name] + ': ' + E.[Environment_Name]
	   FROM [DeploymentPlans] DP
	        INNER JOIN [WorkflowSteps] WS ON WS.[DeploymentPlan_Id] = DP.[DeploymentPlan_Id]
			INNER JOIN [Environments] E ON WS.[Environment_Id] = E.[Environment_Id]
			INNER JOIN [Workflows] W ON W.[Workflow_Id] = WS.[Workflow_Id]
	  WHERE W.[Workflow_Id] = @Workflow_Id
	    AND DP.[DeploymentPlan_Name] IS NULL

	 UPDATE [DeploymentPlans]
	    SET [DeploymentPlan_Name] = W.[Workflow_Name] + ': Build'
	   FROM [DeploymentPlans] DP
			INNER JOIN [Workflows] W ON W.[Build_DeploymentPlan_Id] = DP.[DeploymentPlan_Id]
	  WHERE W.[Workflow_Id] = @Workflow_Id
	    AND DP.[DeploymentPlan_Name] IS NULL

	EXEC [Events_RaiseEvent] 'WKFMOD', NULL, @Workflow_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Workflows_MakeGlobal] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_MoveApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves an approval up or down in sequence.'
GO

IF OBJECT_ID('[Workflows_MoveApproval]') IS NOT NULL
	DROP PROCEDURE [Workflows_MoveApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_MoveApproval]
(
	@Workflow_Id INT,
	@Environment_Id INT,
	@Approval_Sequence INT,
	@Direction_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF @Direction_Code NOT IN ('U', 'D') RETURN

	DECLARE @Old_Sequence INT,
			@New_Sequence INT

	/* Figure out the old and new step sequences */
	SET @Old_Sequence = @Approval_Sequence
	IF @Direction_Code = 'U'
	BEGIN
		/* Moving up */
		SELECT @New_Sequence = COALESCE(MAX([Approval_Sequence]), 1)
		  FROM [PromotionApprovalRequirements]
		 WHERE [Workflow_Id] = @Workflow_Id
		   AND [Environment_Id] = @Environment_Id
		   AND [Approval_Sequence] < @Approval_Sequence
	END
	ELSE
	BEGIN
		/* Moving down */
		SELECT @New_Sequence = COALESCE(MIN([Approval_Sequence]), @Approval_Sequence)
		  FROM [PromotionApprovalRequirements]
		 WHERE [Workflow_Id] = @Workflow_Id
		   AND [Environment_Id] = @Environment_Id
		   AND [Approval_Sequence] > @Approval_Sequence
	END

	IF @Old_Sequence = @New_Sequence ROLLBACK RETURN

	/* Swap sequences */
	UPDATE [PromotionApprovalRequirements]
	   SET [Approval_Sequence] = CASE [Approval_Sequence]
								 WHEN @Old_Sequence THEN @New_Sequence
								 WHEN @New_Sequence THEN @Old_Sequence
								 ELSE [Approval_Sequence] END
	 WHERE [Workflow_Id] = @Workflow_Id
	   AND [Environment_Id] = @Environment_Id


	EXEC [Events_RaiseEvent] 'WKFMDS', NULL, @Workflow_Id, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_MoveApproval] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_MoveWorkflowStep',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves a workflow step to a new sequence.'
GO


IF OBJECT_ID('[Workflows_MoveWorkflowStep]') IS NOT NULL DROP PROCEDURE [Workflows_MoveWorkflowStep]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_MoveWorkflowStep]
(
	@Workflow_Id INT,
	@Step_Sequence INT,
	@New_Step_Sequence INT
)
AS
BEGIN

    DECLARE @Environment_Id INT, @Application_Id INT
	    SET @Environment_Id = (SELECT [Environment_Id] 
	                             FROM [WorkflowSteps]
	                            WHERE [Workflow_Id] = @Workflow_Id
	                              AND [Step_Sequence] = @Step_Sequence)

	IF NOT EXISTS(SELECT * FROM [WorkflowSteps] WHERE [Workflow_Id] = @Workflow_Id AND [Step_Sequence] = @New_Step_Sequence)
	BEGIN
		/* Silently Return or throw an error*/
		Return;
	END
	
	IF EXISTS(SELECT *
			  FROM [Builds] B
				   INNER JOIN [Releases] R
				         ON B.[Application_Id] = @Application_Id
						AND B.[Release_Number] = R.[Release_Number]
			  WHERE R.[Workflow_Id] = @Workflow_Id AND [BuildStatus_Name] = 'Active')
	BEGIN
		/* Silently Return or throw an error*/
		Return;
	END

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [WorkflowSteps] SET [Step_Sequence] = CASE
	  WHEN [Step_Sequence] = @Step_Sequence THEN @New_Step_Sequence  -- Move Target Item
      WHEN (@New_Step_Sequence > @Step_Sequence) AND [Step_Sequence] > @Step_Sequence AND [Step_Sequence] <= @New_Step_Sequence THEN [Step_Sequence] - 1  --If "Move Up (@New_Step_Sequence > @Step_Sequence)"
      WHEN (@New_Step_Sequence < @Step_Sequence) AND [Step_Sequence] < @Step_Sequence AND [Step_Sequence] >= @New_Step_Sequence THEN [Step_Sequence] + 1  --If "Move Down (@New_Step_Sequence < @Step_Sequence)"
      ELSE [Step_Sequence] END
	 WHERE [Workflow_Id] = @Workflow_Id


	--EXEC [Events_RaiseEvent] 'WKFMDS', @Workflow_Id, @Environment_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Workflows_MoveWorkflowStep] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_SetBuildStep',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the build deployment plan or build importer for a workflow.'
GO

IF OBJECT_ID('[Workflows_SetBuildStep]') IS NOT NULL DROP PROCEDURE [Workflows_SetBuildStep]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_SetBuildStep]
(
	@Workflow_Id INT,
	@Build_DeploymentPlan_Id INT,
	@BuildImporterTemplate_Configuration XML
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	/* Update Workflow */
	UPDATE [Workflows]
		SET [Build_DeploymentPlan_Id] = @Build_DeploymentPlan_Id,
			[ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			[ModifiedOn_Date] = GETUTCDATE(),
            [BuildImporterTemplate_Configuration] = @BuildImporterTemplate_Configuration
		WHERE [Workflow_Id] = @Workflow_Id

	EXEC [Events_RaiseEvent] 'WKFMOD', NULL, @Workflow_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_SetBuildStep] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_CloneApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'New_Application_Id',
    /* Description_Text        */ 'Clones an application, but does not copy any history'
GO

IF OBJECT_ID('[Applications_CloneApplication]') IS NOT NULL
	DROP PROCEDURE [Applications_CloneApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_CloneApplication]
(
	@Application_Id INT,
	@LinkAllActionGroups_Indicator YNINDICATOR = 'N',
	@LinkAllDeploymentPlans_Indicator YNINDICATOR = 'N',
	@New_Application_Name VARCHAR(50),
	@New_Application_Id INT = NULL OUT
) AS
BEGIN


SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [Applications] WHERE [Application_Id] = @Application_Id)
		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)

	IF @LinkAllActionGroups_Indicator = 'Y' AND @LinkAllDeploymentPlans_Indicator = 'Y'
		RAISERROR('Cannot link action groups if deployment plans are linked.', 16, 1)

	DECLARE @Application_Name VARCHAR(50)
	SET @Application_Name = (SELECT [Application_Name] FROM [Applications] WHERE [Application_Id] = @Application_Id)

	--==[Applications]==--
	INSERT INTO [Applications] ([Application_Name], [ReleaseNumber_Scheme_Name],[BuildNumber_Scheme_Name],[IssueTracking_Provider_Id],[AllowMultipleActiveReleases_Indicator],[AllowMultipleActiveBuilds_Indicator],[ApplicationGroup_Id],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[VariableSupport_Code])
	SELECT @New_Application_Name, [ReleaseNumber_Scheme_Name],[BuildNumber_Scheme_Name],[IssueTracking_Provider_Id],[AllowMultipleActiveReleases_Indicator],[AllowMultipleActiveBuilds_Indicator],[ApplicationGroup_Id],dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),[VariableSupport_Code]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id
	SET @New_Application_Id = SCOPE_IDENTITY()

	--==[Deployables]==--
	INSERT INTO [Deployables] ([Application_Id],[Deployable_Name],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[DeployableType_Code])
	SELECT @New_Application_Id,[Deployable_Name],dbo.Session_GetUserName(),GETUTCDATE(),[ModifiedBy_User_Name],[ModifiedOn_Date],[DeployableType_Code]
	 FROM [Deployables]
	WHERE [Application_Id] = @Application_Id
	
	--==[DeployableDependencies]==--
	INSERT INTO [DeployableDependencies] ([Deployable_Id],[DependsOn_Deployable_Id])
	SELECT D_NEW.[Deployable_Id],D2_NEW.[Deployable_Id]
	  FROM [DeployableDependencies] DD
	       INNER JOIN [Deployables] D
	               ON D.[Deployable_Id] = DD.[Deployable_Id]
	       INNER JOIN [Deployables] D2
	               ON D2.[Deployable_Id] = DD.[DependsOn_Deployable_Id]
	       INNER JOIN [Deployables] D_NEW
	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
	              AND D_NEW.[Application_Id] = @New_Application_Id
	       INNER JOIN [Deployables] D2_NEW
	               ON D2_NEW.[Deployable_Name] = D2.[Deployable_Name]
	              AND D2_NEW.[Application_Id] = @New_Application_Id
	 WHERE D.[Application_Id] = @Application_Id

	--==[DeployablesImported]==--
	INSERT INTO [DeployablesImported]([Deployable_Id],[Application_Id])
	SELECT [Deployable_Id], @New_Application_Id
	  FROM [DeployablesImported]
	 WHERE [Application_Id] = @Application_Id


     --==[Workflows]==--

     DECLARE @AppWorkflows TABLE ([Workflow_Id] INT NOT NULL, [Workflow_Name] VARCHAR(MAX))
      INSERT @AppWorkflows ([Workflow_Id], [Workflow_Name]) 
      SELECT W.[Workflow_Id], W.[Workflow_Name] 
        FROM [ApplicationWorkflows] AW
  INNER JOIN [Workflows] W ON W.[Workflow_Id] = AW.[Workflow_Id]
       WHERE AW.[Application_Id] = @Application_Id
         AND W.[Global_Indicator] = 'N'

     DECLARE @NewWorkflowIdsCsv VARCHAR(MAX)
     DECLARE @CloneDeploymentPlans_Indicator YNINDICATOR
     DECLARE @WorkflowCloneData_Xml VARCHAR(MAX)
     DECLARE @Workflow_Id INT
     DECLARE @Workflow_Name NVARCHAR(50)
     
     SET @CloneDeploymentPlans_Indicator = CASE WHEN @LinkAllDeploymentPlans_Indicator = 'Y' THEN 'N' ELSE 'Y' END
     
     SET @WorkflowCloneData_Xml = '<WorkflowCloneData CloneDeploymentPlans_Indicator="' + @CloneDeploymentPlans_Indicator + '" LinkAllActionGroups_Indicator="' + @LinkAllActionGroups_Indicator + '">'

     WHILE EXISTS(SELECT * FROM @AppWorkflows)
     BEGIN

        SELECT TOP 1 @Workflow_Id = [Workflow_Id], @Workflow_Name = [Workflow_Name] FROM @AppWorkflows
     
         SET @WorkflowCloneData_Xml = @WorkflowCloneData_Xml + 
            '<Workflow>
                <New_Workflow_Name>' + @Workflow_Name + '</New_Workflow_Name>
                <Workflow_Id>' + CAST(@Workflow_Id AS VARCHAR(MAX)) + '</Workflow_Id>
                <New_Application_Id>' + CAST(@New_Application_Id AS VARCHAR(MAX)) + '</New_Application_Id>
            </Workflow>'

        DELETE @AppWorkflows WHERE [Workflow_Id] = @Workflow_Id

     END
     
     SET @WorkflowCloneData_Xml =  @WorkflowCloneData_Xml + '</WorkflowCloneData>'
	  
     EXEC [Workflows_CloneWorkflow] @WorkflowCloneData_Xml, @NewWorkflowIdsCsv 
	
	/* Set-up Release */
	DECLARE @New_Workflow_Id INT
    DECLARE @New_Workflow_Name VARCHAR(MAX)
	   SELECT TOP 1 @New_Workflow_Id = W.[Workflow_Id], @New_Workflow_Name = [Workflow_Name] 
         FROM [ApplicationWorkflows] AW
   INNER JOIN [Workflows] W ON W.[Workflow_Id] = AW.[Workflow_Id]
        WHERE [Application_Id] = @New_Application_Id
	
	DECLARE @New_Release_Number RELEASENUMBER
	SELECT @New_Release_Number = CASE [ReleaseNumber_Scheme_Name]
	         WHEN 'MajorMinorRevision' THEN '0.0.0'
	         WHEN 'MajorMinor' THEN '0.0'
	         WHEN 'DateBased' THEN '0' END
	  FROM [Applications]
	 WHERE [Application_Id] = @New_Application_Id
	 
	IF @New_Workflow_Id IS NOT NULL
		INSERT INTO [Releases]([Application_Id],[Release_Number],[Workflow_Id],[ReleaseStatus_Name],[Notes_Text],[Target_Date],[Release_Name],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[CancelledBy_User_Name],[CancelledReason_Text],[CancelledOn_Date],[Release_Sequence],[Original_Workflow_Name])
		VALUES (@New_Application_Id,@New_Release_Number,@New_Workflow_Id,'Active',NULL,NULL,'Setup',dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),NULL,NULL,NULL, 1, @New_Workflow_Name)

    IF @New_Workflow_Id IS NOT NULL
        INSERT [ReleaseDeployables] ([Application_Id], [Release_Number], [Deployable_Id], [InclusionType_Code])
        SELECT @New_Application_Id, @New_Release_Number, [Deployable_Id], 'I' FROM [Deployables] WHERE [Application_Id] = @New_Application_Id
    

	--==[VariableDeclarations]==--
	PRINT 'VariableDeclarations...'
    INSERT INTO [VariableDeclarations] 
    (
		[Scope_Code],
		[Variable_Name],
		[Scoped_Application_Id],
		[Scoped_Environment_Id],
		[Required_Indicator],
		[Variable_Configuration],
		[DefaultValue_Text],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[Sensitive_Indicator]
	)
    SELECT [Scope_Code],
		   [Variable_Name],
		   @New_Application_Id,
		   [Scoped_Environment_Id],
		   [Required_Indicator],
		   [Variable_Configuration],
		   [DefaultValue_Text],
		   dbo.Session_GetUserName(),
		   GETUTCDATE(),
		   [Sensitive_Indicator]
     FROM [VariableDeclarations] 
	WHERE [Scoped_Application_Id] = @Application_Id

	--==[BuildSchedules]==--
	PRINT 'BuildSchedules...'
	INSERT INTO [BuildSchedules] ([Schedule_Name],[Application_Id],[Active_Indicator],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[Schedule_Type],[StartTime_Minutes],[EndTime_Minutes],[DailyInterval_Minutes],[Recurrence_Type],[RecurrenceFrequency_Value],[RecurrenceWeeklyDay_Mask],[RecurrenceMonthlyDay_Value],[RecurrenceMonthlyRelativity_Type],[RecurrenceMonthlyRelativityDay_Type],[RecurrenceMonthlyRelativityWeek_Type],[RecurrenceMonthlyRelativityWeekDay_Value],[Url_AllowedIP_CSV],[Url_Username_Text],[Url_Password_Text],[SourceControl_Path_Name],[SourceControl_Provider_Id],[SourceControl_QuietPeriod_Minutes],[ReleaseFilter_ReleaseType_Code])
	SELECT [Schedule_Name],@New_Application_Id,[Active_Indicator],dbo.Session_GetUserName(),GETUTCDATE(),[ModifiedBy_User_Name],[ModifiedOn_Date],[Schedule_Type],[StartTime_Minutes],[EndTime_Minutes],[DailyInterval_Minutes],[Recurrence_Type],[RecurrenceFrequency_Value],[RecurrenceWeeklyDay_Mask],[RecurrenceMonthlyDay_Value],[RecurrenceMonthlyRelativity_Type],[RecurrenceMonthlyRelativityDay_Type],[RecurrenceMonthlyRelativityWeek_Type],[RecurrenceMonthlyRelativityWeekDay_Value],[Url_AllowedIP_CSV],[Url_Username_Text],[Url_Password_Text],[SourceControl_Path_Name],[SourceControl_Provider_Id],[SourceControl_QuietPeriod_Minutes],[ReleaseFilter_ReleaseType_Code]
	  FROM [BuildSchedules]
	 WHERE [Application_Id] = @Application_Id
      
   --==[BuildScheduleVariableDefinitions]==--
   PRINT 'BuildScheduleVariableDefinitions...'
	INSERT INTO [BuildScheduleVariableDefinitions]([Schedule_Id],[VariableDeclaration_Id],[Value_Text])
	 SELECT BS_NEW.[Schedule_Id], VD_NEW.[VariableDeclaration_Id],BSVD.[Value_Text]
	   FROM [BuildSchedules] BS
	        INNER JOIN [BuildScheduleVariableDefinitions] BSVD
	                ON BSVD.[Schedule_Id] = BS.[Schedule_Id]
	        INNER JOIN [VariableDeclarations] VD
	                ON BSVD.[VariableDeclaration_Id] = VD.[VariableDeclaration_Id]
	        INNER JOIN [BuildSchedules] BS_NEW
	                ON BS_NEW.[Schedule_Name] = BS.[Schedule_Name]
	               AND BS_NEW.[Application_Id] = @New_Application_Id
            INNER JOIN [VariableDeclarations] VD_NEW
                    ON VD_NEW.[Scoped_Application_Id] = VD.[Scoped_Application_Id]
                   AND VD_NEW.[Scope_Code] = VD.[Scope_Code]
                   AND VD_NEW.[Variable_Name] = VD.[Variable_Name]
	  WHERE BS.[Application_Id] = @Application_Id

	--==[Privileges]==--
	INSERT INTO [Privileges]([Principal_Name],[PrincipalType_Code],[Role_Id],[Application_Id],[Environment_Id],[PrivilegeType_Code],[Directory_Provider_Id],[ApplicationGroup_Id])
	SELECT [Principal_Name],[PrincipalType_Code],[Role_Id],@New_Application_Id,[Environment_Id],[PrivilegeType_Code],[Directory_Provider_Id],[ApplicationGroup_Id]
	  FROM [Privileges]
	 WHERE [Application_Id] = @Application_Id

	
	INSERT INTO [ConfigurationFiles]([Deployable_Id],[FilePath_Text],[Active_Indicator],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date])
	SELECT D_NEW.[Deployable_Id],[FilePath_Text],[Active_Indicator],dbo.Session_GetUserName(),GETUTCDATE(),CF.[ModifiedBy_User_Name],CF.[ModifiedOn_Date]
	  FROM [ConfigurationFiles] CF
	       INNER JOIN [Deployables] D
			       ON D.[Deployable_Id] = CF.[Deployable_Id]
	       INNER JOIN [Deployables] D_NEW
	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
	              AND D_NEW.[Application_Id] = @New_Application_Id
	 WHERE CF.[Active_Indicator] = 'Y'
	   AND D.[Application_Id] = @Application_Id
	   AND @New_Release_Number IS NOT NULL

	INSERT INTO [ConfigurationFileInstances]([ConfigurationFile_Id],[Instance_Name],[Active_Indicator],[Environment_Id],[Template_Indicator],[Template_Instance_Name],[TransformType_Code])
	SELECT CF_NEW.[ConfigurationFile_Id],[Instance_Name],'Y',[Environment_Id],[Template_Indicator],[Template_Instance_Name],[TransformType_Code]
	  FROM [ConfigurationFileInstances] CFI
	       INNER JOIN [ConfigurationFiles] CF
	               ON CF.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
	       INNER JOIN [Deployables] D
			       ON D.[Deployable_Id] = CF.[Deployable_Id]
	       INNER JOIN [Deployables] D_NEW
	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
	              AND D_NEW.[Application_Id] = @New_Application_Id			       
	       INNER JOIN [ConfigurationFiles] CF_NEW
	               ON CF_NEW.[FilePath_Text] = CF.[FilePath_Text]
	              AND CF_NEW.[Deployable_Id] = D_NEW.[Deployable_Id]
	 WHERE CFI.[Active_Indicator] = 'Y'
	   AND D.[Application_Id] = @Application_Id


	INSERT INTO [ConfigurationFileVersions]
	(
		[ConfigurationFile_Id],
		[Instance_Name],
		[Version_Number],
		[VersionNotes_Text],
		[File_Date],
		[File_Bytes],
		[CreatedBy_User_Name],
		[CreatedOn_Date]
	)
	SELECT CF_NEW.[ConfigurationFile_Id],
	       CFV.[Instance_Name],
		   1,
		   CASE WHEN [File_Bytes] IS NULL THEN NULL ELSE 'Cloned from ' + @Application_Name END,
		   CASE WHEN [File_Bytes] IS NULL THEN NULL ELSE GETUTCDATE() END,
		   [File_Bytes],
		   CASE WHEN [File_Bytes] IS NULL THEN NULL ELSE dbo.Session_GetUserName() END,
		   CASE WHEN [File_Bytes] IS NULL THEN NULL ELSE GETUTCDATE() END
	  FROM [ConfigurationFileVersions_Extended] CFV
		   INNER JOIN (SELECT [ConfigurationFile_Id],[Instance_Name],[Max_Version_Number] = MAX([Version_Number])
						 FROM [ConfigurationFileVersions_Extended]
						GROUP BY [ConfigurationFile_Id],[Instance_Name]) CFVMAX
				   ON CFVMAX.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
				  AND CFVMAX.[Instance_Name] = CFV.[Instance_Name]
				  AND CFVMAX.[Max_Version_Number] = CFV.[Version_Number]
			   INNER JOIN [ConfigurationFileInstances] CFI
					   ON CFI.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
					  AND CFI.[Instance_Name] = CFV.[Instance_Name]
			   INNER JOIN [ConfigurationFiles] CF
					   ON CF.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
			   INNER JOIN [Deployables] D
					   ON D.[Deployable_Id] = CF.[Deployable_Id]
			   INNER JOIN [Deployables] D_NEW
					   ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
					  AND D_NEW.[Application_Id] = @New_Application_Id			       
			   INNER JOIN [ConfigurationFiles] CF_NEW
					   ON CF_NEW.[FilePath_Text] = CF.[FilePath_Text]
					  AND CF_NEW.[Deployable_Id] = D_NEW.[Deployable_Id]
			   INNER JOIN [ConfigurationFileInstances] CFI_NEW
					   ON CFI_NEW.[ConfigurationFile_Id] = CF_NEW.[ConfigurationFile_Id]
					  AND CFI_NEW.[Instance_Name] = CFI.[Instance_Name]
	 WHERE D.[Application_Id] = @Application_Id
	
	/* Database Providers aren't cloned */
	UPDATE [ActionGroupActions]
	   SET [Active_Indicator] = 'N'
	  FROM [ActionGroupActions] AGA
	       INNER JOIN [ActionGroups] AG 
		           ON AG.[ActionGroup_Id] = AGA.[ActionGroup_Id]
	       INNER JOIN [DeploymentPlanActionGroups] DPAG
		           ON DPAG.[ActionGroup_Id] = AG.[ActionGroup_Id]
           INNER JOIN [ApplicationDeploymentPlans] ADP
		           ON ADP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
	 WHERE CAST([Action_Configuration].query('local-name(/*[1])') AS NVARCHAR(MAX)) LIKE 'Inedo.BuildMaster.Extensibility.Actions.Database%'		           
	   AND ADP.[Application_Id] = @New_Application_Id
	   AND @LinkAllActionGroups_Indicator = 'N'
	   AND @LinkAllDeploymentPlans_Indicator = 'N'


	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPNEW', @New_Application_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_CloneApplication] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_InitiateBuildExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Execution_Id',
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Builds_InitiateBuildExecution]') IS NOT NULL DROP PROCEDURE [Builds_InitiateBuildExecution]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_InitiateBuildExecution]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@Start_Date DATETIME = NULL,
	@ContinueFromLastFailed_Indicator YNINDICATOR = 'N',
    @Variables_Xml XML = NULL,
	@Execution_Id INT = NULL OUT
)
AS
BEGIN
	SET @Start_Date = COALESCE(@Start_Date, GETUTCDATE())

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @LatestExecution_Id INT
	SET @LatestExecution_Id = (SELECT MAX([Execution_Id])
	                             FROM [BuildExecutions]
	                            WHERE [Application_Id] = @Application_Id)

	IF @ContinueFromLastFailed_Indicator = 'Y'
	BEGIN

		IF @LatestExecution_Id IS NULL
			RAISERROR('Cannot reexecute: there are no previous executions.', 16, 1)

		IF @LatestExecution_Id <> (SELECT MAX([Execution_Id]) FROM [BuildExecutions] WHERE [Application_Id] = @Application_Id AND [Environment_Id] = @Environment_Id)
			RAISERROR('Cannot reexecute: last execution was associated with a different environment.', 16, 1)

		IF (SELECT [ReexecutionState_Xml] FROM [BuildExecutions] WHERE [Execution_Id] = @LatestExecution_Id) IS NULL
			RAISERROR('Cannot reexecute: a deployment plan was modified since the last execution or there was an unrecoverable error.', 16, 1)

	END

	INSERT INTO [BuildExecutions]
	(
		[Application_Id],
		[Release_Number],
		[Build_Number],
		[Environment_Id],
		[ExecutionStart_Date],
		[ExecutionStatus_Name],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[LastActionReexecutionOf_Execution_Id],
		[WarningLogEntry_Indicator]
	)
	VALUES
	(
		@Application_Id,
		@Release_Number,
		@Build_Number,
		@Environment_Id,
		@Start_Date,
		'Pending',
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		CASE WHEN @ContinueFromLastFailed_Indicator = 'Y' THEN @LatestExecution_Id ELSE NULL END,
		'N'
	)
	
	SET @Execution_Id = SCOPE_IDENTITY()
	
	IF @ContinueFromLastFailed_Indicator = 'N' AND @Variables_Xml IS NOT NULL BEGIN

		INSERT INTO [VariableDefinitions]
		(
			[Variable_Name],
			[Environment_Id],
			[Application_Id],
			[Release_Number],
			[Build_Number],
            [Execution_Id],
			[Value_Text],
			[Sensitive_Indicator]
		)
		SELECT [Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)'),
		       [Environment_Id] = @Environment_Id,
		       [Application_Id] = @Application_Id,
		       [Release_Number] = @Release_Number,
		       [Build_Number] = @Build_Number,
               [Execution_Id] = @Execution_Id,
			   [Value_Text] = LIST.[Variables].[value]('@Value', 'VARCHAR(MAX)'),
			   [Sensitive_Indicator] = COALESCE(VDEC.[Sensitive_Indicator], 'N')
		  FROM @Variables_Xml.[nodes]('/Variables/Variable') AS LIST([Variables])
		       LEFT JOIN [VariableDeclarations] VDEC
		              ON VDEC.[Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)')
		             AND VDEC.[Scoped_Application_Id] = @Application_Id
		             AND VDEC.[Scoped_Environment_Id] = @Environment_Id
		             AND VDEC.[Scope_Code] = 'X'

	END
	ELSE IF @ContinueFromLastFailed_Indicator = 'Y'
	BEGIN

		INSERT INTO [VariableDefinitions]
		(
			[Variable_Name],
			[Environment_Id],
			[Server_Id],
			[ApplicationGroup_Id],
			[Application_Id],
			[Deployable_Id],
			[Release_Number],
			[Build_Number],
            [Execution_Id],
			[Value_Text],
			[Sensitive_Indicator]
		)
		SELECT [Variable_Name],
		       [Environment_Id],
			   [Server_Id],
			   [ApplicationGroup_Id],
			   [Application_Id],
			   [Deployable_Id],
			   [Release_Number],
			   [Build_Number],
               [Execution_Id] = @Execution_Id,
			   [Value_Text],
			   [Sensitive_Indicator]
		  FROM [VariableDefinitions]
		 WHERE [Execution_Id] = @LatestExecution_Id

	END

	EXEC [Events_RaiseEvent] 'EXEINI', @Application_Id, @Execution_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Builds_InitiateBuildExecution] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_PromoteBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Promotes a build to its next environment in its workflow'
GO


IF OBJECT_ID('[Builds_PromoteBuild]') IS NOT NULL DROP PROCEDURE [Builds_PromoteBuild]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_PromoteBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Comments_Text VARCHAR(255) = NULL,
	@ExecutionStart_Date DATETIME = NULL,
	@ForcePromotion_Indicator YNINDICATOR = 'N',
	@DependencyContingent_Indicator YNINDICATOR = 'N',
	@PromoteTo_Environment_Id INT,
	@Promotion_Variables_Xml XML = NULL,
    @Execution_Variables_Xml XML = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Eligible_Indicator CHAR(1)
		   ,@IneligibleReason_Text VARCHAR(100)
		   ,@Workflow_Id INT
		   ,@Next_Step_Sequence INT

	SELECT @Next_Step_Sequence = [Next_Step_Sequence],
	       @Workflow_Id = [Workflow_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	EXEC [Builds_ValidateForPromotion]
			@Application_Id = @Application_Id,
			@Release_Number = @Release_Number, 
			@Build_Number = @Build_Number,
			@Eligible_Indicator = @Eligible_Indicator OUT,
			@IneligibleReason_Text = @IneligibleReason_Text OUT,
			@ForcePromotion_Indicator = @ForcePromotion_Indicator,
			@DependencyContingent_Indicator = @DependencyContingent_Indicator,
			@PromoteTo_Environment_Id = @PromoteTo_Environment_Id
		
	IF (@Eligible_Indicator = 'N')
		RAISERROR('Cannot promote build. %s', 16, 1, @IneligibleReason_Text)

	/* Reject existing builds in environment if multiple active builds are disabled */
	IF EXISTS(SELECT *
					FROM [dbo].[Applications]
				   WHERE [Application_Id] = @Application_Id
					 AND [AllowMultipleActiveBuilds_Indicator] = 'N')
	BEGIN

        DECLARE @BuildsToReject TABLE ( [Build_Number] BUILDNUMBER )

        INSERT @BuildsToReject 
        SELECT [Build_Number] 
          FROM [Builds_Extended]
         WHERE [Application_Id] = @Application_Id
           AND [Release_Number] = @Release_Number
           AND [Build_Number] <> @Build_Number
           AND [BuildStatus_Name] = 'Active'
           AND [Current_Environment_Id] = @PromoteTo_Environment_Id
           
        DECLARE @Tmp_Build_Number BUILDNUMBER

        WHILE EXISTS(SELECT * FROM @BuildsToReject)
        BEGIN

            SELECT TOP 1 @Tmp_Build_Number = [Build_Number] FROM @BuildsToReject
            
            EXEC [dbo].[Builds_RejectBuild]
			    @Application_Id = @Application_Id,
			    @Release_Number = @Release_Number,
			    @Build_Number = @Tmp_Build_Number,
			    @Rejection_Notes = 'Build rejected due to promotion of another build in the same environment.'

            DELETE @BuildsToReject WHERE [Build_Number] = @Tmp_Build_Number

        END

	END
	
	INSERT INTO [BuildPromotions]
		([Application_Id], [Release_Number], [Build_Number], [Environment_Id],
		 [PromotedBy_Name], [Comments_Text], [PromotionStatus_Name], [Forced_Indicator], [Promoted_Date])
	VALUES
		(@Application_Id, @Release_Number, @Build_Number, @PromoteTo_Environment_Id,
		 dbo.Session_GetUserName(), @Comments_Text, 'AwaitingExecution', @ForcePromotion_Indicator, GETUTCDATE())

	-- Remove any scheduled promotion indicator
	DELETE [BuildPromotionsScheduled]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [Environment_Id] = @PromoteTo_Environment_Id

    IF @Promotion_Variables_Xml IS NOT NULL BEGIN
	
		INSERT INTO [VariableDefinitions]
		(
			[Variable_Name],
			[Environment_Id],
			[Application_Id],
			[Release_Number],
			[Build_Number],
			[Value_Text],
			[Sensitive_Indicator]
		)
		SELECT [Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)'),
		       [Environment_Id] = @PromoteTo_Environment_Id,
		       [Application_Id] = @Application_Id,
		       [Release_Number] = @Release_Number,
		       [Build_Number] = @Build_Number,
			   [Value_Text] = LIST.[Variables].[value]('@Value', 'VARCHAR(MAX)'),
			   [Sensitive_Indicator] = COALESCE(VDEC.[Sensitive_Indicator], 'N')
		  FROM @Promotion_Variables_Xml.[nodes]('/Variables/Variable') AS LIST([Variables])
		       LEFT JOIN [VariableDeclarations] VDEC
		              ON VDEC.[Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)')
		             AND VDEC.[Scoped_Application_Id] = @Application_Id
		             AND VDEC.[Scoped_Environment_Id] = @PromoteTo_Environment_Id
		             AND VDEC.[Scope_Code] = 'P'

	END

	EXEC [Events_RaiseEvent] 'BLDPMT',@Application_Id,@Release_Number,@Build_Number,@PromoteTo_Environment_Id

	/* Initiate execution  */
	IF (@ExecutionStart_Date IS NOT NULL) BEGIN
		EXEC [Builds_InitiateBuildExecution]
			@Application_Id = @Application_Id,
			@Release_Number = @Release_Number,
			@Build_Number = @Build_Number,
			@Environment_Id = @PromoteTo_Environment_Id,
			@Start_Date = @ExecutionStart_Date,
            @Variables_Xml = @Execution_Variables_Xml
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
		   
END
GO

GRANT EXECUTE ON [Builds_PromoteBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CompleteAction',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks an action as completed.'
GO


IF OBJECT_ID('[Executions_CompleteAction]') IS NOT NULL DROP PROCEDURE [Executions_CompleteAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CompleteAction]
(
	@BuildExecution_ActionGroupAction_Id INT,
	@ExecutionStatus_Name VARCHAR(10)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [BuildExecution_ActionGroupActions]
	   SET [ExecutionEnded_Date] = GETUTCDATE(),
	       [ExecutionStatus_Name] = @ExecutionStatus_Name,
	       [WarningLogEntry_Indicator] = CASE WHEN @ExecutionStatus_Name = 'Succeeded'
	                                           AND EXISTS(SELECT *
	                                                        FROM [BuildExecution_ActionGroupActionLogEntries] BEAGALE
	                                                       WHERE BEAGALE.[BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id
	                                                         AND BEAGALE.[LogEntry_Level] >= 20
	                                                         AND BEAGALE.[LogEntry_Level] < 30)
	                                          THEN 'Y'
	                                          ELSE 'N'
	                                     END
	 WHERE [BuildExecution_ActionGroupAction_Id] = @BuildExecution_ActionGroupAction_Id

	DELETE [ManualActionsInProgress]
	 WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_ActionGroupAction_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_CompleteAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CreateOrUpdateRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new release or updates an existing one.'
GO

IF OBJECT_ID('[Releases_CreateOrUpdateRelease]') IS NOT NULL
	DROP PROCEDURE [Releases_CreateOrUpdateRelease]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CreateOrUpdateRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Workflow_Id INT = NULL,
	@Target_Date DATETIME,
	@Release_Name VARCHAR(50),
	@Notes_Text VARCHAR(MAX),
	@DeployableIds_Xml XML
	/* XML format example
		<?xml version="1.0" encoding="utf-16"?>
		<ReleaseDeployables>
			<ReleaseDeployable
				Deployable_Id="4"
				InclusionType_Code="R"
				Referenced_Release_Number="3.4"
				Referenced_Application_Id="1" (or not included to indicate NULL)
			/>
		</ReleaseDeployables>
	*/
) AS
BEGIN

IF REPLACE(@Release_Name,' ','') = ''
    SET @Release_Name = NULLIF(REPLACE(@Release_Name,' ',''),'')

DECLARE @Workflow_Name NVARCHAR(50)
    SET @Workflow_Name = (SELECT [Workflow_Name] FROM [Workflows] WHERE [Workflow_Id] = @Workflow_Id)

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS ( SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number )
	BEGIN

		/* Create the new release */
		DECLARE @ReleaseExists_Indicator YNINDICATOR
			   ,@NewerDeployedReleaseExists_Indicator YNINDICATOR
		EXEC [Releases_ReleaseExists]
				 @Application_Id = @Application_Id
				,@Release_Number = @Release_Number
				,@ReleaseExists_Indicator = @ReleaseExists_Indicator OUT
				,@NewerDeployedReleaseExists_Indicator = @NewerDeployedReleaseExists_Indicator OUT

		IF @ReleaseExists_Indicator = 'Y'
		BEGIN
			RAISERROR ('Release number ''%s'' already exists.', 16, 1, @Release_Number)
		END

		IF @NewerDeployedReleaseExists_Indicator = 'Y'
		BEGIN
			RAISERROR ('Release number ''%s'' cannot be created because a newer deployed release already exists.', 16, 1, @Release_Number)
		END

        DECLARE @ReleaseNumber_Scheme_Name VARCHAR(50)
            SET @ReleaseNumber_Scheme_Name = (SELECT [ReleaseNumber_Scheme_Name] FROM [Applications] WHERE [Application_Id] = @Application_Id)

		DECLARE @Sortable_Release_Number DECIMAL(38,0)
		    SET @Sortable_Release_Number = dbo.GetSortableReleaseNumber(@Release_Number, @ReleaseNumber_Scheme_Name)

		DECLARE @Initial_Release_Sequence INT
		    SET @Initial_Release_Sequence = COALESCE((SELECT MAX([Release_Sequence])
			                                            FROM [Releases]
											           WHERE [Application_Id] = @Application_Id), 0) + 1
		INSERT INTO [Releases]
			(
				[Application_Id], 
				[Release_Number], 
				[Workflow_Id], 
				[ReleaseStatus_Name], 
				[Target_Date], 
				[Release_Name], 
				[Notes_Text],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date],
                [Release_Sequence],
				[Original_Workflow_Name]
			)
		VALUES
			(
				@Application_Id, 
				@Release_Number, 
				@Workflow_Id, 
				'Active', 
				@Target_Date, 
				@Release_Name, 
				@Notes_Text,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE(),
                @Initial_Release_Sequence,
				@Workflow_Name
			)

        DECLARE @Release_Sequence INT
		SET @Release_Sequence = (SELECT MAX([Release_Sequence]) + 1
			                       FROM [Releases]
			                      WHERE [Application_Id] = @Application_Id
			                        AND dbo.GetSortableReleaseNumber([Release_Number], @ReleaseNumber_Scheme_Name) < @Sortable_Release_Number)

		IF @Release_Sequence IS NOT NULL AND @Release_Sequence <> @Initial_Release_Sequence
			EXEC [Releases_ResequenceRelease] @Application_Id, @Release_Sequence, @Initial_Release_Sequence

		INSERT INTO [ReleaseDeployables]
			(
				[Application_Id],
				[Release_Number],
				[Deployable_Id],
				[InclusionType_Code],
				[Referenced_Release_Number],
				[Referenced_Application_Id]
			)
		SELECT
				@Application_Id,
				@Release_Number,
				[Deployable_Id] = RDX.[ReleaseDeployable].[value]('@Deployable_Id', 'INT'),
				[InclusionType_Code] = RDX.[ReleaseDeployable].[value]('@InclusionType_Code', 'CHAR(1)'),
				[Referenced_Release_Number] = RDX.[ReleaseDeployable].[value]('@Referenced_Release_Number', 'RELEASENUMBER'),
				[Referenced_Application_Id] = RDX.[ReleaseDeployable].[value]('@Referenced_Application_Id', 'INT')
		FROM @DeployableIds_Xml.[nodes]('/ReleaseDeployables/ReleaseDeployable') AS RDX(ReleaseDeployable)

		INSERT INTO [ReleaseConfigurationFiles]
		(
			[Application_Id],
			[Release_Number],
			[ConfigurationFile_Id],
			[Version_Number]
		)
		SELECT @Application_Id,
		       @Release_Number,
			   CFV.[ConfigurationFile_Id],
			   MAX(CFV.[Version_Number])
		  FROM [ConfigurationFileVersions] CFV
		       INNER JOIN [ConfigurationFiles] CF
			           ON CF.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
			   INNER JOIN [Deployables] D
			           ON D.[Deployable_Id] = CF.[Deployable_Id]
		 WHERE D.[Application_Id] = @Application_Id
		 GROUP BY CFV.[ConfigurationFile_Id]

		EXEC [Events_RaiseEvent] 'RELNEW', @Application_Id, @Release_Number

	END
	ELSE
	BEGIN
	
		/* verify that there are no active builds for this release if we're including a workflow id */
		DECLARE @Old_Workflow_Id INT
		
		SELECT @Old_Workflow_Id = [Workflow_Id] FROM [Releases] 
		WHERE [Application_Id] = @Application_Id 
		AND [Release_Number] = @Release_Number
		
		EXEC [dbo].[Builds_GetBuilds] @Application_Id, @Release_Number, 'Active'
		
		IF (@Workflow_Id IS NOT NULL 
			AND @Workflow_Id <> @Old_Workflow_Id 
			AND @@ROWCOUNT > 0)
        BEGIN
			RAISERROR('A release''s workflow may not be changed if the release has any active builds', 16, 1)
			ROLLBACK
        END

		/* Update the existing release */
		UPDATE [Releases]
		   SET [Target_Date] = @Target_Date
			  ,[Release_Name] = @Release_Name
			  ,[Workflow_Id] = @Workflow_Id
			  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[Original_Workflow_Name] = @Workflow_Name
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number

		DELETE
		  FROM [ReleaseDeployables]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number

		INSERT INTO [ReleaseDeployables]
			(
				[Application_Id],
				[Release_Number],
				[Deployable_Id],
				[InclusionType_Code],
				[Referenced_Release_Number],
				[Referenced_Application_Id]
			)
		SELECT
				@Application_Id,
				@Release_Number,
				[Deployable_Id] = RDX.[ReleaseDeployable].[value]('@Deployable_Id', 'INT'),
				[InclusionType_Code] = RDX.[ReleaseDeployable].[value]('@InclusionType_Code', 'CHAR(1)'),
				[Referenced_Release_Number] = RDX.[ReleaseDeployable].[value]('@Referenced_Release_Number', 'RELEASENUMBER'),
				[Referenced_Application_Id] = RDX.[ReleaseDeployable].[value]('@Referenced_Application_Id', 'INT')
		FROM @DeployableIds_Xml.[nodes]('/ReleaseDeployables/ReleaseDeployable') AS RDX(ReleaseDeployable)

		EXEC [Events_RaiseEvent] 'RELMOD', @Application_Id, @Release_Number

		UPDATE [BuildExecutions]
		   SET [ReexecutionState_Xml] = NULL
		 WHERE [Application_Id] = @Application_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_CreateOrUpdateRelease] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_CreateOrUpdateWorkflowStep',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Step_Sequence',
    /* Description_Text        */ 'Creates a new workflow step or updates an existing one.'
GO


IF OBJECT_ID('[Workflows_CreateOrUpdateWorkflowStep]') IS NOT NULL DROP PROCEDURE [Workflows_CreateOrUpdateWorkflowStep]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_CreateOrUpdateWorkflowStep]
(
	@Workflow_Id INT,
	@Environment_Id INT,
	@DeploymentPlan_Id INT,
	@AutoPromote_Indicator YNINDICATOR,
	@Step_Sequence INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Step_Sequence IS NULL
	BEGIN

		SET @Step_Sequence = COALESCE(
			(SELECT MAX([Step_Sequence])
		       FROM [WorkflowSteps]
		      WHERE [Workflow_Id] = @Workflow_Id),
			0) + 1

		INSERT INTO [WorkflowSteps]
		(
			[Workflow_Id],
			[Environment_Id],
			[Step_Sequence],
			[DeploymentPlan_Id],
			[AutoPromote_Indicator]
		)
		VALUES
		(
			@Workflow_Id,
			@Environment_Id,
			@Step_Sequence,
			@DeploymentPlan_Id,
			@AutoPromote_Indicator
		)

	END
	ELSE
	BEGIN

		UPDATE [WorkflowSteps]
		   SET [DeploymentPlan_Id] = @DeploymentPlan_Id,
		       [AutoPromote_Indicator] = @AutoPromote_Indicator
		 WHERE [Workflow_Id] = @Workflow_Id
		   AND [Step_Sequence] = @Step_Sequence

        SELECT @Environment_Id = [Environment_Id]
          FROM [WorkflowSteps]
         WHERE [Workflow_Id] = @Workflow_Id
		   AND [Step_Sequence] = @Step_Sequence

	END

	EXEC [Events_RaiseEvent] 'WKFMDS', NULL, @Workflow_Id, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_CreateOrUpdateWorkflowStep] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_CompletePromotion',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Builds_CompletePromotion]') IS NOT NULL DROP PROCEDURE [Builds_CompletePromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_CompletePromotion]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@StatusOverriddenReason_Text VARCHAR(100) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Workflow_Id INT
	DECLARE @Step_Sequence INT
	DECLARE @Application_Name VARCHAR(50)
	DECLARE @NotificationTitle_Text VARCHAR(100)
	DECLARE @NotificationDetails_Text VARCHAR(MAX)
    DECLARE @AutoIncrementRelease_Indicator YNINDICATOR
    DECLARE @AutoDeployRelease_Indicator YNINDICATOR
    DECLARE @AutoCancelReleases_Indicator YNINDICATOR

	SELECT @Workflow_Id = [Workflow_Id], 
           @AutoIncrementRelease_Indicator = [AutoIncrementRelease_Indicator],
           @AutoDeployRelease_Indicator = [AutoDeployRelease_Indicator],
           @AutoCancelReleases_Indicator = [AutoCancelReleases_Indicator]
      FROM [Releases_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	/* Skip if the promotion is Already Completed */
	IF EXISTS(SELECT * FROM [BuildPromotions]
	           WHERE [Application_Id] = @Application_Id
	             AND [Release_Number] = @Release_Number
	             AND [Build_Number] = @Build_Number
	             AND [Environment_Id] = @Environment_Id
	             AND [PromotionStatus_Name] = 'Completed')
		BEGIN COMMIT RETURN END

	/* Validate Status */
	IF NOT EXISTS(SELECT * FROM [BuildPromotions]
	               WHERE [Application_Id] = @Application_Id
	                 AND [Release_Number] = @Release_Number
	                 AND [Build_Number] = @Build_Number
	                 AND [Environment_Id] = @Environment_Id
	                 AND [PromotionStatus_Name] IN ('AwaitingExecution','Failed'))
			RAISERROR('Build Promotion must be AwaitingExecution or Failed', 16, 1)

	/* if there are no Promotion Contingiencies that aren't Completed... */
	IF NOT EXISTS(
		SELECT *
		  FROM [BuildPromotionContingencies] BPC
		       INNER JOIN [BuildPromotions] BP
		               ON BPC.[Contingent_Application_Id] = BP.[Application_Id]
		              AND BPC.[Contingent_Release_Number] = BP.[Release_Number]
		              AND BPC.[Contingent_Build_Number] = BP.[Build_Number]
		              AND BPC.[Contingent_Environment_Id] = BP.[Environment_Id]
		 WHERE BPC.[Application_Id] = @Application_Id
		   AND BPC.[Release_Number] = @Release_Number
		   AND BPC.[Build_Number] = @Build_Number
		   AND BPC.[Environment_Id] = @Environment_Id
		   AND BP.[PromotionStatus_Name] <> 'Completed')
	BEGIN


		/* Do the update */
		/* Special CASE logic for when build is being overridden (@StatusOverriddenReason_Text is not null) */
		UPDATE [BuildPromotions]
		   SET [PromotionStatus_Name] = 'Completed'
			  ,[StatusOverriddenBy_User_Name] = CASE WHEN @StatusOverriddenReason_Text IS NULL THEN NULL ELSE dbo.Session_GetUserName() END
			  ,[StatusOverriddenReason_Text] = @StatusOverriddenReason_Text
			  ,[StatusOverridden_Date] = CASE WHEN @StatusOverriddenReason_Text IS NULL THEN NULL ELSE GETUTCDATE() END
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] = @Environment_Id

		/* Find the current Sequence */
		SET @Step_Sequence = 
			(SELECT [Step_Sequence] FROM [WorkflowSteps]
			  WHERE [Workflow_Id] = @Workflow_Id 
				AND [Environment_Id] = @Environment_Id)
				
		/* if there is no next step (i.e. deployed to Production), the release should be auto-deployed, and the release status is active */
		IF EXISTS(SELECT * FROM [Workflows]
		           WHERE [Workflow_Id] = @Workflow_Id
		             AND [AutoDeployRelease_Indicator]='Y')
		 AND NOT EXISTS(SELECT * FROM [WorkflowSteps]
					     WHERE [Workflow_Id] = @Workflow_Id 
						   AND [Step_Sequence] > @Step_Sequence)
		 AND EXISTS(SELECT * FROM [Releases]
		             WHERE  [Application_Id] = @Application_Id
		                AND [Release_Number] = @Release_Number
		                AND [ReleaseStatus_Name] = 'Active')
		BEGIN
			
            EXEC [Builds_DeployBuild] @Application_Id, @Release_Number, @Build_Number, @Workflow_Id

		END
		
		EXEC [Events_RaiseEvent] 'BLDCPP',@Application_Id,@Release_Number,@Build_Number,@Environment_Id
	
	END


    IF EXISTS(SELECT * FROM [BuildPromotionContingencies]
                      WHERE [Application_Id] = @Application_Id
			            AND [Release_Number] = @Release_Number
			            AND [Build_Number] = @Build_Number 
			            AND [Environment_Id] = @Environment_Id)
    BEGIN

        DECLARE @BPC TABLE
        (
            [Key] UNIQUEIDENTIFIER,
            [Application_Id] INT,
            [Release_Number] RELEASENUMBER,
            [Build_Number] BUILDNUMBER,
            [Environment_Id] INT
        )

        DECLARE @Tmp_Key UNIQUEIDENTIFIER,
                @Tmp_Application_Id INT,
			    @Tmp_Release_Number RELEASENUMBER, 
			    @Tmp_Build_Number BUILDNUMBER, 
			    @Tmp_Environment_Id INT

        INSERT @BPC ([Key], [Application_Id], [Release_Number], [Build_Number], [Environment_Id])
        SELECT NEWID(), BPC.[Application_Id], BPC.[Release_Number], BPC.[Build_Number], BPC.[Environment_Id]
			    FROM [BuildPromotionContingencies] BPC
			        INNER JOIN [BuildPromotions_Extended] BP
			                ON BP.[Application_Id] = BPC.[Application_Id]
			                AND BP.[Release_Number] = BPC.[Release_Number]
			                AND BP.[Build_Number] = BPC.[Build_Number] 
			                AND BP.[Environment_Id] = BPC.[Environment_Id]
			    WHERE BP.[Last_ExecutionStatus_Name] = 'Succeeded'
			    AND BP.[PromotionStatus_Name] = 'AwaitingExecution'
			    AND BPC.[Contingent_Application_Id] = @Application_Id
			    AND BPC.[Contingent_Release_Number] = @Release_Number
			    AND BPC.[Contingent_Build_Number] = @Build_Number
			    AND BPC.[Contingent_Environment_Id] = @Environment_Id

        WHILE EXISTS(SELECT * FROM @BPC)
        BEGIN
            
            SELECT TOP 1 
                    @Tmp_Key = [Key],  
                    @Tmp_Application_Id = [Application_Id], 
                    @Tmp_Release_Number = [Release_Number], 
                    @Tmp_Build_Number = [Build_Number], 
                    @Tmp_Environment_Id = [Environment_Id]
                  FROM @BPC

            EXEC [Builds_CompletePromotion]
				    @Application_Id = @Tmp_Application_Id,
				    @Release_Number = @Tmp_Release_Number,
				    @Build_Number = @Tmp_Build_Number,
				    @Environment_Id = @Tmp_Environment_Id	

            DELETE @BPC WHERE [Key] = @Tmp_Key

        END

    END
	
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Builds_CompletePromotion] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_CreateBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Build_Number',
    /* Description_Text        */ 'Creates a new build of an application and optionally promotes it to the first environment.',
    /* Remarks_Text            */ 'To specify build variable values, the Variables_Xml argument should be a string in the format:
	<Variables>
		<Variable Name="VARIABLE_NAME" Value="VARIABLE_VALUE" />
	</Variables>'
GO

IF OBJECT_ID('[Builds_CreateBuild]') IS NOT NULL DROP PROCEDURE [Builds_CreateBuild]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_CreateBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@PromoteBuild_Indicator YNINDICATOR = NULL,
	@StartExecution_Indicator YNINDICATOR = NULL,
	@ExecutionStart_Date DATETIME = NULL,
	@Requested_Build_Number BUILDNUMBER = NULL,
	@BuildVariables_Xml XML = NULL,
	@PromotionVariables_Xml XML = NULL,
    @ExecutionVariables_Xml XML = NULL,
	@BuildImporter_Configuration XML = NULL,
	@Build_Number BUILDNUMBER OUT
)
AS
BEGIN

	/* Set defaults */
	SET @PromoteBuild_Indicator = COALESCE(@PromoteBuild_Indicator, 'N')
	SET @StartExecution_Indicator = COALESCE(@StartExecution_Indicator, @PromoteBuild_Indicator)
	
	IF @PromoteBuild_Indicator = 'Y' AND @StartExecution_Indicator = 'Y' BEGIN
		SET @ExecutionStart_Date = CASE
									WHEN @ExecutionStart_Date >= GETUTCDATE() THEN @ExecutionStart_Date
									ELSE GETUTCDATE()
								   END
	END
	ELSE BEGIN
		SET @ExecutionStart_Date = NULL
	END

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF NOT EXISTS(SELECT * FROM [Releases] 
	               WHERE [Application_Id] = @Application_Id
	                 AND [Release_Number] = @Release_Number
	                 AND [ReleaseStatus_Name] = 'Active')
		RAISERROR('Release "%s" for Application #%i is not an active release or does not exist.', 16, 1, @Release_Number, @Application_Id)
		
		IF @Requested_Build_Number IS NOT NULL BEGIN
			IF EXISTS (SELECT *
						 FROM [Builds]
						WHERE [Application_Id] = @Application_Id
						  AND [Release_Number] = @Release_Number
						  AND [Build_Number] = @Requested_Build_Number)
				RAISERROR('Build "%s" of release "%s" for Application #%i already exists.', 16, 1, @Requested_Build_Number, @Release_Number, @Application_Id)

            IF EXISTS (SELECT *
						 FROM [Builds] B
                   INNER JOIN [Applications] A 
                           ON A.[Application_Id] = B.[Application_Id]
						WHERE B.[Application_Id] = @Application_Id
						  AND B.[Build_Number] = @Requested_Build_Number
                          AND A.[BuildNumber_Scheme_Name] = 'Unique')
				RAISERROR('Unique build numbers must be unique in the context of an application. Build number "%s" in Application #%i already exists.', 16, 1, @Requested_Build_Number, @Application_Id)

			SET @Build_Number = @Requested_Build_Number
		END
		ELSE BEGIN
		    SET @Build_Number = dbo.GetNextBuildNumber(
		            @Application_Id,
					(SELECT TOP 1 [Build_Number]
					   FROM [Builds]
					  WHERE [Application_Id] = @Application_Id
					    AND [Release_Number] = @Release_Number
					  ORDER BY [Build_Sequence] DESC))
		END

	DECLARE @Build_Sequence INT
	SET @Build_Sequence = COALESCE((SELECT MAX([Build_Sequence])
	                                  FROM [Builds]
							         WHERE [Application_Id] = @Application_Id
							           AND [Release_Number] = @Release_Number), 0) + 1

	INSERT INTO [Builds]
	(	
		[Application_Id], 
		[Release_Number], 
		[Build_Number], 
		[BuildStatus_Name],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[ModifiedBy_User_Name],
		[ModifiedOn_Date],
		[BuildImporter_Configuration],
		[Build_Sequence]
	)
	VALUES
	(
		@Application_Id, 
		@Release_Number, 
		@Build_Number, 
		'Active',
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		@BuildImporter_Configuration,
		@Build_Sequence
	)

	IF @BuildVariables_Xml IS NOT NULL BEGIN

		INSERT INTO [VariableDefinitions]
		(
			[Variable_Name],
			[Application_Id],
			[Release_Number],
			[Build_Number],
			[Value_Text],
			[Sensitive_Indicator]
		)
		SELECT [Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)'),
		       [Application_Id] = @Application_Id,
		       [Release_Number] = @Release_Number,
		       [Build_Number] = @Build_Number,
			   [Value_Text] = LIST.[Variables].[value]('@Value', 'VARCHAR(MAX)'),
			   [Sensitive_Indicator] = COALESCE(VDEC.[Sensitive_Indicator], 'N')
		  FROM @BuildVariables_Xml.[nodes]('/Variables/Variable') AS LIST([Variables])
		       LEFT JOIN [VariableDeclarations] VDEC
		               ON VDEC.[Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)')
		              AND VDEC.[Scoped_Application_Id] = @Application_Id
		              AND VDEC.[Scope_Code] = 'B'

	END

	EXEC [Events_RaiseEvent] 'BLDNEW', @Application_Id, @Release_Number, @Build_Number

	DECLARE @Build_DeploymentPlan_Id INT

	SELECT @Build_DeploymentPlan_Id = W.[Build_DeploymentPlan_Id]
	  FROM [Releases] R
	       INNER JOIN [Workflows] W
		           ON W.[Workflow_Id] = R.[Workflow_Id]
	 WHERE R.[Application_Id] = @Application_Id
	   AND R.[Release_Number] = @Release_Number

	IF @Build_DeploymentPlan_Id IS NULL AND @BuildImporter_Configuration IS NULL
	BEGIN
		/* Promote the Build */
		IF @PromoteBuild_Indicator = 'Y' BEGIN
			DECLARE @Next_Environment_Id INT
			SET @Next_Environment_Id = (
				SELECT [Next_Environment_Id] FROM [Builds_Extended]
				 WHERE [Application_Id] = @Application_Id
				   AND [Release_Number] = @Release_Number
				   AND [Build_Number] = @Build_Number)

			EXEC [Builds_PromoteBuild]
				@Application_Id = @Application_Id,
				@Release_Number = @Release_Number,
				@Build_Number = @Build_Number,
				@Comments_Text = NULL,
				@ExecutionStart_Date = @ExecutionStart_Date,
				@PromoteTo_Environment_Id = @Next_Environment_Id,
				@Promotion_Variables_Xml = @PromotionVariables_Xml,
		        @Execution_Variables_Xml = @ExecutionVariables_Xml
		END
	END
	ELSE
	BEGIN

		/* Reject existing unpromoted builds if multiple active builds is disabled */
		IF EXISTS(SELECT *
						FROM [dbo].[Applications]
					   WHERE [Application_Id] = @Application_Id
						 AND [AllowMultipleActiveBuilds_Indicator] = 'N')
		BEGIN
            
            DECLARE @BuildsToReject TABLE ( [Build_Number] BUILDNUMBER )

            INSERT @BuildsToReject 
            SELECT [Build_Number] 
              FROM [Builds_Extended]
             WHERE [Application_Id] = @Application_Id
               AND [Release_Number] = @Release_Number
               AND [Build_Number] <> @Build_Number
               AND [BuildStatus_Name] = 'Active'
               AND [Current_Environment_Id] IS NULL
           
            DECLARE @Tmp_Build_Number BUILDNUMBER

            WHILE EXISTS(SELECT * FROM @BuildsToReject)
            BEGIN

                SELECT TOP 1 @Tmp_Build_Number = [Build_Number] FROM @BuildsToReject
            
                EXEC [dbo].[Builds_RejectBuild]
			        @Application_Id = @Application_Id,
			        @Release_Number = @Release_Number,
			        @Build_Number = @Tmp_Build_Number,
			        @Rejection_Notes = 'Build rejected due to creation of new build.'

                DELETE @BuildsToReject WHERE [Build_Number] = @Tmp_Build_Number

            END

		END

		/* Build DeploymentPlan is present */
		EXEC [Builds_InitiateBuildExecution]
			@Application_Id = @Application_Id,
			@Release_Number = @Release_Number,
			@Build_Number = @Build_Number,
			@Environment_Id = NULL,
			@Variables_Xml = @ExecutionVariables_Xml

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_CreateBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_CreateDependentBuilds',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates new builds in all dependent applications a build.',
    /* Remarks_Text            */ 'To specify build variable values, the Variables_Xml argument should be a string in the format:
	<Variables>
		<Variable Name="VARIABLE_NAME" Value="VARIABLE_VALUE" />
	</Variables>'
GO

IF OBJECT_ID('[Builds_CreateDependentBuilds]') IS NOT NULL DROP PROCEDURE [Builds_CreateDependentBuilds]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_CreateDependentBuilds]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,

	@ExecutionStart_Date DATETIME = NULL,
	@OverrideApprovals_Indicator YNINDICATOR = 'N',
	@Variables_Xml XML = NULL,

	@PromoteFor_Release_Number RELEASENUMBER = NULL,
	@ContinueOnFail_Indicator YNINDICATOR = 'N'	
)
AS
BEGIN


SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	/* Verify Build */
	IF NOT EXISTS(SELECT * FROM [Builds]
	               WHERE [Application_Id] = @Application_Id
	                 AND [Release_Number] = @Release_Number
	                 AND [Build_Number] = @Build_Number
	                 AND [BuildStatus_Name] <> 'Rejected')
		RAISERROR('Build %s does not exist or has been rejected', 16, 1, @Build_Number)

	/* Verify Current Environment Id */		
	DECLARE @Current_Environment_Id INT
	SELECT @Current_Environment_Id = [Current_Environment_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	IF @Current_Environment_Id IS NULL
		RAISERROR('Build %s does not have a current environment.', 16, 1, @Build_Number)

	
	/* Find All Dependent Releases */
	SELECT RE.*
	  INTO #DependentReleases
	  FROM [Releases_Extended] RE
	 WHERE EXISTS 
           (
	         SELECT * FROM [ReleaseDeployables] RD
	          WHERE RE.[Application_Id] = RD.[Application_Id]
	            AND RE.[Release_Number] = RD.[Release_Number]
	            AND RD.[Referenced_Application_Id] = @Application_Id
	            AND RD.[Referenced_Release_Number] = @Release_Number
	            
	       )
	   AND NOT ([Application_Id] = @Application_Id AND 
	            [Release_Number] = @Release_Number)
	   AND (@PromoteFor_Release_Number IS NULL OR [Release_Number] = @PromoteFor_Release_Number)
	   AND RE.[ReleaseStatus_Name] = 'Active'
	   
	/* "Declare" a temp table */
	SELECT * INTO #CreatedBuilds FROM [Builds_Extended] WHERE 1=0
	DECLARE @DependentReleasesCursor CURSOR,
	        @CUR_Application_Id INT,
	        @CUR_Release_Number RELEASENUMBER, 
	        @CUR_Build_Number BUILDNUMBER, 
	        @CUR_Next_Environment_Id INT,
	        @DeferredError VARCHAR(MAX),
	        @Eligible_Indicator CHAR(1),
	        @IneligibleReason_Text VARCHAR(MAX)

	/* Create Builds, Promote, and Insert Contingencies */
    SET @DependentReleasesCursor = CURSOR FOR
        SELECT [Application_Id], [Release_Number]
          FROM #DependentReleases
          
	OPEN @DependentReleasesCursor
	FETCH NEXT FROM @DependentReleasesCursor INTO @CUR_Application_Id, @CUR_Release_Number
		WHILE @@FETCH_STATUS <> -1
		BEGIN
			
			SET @CUR_Build_Number = NULL
			
			EXEC [Builds_CreateBuild] 
			   @Application_Id = @CUR_Application_Id
			  ,@Release_Number = @CUR_Release_Number
			  ,@Requested_Build_Number = NULL
			  ,@PromoteBuild_Indicator = 'N'
			  ,@StartExecution_Indicator = 'N'
			  ,@ExecutionStart_Date = NULL
			  ,@BuildVariables_Xml = @Variables_Xml
              ,@PromotionVariables_Xml = NULL
			  ,@Build_Number  = @CUR_Build_Number OUTPUT
			  
			INSERT INTO #CreatedBuilds
			SELECT * FROM [Builds_Extended] 
			 WHERE [Application_Id] = @CUR_Application_Id
			   AND [Release_Number] = @CUR_Release_Number
			   AND [Build_Number] = @CUR_Build_Number

			/* Get Build's @CUR_Next_Environment_Id for Promote */
			SELECT @CUR_Next_Environment_Id = [Next_Environment_Id] 
			  FROM #CreatedBuilds
			 WHERE [Application_Id] = @CUR_Application_Id
			   AND [Release_Number] = @CUR_Release_Number
			   AND [Build_Number] = @CUR_Build_Number
			   
			/* Do the Promotion */
			EXECUTE [Builds_PromoteBuild] 
			   @Application_Id = @CUR_Application_Id
			  ,@Release_Number = @CUR_Release_Number
			  ,@Build_Number = @CUR_Build_Number
			  ,@Comments_Text = 'Dependency Promotion'
			  ,@ExecutionStart_Date = NULL
			  ,@ForcePromotion_Indicator = @OverrideApprovals_Indicator
			  ,@DependencyContingent_Indicator = 'Y'
			  ,@PromoteTo_Environment_Id = @CUR_Next_Environment_Id

			/* Insert Contingencies */
			INSERT INTO [BuildPromotionContingencies]
				([Application_Id],[Release_Number],[Build_Number],[Environment_Id]
				,[Contingent_Application_Id],[Contingent_Release_Number],[Contingent_Build_Number],[Contingent_Environment_Id])
			VALUES
				(@CUR_Application_Id,@CUR_Release_Number,@CUR_Build_Number,@CUR_Next_Environment_Id
				,@Application_Id,@Release_Number,@Build_Number,@Current_Environment_Id)			

			FETCH NEXT FROM @DependentReleasesCursor INTO @CUR_Application_Id, @CUR_Release_Number
		END
	CLOSE @DependentReleasesCursor
	
	/* Execute Dependents whose Contingencies match Dependencies */
	DECLARE @CreatedBuildsCursor CURSOR
    SET @CreatedBuildsCursor = CURSOR FOR
        SELECT [Application_Id], [Release_Number], [Build_Number], [Next_Environment_Id]
          FROM #CreatedBuilds CB
         WHERE [Application_Id] IN
               (
                 SELECT _RD.[Application_Id]
                   FROM [ReleaseDeployables] _RD
                        LEFT JOIN [BuildPromotionContingencies] _BPC
                               ON _RD.[Referenced_Application_Id] = _BPC.[Contingent_Application_Id]
                              AND _RD.[Referenced_Release_Number] = _BPC.[Contingent_Release_Number]
                  WHERE _RD.[InclusionType_Code] = 'R'
                    AND _RD.[Application_Id] = CB.[Application_Id]
                    AND _RD.[Release_Number] = CB.[Release_Number]
                    AND _BPC.[Contingent_Application_Id] IS NOT NULL
               )
    OPEN @CreatedBuildsCursor
	FETCH NEXT FROM @CreatedBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		WHILE @@FETCH_STATUS <> -1
		BEGIN	

			/* Initiate execution  */
			IF (@ExecutionStart_Date IS NOT NULL) BEGIN
				EXEC [Builds_InitiateBuildExecution]
					@Application_Id = @CUR_Application_Id,
					@Release_Number = @CUR_Release_Number,
					@Build_Number = @CUR_Build_Number,
					@Environment_Id = @CUR_Next_Environment_Id,
					@Start_Date = @ExecutionStart_Date
			END

			FETCH NEXT FROM @CreatedBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		END
	CLOSE @CreatedBuildsCursor	


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_CreateDependentBuilds] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_PromoteDependents',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Promotes all dependents for a given promotion'
GO


IF OBJECT_ID('[Builds_PromoteDependents]') IS NOT NULL DROP PROCEDURE [Builds_PromoteDependents]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_PromoteDependents]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
--	@Comments_Text VARCHAR(255) = NULL,
	@ExecutionStart_Date DATETIME = NULL,
	@OverrideApprovals_Indicator YNINDICATOR = 'N',
	
	@From_Environment_Id INT = NULL,
	@To_Environment_Id INT = NULL,
	
	@PromoteFor_Release_Number RELEASENUMBER = NULL,

	@ContinueOnFailedPromote_Indicator YNINDICATOR = 'N'
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	/* Verify Build */
	IF NOT EXISTS(SELECT * FROM [Builds]
	               WHERE [Application_Id] = @Application_Id
	                 AND [Release_Number] = @Release_Number
	                 AND [Build_Number] = @Build_Number
	                 AND [BuildStatus_Name] <> 'Rejected')
		RAISERROR('Build %s does not exist or has been rejected', 16, 1, @Build_Number)

	/* Verify Current Environment Id */		
	DECLARE @Current_Environment_Id INT
	SELECT @Current_Environment_Id = [Current_Environment_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	IF @Current_Environment_Id IS NULL
		RAISERROR('Build %s does not have a current environment.', 16, 1, @Build_Number)
	
	/* Find All Dependent Builds */
	SELECT DISTINCT B.[Application_Id], B.[Release_Number], B.[Build_Number], B.[Next_Environment_Id] -- distinct to filter out multiple dependencies against same deployable
	  INTO #DependentBuilds
	  FROM [Builds_Extended] B
	       INNER JOIN [ReleaseDeployables] RD
	               ON B.[Application_Id] = RD.[Application_Id]
	              AND B.[Release_Number] = RD.[Release_Number]
	              AND RD.[InclusionType_Code] = 'R'
	 WHERE [BuildStatus_Name] = 'Active'
	   AND RD.[Referenced_Application_Id] = @Application_Id
	   AND RD.[Referenced_Release_Number] = @Release_Number
	   AND (@From_Environment_Id IS NULL OR B.[Current_Environment_Id] = @From_Environment_Id)
	   AND (@To_Environment_Id IS NULL OR B.[Next_Environment_Id] = @To_Environment_Id)
	   AND (@PromoteFor_Release_Number IS NULL OR B.[Release_Number] = @PromoteFor_Release_Number)

	DECLARE @DependentBuildsCursor CURSOR,
	        @CUR_Application_Id INT,
	        @CUR_Release_Number RELEASENUMBER, 
	        @CUR_Build_Number BUILDNUMBER, 
	        @CUR_Next_Environment_Id INT,
	        @DeferredError VARCHAR(MAX),
	        @Eligible_Indicator CHAR(1),
	        @IneligibleReason_Text VARCHAR(MAX)

	/* Promote (with no execute) and Insert Contingencies */
    SET @DependentBuildsCursor = CURSOR FOR
        SELECT [Application_Id], [Release_Number], [Build_Number], [Next_Environment_Id]
          FROM #DependentBuilds
	OPEN @DependentBuildsCursor
	FETCH NEXT FROM @DependentBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		WHILE @@FETCH_STATUS <> -1
		BEGIN
		
			EXEC [Builds_ValidateForPromotion]
			   @Application_Id = @CUR_Application_Id
			  ,@Release_Number = @CUR_Release_Number
			  ,@Build_Number = @CUR_Build_Number
			  ,@Eligible_Indicator = @Eligible_Indicator OUT
			  ,@IneligibleReason_Text = @IneligibleReason_Text OUT
			  ,@ForcePromotion_Indicator = @OverrideApprovals_Indicator
			  ,@DependencyContingent_Indicator = 'Y'
			  ,@PromoteTo_Environment_Id = @CUR_Next_Environment_Id

			IF @Eligible_Indicator = 'Y' BEGIN

				EXECUTE [Builds_PromoteBuild] 
				   @Application_Id = @CUR_Application_Id
				  ,@Release_Number = @CUR_Release_Number
				  ,@Build_Number = @CUR_Build_Number
				  ,@Comments_Text = 'Dependency Promotion'
				  ,@ExecutionStart_Date = NULL
				  ,@ForcePromotion_Indicator = @OverrideApprovals_Indicator
				  ,@DependencyContingent_Indicator = 'Y'
				  ,@PromoteTo_Environment_Id = @CUR_Next_Environment_Id
				  
				INSERT INTO [BuildPromotionContingencies]
					([Application_Id],[Release_Number],[Build_Number],[Environment_Id]
					,[Contingent_Application_Id],[Contingent_Release_Number],[Contingent_Build_Number],[Contingent_Environment_Id])
				VALUES
					(@CUR_Application_Id,@CUR_Release_Number,@CUR_Build_Number,@CUR_Next_Environment_Id
					,@Application_Id,@Release_Number,@Build_Number,@Current_Environment_Id)

			END ELSE BEGIN
				IF @ContinueOnFailedPromote_Indicator='N' BEGIN
					RAISERROR('Cannot Promote Build. %s', 16, 1, @IneligibleReason_Text)
				END ELSE BEGIN
					SET @DeferredError = @DeferredError 
						+ 'Build #' + @CUR_Build_Number + ' ' + ' for Application #' + @CUR_Application_Id + ' '
						+ 'cannot be be promoted: ' + @IneligibleReason_Text
					DELETE #DependentBuilds
					 WHERE CURRENT OF @DependentBuildsCursor
				END
			END

			FETCH NEXT FROM @DependentBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		END
	CLOSE @DependentBuildsCursor
	
	/* Execute Dependents whose Contingencies match Dependencies */
    SET @DependentBuildsCursor = CURSOR FOR
        SELECT [Application_Id], [Release_Number], [Build_Number], [Next_Environment_Id]
          FROM #DependentBuilds DB
         WHERE [Application_Id] IN
               (
                 SELECT _BPC.[Application_Id]
                   FROM [ReleaseDeployables] _RD
                        LEFT JOIN [BuildPromotionContingencies] _BPC
                               ON _RD.[Referenced_Application_Id] = _BPC.[Contingent_Application_Id]
                              AND _RD.[Referenced_Release_Number] = _BPC.[Contingent_Release_Number]
                  WHERE _RD.[InclusionType_Code] = 'R'
                    AND _RD.[Application_Id] = DB.[Application_Id]
                    AND _RD.[Release_Number] = DB.[Release_Number]
                    AND _BPC.[Contingent_Application_Id] IS NOT NULL
               )
    OPEN @DependentBuildsCursor
	FETCH NEXT FROM @DependentBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		WHILE @@FETCH_STATUS <> -1
		BEGIN	

			/* Initiate execution  */
			IF (@ExecutionStart_Date IS NOT NULL) BEGIN
				EXEC [Builds_InitiateBuildExecution]
					@Application_Id = @CUR_Application_Id,
					@Release_Number = @CUR_Release_Number,
					@Build_Number = @CUR_Build_Number,
					@Environment_Id = @CUR_Next_Environment_Id,
					@Start_Date = @ExecutionStart_Date
			END

			FETCH NEXT FROM @DependentBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		END
	CLOSE @DependentBuildsCursor
	
	COMMIT
	
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
 
BEGIN TRY
	IF @DeferredError IS NOT NULL
		RAISERROR('Promotion error(s) occured. %s', 16, 1, @DeferredError)
END TRY BEGIN CATCH
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_PromoteDependents] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CompleteExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks an execution as completed.'
GO


IF OBJECT_ID('[Executions_CompleteExecution]') IS NOT NULL DROP PROCEDURE [Executions_CompleteExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CompleteExecution]
(
	@Execution_Id INT,
	@ExecutionStatus_Name VARCHAR(10),
	@ReexecutionState_Xml XML = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [BuildExecutions]
	  WITH (TABLOCK)
	   SET [ExecutionStatus_Name] = @ExecutionStatus_Name,
	       [WarningLogEntry_Indicator] = CASE WHEN @ExecutionStatus_Name = 'Succeeded'
	                                           AND EXISTS(SELECT *
	                                                        FROM [BuildExecution_ActionGroupActions] BEAGA
	                                                             INNER JOIN [BuildExecution_DeploymentPlans] BEDP
	                                                                     ON BEDP.[BuildExecution_DeploymentPlan_Id] = BEAGA.[BuildExecution_DeploymentPlan_Id]
	                                                       WHERE BEDP.[Execution_Id] = @Execution_Id
	                                                         AND BEAGA.[WarningLogEntry_Indicator] = 'Y')
	                                          THEN 'Y'
	                                          ELSE 'N'
	                                     END,
	       [ExecutionEnded_Date] = GETUTCDATE(),
	       [ReexecutionState_Xml] = @ReexecutionState_Xml
	 WHERE [Execution_Id] = @Execution_Id

	DECLARE @Application_Id INT,
		@Release_Number RELEASENUMBER,
		@Build_Number BUILDNUMBER,
		@Environment_Id INT

	SELECT @Application_Id = [Application_Id],
	       @Release_Number = [Release_Number],
	       @Build_Number = [Build_Number],
	       @Environment_Id = [Environment_Id]
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] = @Execution_Id

	EXEC [Events_RaiseEvent] 'EXECPT', @Application_Id, @Execution_Id

	/* if successful and there are no promotion contingencies, then...  */
	IF @ExecutionStatus_Name = 'Succeeded' AND @Environment_Id IS NOT NULL AND NOT EXISTS(
			SELECT *
			FROM [BuildPromotionContingencies] BPC
				INNER JOIN [BuildPromotions] BP
						ON BPC.[Contingent_Application_Id] = BP.[Application_Id]
						AND BPC.[Contingent_Release_Number] = BP.[Release_Number]
						AND BPC.[Contingent_Build_Number] = BP.[Build_Number]
						AND BPC.[Contingent_Environment_Id] = BP.[Environment_Id]
			WHERE BPC.[Application_Id] = @Application_Id
			AND BPC.[Release_Number] = @Release_Number
			AND BPC.[Build_Number] = @Build_Number
			AND BPC.[Environment_Id] = @Environment_Id
			AND BP.[PromotionStatus_Name] <> 'Completed')
	BEGIN
		-- complete the promotion
		EXEC [Builds_CompletePromotion]
			@Application_Id = @Application_Id,
			@Release_Number = @Release_Number,
			@Build_Number = @Build_Number,
			@Environment_Id = @Environment_Id
	END
	ELSE IF @ExecutionStatus_Name = 'Failed'
	BEGIN
		UPDATE [BuildPromotions]
		   SET [PromotionStatus_Name] = 'Failed'
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] = @Environment_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_CompleteExecution] TO [BuildMasterUser_Role]


GO
